package org.atlas.streetgeocoder;

import com.vividsolutions.jts.geom.LineSegment;
import java.sql.Connection;
import org.atlas.geocoding.ConfigurationDialog;
import org.atlas.hibernatedata.GeocoderConfig;
import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.Source;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.hibernate.classic.Session;
import org.hibernate.dialect.Dialect;
import org.atlas.util.DataBaseUtils;
import org.atlas.util.FrmWait;
import org.atlas.util.HibernateUtil;
import org.atlas.util.WindowUtils;

public class ConfigDialog extends ConfigurationDialog {

    public ConfigDialog(Project p, HibernateUtil hu, GeocoderConfig g) {
        initComponents();
        setProject(p);
        setHibernate(hu);
        setConfig(g);
        Source[] srcs = new Source[getProject().getSources().size()];
        getProject().getSources().toArray(srcs);

        ArrayList<Source> lsrcs = new ArrayList<Source>();

        for (int i = 0; i < srcs.length; i++) {
            Source source = srcs[i];
            boolean valid = false;
            if (source.getGeomClass().equals(com.vividsolutions.jts.geom.LineString.class)) {
                valid = true;
            }

            if (source.getGeomClass().equals(com.vividsolutions.jts.geom.MultiLineString.class)) {
                valid = true;
            }

            if (valid) {
                lsrcs.add(source);
            }
        }

        if (lsrcs.size() > 0) {
            cmbSource.addItem("[SELECT SOURCE]");
            for (int i = 0; i < lsrcs.size(); i++) {
                cmbSource.addItem(lsrcs.get(i));
            }
        } else {
            cmbSource.addItem("[NO SOURCES FOUND]");
            cmbName.addItem("[NO SOURCES FOUND]");
            cmbSource.setEnabled(false);
        }
        cmbName.setEnabled(false);

    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jButton1 = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        cmbSource = new javax.swing.JComboBox();
        jLabel2 = new javax.swing.JLabel();
        cmbName = new javax.swing.JComboBox();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Config Dialog");
        setLocationByPlatform(true);
        setModal(true);
        setResizable(false);

        jButton1.setText("Continue");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel1.setText("Select streets source");

        cmbSource.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbSourceActionPerformed(evt);
            }
        });

        jLabel2.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel2.setText("Select street's name field");

        cmbName.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbNameActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jLabel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jLabel2, javax.swing.GroupLayout.DEFAULT_SIZE, 219, Short.MAX_VALUE))
                        .addGap(10, 10, 10)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(cmbSource, 0, 249, Short.MAX_VALUE)
                            .addComponent(cmbName, javax.swing.GroupLayout.Alignment.LEADING, 0, 249, Short.MAX_VALUE)))
                    .addComponent(jButton1, javax.swing.GroupLayout.Alignment.TRAILING))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(19, 19, 19)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cmbSource, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cmbName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jButton1)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed

        if (cmbSource.getSelectedIndex() <= 0 || cmbName.getSelectedIndex() <= 0) {
            WindowUtils.msgInfo(this, "Please complete the form");
            return;
        }

        final Source src = (Source) cmbSource.getSelectedItem();
        getConfig().setConfiguration(src.getCodesour() + "#" + cmbName.getSelectedItem());
        FrmWait f = new FrmWait() {

            @Override
            public void action() {
                Dialect d = getHibernate().getConnectionProperties().getDialect();
                String type = d.getTypeName(Types.VARBINARY);
                String sql;
                Session sess = null;
                Connection con = null;
                try {
                    con = getHibernate().getConnection();
                    src.loadFeatures(con);
                    con.close();

                    org.atlas.service.Feature[] fts = src.getFeatures();
                    StreetData[] sdata = new StreetData[fts.length];
                    LineSegment[][] segs = LinearReferencing.findLineSegments(fts);

                    for (int i = 0; i < fts.length; i++) {
                        org.atlas.service.Feature feat = fts[i];
                        sdata[i] = LinearReferencing.findStreetData(feat, fts, segs);
                    }

                    con = getHibernate().getConnection();
                    Statement st = con.createStatement();

                    try {
                        sql = String.format("ALTER TABLE \"%s\" DROP COLUMN binmdata;", src.getTablsour());
                        st.execute(sql);
                    } catch (Exception ex) {
                        System.out.println("Unable to drop");
                    }


                    try {
                        sql = String.format("ALTER TABLE \"%s\" DROP COLUMN binintersectdata;", src.getTablsour());
                        st.execute(sql);
                    } catch (Exception ex) {
                        System.out.println("Unable to drop");
                    }

                    sql = String.format("ALTER TABLE \"%s\" ADD COLUMN binmdata %s;", src.getTablsour(), type);
                    st.execute(sql);

                    sql = String.format("ALTER TABLE \"%s\" ADD COLUMN binintersectdata %s;", src.getTablsour(), type);
                    st.execute(sql);

                    sql = String.format("UPDATE \"%s\" SET binmdata = ?, binintersectdata = ? WHERE atlas27_id = ?", src.getTablsour());
                    PreparedStatement ps = con.prepareStatement(sql);
                    for (int i = 0; i < sdata.length; i++) {
                        ps.setBytes(1, sdata[i].getBinMData());
                        ps.setBytes(2, sdata[i].getBinCrossing());
                        ps.setInt(3, fts[i].getId());
                        ps.executeUpdate();
                    }

                    st.close();
                    con.close();


                    sess = getHibernate().begin();
                    sess.update(getConfig());
                    getHibernate().commitAndClose(sess);
                    setCancelled(false);
                } catch (Exception ex) {
                    Logger.getLogger(ConfigDialog.class.getName()).log(Level.SEVERE, null, ex);
                    getHibernate().close(sess);
                }
            }
        };
        f.setVisible(true);
        this.dispose();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void cmbSourceActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmbSourceActionPerformed
        cmbName.removeAllItems();
        if (cmbSource.getSelectedIndex() > 0) {
            ArrayList<String> lst = new ArrayList<String>();
            Source src = (Source) cmbSource.getSelectedItem();
            for (int i = 0; i < src.getFieldNames().size(); i++) {
                if (DataBaseUtils.isText(src.getFieldTypes().get(i).intValue())) {
                    lst.add(src.getFieldNames().get(i));
                }
            }

            if (lst.size() > 0) {
                cmbName.addItem("[SELECT FIELD]");
                cmbName.setEnabled(true);
                for (int i = 0; i < lst.size(); i++) {
                    cmbName.addItem(lst.get(i));
                }
            } else {
                cmbName.addItem("[NO FIELDS FOUND]");
                cmbName.setEnabled(false);
            }

        } else {
            cmbName.addItem("[SELECT SOURCE]");
            cmbName.setEnabled(false);
        }
}//GEN-LAST:event_cmbSourceActionPerformed

    private void cmbNameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmbNameActionPerformed
        
}//GEN-LAST:event_cmbNameActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox cmbName;
    private javax.swing.JComboBox cmbSource;
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    // End of variables declaration//GEN-END:variables
}
