package org.atlas.streetgeocoder;

import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.io.WKBReader;

import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import javax.swing.JDialog;
import org.atlas.geocoding.ConfigurationDialog;
import org.atlas.geocoding.Geocoder;
import org.atlas.geocoding.Result;
import org.atlas.hibernatedata.GeocoderConfig;
import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.Source;
import org.atlas.util.ConnectionProperties;
import org.atlas.util.DataBaseUtils;
import org.atlas.util.HibernateUtil;
import org.atlas.util.PostGis;

public class StreetGeocoder extends Geocoder {

    private GeocoderConfig gc;
    private Project project;
    private Map<String, StreetData> nameMap;
    private Map<Integer, StreetData> idMap;

    public String getStandardizedAddress(String address) {
        String res = new String(address);
        res = StreetGeocoder.getCleanAddress(res);
        res = Pattern.compile("\\b(CL|CALLE|CLE|CLLE|CAL|CLL)\\b", Pattern.CASE_INSENSITIVE).matcher(res).replaceAll("CL");
        res = Pattern.compile("\\b(CR|CRA|KRA|KR|CARRERA|CRRA|KRRA)\\b", Pattern.CASE_INSENSITIVE).matcher(res).replaceAll("CR");
        res = Pattern.compile("\\b(n0|#|no|num|n)\\b", Pattern.CASE_INSENSITIVE).matcher(res).replaceAll(" ");
        //TODO REEMPLAZOS PARA LOS PTOS CARDINALES
        res = StreetGeocoder.getSingleSpaced(res);
        return res;
    }

    @Override
    public boolean isCapableFor(String address) {
        if (iFind("\\b(CL|CALLE|CLE|CLLE|CLL)\\b", address)) {
            return true;
        }
        if (iFind("\\b(CR|CRA|KRA|KR|CARRERA|CRRA|KRRA)\\b", address)) {
            return true;
        }
        return false;
    }

    @Override
    public String[] parseAdress(String address) throws IllegalArgumentException {
        int sta = 1;
        String resid = "";
        String kind = "";
        String numbe = "";
        String inter = "";
        String house = "";

        for (int i = 0; i < address.length(); i++) {
            String l = address.substring(i, i + 1);
            switch (sta) {
                case 1:
                    if (l.matches("C")) {
                        sta = 2;
                        kind = l;
                    } else {
                        resid += l;
                    }
                    break;
                case 2:
                    if (l.matches("[LR]")) {
                        kind += l;
                        sta = 3;
                    } else {
                        resid += kind;
                        resid += l;
                        kind = "";
                        sta = 1;
                    }
                    break;
                case 3:
                    if (l.matches("\\s")) {
                        sta = 4;
                    } else {
                        resid += kind;
                        resid += l;
                        kind = "";
                        sta = 1;
                    }
                    break;
                case 4:
                    if (l.matches("\\d")) {
                        sta = 5;
                        numbe += l;
                    }
                    break;
                case 5:
                    if (l.matches("\\d")) {
                        numbe += l;
                    } else if (l.matches("[\\s]")) {
                        numbe += l;
                        sta = 6;
                    } else if (l.matches("[\\wÑ]")) {
                        numbe += (" " + l);
                        sta = 6;
                    }
                    break;
                case 6:
                    if (l.matches("\\d")) {
                        inter += l;
                        sta = 7;
                    } else if (l.matches("[\\s\\wÑ]")) {
                        numbe += l;
                    }
                    break;
                case 7:
                    if (l.matches("\\d")) {
                        inter += l;
                    } else if (l.matches("[\\s]")) {
                        inter += l;
                        sta = 8;
                    } else if (l.matches("[\\wÑ]")) {
                        inter += (" " + l);
                        sta = 8;
                    }
                    break;
                case 8:
                    if (l.matches("\\d")) {
                        house += l;
                        sta = 9;
                    } else if (l.matches("[\\s\\wÑ]")) {
                        inter += l;
                    }
                    break;
                case 9:
                    if (l.matches("\\d")) {
                        house += l;
                    } else if (l.matches("[\\s\\wÑ]")) {
                        resid += l;
                        sta = 10;
                    }
                    break;
                case 10:
                    resid += l;
                    break;
            }
        }

        if (sta < 7) {
            throw new IllegalArgumentException("La dirección no tiene los componentes requeridos " + address);
        }

        String[] res = new String[5];
        res[0] = Geocoder.getSingleSpaced(kind);
        res[1] = Geocoder.getSingleSpaced(numbe);
        res[2] = Geocoder.getSingleSpaced(inter);
        res[3] = Geocoder.getSingleSpaced(house);
        res[4] = Geocoder.getSingleSpaced(resid);

        return res;
    }

    private List<Result> findAddresses(StreetData main, StreetData sec, int numb, String name) throws Exception {

        int id = sec.getId();
        List<Result> rs = new ArrayList<Result>();
        for (int i = 0; i < main.getCrossing().length; i++) {
            StreetCrossing sc = main.getCrossing()[i];
            if (sc.getAgainst() == id) {

                double m1 = sc.getM();
                double m2 = Double.NaN;

                StreetCrossing s1 = null;
                StreetCrossing s2 = null;

                if (i > 0) {
                    s1 = main.getCrossing()[i - 1];
                }

                if (i < main.getCrossing().length - 1) {
                    s2 = main.getCrossing()[i + 1];
                }

                if (s1 != null && s2 == null) {
                    m2 = s1.getM();
                } else if (s1 == null && s2 != null) {
                    m2 = s2.getM();
                } else if (s1 != null && s2 != null) {

                    String name1 = idMap.get(s1.getAgainst()).getName();
                    String name2 = idMap.get(s2.getAgainst()).getName();

                    int n1 = -1;
                    int n2 = -1;

                    try {
                        n1 = Integer.parseInt(name1.replaceAll("[^\\d]", ""));
                    } catch (Exception ex) {
                        n1 = -1;
                    }
                    try {
                        n2 = Integer.parseInt(name2.replaceAll("[^\\d]", ""));
                    } catch (Exception ex) {
                        n2 = -1;
                    }

                    if (n1 > n2) {
                        m2 = s1.getM();
                    } else {
                        m2 = s2.getM();
                    }
                } else {
                    m2 = m1;
                }


                if (numb < 0) {
                    numb = (int) ((m1 + m2) / 2);
                }

                double dist;
                if (numb <= Math.abs(m2 - m1)) {
                    dist = m1 + ((m2 - m1) * Math.abs(numb / (m2 - m1)));
                } else {
                    dist = (m1 + m2) / 2;
                }

                if (dist == m1) {
                    if (m1 > m2) {
                        dist -= 0.01;
                    } else {
                        dist += 0.01;
                    }
                }

                Point2D casa = LinearReferencing.getCoordinate(main, dist);
                Point2D esquina = LinearReferencing.getCoordinate(main, m1);

                if (numb % 2 == 0 || numb == 0) {
                    rs.add(new Result(name, findSidePoint(esquina, casa, 10)));
                } else {
                    rs.add(new Result(name, findSidePoint(esquina, casa, -10)));
                }
            }
        }

        if (rs.size() == 0) {
            throw new Exception("Las calles " + main.getName() + " " + sec.getName() + " No se intersectan");
        }

        return rs;
    }

    @Override
    public Result[] locateAddress(String[] parts) throws Exception {
        ArrayList<Result> rs = new ArrayList<Result>();
        ArrayList<StreetData> mains = new ArrayList<StreetData>();
        ArrayList<StreetData> cross = new ArrayList<StreetData>();

        String first = Geocoder.getSingleSpaced(parts[0] + " " + parts[1]).toLowerCase();
        String second;
        int num;
        try {
            num = Integer.parseInt(parts[3]);
        } catch (NumberFormatException numberFormatException) {
            num = -1;
        }



        if (parts[0].equals("CL")) {
            second = Geocoder.getSingleSpaced("CR " + parts[2]).toLowerCase();
        } else {
            second = Geocoder.getSingleSpaced("CL " + parts[2]).toLowerCase();
        }


        StreetData main = nameMap.get(first);
        if (main == null) {
            Iterator<StreetData> ite = nameMap.values().iterator();
            Pattern pat = Pattern.compile(parts[0] + " " + parts[1].replaceAll("[^\\d]", "") + "[.]{0,}", Pattern.CASE_INSENSITIVE);
            while (ite.hasNext()) {
                StreetData i = ite.next();
                if (pat.matcher(i.getName()).matches()) {
                    mains.add(i);
                }
            }
        } else {
            mains.add(main);
        }

        if (mains.size() == 0) {
            throw new Exception("No se encuentra " + first);
        }

        StreetData intesec = nameMap.get(second);
        if (intesec == null) {
            Iterator<StreetData> ite = nameMap.values().iterator();
            Pattern pat;

            if (parts[0].equals("CL")) {
                pat = Pattern.compile("CR " + parts[2].replaceAll("[^\\d]", "") + "[.]{0,}", Pattern.CASE_INSENSITIVE);
            } else {
                pat = Pattern.compile("CL " + parts[2].replaceAll("[^\\d]", "") + "[.]{0,}", Pattern.CASE_INSENSITIVE);
            }

            while (ite.hasNext()) {
                StreetData i = ite.next();
                if (pat.matcher(i.getName()).matches()) {
                    cross.add(i);
                }
            }
        } else {
            cross.add(intesec);
        }

        if (cross.size() == 0) {
            throw new Exception("No se encuentra " + second);
        }


        for (int i = 0; i < mains.size(); i++) {
            StreetData mainstreet = mains.get(i);
            for (int j = 0; j < cross.size(); j++) {
                String name = mainstreet.getName() + " No " + cross.get(j).getName().substring(3) + " - " + parts[3];
                rs.addAll(findAddresses(mainstreet, cross.get(j), num, name));
            }
        }

        Result[] ars = new Result[rs.size()];
        rs.toArray(ars);
        return ars;
    }

    public static void main(String[] args) throws IOException, ClassNotFoundException {

    }

    @Override
    public String getName() {
        return "Street Crossing geocoder";
    }

    @Override
    public String getDescription() {
        return "Street Geocoder designed to meet the requirements of our city";
    }

    @Override
    public JDialog getAboutDialog() {
        return new AboutDlg();
    }

    @Override
    public ConfigurationDialog getConfigurationDialog(HibernateUtil hibernate, Project project, GeocoderConfig config) {       
        
        return new ConfigDialog(project, hibernate, config);
    }

    @Override
    public void setGeocoderConfig(GeocoderConfig gc) {
        this.gc = gc;
    }

    @Override
    public void init(ConnectionProperties c) throws IOException, SQLException, ClassNotFoundException, Exception {
        String conf = gc.getConfiguration();
        int srcId = Integer.parseInt(conf.substring(0, conf.indexOf("#")));

        Iterator it = getProject().getSources().iterator();
        Source src = null;
        while (it.hasNext()) {
            Source t = (Source) it.next();
            if (t.getCodesour() == srcId) {
                src = t;
            }
        }

        if (src == null) {
            throw new Exception("Source " + srcId + " no found, please reconfigure");
        }

        String tabName = src.getTablsour();
        String fldName = conf.substring(conf.indexOf("#") + 1, conf.length());

        Connection conn = DataBaseUtils.createConnection(c);
        Statement sta = conn.createStatement();


        boolean postgis = false;
        if (PostGis.isPostgisInstalled(sta)) {
            if (PostGis.isScriptPresent(sta)) {
                postgis = true;
            }
        }

        String geom;

        if (postgis) {
            geom = String.format("asbinary(\"%s\") as %s ", src.getFielsour(), src.getFielsour());
        } else {
            geom = src.getFielsour();
        }

        String sql = String.format("SELECT atlas27_id, %s , %s, binmdata, binintersectdata   FROM \"%s\";", geom, fldName, tabName);
        ResultSet rs = sta.executeQuery(sql);

        nameMap = new HashMap<String, StreetData>();
        idMap = new HashMap<Integer, StreetData>();
        WKBReader reader = new WKBReader();

        while (rs.next()) {
            StreetData sd = new StreetData();
            sd.setBinCrossing(rs.getBytes("binintersectdata"));
            sd.setBinMData(rs.getBytes("binmdata"));
            sd.setId(rs.getInt("atlas27_id"));
            sd.setName(rs.getString("name"));
            sd.setData((MultiLineString) reader.read(rs.getBytes(src.getFielsour())));
            nameMap.put(sd.getName(), sd);
            idMap.put(sd.getId(), sd);
        }
    }

    /**
     * 
     * @param esquina
     * @param casa
     * @param dist
     * @return
     */
    public static Point2D findSidePoint(Point2D esquina, Point2D casa, int dist) {
        double dx = casa.getX() - esquina.getX();
        double dy = casa.getY() - esquina.getY();

//        if(dx*dy==0){
//            return esquina;
//        }

        double ang = Math.acos(dx / Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2)));
        if (dy < 0) {
            ang = (2 * Math.PI) - ang;
        }

        AffineTransform at = new AffineTransform();
        at.translate(casa.getX(), casa.getY());
        at.rotate(ang - (Math.PI / 2));

        Point2D p1 = new Point2D.Double(dist, 0);
        Point2D p2 = new Point2D.Double();
        at.transform(p1, p2);
        return p2;
    }

    @Override
    public Project getProject() {
        return project;
    }

    @Override
    public void setProject(Project project) {
        this.project = project;
    }

    @Override
    public boolean isUsingSource(int idsour) {
        if(gc==null)throw new IllegalStateException("SetGeocoderConfig must be called first.");
        if(gc.getConfiguration()==null)return false;
        if(gc.getConfiguration().length()==0)return false;
        String[] parts = gc.getConfiguration().split("#");
        if(parts.length==2){
            int id = Integer.parseInt(parts[0]);
            if(id == idsour){
                return true;
            }
        }
        return false;
    }
}
