/*
 * Created on 30 de abril de 2007, 07:46 PM
 */

package windows.connections;

import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.net.URL;
import java.util.Collections;
import java.util.Comparator;
import javax.swing.DefaultComboBoxModel;
import org.atlas.util.Configuration;
import org.atlas.i18n.LocaleUtil;
import org.atlas.util.ServiceConnection;
import org.atlas.util.WindowUtils;

public class FrmConnectionProps extends javax.swing.JDialog {
    
    private Configuration config;
    private ServiceConnection serviceConnection;
    private boolean userCancel;
        
    public FrmConnectionProps() {
        super((Frame)null, true);
        initComponents();
        this.addWindowListener(new WindowAdapter(){
            @Override
            public void windowClosing(WindowEvent windowEvent) {
                userCancel = true;
            }
        });
        
        WindowUtils.setPanelEnabled(this.pnlConnProps,false);
        config = Configuration.grantReadConfiguration("config.xml");
        noConnection();
        loadComboCon();
        org.atlas.util.WindowUtils.center(this);
        org.atlas.util.WindowUtils.setFrameIcon(this,"/media/appicon.png");
    }
    
    private void loadComboCon(){
        DefaultComboBoxModel model = new DefaultComboBoxModel();
        model.addElement(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("select_conn"));
        model.addElement(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("new_connection"));
        for (int i = 0; i < config.getConnections().size(); i++) {
            model.addElement(config.getConnections().get(i));
        }
        
        this.cmbConn.setModel(model);
        int index = this.config.getCurrentConnection();
        if(index>=0){
            ServiceConnection sc = (ServiceConnection)this.config.getConnections().get(index);
            this.cmbConn.setSelectedItem(sc);
        }
    }
    
    private int getCurrentConnectionIndex(){
        for (int i = 0; i < this.config.getConnections().size(); i++) {
            if(this.getServiceConnection()==config.getConnections().get(i)){
                return i;
            }
        }
        return -1;
    }
    
    private void clearForm(){
        this.txtConnName.setText("");
        this.txtHost.setText("");
        this.txtUser.setText("");
        this.txtPass.setText("");
    }
    
    private void newConnection(){
        clearForm();
        WindowUtils.setPanelEnabled(this.pnlConnProps,true);
        this.txtConnName.setEnabled(true);
        this.txtConnName.requestFocus(true);
        this.btnSave.setEnabled(true);
        this.btnDelete.setEnabled(false);
        this.btnConnect.setEnabled(false);
    }
    
    private void noConnection(){
        clearForm();
        WindowUtils.setPanelEnabled(this.pnlConnProps,false);
        this.txtConnName.setEnabled(false);
        this.btnSave.setEnabled(false);
        this.btnDelete.setEnabled(false);
        this.btnConnect.setEnabled(false);
    }
    
    private void showConnection(ServiceConnection cp){
        this.txtConnName.setText(cp.getName());
        this.txtHost.setText(cp.getAppUrl());
        this.txtUser.setText(cp.getUsername());
        this.txtPass.setText(cp.getPassword());
        WindowUtils.setPanelEnabled(this.pnlConnProps,true);
        this.txtConnName.setEnabled(true);
        this.btnSave.setEnabled(true);
        this.btnDelete.setEnabled(true);
        this.btnConnect.setEnabled(true);
    }
    
    private ServiceConnection generateConnectioProperties() throws IllegalStateException{
        if(this.txtConnName.getText().trim().equals(""))throw new IllegalStateException(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("req_name"));
        if(this.txtHost.getText().trim().equals(""))throw new IllegalStateException(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("req_host"));
        if(this.txtUser.getText().trim().equals(""))throw new IllegalStateException(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("uname_req"));
        if(this.txtPass.getPassword().toString().trim().equals(""))throw new IllegalStateException(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("pass_req"));
        try {
            new URL(txtHost.getText());
        } catch (MalformedURLException ex) {
            throw new IllegalStateException(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("inval_host"));
        }
        ServiceConnection conn = new ServiceConnection();
        conn.setName(this.txtConnName.getText());
        conn.setAppUrl(this.txtHost.getText());
        conn.setUsername(this.txtUser.getText());
        conn.setPassword(new String(this.txtPass.getPassword()));
        return conn;
    }
    
    
    
    private void setUserCancel(boolean userCancel) {
        this.userCancel = userCancel;
    }
    
    private void save() throws Exception{
        ServiceConnection cp;
        try {
            cp = generateConnectioProperties();
        } catch (IllegalStateException ex) {
            WindowUtils.msgError(this,ex.getMessage());
            throw ex;
        }
        
        this.config.getConnections().add(cp);
        if(this.getServiceConnection() != null){
            this.config.getConnections().remove(getServiceConnection());
        }
        
        this.serviceConnection = cp;
        Collections.sort(config.getConnections(),new Comparator(){
            public int compare(Object ob1, Object ob2) {
                return ob1.toString().compareTo(ob2.toString());
            }
        });
        this.config.setCurrentConnection(this.getCurrentConnectionIndex());
        try {
            this.config.writeConfiguration("config.xml");
            loadComboCon();
        } catch (Exception ex) {
            this.config.getConnections().remove(cp);
            WindowUtils.msgError(this,LocaleUtil.getString("windows/connections/messages","writeconferror"));
            throw ex;
        }
    }
    
    
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        pnlConnProps = new javax.swing.JPanel();
        jLabel6 = new javax.swing.JLabel();
        txtConnName = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        txtHost = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        txtUser = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        txtPass = new javax.swing.JPasswordField();
        jPanel3 = new javax.swing.JPanel();
        lblConnections = new javax.swing.JLabel();
        cmbConn = new javax.swing.JComboBox();
        pnlConnBtns = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        btnCancel = new javax.swing.JButton();
        btnDelete = new javax.swing.JButton();
        btnSave = new javax.swing.JButton();
        btnConnect = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("con_props"));
        setResizable(false);
        pnlConnProps.setLayout(new java.awt.GridLayout(4, 2, 5, 5));

        jLabel6.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel6.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("conn_name"));
        jLabel6.setMaximumSize(new java.awt.Dimension(74, 15));
        jLabel6.setMinimumSize(new java.awt.Dimension(74, 15));
        pnlConnProps.add(jLabel6);

        pnlConnProps.add(txtConnName);

        jLabel7.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel7.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("URL:"));
        pnlConnProps.add(jLabel7);

        pnlConnProps.add(txtHost);

        jLabel3.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel3.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("username"));
        pnlConnProps.add(jLabel3);

        pnlConnProps.add(txtUser);

        jLabel4.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel4.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("password"));
        pnlConnProps.add(jLabel4);

        pnlConnProps.add(txtPass);

        jPanel3.setLayout(new java.awt.GridLayout(1, 2, 5, 5));

        lblConnections.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        lblConnections.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("connections"));
        jPanel3.add(lblConnections);

        cmbConn.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        cmbConn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbConnActionPerformed(evt);
            }
        });

        jPanel3.add(cmbConn);

        pnlConnBtns.setLayout(new java.awt.GridLayout(1, 4, 3, 3));

        jPanel1.setLayout(new java.awt.GridLayout(1, 0, 5, 5));

        btnCancel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/process-stop.png")));
        btnCancel.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("Cancel"));
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        jPanel1.add(btnCancel);

        btnDelete.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/trash.png")));
        btnDelete.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("delete"));
        btnDelete.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDeleteActionPerformed(evt);
            }
        });

        jPanel1.add(btnDelete);

        btnSave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/save.png")));
        btnSave.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("save"));
        btnSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSaveActionPerformed(evt);
            }
        });

        jPanel1.add(btnSave);

        btnConnect.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/connect.png")));
        btnConnect.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("continue"));
        btnConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnConnectActionPerformed(evt);
            }
        });

        jPanel1.add(btnConnect);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addGap(596, 596, 596)
                .addComponent(pnlConnBtns, javax.swing.GroupLayout.DEFAULT_SIZE, 0, Short.MAX_VALUE))
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, 576, Short.MAX_VALUE)
                .addContainerGap())
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addGap(12, 12, 12)
                        .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 564, Short.MAX_VALUE))
                    .addComponent(pnlConnProps, javax.swing.GroupLayout.DEFAULT_SIZE, 576, Short.MAX_VALUE))
                .addContainerGap())
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 576, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(pnlConnBtns, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pnlConnProps, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    private void btnSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSaveActionPerformed
        try {
            save();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }//GEN-LAST:event_btnSaveActionPerformed
    
    private void btnDeleteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDeleteActionPerformed
        int index = this.cmbConn.getSelectedIndex();
        if(this.cmbConn.getSelectedIndex()>=2){
            if(WindowUtils.msgYesNo(this, LocaleUtil.getString("windows/connections/messages","want_to_delete"))){
                ServiceConnection cp = (ServiceConnection)cmbConn.getSelectedItem();
                config.getConnections().remove(cp);
                config.setCurrentConnection(-1);
                this.serviceConnection = null;
                Collections.sort(config.getConnections(),new Comparator(){
                    public int compare(Object ob1, Object ob2) {
                        return ob1.toString().compareTo(ob2.toString());
                    }
                });
                try {
                    this.config.writeConfiguration("config.xml");
                    loadComboCon();
                    noConnection();
                } catch (Exception ex) {
                    this.config.getConnections().remove(cp);
                    WindowUtils.msgError(this,LocaleUtil.getString("windows/connections/messages","writeconferror"));
                }
            }
        }
    }//GEN-LAST:event_btnDeleteActionPerformed
    
    private void cmbConnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmbConnActionPerformed
        int index = this.cmbConn.getSelectedIndex();
        if(this.cmbConn.getSelectedIndex()>=0){
            if(this.cmbConn.getSelectedIndex()==0){
                noConnection();
                this.serviceConnection = null;
            }else if(this.cmbConn.getSelectedIndex()==1){
                newConnection();
                this.serviceConnection = null;
            }else{
                ServiceConnection cp = (ServiceConnection)this.cmbConn.getSelectedItem();
                this.serviceConnection = cp;
                showConnection(cp);
            }
        }
    }//GEN-LAST:event_cmbConnActionPerformed
    
    private void btnConnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnConnectActionPerformed
        try {
            save();
            userCancel = false;
        this.dispose();
        } catch (Exception ex) {
            ex.printStackTrace();
            return;
        }
    }//GEN-LAST:event_btnConnectActionPerformed
    
    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        this.userCancel = true;
        this.setVisible(false);
    }//GEN-LAST:event_btnCancelActionPerformed
        
    public ServiceConnection getServiceConnection() {
        return serviceConnection;
    }
    
    public boolean isUserCancel() {
        return userCancel;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnConnect;
    private javax.swing.JButton btnDelete;
    private javax.swing.JButton btnSave;
    private javax.swing.JComboBox cmbConn;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JLabel lblConnections;
    private javax.swing.JPanel pnlConnBtns;
    private javax.swing.JPanel pnlConnProps;
    private javax.swing.JTextField txtConnName;
    private javax.swing.JTextField txtHost;
    private javax.swing.JPasswordField txtPass;
    private javax.swing.JTextField txtUser;
    // End of variables declaration//GEN-END:variables
    
}
