/*
 * Legends.java
 *
 * Created on 23 de marzo de 2008, 12:29 PM
 */
package windows.controls;

import org.atlas.hibernatedata.Layer;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.SystemColor;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;

/**
 *
 * @author  Mario
 */
public class Legends extends javax.swing.JPanel {

    private int currentLayer = -1;
    private Layer[] layers = new Layer[0];
    private BufferedImage[] legends = new BufferedImage[0];
    private String wmsUrl = "";
    private static final int alto = 20;    
    private int[] pos = new int[0];
    private BufferedImage opened;
    private BufferedImage closed;

    /** Creates new form Legends */
    public Legends() {
        try {
            opened = ImageIO.read(this.getClass().getResourceAsStream("/windows/controls/opened.png"));
            closed = ImageIO.read(this.getClass().getResourceAsStream("/windows/controls/closed.png"));
            initComponents();
        } catch (IOException ex) {
            Logger.getLogger(Legends.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * 
     * @param name 
     * @return 
     * @throws java.lang.Exception
     */
    public BufferedImage getLeyend(String name) throws Exception {
        URL dir = new URL(getWmsUrl() + "?admin_entrace=true&request=GetLegend&layers=" + name);
        HttpURLConnection con = (HttpURLConnection) dir.openConnection();
        if (con.getResponseCode() == HttpURLConnection.HTTP_OK) {
            if (con.getContentType().startsWith("image/")) {
                InputStream is = con.getInputStream();
                BufferedImage img = ImageIO.read(is);
                is.close();
                return img;
            }
        }
        throw new Exception("HTTP error");
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                formMouseReleased(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    private void formMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseReleased

        int v = evt.getY();
        int curr = -1;

        for (int i = 0; i < layers.length - 1; i++) {
            if (pos[i] <= v && v <= pos[i + 1]) {
                curr = i;
                break;
            }
        }

        if (curr == -1) {
            curr = layers.length - 1;
        }

        if (curr == currentLayer) {
            currentLayer = -1;
        } else {
            currentLayer = curr;
        }
        
        openLayer();
    }//GEN-LAST:event_formMouseReleased

    /**
     * 
     * @return
     */
    public Layer[] getLayers() {
        return layers;
    }

    /**
     * 
     * @param layers
     */
    public void setLayers(Layer[] layers) {
        this.layers = layers;
        legends = new BufferedImage[layers.length];
        pos = new int[layers.length];
        currentLayer = -1;
        openLayer();
    }

    /**
     * 
     * @return
     */
    public String getWmsUrl() {
        return wmsUrl;
    }

    /**
     * 
     * @param wmsUrl
     */
    public void setWmsUrl(String wmsUrl) {
        this.wmsUrl = wmsUrl;
    }

    private void openLayer() {        
        int y = - alto;
        for (int i = 0; i < layers.length; i++) {
            y += alto;
            pos[i] = y;
            if (i == currentLayer) {
                if (legends[i] == null) {
                    try {
                        legends[i] = getLeyend(layers[i].getNameLaye());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                y += (legends[i].getHeight() + 1);
            }
        }
                
        int altoC = y + alto;
        setPreferredSize(new Dimension(100, altoC));
        revalidate();
        repaint();
        
    }

    @Override
    public void paint(Graphics g) {                
        int y = - alto;        
        super.paint(g);
        for (int i = 0; i < layers.length; i++) {

            SystemColor back;
            SystemColor text;
            Layer layer = layers[i];
            BufferedImage ic;
            y += alto;
            
            if (i == currentLayer) {
                ic = opened;
                back = java.awt.SystemColor.textHighlight;
                text = java.awt.SystemColor.textHighlightText;
            } else {
                ic = closed;
                back = java.awt.SystemColor.text;
                text = java.awt.SystemColor.textText;
            }

            g.setColor(back);
            g.fillRect(0, y, getWidth()-1 , alto);
            g.setColor(java.awt.SystemColor.controlDkShadow);
            g.drawRect(0, y, getWidth()-1 , alto);
            g.setColor(text);

            g.drawImage(ic, 4, y + (alto - 10) / 2, null);

            int vPos = (int) (y + ((double) alto / 2) + (((double) g.getFontMetrics().getAscent()) / 2));
            g.drawString(layer.getDesclaye(), 20, vPos);

            if (i == currentLayer) {
                g.setColor(new Color(255, 255, 255));
                g.fillRect(0, y + alto, getWidth()-1, legends[i].getHeight() + 1);
                g.setColor(java.awt.SystemColor.controlDkShadow);
                g.drawRect(0, y + alto, getWidth()-1, legends[i].getHeight() + 1);
                g.drawImage(legends[i], 1, y + alto + 1, null);
                y += legends[i].getHeight();
                y += 1;
            }
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
