/*
 * Created on 9 de junio de 2007, 08:08 AM
 */
package windows.controls;

import org.atlas.exceptions.HTTPRequestException;
import org.atlas.hibernatedata.Layer;
import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.Service;
import org.atlas.hibernatedata.Source;
import org.atlas.i18n.LocaleUtil;
import org.atlas.i18n.StringProvider;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.ConnectException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import org.hibernate.classic.Session;
import org.atlas.util.HibernateUtil;
import org.atlas.util.ServiceConnection;
import org.atlas.util.WindowUtils;
import org.atlas.util.FrmWait;
import windows.fonts.FrmFonts;
import windows.layers.FrmLayerProps;
import windows.sources.FrmLayerSrc;
import windows.symbology.FrmSymbLine;
import windows.symbology.FrmSymbPoint;
import windows.symbology.FrmSymbPoly;

public class PnlLayers extends javax.swing.JPanel {

    private List layerExistsChangeListeners = new ArrayList();
    private List sourceChangeListeners = new ArrayList();
    private List styleChangeListeners = new ArrayList();
    private List scaleDenomChangeListeners = new ArrayList();
    private List orderChangeListeners = new ArrayList();
    private List showHideChangeListeners = new ArrayList();
    private List selectionChangeListeners = new ArrayList();
    private HibernateUtil hibernate;
    private LayerTableModel model;
    private Project project;
    private Service service;
    private ServiceConnection serviceConnection;
    private PnlProjects pnlProyect;
    private PnlViewer pnlViewer;
    private List projectChangeListeners = new ArrayList();
    private StringProvider sp = new StringProvider("windows/controls/messages");
    private boolean sendSelecionChanges = true;

    public void addScaleDenomChangeListener(ActionListener al) {
        scaleDenomChangeListeners.add(al);
    }

    public void removeScaleDenomChangeListener(ActionListener al) {
        scaleDenomChangeListeners.remove(al);
    }

    public void addOrderChangeListener(ActionListener al) {
        orderChangeListeners.add(al);
    }

    public void removeOrderChangeListener(ActionListener al) {
        orderChangeListeners.remove(al);
    }

    public void addStyleChangeListener(ActionListener al) {
        styleChangeListeners.add(al);
    }

    public void removeStyleChangeListener(ActionListener al) {
        styleChangeListeners.remove(al);
    }

    public void addLayerDeleteChangeListener(ActionListener al) {
        layerExistsChangeListeners.add(al);
    }

    public void removeLayerDeleteChangeListener(ActionListener al) {
        layerExistsChangeListeners.remove(al);
    }

    public void addSourceChangeListener(ActionListener al) {
        sourceChangeListeners.add(al);
    }

    public void removeSourceChangeListener(ActionListener al) {
        sourceChangeListeners.remove(al);
    }

    public void addShowHideChangeListener(ActionListener al) {
        showHideChangeListeners.add(al);
    }

    public void removeShowHideChangeListener(ActionListener al) {
        showHideChangeListeners.remove(al);
    }

    public void addSelectionChangeListener(ActionListener al) {
        selectionChangeListeners.add(al);
    }

    public void removeSelectionChangeListener(ActionListener al) {
        selectionChangeListeners.remove(al);
    }

    public void addProjectChangeListener(ActionListener al) {
        projectChangeListeners.add(al);
    }

    public void removeProjectChangeListener(ActionListener al) {
        projectChangeListeners.remove(al);
    }

    public Layer[] getVisibleLayers() {
        return this.model.getVisibleLayers();
    }

    public void fireListeners(List cl) {
        ActionEvent ae = new ActionEvent(this, 0, "");
        for (int i = 0; i < cl.size(); i++) {
            ((ActionListener) cl.get(i)).actionPerformed(ae);
        }
    }

    public PnlLayers() {
        initComponents();
        this.model = new LayerTableModel();
        this.tblLayers.setModel(model);
        this.model.addTableModelListener(new TableModelListener() {

            public void tableChanged(TableModelEvent ev) {
                if (sendSelecionChanges) {
                    if (ev.getColumn() == 0) {
                        fireListeners(showHideChangeListeners);
                    }
                }
            }
        });

        this.tblLayers.getColumnModel().getColumn(0).setMinWidth(40);
        this.tblLayers.getColumnModel().getColumn(0).setPreferredWidth(50);
        this.tblLayers.getColumnModel().getColumn(0).setMaxWidth(60);

        this.tblLayers.getColumnModel().getColumn(2).setMinWidth(40);
        this.tblLayers.getColumnModel().getColumn(2).setPreferredWidth(80);
        this.tblLayers.getColumnModel().getColumn(2).setMaxWidth(120);

        this.tblLayers.getColumnModel().getColumn(3).setMinWidth(40);
        this.tblLayers.getColumnModel().getColumn(3).setPreferredWidth(80);
        this.tblLayers.getColumnModel().getColumn(3).setMaxWidth(120);

        this.tblLayers.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent ev) {
                if (!ev.getValueIsAdjusting()) {
                    fireListeners(selectionChangeListeners);
                }
            }
        });

        tblLayers.getSelectionModel().setSelectionMode(tblLayers.getSelectionModel().SINGLE_SELECTION);
        WindowUtils.setPanelEnabled(this.panelCapa, false);
        
    }

    public void setHibernate(HibernateUtil hibernate) {
        this.hibernate = hibernate;
    }

    private void saveDenomScale(final boolean min, final boolean max, final double scale) {
        final Layer layer = getCheckedLayer();
        if (layer != null) {
            FrmWait w = new FrmWait() {

                public void action() {
                    Session sess = null;
                    try {
                        sess = hibernate.begin();
                        sess.update(layer);
                        if (min) {
                            layer.setMinscaledeno(scale);
                            if (!max && layer.getMinscaledeno() >= layer.getMaxscaledeno()) {
                                layer.setMaxscaledeno(0);
                            }
                        }
                        if (max) {
                            layer.setMaxscaledeno(scale);
                            if (!min && layer.getMinscaledeno() >= layer.getMaxscaledeno()) {
                                layer.setMinscaledeno(0);
                            }
                        }

                    } catch (Exception ex) {
                        ex.printStackTrace();
                    } finally {
                        try {
                            hibernate.commitAndClose(sess);
                            try {
                                service.updateCapabilities(hibernate, serviceConnection);
                                int i = tblLayers.getSelectedRow();
                                loadModel();
                                tblLayers.getSelectionModel().setSelectionInterval(i, i);
                                fireListeners(scaleDenomChangeListeners);
                            } catch (Exception ex) {
                                ex.printStackTrace();
                            }
                        } catch (Exception ex) {
                            System.out.println(ex);
                            hibernate.close(sess);
                        }
                    }
                }
            };
            w.setMessage(sp.getString("saving_changes"));
            w.setVisible(true);
        }
    }

    private void saveZorder() throws Exception {
        int cont = this.model.getRowCount();
        Session sess = hibernate.begin();
        for (int i = 0; i < cont; i++) {
            this.model.getLayerAt(i).setZorder(i);
            sess.update(this.model.getLayerAt(i));
        }
        hibernate.commitAndClose(sess);
    }

    public void setProject(final Project iproject) {
        new FrmWait() {
            @Override
            public void action() {
                project = iproject;
                try {
                    projectChange(project);
                } catch (Exception ex) {
                    ex.printStackTrace();
                    WindowUtils.msgError(null, "Unexpected error");
                    dispose();
                }
            }
        }.setVisible(true);
    }

    private void projectChange(Project p) throws Exception {
        if (project != null) {
            loadLayers();
            loadModel();
            setEnabled(true);
            legends1.setWmsUrl(serviceConnection.getWmsUrl());
        } else {
            model.resetModel();
            setEnabled(false);
        }
        
    }

    @Override
    public void setEnabled(boolean b) {
        jTabbedPane1.setSelectedIndex(0);
        WindowUtils.setPanelEnabled(panelCapa, b);
        WindowUtils.setPanelEnabled(jTabbedPane1, b);
        WindowUtils.setPanelEnabled(jPanel1, b);
        WindowUtils.setPanelEnabled(jPanel2, b);
        tblLayers.setEnabled(b);
    }

    private void loadLayers() throws Exception {
        Session sess = hibernate.begin();
        sess.update(getProject());
        Object[] tmp = getProject().getLayers().toArray();
        if (tmp.length > 0) {
            for (int i = 0; i < tmp.length; i++) {
                Layer l = (Layer) tmp[i];
                sess.update(l);
                Source src = l.getSource();
                if (src != null) {
                    sess.update(src);
                    src.getTablsour();
                }
            }
        }
        hibernate.close(sess);
    }

    private void loadModel() {
        model.resetModel();
        Object[] tmp = getProject().getLayers().toArray();
        if (tmp.length > 0) {
            for (int i = 0; i < tmp.length; i++) {
                model.addLayer((Layer) tmp[i]);
            }
            model.sortList();
        }
        legends1.setLayers(model.getLayers());
    }

    public int[] getSelectionCodes() {
        int cont = this.model.getCheckedCount();
        int[] res = new int[cont];
        int x = 0;
        for (int i = 0; i < this.model.getRowCount(); i++) {
            if (this.model.getBooleanAt(i)) {
                res[x] = this.model.getLayerAt(i).getCodelaye();
                x++;
            }
        }
        return res;
    }

    public void reset() {
        this.hibernate = null;
        this.project = null;
        this.serviceConnection = null;
        this.service = null;
        this.model.resetModel();
        setEnabled(false);
        fireListeners(projectChangeListeners);
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        menu = new javax.swing.JPopupMenu();
        rm_min_scal = new javax.swing.JMenuItem();
        rm_max_scal = new javax.swing.JMenuItem();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        tblLayers = new javax.swing.JTable();
        panelCapa = new javax.swing.JPanel();
        btnEditFont = new javax.swing.JButton();
        btnAddLayer = new javax.swing.JButton();
        btnDelLayer = new javax.swing.JButton();
        bntAdminAttributes = new javax.swing.JButton();
        btnEditSymbology = new javax.swing.JButton();
        btnMoveLayerUp = new javax.swing.JButton();
        btnMoveLayerDown = new javax.swing.JButton();
        btnEditLayer = new javax.swing.JButton();
        btnMinScale = new javax.swing.JButton();
        btnMaxScale = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        legends1 = new windows.controls.Legends();

        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/controls/messages",LocaleUtil.getLocale()); // NOI18N
        rm_min_scal.setText(bundle.getString("rm_min_scale")); // NOI18N
        rm_min_scal.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rm_min_scalActionPerformed(evt);
            }
        });
        menu.add(rm_min_scal);

        rm_max_scal.setText(bundle.getString("rm_max_scale")); // NOI18N
        rm_max_scal.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rm_max_scalActionPerformed(evt);
            }
        });
        menu.add(rm_max_scal);

        jTabbedPane1.setEnabled(false);

        tblLayers.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {},
                {},
                {},
                {}
            },
            new String [] {

            }
        ));
        tblLayers.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                tblLayersMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                tblLayersMouseReleased(evt);
            }
        });
        jScrollPane1.setViewportView(tblLayers);

        panelCapa.setLayout(new java.awt.GridBagLayout());

        btnEditFont.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/format-text-bold.png"))); // NOI18N
        btnEditFont.setDefaultCapable(false);
        btnEditFont.setIconTextGap(0);
        btnEditFont.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnEditFont.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnEditFontActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        panelCapa.add(btnEditFont, gridBagConstraints);

        btnAddLayer.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/plus.png"))); // NOI18N
        btnAddLayer.setIconTextGap(0);
        btnAddLayer.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnAddLayer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddLayerActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        panelCapa.add(btnAddLayer, gridBagConstraints);

        btnDelLayer.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/trash.png"))); // NOI18N
        btnDelLayer.setDefaultCapable(false);
        btnDelLayer.setIconTextGap(0);
        btnDelLayer.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnDelLayer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDelLayerActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 10);
        panelCapa.add(btnDelLayer, gridBagConstraints);

        bntAdminAttributes.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/table.png"))); // NOI18N
        bntAdminAttributes.setDefaultCapable(false);
        bntAdminAttributes.setIconTextGap(0);
        bntAdminAttributes.setMargin(new java.awt.Insets(0, 0, 0, 0));
        bntAdminAttributes.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bntAdminAttributesActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        panelCapa.add(bntAdminAttributes, gridBagConstraints);

        btnEditSymbology.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/paint.png"))); // NOI18N
        btnEditSymbology.setDefaultCapable(false);
        btnEditSymbology.setIconTextGap(0);
        btnEditSymbology.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnEditSymbology.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnEditSymbologyActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 10);
        panelCapa.add(btnEditSymbology, gridBagConstraints);

        btnMoveLayerUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/up.png"))); // NOI18N
        btnMoveLayerUp.setDefaultCapable(false);
        btnMoveLayerUp.setIconTextGap(0);
        btnMoveLayerUp.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnMoveLayerUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnMoveLayerUpActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 8;
        gridBagConstraints.gridy = 0;
        panelCapa.add(btnMoveLayerUp, gridBagConstraints);

        btnMoveLayerDown.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/down.png"))); // NOI18N
        btnMoveLayerDown.setDefaultCapable(false);
        btnMoveLayerDown.setIconTextGap(0);
        btnMoveLayerDown.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnMoveLayerDown.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnMoveLayerDownActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 9;
        gridBagConstraints.gridy = 0;
        panelCapa.add(btnMoveLayerDown, gridBagConstraints);

        btnEditLayer.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/edit.png"))); // NOI18N
        btnEditLayer.setDefaultCapable(false);
        btnEditLayer.setIconTextGap(0);
        btnEditLayer.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnEditLayer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnEditLayerActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        panelCapa.add(btnEditLayer, gridBagConstraints);

        btnMinScale.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/min_deno.png"))); // NOI18N
        btnMinScale.setDefaultCapable(false);
        btnMinScale.setIconTextGap(0);
        btnMinScale.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnMinScale.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnMinScaleActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 0;
        panelCapa.add(btnMinScale, gridBagConstraints);

        btnMaxScale.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/max_deno.png"))); // NOI18N
        btnMaxScale.setDefaultCapable(false);
        btnMaxScale.setIconTextGap(0);
        btnMaxScale.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnMaxScale.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnMaxScaleActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 10);
        panelCapa.add(btnMaxScale, gridBagConstraints);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(panelCapa, javax.swing.GroupLayout.PREFERRED_SIZE, 320, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 442, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(panelCapa, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 331, Short.MAX_VALUE)
                .addContainerGap())
        );

        java.util.ResourceBundle bundle1 = java.util.ResourceBundle.getBundle("windows/controls/messages"); // NOI18N
        jTabbedPane1.addTab(bundle1.getString("layers"), jPanel1); // NOI18N

        jScrollPane2.setViewportView(legends1);

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 442, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 370, Short.MAX_VALUE)
                .addContainerGap())
        );

        jTabbedPane1.addTab(bundle1.getString("legends"), jPanel2); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 467, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 420, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    private void rm_max_scalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rm_max_scalActionPerformed
        saveDenomScale(false, true, 0);
    }//GEN-LAST:event_rm_max_scalActionPerformed

    private void rm_min_scalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rm_min_scalActionPerformed
        saveDenomScale(true, false, 0);
    }//GEN-LAST:event_rm_min_scalActionPerformed

    private void btnMaxScaleActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnMaxScaleActionPerformed
        saveDenomScale(false, true, this.pnlViewer.getWmsScale());
    }//GEN-LAST:event_btnMaxScaleActionPerformed

    private void btnMinScaleActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnMinScaleActionPerformed
        saveDenomScale(true, false, this.pnlViewer.getWmsScale());
    }//GEN-LAST:event_btnMinScaleActionPerformed

    private void tblLayersMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tblLayersMouseReleased
        if (evt.isPopupTrigger()) {
            int r = this.tblLayers.rowAtPoint(evt.getPoint());
            this.tblLayers.getSelectionModel().setSelectionInterval(r, r);
            this.menu.show(this.tblLayers, evt.getX(), evt.getY());
        }
    }//GEN-LAST:event_tblLayersMouseReleased

    private void tblLayersMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_tblLayersMousePressed
        if (evt.isPopupTrigger()) {
            int r = this.tblLayers.rowAtPoint(evt.getPoint());
            this.tblLayers.getSelectionModel().setSelectionInterval(r, r);
            this.menu.show(this.tblLayers, evt.getX(), evt.getY());
        }
    }//GEN-LAST:event_tblLayersMousePressed

    private Layer getCheckedLayer() {
        int index = this.tblLayers.getSelectedRow();
        if (index >= 0) {
            return model.getLayerAt(index);
        } else {
            WindowUtils.msgInfo(this.getRootPane(), java.util.ResourceBundle.getBundle("windows/controls/messages", LocaleUtil.getLocale()).getString("sellayer"));
        }
        return null;
    }

    public Layer[] getSelectedLayers() {
        if (tblLayers.getSelectedRowCount() > 0) {
            int indexes[] = tblLayers.getSelectedRows();
            ArrayList layers = new ArrayList();
            for (int i = 0; i < indexes.length; i++) {
                Layer layer = model.getLayerAt(indexes[i]);
                if (layer.getSource() != null) {
                    layers.add(layer);
                }
            }

            if (layers.size() > 0) {
                Layer[] ls = new Layer[layers.size()];
                layers.toArray(ls);
                return ls;
            }
        }
        return null;
    }

    private void bntAdminAttributesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bntAdminAttributesActionPerformed
        final Layer layer = getCheckedLayer();
        if (layer != null) {
            FrmWait frm = new FrmWait() {

                @Override
                public void action() {
                    sendSelecionChanges = false;
                    FrmLayerSrc frm = new FrmLayerSrc(layer, hibernate, serviceConnection);
                    frm.setVisible(true);
                    if (layer.getSource() != null) {
                        model.setValueAt(true, tblLayers.getSelectedRow(), 0);
                    } else {
                        model.setValueAt(false, tblLayers.getSelectedRow(), 0);
                    }
                    fireListeners(sourceChangeListeners);
                    sendSelecionChanges = true;
                }
            };
            frm.setVisible(true);
        }
    }//GEN-LAST:event_bntAdminAttributesActionPerformed

    private void btnEditLayerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnEditLayerActionPerformed
        Layer layer = getCheckedLayer();
        if (layer != null) {
            FrmLayerProps frm = new FrmLayerProps(this.hibernate, this.getProject(), layer, serviceConnection);
            frm.setVisible(true);
            if (!frm.isUserCancel()) {
                int i = tblLayers.getSelectedRow();
                loadModel();
                tblLayers.getSelectionModel().setSelectionInterval(i, i);
                fireListeners(scaleDenomChangeListeners);
            }
        }
    }//GEN-LAST:event_btnEditLayerActionPerformed

    private void btnMoveLayerDownActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnMoveLayerDownActionPerformed

        int sel = this.tblLayers.getSelectedRow();
        if (sel < this.tblLayers.getRowCount() - 1) {
            if (this.model.moveRowDown(sel)) {
                this.tblLayers.getSelectionModel().setSelectionInterval(sel + 1, sel + 1);
                try {
                    this.saveZorder();
                    service.updateCapabilities(this.hibernate, this.serviceConnection); //TODO Test ?
                    fireListeners(orderChangeListeners);
                } catch (Exception ex) {
                    this.model.moveRowUp(sel + 1);
                    ex.printStackTrace();
                    WindowUtils.msgError(this.getRootPane(), LocaleUtil.getString("windows/controls/messages", "unexerror"));
                }
            } else {
                this.tblLayers.getSelectionModel().setSelectionInterval(sel, sel);
            }
        }
    }//GEN-LAST:event_btnMoveLayerDownActionPerformed

    private void btnMoveLayerUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnMoveLayerUpActionPerformed
        int sel = this.tblLayers.getSelectedRow();
        if (sel > 0) {
            if (this.model.moveRowUp(sel)) {
                tblLayers.getSelectionModel().setSelectionInterval(sel - 1, sel - 1);
                try {
                    this.saveZorder();
                    this.service.updateCapabilities(this.hibernate, this.serviceConnection); //TODO Test ?
                    fireListeners(orderChangeListeners);
                } catch (Exception ex) {
                    this.model.moveRowDown(sel - 1);
                    ex.printStackTrace();
                    WindowUtils.msgError(this.getRootPane(), LocaleUtil.getString("windows/controls/messages", "unexerror"));
                }
            } else {
                this.tblLayers.getSelectionModel().setSelectionInterval(sel, sel);
            }
        }
    }//GEN-LAST:event_btnMoveLayerUpActionPerformed

    private void btnEditSymbologyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnEditSymbologyActionPerformed
        final Layer layer = getCheckedLayer();
        if (layer != null) {
            if (layer.getSource() == null) {
                WindowUtils.msgInfo(null, LocaleUtil.getString("windows/controls/messages", "no_source"));
                return;
            }

            FrmWait frm = new FrmWait() {

                @Override
                public void action() {
                    try {
                        if (layer.getSource().getGeomClass().equals(com.vividsolutions.jts.geom.Polygon.class) || layer.getSource().getGeomClass().equals(com.vividsolutions.jts.geom.MultiPolygon.class)) {
                            FrmSymbPoly frm = new FrmSymbPoly(layer, hibernate);
                            frm.setVisible(true);
                        } else if (layer.getSource().getGeomClass().equals(com.vividsolutions.jts.geom.Point.class) || layer.getSource().getGeomClass().equals(com.vividsolutions.jts.geom.MultiPoint.class)) {
                            FrmSymbPoint frm = new FrmSymbPoint(layer, hibernate);
                            frm.setVisible(true);
                        } else if (layer.getSource().getGeomClass().equals(com.vividsolutions.jts.geom.LineString.class) || layer.getSource().getGeomClass().equals(com.vividsolutions.jts.geom.MultiLineString.class)) {
                            FrmSymbLine frm = new FrmSymbLine(layer, hibernate);
                            frm.setVisible(true);
                        } else {
                            System.err.println("There is no dialog capable of handling " + layer.getSource().getGeomClass().getSimpleName());
                        }
                        service.sendWMSReload(serviceConnection);
                        int i = tblLayers.getSelectedRow();
                        loadModel();
                        tblLayers.getSelectionModel().setSelectionInterval(i, i);
                        fireListeners(styleChangeListeners);

                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
            };
            frm.setVisible(true);
        }
    }//GEN-LAST:event_btnEditSymbologyActionPerformed

    private void btnDelLayerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDelLayerActionPerformed
        int index = this.tblLayers.getSelectedRow();
        if (index >= 0) {
            if (WindowUtils.msgYesNo(this.getRootPane(), LocaleUtil.getString("windows/controls/messages", "msgError2"))) {
                final Layer layer = model.getLayerAt(index);

                FrmWait frm = new FrmWait() {

                    @Override
                    public void action() {
                        try {
                            layer.remove(hibernate);
                            loadModel();
                            saveZorder();

                            Service servicei = null;
                            try {
                                servicei = Service.getService(hibernate);
                                servicei.updateCapabilities(hibernate, serviceConnection);
                                fireListeners(layerExistsChangeListeners);
                            } catch (ConnectException ex) {
                                WindowUtils.msgError(this, LocaleUtil.getString("windows/service/messages", "notlisten"));
                            } catch (HTTPRequestException ex) {
                                LocaleUtil.getString("windows/service/messages", "statecode");
                                WindowUtils.msgInfo(this, " " + ex.getStatusCode() + " " + servicei.getServiceURL());
                            } catch (Exception ex) {
                                ex.printStackTrace();
                            }

                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                    }
                };
                frm.setVisible(true);
            }
        } else {
            WindowUtils.msgError(null, java.util.ResourceBundle.getBundle("windows/controls/messages", LocaleUtil.getLocale()).getString("sellayer"));
        }
    }//GEN-LAST:event_btnDelLayerActionPerformed

    private void btnAddLayerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddLayerActionPerformed
        FrmLayerProps frm = new FrmLayerProps(this.hibernate, this.getProject(), serviceConnection);
        frm.setVisible(true);
        if (!frm.isUserCancel()) {
            loadModel();
            int i = tblLayers.getRowCount() - 1;
            tblLayers.getSelectionModel().setSelectionInterval(i, i);
        }
    }//GEN-LAST:event_btnAddLayerActionPerformed

    private void btnEditFontActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnEditFontActionPerformed
        final Layer layer = getCheckedLayer();
        if (layer != null) {
            try {
                if (layer.getSource() == null) {
                    WindowUtils.msgInfo(null, LocaleUtil.getString("windows/controls/messages", "no_source"));
                    return;
                }


                FrmFonts frmf = new FrmFonts(layer, hibernate, serviceConnection);
                frmf.setVisible(true);

                if (!frmf.isChanged()) {
                    return;
                }

                FrmWait frm = new FrmWait() {

                    @Override
                    public void action() {
                        try {

                            service.sendWMSReload(serviceConnection);
                            int i = tblLayers.getSelectedRow();
                            loadModel();
                            tblLayers.getSelectionModel().setSelectionInterval(i, i);
                            fireListeners(styleChangeListeners);
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                    }
                };
                frm.setVisible(true);
            } catch (Exception ex) {
                Logger.getLogger(PnlLayers.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
}//GEN-LAST:event_btnEditFontActionPerformed

    private class LayerTableModel extends AbstractTableModel {

        private class Row implements Comparable {

            private Layer l;
            private boolean render;

            public Row(Layer l) {
                this.setL(l);
                if (l.getSource() != null) {
                    this.setRender(true);
                    return;
                }
                this.setRender(false);
            }

            public boolean getRender() {
                return render;
            }

            public void setRender(boolean render) {
                this.render = render;
            }

            public Layer getL() {
                return l;
            }

            public void setL(Layer l) {
                this.l = l;
            }

            public int compareTo(Object o) {
                Row r = (Row) o;
                return this.getL().compareTo(r.getL());
            }
        }
        private List<Row> rows;
        private Class[] types = {Boolean.class, String.class, String.class, String.class};
        private String[] names = {"", sp.getString("layeName"), sp.getString("min_scal"), sp.getString("max_scal")};
        private DecimalFormat df = new DecimalFormat("#");

        public Layer[] getLayers() {
            Layer[] rta = new Layer[rows.size()];
            for (int i = 0; i < rta.length; i++) {
                rta[i] = rows.get(i).getL();
            }
            return rta;
        }

        public LayerTableModel() {
            this.rows = new ArrayList<Row>();
        }

        public void addLayer(Layer l) {
            this.rows.add(new Row(l));
        }

        public void sortList() {
            Collections.sort(this.rows);
            fireTableDataChanged();
        }

        @Override
        public Class getColumnClass(int columnIndex) {
            return types[columnIndex];
        }

        @Override
        public String getColumnName(int column) {
            return names[column];
        }

        @Override
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            if (columnIndex == 0) {
                if (rows.get(rowIndex).getL().getSource() != null) {
                    return true;
                }
            }
            return false;
        }

        @Override
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            if (columnIndex == 0) {
                this.rows.get(rowIndex).setRender((Boolean) aValue);
                fireTableCellUpdated(rowIndex, columnIndex);
            }
        }

        public Object getValueAt(int row, int column) {

            if (column == 0) {
                return new Boolean(rows.get(row).getRender());
            } else if (column == 1) {
                return rows.get(row).getL().getDesclaye();
            } else if (column == 2) {
                if (rows.get(row).getL().getMinscaledeno() == 0) {
                    return "ND";
                }
                return df.format(rows.get(row).getL().getMinscaledeno());
            } else if (column == 3) {
                if (rows.get(row).getL().getMaxscaledeno() == 0) {
                    return "ND";
                }
                return df.format(rows.get(row).getL().getMaxscaledeno());
            }
            return null;
        }

        public int getRowCount() {
            return this.rows.size();
        }

        public int getColumnCount() {
            return 4;
        }

        public void resetModel() {
            this.rows = new ArrayList<Row>();
            this.fireTableDataChanged();
        }

        public boolean moveRowUp(int index) {
            if (index > 0) {
                Row tmp = this.rows.get(index - 1);
                this.rows.set(index - 1, this.rows.get(index));
                this.rows.set(index, tmp);
                this.fireTableDataChanged();
                return true;
            }
            return false;
        }

        public boolean moveRowDown(int index) {
            if (index < this.rows.size() - 1) {
                Row tmp = this.rows.get(index + 1);
                this.rows.set(index + 1, this.rows.get(index));
                this.rows.set(index, tmp);
                this.fireTableDataChanged();
                return true;
            }
            return false;
        }

        public Layer getLayerAt(int index) {
            return this.rows.get(index).getL();
        }

        public boolean getBooleanAt(int index) {
            return this.rows.get(index).getRender();
        }

        public int getCheckedCount() {
            int cont = 0;
            for (int i = 0; i < this.rows.size(); i++) {
                if (this.rows.get(i).getRender()) {
                    cont++;
                }
            }
            return cont;
        }

        public Layer[] getVisibleLayers() {
            int cont = getCheckedCount();
            Layer[] res = new Layer[cont];

            for (int i = 0,  j = 0; i < this.rows.size(); i++) {
                if (this.rows.get(i).getRender()) {
                    res[j] = this.rows.get(i).getL();
                    j++;
                }
            }
            return res;
        }
    }

    public ServiceConnection getServiceConnection() {
        return serviceConnection;
    }

    public void setServiceConnection(ServiceConnection serviceConnection) {
        this.serviceConnection = serviceConnection;
    }

    public Service getService() {
        return service;
    }

    public void setService(Service service) {
        this.service = service;
    }

    public Project getProject() {
        return project;
    }

    public PnlProjects getPnlProyect() {
        return pnlProyect;
    }

    public void setPnlProyect(PnlProjects pnlProj) {
        this.pnlProyect = pnlProj;
        pnlProyect.addProjectChangeListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                setProject(pnlProyect.getSelectedProject());
                fireListeners(projectChangeListeners);
            }
        });
    }

    public PnlViewer getPnlViewer() {
        return pnlViewer;
    }

    public void setPnlViewer(PnlViewer pnlViewer) {
        this.pnlViewer = pnlViewer;
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bntAdminAttributes;
    private javax.swing.JButton btnAddLayer;
    private javax.swing.JButton btnDelLayer;
    private javax.swing.JButton btnEditFont;
    private javax.swing.JButton btnEditLayer;
    private javax.swing.JButton btnEditSymbology;
    private javax.swing.JButton btnMaxScale;
    private javax.swing.JButton btnMinScale;
    private javax.swing.JButton btnMoveLayerDown;
    private javax.swing.JButton btnMoveLayerUp;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTabbedPane jTabbedPane1;
    private windows.controls.Legends legends1;
    private javax.swing.JPopupMenu menu;
    private javax.swing.JPanel panelCapa;
    private javax.swing.JMenuItem rm_max_scal;
    private javax.swing.JMenuItem rm_min_scal;
    private javax.swing.JTable tblLayers;
    // End of variables declaration//GEN-END:variables
}
