package windows.controls;


import org.atlas.desk.events.DisplayEvent;
import org.atlas.desk.events.DisplayListener;
import org.atlas.desk.events.FeatureInfoEvent;
import org.atlas.desk.events.FeatureInfoListener;
import org.atlas.desk.events.GeocodingEvent;
import org.atlas.desk.events.GeocodingListener;
import org.atlas.desk.events.HttpRequestEvent;
import org.atlas.desk.events.HttpRequestListener;
import org.atlas.desk.events.MouseMoveEvent;
import org.atlas.desk.events.MouseMoveListener;
import org.atlas.desk.gui.MapPanel;
import org.atlas.geocoding.Result;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
import org.atlas.desk.mashup.Marker;
import org.atlas.util.ServiceConnection;
import org.atlas.util.WindowUtils;
import org.atlas.desk.viewers.GMLViewer;
import org.atlas.desk.viewers.HTMLViewer;

public class PnlViewer extends javax.swing.JPanel {

    private PnlLayers pnlLayers;
    private MapPanel mapPanel;
    private DecimalFormat df = new DecimalFormat("#.####");
    GMLViewer Gviewer = new GMLViewer((JFrame) null, false);
    HTMLViewer Hviewer = new HTMLViewer((JFrame) null, false);

    private BufferedImage norm;
    
    public PnlViewer() {
        try {
            initComponents();
            mapPanel = (MapPanel) this.jmapPanel;
            mapPanel.setAdminEntrace(true);
            mapPanel.setMouseWheelEnabled(true);
            norm = ImageIO.read(this.getClass().getResourceAsStream("/media/home.png"));

            WindowUtils.center(Gviewer);
            WindowUtils.center(Hviewer);

            mapPanel.getEventManager().addGeocodingListener(new GeocodingListener() {

                public void actionPerformed(GeocodingEvent ev) {                    
                    if(ev.getAddresses().length == 0){
                        WindowUtils.msgWarning(jPanel1, java.util.ResourceBundle.getBundle("windows/controls/messages").getString("unable_to_locate"));
                        return;
                    }
                    mapPanel.setZoomLevel(-12);
                    Result r = ev.getAddresses()[0];
                    mapPanel.setCenter(r.getLocation());
                    Marker m = new Marker(mapPanel, r.getLocation(), norm);
                    mapPanel.removeAllMarkers();
                    mapPanel.addMarker(m);
                    mapPanel.updateView();
                    
                }
            });

            mapPanel.getEventManager().addMouseMoveListener(new MouseMoveListener() {

                public void actionPerformed(MouseMoveEvent ev) {
                    Point2D map = ev.getXYCoordinate();
                    String x = df.format(map.getX());
                    String y = df.format(map.getY());
                    lblPos.setText("x = " + x + " - y = " + y);
                }
            });

            mapPanel.getEventManager().addFeatureInfoListener(new FeatureInfoListener() {

                public void actionPerformed(FeatureInfoEvent ev) {
                    long milis = System.currentTimeMillis();

                    if (mapPanel.getCurrentFeatureInfoFormat().equals("text/html")) {
                        Hviewer.setHtml(ev.getFeatureInfo());
                        Hviewer.setVisible(true);
                        Hviewer.setAlwaysOnTop(true);
                    } else if (mapPanel.getCurrentFeatureInfoFormat().equals("application/vnd.ogc.gml")) {
                        Gviewer.setUrlGML(ev.getUrl());
                        Gviewer.setVisible(true);
                        Gviewer.setAlwaysOnTop(true);
                    }

                    System.out.println(ev.getUrl());
                    System.out.println("Feature Info Request took: " + (System.currentTimeMillis() - milis));
                }
            });

            mapPanel.getEventManager().addDisplayListener(new DisplayListener() {

                public void actionPerformed(DisplayEvent ev) {
                    lblScale.setText("Scale: " + df.format(ev.getWmsScale()));
                }
            });

            mapPanel.getEventManager().addHttpRequestListener(new HttpRequestListener() {

                public void actionPerformed(HttpRequestEvent ev) {
                    //System.out.println(ev.getUrl());
                    //jEditorPane1.setText(jEditorPane1.getText()+ev.getUrl()+"\n");
                }
            });

            WindowUtils.setPanelEnabled(this.panelVisor, false);
        } catch (IOException ex) {
            Logger.getLogger(PnlViewer.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    
    @Override
    public void setEnabled(boolean enabled){
        WindowUtils.setPanelChildrenEnabled(panelVisor, enabled);
        WindowUtils.setPanelChildrenEnabled(jPanel1, enabled);
        WindowUtils.setPanelChildrenEnabled(jPanel2, enabled);
        mapPanel.setMode(MapPanel.NOMODESELECTED);
        buttonGroup1.clearSelection();
    }
    
    public double getWmsScale() {
        return mapPanel.getWMScale();
    }

    public PnlLayers getPnlLayers() {
        return pnlLayers;
    }

    public void setPnlLayers(PnlLayers pnlLyr) {
        this.pnlLayers = pnlLyr;
        this.pnlLayers.setPnlViewer(this);        
        ActionListener reloadAll = new ActionListener() {
            
            public void actionPerformed(ActionEvent e) {
                org.atlas.hibernatedata.Project p = pnlLayers.getProject();
                txtAddress.setText("");
                lblPos.setText("");
                lblScale.setText("");
                mapPanel.removeAllMarkers();
                try {
                    if (p != null) {
                        mapPanel.clearCache();
                        ServiceConnection con = pnlLayers.getServiceConnection();
                        mapPanel.setURL(new URL(con.getAppUrl()));
                        mapPanel.setCurrentImageFormat("image/png");
                        mapPanel.setCurrentFeatureInfoFormat(getFormatFeatureInfo());
                        org.atlas.clientdata.Project proj = getProjectByName(p.getNameProj());
                        if (proj.getLayers().size() > 0) {                            
                            mapPanel.setCurrentProject(proj);
                            if (loadVisibleLayers()) {
                                setEnabled(true);
                                mapPanel.zoomToProject();                             
                                return;
                            }
                        }
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
                setEnabled(false);
                mapPanel.updateView();
            }
        };

        ActionListener redraw = new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                mapPanel.clearCache();
                if (loadVisibleLayers()) {
                    WindowUtils.setPanelEnabled(panelVisor, true);
                    mapPanel.setEnabled(true);
                } else {
                    WindowUtils.setPanelEnabled(panelVisor, false);
                    mapPanel.setEnabled(false);
                    lblPos.setText("");
                    lblScale.setText("");
                }
                mapPanel.updateView();
            }
        };

        ActionListener reloadRedraw = new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    mapPanel.clearCache();
                    ServiceConnection con = pnlLayers.getServiceConnection();
                    mapPanel.setURL(new URL(con.getAppUrl()));
                    mapPanel.setCurrentImageFormat("image/png");
                    mapPanel.setCurrentFeatureInfoFormat(getFormatFeatureInfo());
                    org.atlas.clientdata.Project proj = getProjectByName(pnlLayers.getProject().getNameProj());

                    Point2D c = mapPanel.getCenter();
                    int lod   = mapPanel.getZoomLevel();
                    mapPanel.setCurrentProject(proj);
                    if (c != null) {
                        mapPanel.setCenter(c);
                        mapPanel.setZoomLevel(lod);
                    }

                    if (loadVisibleLayers()) {
                        WindowUtils.setPanelEnabled(panelVisor, true);
                        mapPanel.setEnabled(true);
                    } else {
                        WindowUtils.setPanelEnabled(panelVisor, false);
                        mapPanel.setEnabled(false);
                        lblPos.setText("");
                        lblScale.setText("");
                    }
                    mapPanel.updateView();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        };

        /*ActionListener selectionChange = new ActionListener() {

            public void actionPerformed(ActionEvent actionEvent) {

                org.atlas.hibernatedata.Layer layersSer[] = null;
                org.atlas.clientdata.Layer layersCli[] = null;

                if (pnlLayers.getSelectedLayers() != null) {
                    layersSer = pnlLayers.getSelectedLayers();
                    layersCli = new org.atlas.clientdata.Layer[layersSer.length];
                    for (int i = 0; i < layersSer.length; i++) {
                        layersCli[i] = getLayerByName(mapPanel.getCurrentProject(), layersSer[i].getNameLaye());
                    }
                } else {
                    if (mapPanel.getCurrentProject() != null) {
                        List layers = mapPanel.getCurrentProject().getLayers();
                        layersCli = new org.atlas.clientdata.Layer[layers.size()];
                        layers.toArray(layersCli);
                    }
                }
                mapPanel.setFeatureInfoLayers(layersCli);
            }
        };*/

        pnlLayers.addLayerDeleteChangeListener(reloadAll);

        pnlLayers.addSourceChangeListener(reloadAll);

        //pnlLayers.addSourceChangeListener(selectionChange);

        pnlLayers.addProjectChangeListener(reloadAll);

        //pnlLayers.addProjectChangeListener(selectionChange);

        pnlLayers.addShowHideChangeListener(redraw);

        pnlLayers.addStyleChangeListener(redraw);

        pnlLayers.addOrderChangeListener(redraw);

        pnlLayers.addScaleDenomChangeListener(reloadRedraw);

        //pnlLayers.addSelectionChangeListener(selectionChange);

    }

    private boolean loadVisibleLayers() {
        org.atlas.hibernatedata.Layer[] lrs = pnlLayers.getVisibleLayers();
        int cont = lrs.length;
        org.atlas.clientdata.Layer[] lyrs = new org.atlas.clientdata.Layer[cont];
        for (int i = 0; i < cont; i++) {
            lyrs[i] = getLayerByName(mapPanel.getCurrentProject(), lrs[i].getNameLaye());
        }
        mapPanel.setLayers(lyrs);
        if (cont > 0) {
            return true;
        }
        return false;
    }

    private org.atlas.clientdata.Project getProjectByName(String name) throws IllegalArgumentException {
        Object[] prs = mapPanel.getProjects();
        for (int i = 0; i < prs.length; i++) {
            org.atlas.clientdata.Project cp = (org.atlas.clientdata.Project) prs[i];
            if (cp.getName().equals(name)) {
                return cp;
            }
        }
        throw new IllegalArgumentException("There is no project with name " + name);
    }

    private org.atlas.clientdata.Layer getLayerByName(org.atlas.clientdata.Project prj, String name) throws IllegalArgumentException {
        for (int i = 0; i < prj.getLayers().size(); i++) {
            org.atlas.clientdata.Layer l = (org.atlas.clientdata.Layer) prj.getLayers().get(i);
            if (l.getName().equals(name)) {
                return l;
            }
        }
        throw new IllegalArgumentException("There is no layer with name " + name);
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jScrollPane1 = new javax.swing.JScrollPane();
        jEditorPane1 = new javax.swing.JEditorPane();
        buttonGroup1 = new javax.swing.ButtonGroup();
        btnZoomLayer = new javax.swing.JButton();
        panelVisor = new javax.swing.JPanel();
        btnZoomFit = new javax.swing.JButton();
        tglZoomIn = new javax.swing.JToggleButton();
        tglPan = new javax.swing.JToggleButton();
        tglZoomOut = new javax.swing.JToggleButton();
        tglZoomPoint = new javax.swing.JToggleButton();
        tglInfo = new javax.swing.JToggleButton();
        formatFeatInfo = new javax.swing.JComboBox();
        txtAddress = new javax.swing.JTextField();
        btnGeoc = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jmapPanel = new MapPanel();
        jPanel2 = new javax.swing.JPanel();
        lblScale = new javax.swing.JLabel();
        lblPos = new javax.swing.JLabel();

        jScrollPane1.setViewportView(jEditorPane1);

        btnZoomLayer.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/zoom_layer.png"))); // NOI18N
        btnZoomLayer.setToolTipText("Agregar una Capa");
        buttonGroup1.add(btnZoomLayer);
        btnZoomLayer.setDefaultCapable(false);
        btnZoomLayer.setIconTextGap(0);
        btnZoomLayer.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnZoomLayer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnZoomLayerActionPerformed(evt);
            }
        });

        panelVisor.setLayout(new java.awt.GridBagLayout());

        btnZoomFit.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/zoom-fit.png"))); // NOI18N
        buttonGroup1.add(btnZoomFit);
        btnZoomFit.setIconTextGap(0);
        btnZoomFit.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnZoomFit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnZoomFitActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        panelVisor.add(btnZoomFit, gridBagConstraints);

        buttonGroup1.add(tglZoomIn);
        tglZoomIn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/zoom-in.png"))); // NOI18N
        tglZoomIn.setMargin(new java.awt.Insets(0, 0, 0, 0));
        tglZoomIn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tglZoomInActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        panelVisor.add(tglZoomIn, gridBagConstraints);

        buttonGroup1.add(tglPan);
        tglPan.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/move.png"))); // NOI18N
        tglPan.setMargin(new java.awt.Insets(0, 0, 0, 0));
        tglPan.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tglPanActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 10);
        panelVisor.add(tglPan, gridBagConstraints);

        buttonGroup1.add(tglZoomOut);
        tglZoomOut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/zoom-out.png"))); // NOI18N
        tglZoomOut.setMargin(new java.awt.Insets(0, 0, 0, 0));
        tglZoomOut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tglZoomOutActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        panelVisor.add(tglZoomOut, gridBagConstraints);

        buttonGroup1.add(tglZoomPoint);
        tglZoomPoint.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/zoom-point.png"))); // NOI18N
        tglZoomPoint.setMargin(new java.awt.Insets(0, 0, 0, 0));
        tglZoomPoint.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tglZoomPointActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        panelVisor.add(tglZoomPoint, gridBagConstraints);

        buttonGroup1.add(tglInfo);
        tglInfo.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/info.png"))); // NOI18N
        tglInfo.setMargin(new java.awt.Insets(0, 0, 0, 0));
        tglInfo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tglInfoActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 0;
        panelVisor.add(tglInfo, gridBagConstraints);

        formatFeatInfo.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "HTML", "GML" }));
        formatFeatInfo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                formatFeatInfoActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.ipadx = 22;
        gridBagConstraints.ipady = 6;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 0);
        panelVisor.add(formatFeatInfo, gridBagConstraints);

        txtAddress.setEnabled(false);
        txtAddress.setPreferredSize(new java.awt.Dimension(180, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
        panelVisor.add(txtAddress, gridBagConstraints);

        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/controls/messages"); // NOI18N
        btnGeoc.setText(bundle.getString("locate")); // NOI18N
        btnGeoc.setEnabled(false);
        btnGeoc.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnGeocActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        panelVisor.add(btnGeoc, gridBagConstraints);

        jPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        jmapPanel.setEnabled(false);

        javax.swing.GroupLayout jmapPanelLayout = new javax.swing.GroupLayout(jmapPanel);
        jmapPanel.setLayout(jmapPanelLayout);
        jmapPanelLayout.setHorizontalGroup(
            jmapPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 799, Short.MAX_VALUE)
        );
        jmapPanelLayout.setVerticalGroup(
            jmapPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 372, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jmapPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jmapPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        jPanel2.setLayout(new java.awt.GridLayout(1, 0, 5, 0));

        lblScale.setText("Scale:");
        lblScale.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        lblScale.setEnabled(false);
        jPanel2.add(lblScale);

        lblPos.setText("Position:");
        lblPos.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        lblPos.setEnabled(false);
        jPanel2.add(lblPos);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(layout.createSequentialGroup()
                .addComponent(panelVisor, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(266, Short.MAX_VALUE))
            .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, 803, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(panelVisor, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents
    private void formatFeatInfoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_formatFeatInfoActionPerformed
        mapPanel.setCurrentFeatureInfoFormat(getFormatFeatureInfo());
    }//GEN-LAST:event_formatFeatInfoActionPerformed

    private String getFormatFeatureInfo() {
        if (formatFeatInfo.getSelectedIndex() == 0) {
            return "text/html";
        } else {
            return "application/vnd.ogc.gml";
        }
    }

    private void tglInfoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tglInfoActionPerformed
        if (this.tglInfo.isSelected()) {
            mapPanel.setMode(MapPanel.MODEFEATUREINFO);
        } else {
            mapPanel.setMode(MapPanel.NOMODESELECTED);
        }
    }//GEN-LAST:event_tglInfoActionPerformed

    private void tglZoomPointActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tglZoomPointActionPerformed
        if (this.tglZoomPoint.isSelected()) {
            mapPanel.setMode(MapPanel.MODEZOOMBOX);
        } else {
            mapPanel.setMode(MapPanel.NOMODESELECTED);
        }
    }//GEN-LAST:event_tglZoomPointActionPerformed

    private void tglZoomOutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tglZoomOutActionPerformed
        if (this.tglZoomOut.isSelected()) {
            mapPanel.setMode(MapPanel.MODEZOOMMINUS);
        } else {
            mapPanel.setMode(MapPanel.NOMODESELECTED);
        }
    }//GEN-LAST:event_tglZoomOutActionPerformed

    private void btnZoomLayerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnZoomLayerActionPerformed
// TODO add your handling code here:
    }//GEN-LAST:event_btnZoomLayerActionPerformed

    private void tglPanActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tglPanActionPerformed
        if (this.tglPan.isSelected()) {
            mapPanel.setMode(MapPanel.MODEPAN);
        } else {
            mapPanel.setMode(MapPanel.NOMODESELECTED);
        }
    }//GEN-LAST:event_tglPanActionPerformed

    private void tglZoomInActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tglZoomInActionPerformed
        if (this.tglZoomIn.isSelected()) {
            mapPanel.setMode(MapPanel.MODEZOOMPLUS);
        } else {
            mapPanel.setMode(MapPanel.NOMODESELECTED);
        }
    }//GEN-LAST:event_tglZoomInActionPerformed

    private void btnZoomFitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnZoomFitActionPerformed
        try {
            mapPanel.zoomToProject();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }//GEN-LAST:event_btnZoomFitActionPerformed

    private void btnGeocActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnGeocActionPerformed
        try {
            mapPanel.sendGeocoderRequest(txtAddress.getText());
        } catch (Exception ex) {
            Logger.getLogger(PnlViewer.class.getName()).log(Level.SEVERE, null, ex);
        }
        
}//GEN-LAST:event_btnGeocActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnGeoc;
    private javax.swing.JButton btnZoomFit;
    private javax.swing.JButton btnZoomLayer;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JComboBox formatFeatInfo;
    private javax.swing.JEditorPane jEditorPane1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JPanel jmapPanel;
    private javax.swing.JLabel lblPos;
    private javax.swing.JLabel lblScale;
    private javax.swing.JPanel panelVisor;
    private javax.swing.JToggleButton tglInfo;
    private javax.swing.JToggleButton tglPan;
    private javax.swing.JToggleButton tglZoomIn;
    private javax.swing.JToggleButton tglZoomOut;
    private javax.swing.JToggleButton tglZoomPoint;
    private javax.swing.JTextField txtAddress;
    // End of variables declaration//GEN-END:variables
}
