package windows.fonts;

import org.atlas.hibernatedata.Layer;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import org.atlas.util.HibernateUtil;
import org.atlas.i18n.LocaleUtil;
import java.awt.Font;
import java.awt.Graphics;
import java.sql.Connection;
import java.util.Iterator;
import java.util.List;
import javax.swing.JPanel;
import org.hibernate.classic.Session;
import org.atlas.util.FrmWait;
import org.atlas.util.ServiceConnection;
import org.atlas.util.WindowUtils;

public class FrmFonts extends javax.swing.JDialog {

    private Layer layer;
    private HibernateUtil hibernate;
    private ServiceConnection serviceConnection;
    private boolean changed = false;
    
    public FrmFonts(Layer layer, HibernateUtil hibernate, ServiceConnection serviceConnection) throws Exception {
        super((JFrame) null, true);
        initComponents();
        WindowUtils.center(this);
        WindowUtils.setFrameIcon(this, "/media/appicon.png");
        setLayer(layer);
        setHibernate(hibernate);
        setServiceConnection(serviceConnection);
        loadInterface();
    }

    private void setUIEnabled(boolean enabled) {
        WindowUtils.setPanelEnabled(jPanel1, enabled);
        WindowUtils.setPanelEnabled(jPanel2, enabled);
        WindowUtils.setPanelEnabled(jPanel3, enabled);
    }

    private void saveInterface(){
        if (cmbFields.getSelectedIndex() < 1&&chkUse.isSelected()) {            
            String fld = java.util.ResourceBundle.getBundle("windows/fonts/messages",LocaleUtil.getLocale()).getString("please_sel_fld");
            WindowUtils.msgInfo(this, fld);
            return;
        }

        getLayer().getSymbology().setLabelFieldName((String) cmbFields.getSelectedItem());
        getLayer().getSymbology().setFontName((String) cmbFamily.getSelectedItem());
        getLayer().getSymbology().setFontStyle(getFontStyle());
        getLayer().getSymbology().setFontSize(Integer.parseInt(spnSize.getValue().toString()));
        getLayer().getSymbology().setPriority(cmbPriority.getSelectedIndex()+1);
        getLayer().getSymbology().setLabeled(chkUse.isSelected());

        FrmWait wait = new FrmWait() {
            @Override
            public void action() {
                try {
                    Session sess = hibernate.begin();
                    sess.update(getLayer());
                    sess.update(getLayer().getSymbology());
                    hibernate.commitAndClose(sess);
                    setChanged(true);
                } catch (Exception ex) {
                    Logger.getLogger(FrmFonts.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        };
        wait.setVisible(true);
        dispose();    
    }

    private void loadInterface() throws Exception {
        Connection con = hibernate.getConnection();
        layer.getSource().loadFieldMetadata(con);
        con.close();

        List<String> names = getLayer().getSource().getFieldNames();
        cmbFields.removeAllItems();
        
        String sel = java.util.ResourceBundle.getBundle("windows/fonts/messages",LocaleUtil.getLocale()).getString("select_fld");
        
        cmbFields.addItem(sel);
        Iterator<String> it = names.iterator();
        cmbFields.setSelectedIndex(0);
        for (int i = 1; it.hasNext(); i++) {
            String name = it.next();
            cmbFields.addItem(name);
            if (name.equals(getLayer().getSymbology().getLabelFieldName())) {
                cmbFields.setSelectedIndex(i);
            }
        }


        cmbFamily.removeAllItems();
        String[] fts = {"Serif", "SansSerif", "Monospaced", "Dialog", "DialogInput"};

        for (int i = 0; i < fts.length; i++) {
            cmbFamily.addItem(fts[i]);
            if (fts[i].equals(getLayer().getSymbology().getFontName())) {
                cmbFamily.setSelectedIndex(i);
            }
        }

        if (getLayer().getSymbology().getFontStyle() == Font.PLAIN) {
            chkBold.setSelected(false);
            chkItalic.setSelected(false);
        } else if (getLayer().getSymbology().getFontStyle() == Font.BOLD) {
            chkBold.setSelected(true);
            chkItalic.setSelected(false);
        } else if (getLayer().getSymbology().getFontStyle() == Font.ITALIC) {
            chkBold.setSelected(false);
            chkItalic.setSelected(true);
        } else if (getLayer().getSymbology().getFontStyle() == Font.ITALIC + Font.BOLD) {
            chkBold.setSelected(true);
            chkItalic.setSelected(true);
        }

        spnSize.setValue(getLayer().getSymbology().getFontSize());

        chkUse.setSelected(layer.getSymbology().isLabeled());
        
        
        
        String high = java.util.ResourceBundle.getBundle("windows/fonts/messages",LocaleUtil.getLocale()).getString("high");
        String med  = java.util.ResourceBundle.getBundle("windows/fonts/messages",LocaleUtil.getLocale()).getString("medium");
        String low  = java.util.ResourceBundle.getBundle("windows/fonts/messages",LocaleUtil.getLocale()).getString("low");
        
        cmbPriority.removeAllItems();
        cmbPriority.addItem(high);
        cmbPriority.addItem(med);
        cmbPriority.addItem(low);        
        cmbPriority.setSelectedIndex(getLayer().getSymbology().getPriority()-1);
        
        setUIEnabled(layer.getSymbology().isLabeled());
    }

    private int getFontStyle() {
        boolean b = chkBold.isSelected();
        boolean i = chkItalic.isSelected();

        int style = Font.PLAIN;

        if (b && !i) {
            style = Font.BOLD;
        } else if (!b && i) {
            style = Font.ITALIC;
        } else if (b && i) {
            style = Font.BOLD + Font.ITALIC;
        }
        return style;
    }

    private void dibujarEjemplo(Graphics g) {
        if (jPanel1.isEnabled()) {
            String label = java.util.ResourceBundle.getBundle("windows/fonts/messages", LocaleUtil.getLocale()).getString("label");
            int y = jPanel1.getHeight() / 2;
            Font f = new Font((String) this.cmbFamily.getSelectedItem(), getFontStyle(), (Integer)spnSize.getValue());
            g.setFont(f);            
            g.drawString(label, 10, y);
        }
    }


    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new JPanel(){
            public void paint(Graphics g){
                super.paint(g);
                dibujarEjemplo(g);
            }
        };
        jPanel2 = new javax.swing.JPanel();
        lblField = new javax.swing.JLabel();
        cmbFields = new javax.swing.JComboBox();
        jLabel1 = new javax.swing.JLabel();
        cmbFamily = new javax.swing.JComboBox();
        jLabel2 = new javax.swing.JLabel();
        jPanel3 = new javax.swing.JPanel();
        chkBold = new javax.swing.JCheckBox();
        chkItalic = new javax.swing.JCheckBox();
        jLabel3 = new javax.swing.JLabel();
        spnSize = new javax.swing.JSpinner();
        jLabel4 = new javax.swing.JLabel();
        cmbPriority = new javax.swing.JComboBox();
        chkUse = new javax.swing.JCheckBox();
        jPanel4 = new javax.swing.JPanel();
        jButton2 = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/fonts/messages",LocaleUtil.getLocale()); // NOI18N
        setTitle(bundle.getString("labeling_man")); // NOI18N
        setResizable(false);

        jPanel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 176, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 57, Short.MAX_VALUE)
        );

        jPanel2.setLayout(new java.awt.GridLayout(5, 2, 10, 10));

        java.util.ResourceBundle bundle1 = java.util.ResourceBundle.getBundle("windows/symbology/messages",LocaleUtil.getLocale()); // NOI18N
        lblField.setText(bundle1.getString("fields")); // NOI18N
        jPanel2.add(lblField);

        cmbFields.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        jPanel2.add(cmbFields);

        java.util.ResourceBundle bundle2 = java.util.ResourceBundle.getBundle("windows/fonts/messages"); // NOI18N
        jLabel1.setText(bundle2.getString("family")); // NOI18N
        jPanel2.add(jLabel1);

        cmbFamily.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Serif", "SansSerif", "Monospaced", "Dialog", "DialogInput" }));
        cmbFamily.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbFamilyActionPerformed(evt);
            }
        });
        jPanel2.add(cmbFamily);

        jLabel2.setText(bundle2.getString("style")); // NOI18N
        jPanel2.add(jLabel2);

        jPanel3.setLayout(new java.awt.GridLayout(1, 2, 15, 0));

        chkBold.setText(bundle2.getString("bold")); // NOI18N
        chkBold.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chkBoldActionPerformed(evt);
            }
        });
        jPanel3.add(chkBold);

        chkItalic.setText(bundle2.getString("italic")); // NOI18N
        chkItalic.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chkItalicActionPerformed(evt);
            }
        });
        jPanel3.add(chkItalic);

        jPanel2.add(jPanel3);

        jLabel3.setText(bundle2.getString("font_size")); // NOI18N
        jPanel2.add(jLabel3);

        spnSize.setModel(new javax.swing.SpinnerNumberModel(3, 3, 12, 1));
        spnSize.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                spnSizeStateChanged(evt);
            }
        });
        jPanel2.add(spnSize);

        jLabel4.setText(bundle2.getString("priority")); // NOI18N
        jPanel2.add(jLabel4);

        cmbPriority.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        jPanel2.add(cmbPriority);

        chkUse.setText(bundle2.getString("use_labels")); // NOI18N
        chkUse.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chkUseActionPerformed(evt);
            }
        });

        jPanel4.setLayout(new java.awt.GridLayout(1, 2, 10, 0));

        jButton2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/process-stop.png"))); // NOI18N
        java.util.ResourceBundle bundle3 = java.util.ResourceBundle.getBundle("windows/spatial_ref_sys/messages",LocaleUtil.getLocale()); // NOI18N
        jButton2.setText(bundle3.getString("cancel_key")); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel4.add(jButton2);

        jButton1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/go-next.png"))); // NOI18N
        jButton1.setText(bundle.getString("continue")); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel4.add(jButton1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(chkUse)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, 304, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 18, Short.MAX_VALUE)
                                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)))
                        .addGap(10, 10, 10))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, 392, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap())))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(chkUse)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 27, Short.MAX_VALUE)
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private void cmbFamilyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cmbFamilyActionPerformed
        jPanel1.repaint();
}//GEN-LAST:event_cmbFamilyActionPerformed

    private void chkBoldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_chkBoldActionPerformed
        jPanel1.repaint();
    }//GEN-LAST:event_chkBoldActionPerformed

    private void chkItalicActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_chkItalicActionPerformed
        jPanel1.repaint();
    }//GEN-LAST:event_chkItalicActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        try {
            saveInterface();//GEN-LAST:event_jButton1ActionPerformed
        } catch (Exception ex) {
            Logger.getLogger(FrmFonts.class.getName()).log(Level.SEVERE, null, ex);
        }
    }                                        

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        this.setVisible(false);
    }//GEN-LAST:event_jButton2ActionPerformed

    private void chkUseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_chkUseActionPerformed
        setUIEnabled(chkUse.isSelected());
}//GEN-LAST:event_chkUseActionPerformed

    private void spnSizeStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_spnSizeStateChanged
        jPanel1.repaint();
    }//GEN-LAST:event_spnSizeStateChanged

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox chkBold;
    private javax.swing.JCheckBox chkItalic;
    private javax.swing.JCheckBox chkUse;
    private javax.swing.JComboBox cmbFamily;
    private javax.swing.JComboBox cmbFields;
    private javax.swing.JComboBox cmbPriority;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JLabel lblField;
    private javax.swing.JSpinner spnSize;
    // End of variables declaration//GEN-END:variables
    private Layer getLayer() {
        return layer;
    }

    private void setLayer(Layer layer) {
        this.layer = layer;
    }

    private HibernateUtil getHibernate() {
        return hibernate;
    }

    private void setHibernate(HibernateUtil hibernate) {
        this.hibernate = hibernate;
    }

    private ServiceConnection getServiceConnection() {
        return serviceConnection;
    }

    private void setServiceConnection(ServiceConnection serviceConnection) {
        this.serviceConnection = serviceConnection;
    }

    public boolean isChanged() {
        return changed;
    }

    public void setChanged(boolean changed) {
        this.changed = changed;
    }
}
