package windows.geocoding;

import org.atlas.geocoding.ConfigurationDialog;
import org.atlas.geocoding.Geocoder;
import org.atlas.hibernatedata.GeocoderConfig;
import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.Source;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import org.atlas.i18n.LocaleUtil;
import org.atlas.i18n.StringProvider;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import org.hibernate.classic.Session;
import org.hibernate.classic.Session;
import org.atlas.util.HibernateUtil;
import org.atlas.util.Plugins;
import org.atlas.util.WindowUtils;
import org.atlas.util.FrmWait;
import org.atlas.util.ServiceConnection;

public class FrmAdminGeocoders extends javax.swing.JDialog {

    private HibernateUtil hibernate;
    private Project project;
    private StringProvider prov = new StringProvider("windows/geocoding/messages");
    private PluginsTableModel ptm = new PluginsTableModel();
    private ServiceConnection sc;

    public FrmAdminGeocoders(HibernateUtil hibernate, Project project, ServiceConnection sc) {
        super((JFrame) null, true);
        initComponents();        
        WindowUtils.setFrameIcon(this, "/media/appicon.png");

        jComboBox1.addItem(prov.getString("select_geo"));
        Geocoder[] gcs = Plugins.getInstalledPlugins();
        for (int i = 0; i < gcs.length; i++) {
            jComboBox1.addItem(gcs[i]);
        }

        tblPlug.setModel(ptm);

        this.hibernate = hibernate;
        this.project   = project;
        this.sc        = sc;
        
        tblPlug.getSelectionModel().setSelectionMode(tblPlug.getSelectionModel().SINGLE_SELECTION);
        tblPlug.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent e) {
                boolean sta = false;
                if (getSelected() != null) {
                    sta = true;
                }
                btnAbout.setEnabled(sta);
                btnConf.setEnabled(sta);
                btnRem.setEnabled(sta);
            }
        });


        Session sess = null;
        Source[] srcs = new Source[0];
        GeocoderConfig[] cfgs = new GeocoderConfig[0];
        try {
            sess = hibernate.begin();
            sess.update(project);

            srcs = new Source[project.getSources().size()];
            project.getSources().toArray(srcs);

            cfgs = new GeocoderConfig[project.getGeocoderConfigs().size()];
            project.getGeocoderConfigs().toArray(cfgs);

        } catch (Exception ex) {
            ex.printStackTrace();
            this.dispose();
        } finally {
            hibernate.close(sess);
        }

        if (srcs.length > 0) {
            try {
                Connection conn = hibernate.getConnection();
                for (int i = 0; i < srcs.length; i++) {
                    Source source = srcs[i];
                    source.loadFieldMetadata(conn);
                }
            } catch (Exception ex) {
                Logger.getLogger(FrmAdminGeocoders.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

        cargarLista();
    }

    private Object getSelected() {
        int index = tblPlug.getSelectedRow();
        if (index >= 0) {
            if (ptm.getList().get(index) instanceof Geocoder) {
                return (Geocoder) ptm.getList().get(index);
            }
        }
        return null;
    }

    private void cargarLista() {
        GeocoderConfig[] cfgs = new GeocoderConfig[project.getGeocoderConfigs().size()];
        project.getGeocoderConfigs().toArray(cfgs);
        ptm.clear();
        for (int i = 0; i < cfgs.length; i++) {
            GeocoderConfig config = cfgs[i];
            try {
                Geocoder geo = (Geocoder) Class.forName(config.getClassname()).newInstance();
                ptm.add(geo);
            } catch (Exception ex) {
                Logger.getLogger(FrmAdminGeocoders.class.getName()).log(Level.SEVERE, null, ex);
                ptm.add(config.getClassname());
            }
        }
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jButton2 = new javax.swing.JButton();
        btnAdd = new javax.swing.JButton();
        btnRem = new javax.swing.JButton();
        jComboBox1 = new javax.swing.JComboBox();
        jScrollPane1 = new javax.swing.JScrollPane();
        tblPlug = new javax.swing.JTable();
        btnAbout = new javax.swing.JButton();
        btnConf = new javax.swing.JButton();

        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/geocoding/messages",LocaleUtil.getLocale()); // NOI18N
        setTitle(bundle.getString("title_admin")); // NOI18N
        setLocationByPlatform(true);

        jButton2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/volver.png"))); // NOI18N
        java.util.ResourceBundle bundle1 = java.util.ResourceBundle.getBundle("windows/icons/messages",LocaleUtil.getLocale()); // NOI18N
        jButton2.setText(bundle1.getString("exit")); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        btnAdd.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/plus.png"))); // NOI18N
        btnAdd.setEnabled(false);
        btnAdd.setIconTextGap(0);
        btnAdd.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddActionPerformed(evt);
            }
        });

        btnRem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/trash.png"))); // NOI18N
        btnRem.setDefaultCapable(false);
        btnRem.setEnabled(false);
        btnRem.setIconTextGap(0);
        btnRem.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnRem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRemActionPerformed(evt);
            }
        });

        jComboBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox1ActionPerformed(evt);
            }
        });

        tblPlug.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(tblPlug);

        btnAbout.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/about.png"))); // NOI18N
        btnAbout.setEnabled(false);
        btnAbout.setIconTextGap(0);
        btnAbout.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAboutActionPerformed(evt);
            }
        });

        btnConf.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/config.png"))); // NOI18N
        btnConf.setDefaultCapable(false);
        btnConf.setEnabled(false);
        btnConf.setIconTextGap(0);
        btnConf.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnConf.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnConfActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 109, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, 0, 0, Short.MAX_VALUE)
                        .addGroup(layout.createSequentialGroup()
                            .addComponent(jComboBox1, javax.swing.GroupLayout.PREFERRED_SIZE, 255, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGap(26, 26, 26)
                            .addComponent(btnAdd, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGap(18, 18, 18)
                            .addComponent(btnAbout)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(btnConf)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(btnRem)
                            .addGap(4, 4, 4))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jComboBox1, javax.swing.GroupLayout.PREFERRED_SIZE, 29, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnAdd)
                    .addComponent(btnAbout)
                    .addComponent(btnConf)
                    .addComponent(btnRem))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 182, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 34, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private void btnRemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRemActionPerformed

        if (!WindowUtils.msgYesNo(this, prov.getString("want_to_remove"))) {
            return;
        }

        GeocoderConfig rem = null;
        String remclass = ((Geocoder) getSelected()).getClass().getName();


        GeocoderConfig[] cfgs = new GeocoderConfig[project.getGeocoderConfigs().size()];
        project.getGeocoderConfigs().toArray(cfgs);

        for (int i = 0; i < cfgs.length; i++) {
            GeocoderConfig config = cfgs[i];
            if (config.getClassname().equals(remclass)) {
                rem = config;
                break;
            }
        }

        final GeocoderConfig del = rem;
        FrmWait frm = new FrmWait() {

            @Override
            public void action() {
                GeocoderConfig gc = new GeocoderConfig();
                try {
                    Session sess = hibernate.begin();
                    sess.delete(del);
                    sess.update(project);
                    project.getGeocoderConfigs().remove(del);
                    hibernate.commitAndClose(sess);
                    cargarLista();
                    sc.sendGeocoderReload();
                } catch (Exception ex) {
                    Logger.getLogger(FrmAdminGeocoders.class.getName()).log(Level.SEVERE, null, ex);
                    project.getGeocoderConfigs().remove(gc);
                    WindowUtils.msgError(this, prov.getString("unex_error"));
                }
            }
        };
        frm.setVisible(true);

}//GEN-LAST:event_btnRemActionPerformed

    private void btnAddActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddActionPerformed

        Geocoder add = (Geocoder) jComboBox1.getSelectedItem();
        String addclass = add.getClass().getName();

        GeocoderConfig[] cfgs = new GeocoderConfig[project.getGeocoderConfigs().size()];
        project.getGeocoderConfigs().toArray(cfgs);
        boolean found = false;
        for (int i = 0; i < cfgs.length && !found; i++) {
            GeocoderConfig config = cfgs[i];
            if (config.getClassname().equals(addclass)) {
                found = true;
            }
        }

        if (found) {
            WindowUtils.msgInfo(this, prov.getString("geo_already_in_list"));
            return;
        }

        FrmWait frm = new FrmWait() {

            @Override
            public void action() {
                GeocoderConfig gc = new GeocoderConfig();
                try {
                    gc.setClassname(((Geocoder) jComboBox1.getSelectedItem()).getClass().getName());
                    Session sess = hibernate.begin();
                    sess.save(gc);
                    sess.update(project);
                    project.getGeocoderConfigs().add(gc);
                    gc.setProject(project);
                    hibernate.commitAndClose(sess);
                    cargarLista();
                } catch (Exception ex) {
                    Logger.getLogger(FrmAdminGeocoders.class.getName()).log(Level.SEVERE, null, ex);
                    project.getGeocoderConfigs().remove(gc);
                    WindowUtils.msgError(this, prov.getString("unex_error"));
                }
            }
        };
        frm.setVisible(true);
}//GEN-LAST:event_btnAddActionPerformed

    private class PluginsTableModel extends AbstractTableModel {

        private String[] names = {prov.getString("name"), prov.getString("description")};
        private List list = new ArrayList();

        private void sort() {
            Collections.sort(getList(), new Comparator() {

                public int compare(Object o1, Object o2) {
                    return ((Geocoder) o1).getName().compareTo(((Geocoder) o2).getName());
                }
            });
        }

        public void add(Object o) {
            getList().add(o);
            sort();
            fireTableDataChanged();
        }

        public void remove(Object o) {
            getList().remove(o);
            sort();
            fireTableDataChanged();
        }

        public void clear() {
            getList().clear();
            fireTableDataChanged();
        }

        @Override
        public Class<?> getColumnClass(int columnIndex) {
            return super.getColumnClass(columnIndex);
        }

        @Override
        public String getColumnName(int column) {
            return names[column];
        }

        @Override
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }

        public int getRowCount() {
            return getList().size();
        }

        public int getColumnCount() {
            return 2;
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            Object obj = null;
            if (getList().get(rowIndex) instanceof Geocoder) {
                Geocoder g = (Geocoder) getList().get(rowIndex);
                switch (columnIndex) {
                    case 0:
                        obj = g.getName();
                        break;
                    case 1:
                        obj = g.getDescription();
                        break;
                }
            } else {
                switch (columnIndex) {
                    case 0:
                        obj = getList().get(rowIndex);
                        break;
                    case 1:
                        obj = "";
                        break;
                }
            }
            return obj;
        }

        public List getList() {
            return list;
        }
    }

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        this.setVisible(false);
    }//GEN-LAST:event_jButton2ActionPerformed

    private void btnAboutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAboutActionPerformed
        Object o = getSelected();
        if (o == null) {
            return;
        }
        ((Geocoder) o).getAboutDialog().setVisible(true);
    }//GEN-LAST:event_btnAboutActionPerformed

    private void btnConfActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnConfActionPerformed
        Object obj = getSelected();
        if (obj == null) {
            return;
        }
        
        GeocoderConfig config = null;
        String remclass = ((Geocoder) obj).getClass().getName();

        GeocoderConfig[] cfgs = new GeocoderConfig[project.getGeocoderConfigs().size()];
        project.getGeocoderConfigs().toArray(cfgs);

        for (int i = 0; i < cfgs.length; i++) {
            GeocoderConfig conf = cfgs[i];
            if (conf.getClassname().equals(remclass)) {
                config = conf;
                break;
            }
        }
        
        ConfigurationDialog cd = ((Geocoder) obj).getConfigurationDialog(hibernate, project, config);
        
        cd.setVisible(true);
        if(!cd.isCancelled()){
            try {
                sc.sendGeocoderReload();
            } catch (Exception ex) {
                org.atlas.util.WindowUtils.msgError(this, "Unexpected error "+ex.getMessage());
            }
        }
}//GEN-LAST:event_btnConfActionPerformed

    private void jComboBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox1ActionPerformed
        if (jComboBox1.getSelectedIndex() > 0) {
            btnAdd.setEnabled(true);
            return;
        }
        btnAdd.setEnabled(false);
    }//GEN-LAST:event_jComboBox1ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAbout;
    private javax.swing.JButton btnAdd;
    private javax.swing.JButton btnConf;
    private javax.swing.JButton btnRem;
    private javax.swing.JButton jButton2;
    private javax.swing.JComboBox jComboBox1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable tblPlug;
    // End of variables declaration//GEN-END:variables
}
