package windows.icons;

import org.atlas.hibernatedata.Icon;
import org.atlas.i18n.LocaleUtil;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JPanel;

public class IconsPanel extends javax.swing.JPanel {

    private List itemsAdded = new ArrayList(); //listener
    private List itemsRemoved = new ArrayList(); //listener
    private List itemsSelected = new ArrayList(); //listener
    private List icons = new ArrayList();
    private int rowCapa;
    private List currIcons = new ArrayList();
    private boolean multipleSelectionAllowed = true;

    public IconsPanel() {
        initComponents();
    }

    public void setSelectedIcon(Icon icon) {
        double y = 0;
        for (int i = 0; i < icons.size(); i++) {
            Icon ic = (Icon) icons.get(i);
            if (ic.equals(icon)) {
                y = Math.floor(((double)i) / rowCapa) * Icon.THUMB_HEIGHT;
                break;
            }
        }

        currIcons.clear();
        currIcons.add(icon);
        Rectangle r = new Rectangle(10, (int)y, 10, Icon.THUMB_HEIGHT);
        panel.scrollRectToVisible(r);
        panel.revalidate();
    }

    public void fireListener(List l) {
        if (l.size() == 0) {
            return;
        }

        ActionEvent ae = new ActionEvent(this, -1, "");
        for (int i = 0; i <
                l.size(); i++) {
            ((ActionListener) l.get(i)).actionPerformed(ae);
        }

    }

    @Override
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        arrangeIcons();
    }

    public void addIcon(Icon icon) {
        try {
            icons.add(icon);
            Collections.sort(icons, new Comparator(){
                public int compare(Object o1, Object o2) {
                    return ((Icon)o1).getDescicon().compareTo(((Icon)o2).getDescicon());
                }
            });
            icon.createThumb();
            arrangeIcons();
            fireItemsAddedAction();
        } catch (Exception ex) {
            Logger.getLogger(IconsPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void removeIcon(Icon icon) {
        this.icons.remove(icon);
        this.currIcons.remove(icon);
        arrangeIcons();
        fireItemsRemovedAction();
    }

    public void removeAllIcons() {
        this.currIcons.clear();
        this.icons.clear();
        arrangeIcons();
        fireItemsRemovedAction();
    }

    public int getIconCount() {
        return icons.size();
    }

    private void draw(Graphics gr) {
        Graphics2D g = (Graphics2D) gr;
        g.clearRect(0, 0, this.panel.getWidth(), this.panel.getHeight());
        if (isEnabled()) {
            if (icons.size() > 0) {
                int row = 0;
                int col = 0;
                try {
                    for (int i = 0; i <
                            icons.size(); i++) {
                        Icon ic = (Icon) icons.get(i);
                        BufferedImage t = ic.getThumb();
                        g.drawImage(t, col * Icon.THUMB_WIDTH, row * Icon.THUMB_HEIGHT, null);
                        if (currIcons.contains(ic)) {
                            g.setStroke(new BasicStroke(3.0f));
                            g.setColor(new Color(8, 108, 180, 64));
                            g.fillRect(col * Icon.THUMB_WIDTH, row * Icon.THUMB_HEIGHT, Icon.THUMB_WIDTH, Icon.THUMB_HEIGHT);
                            g.setColor(new Color(8, 108, 180, 255));
                            g.drawRect(col * Icon.THUMB_WIDTH, row * Icon.THUMB_HEIGHT, Icon.THUMB_WIDTH, Icon.THUMB_HEIGHT);
                        }

                        if (col == rowCapa - 1) {
                            col = 0;
                            row++;

                        } else {
                            col++;
                        }

                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }

            } else {
                String cad = LocaleUtil.getString("windows/icons/messages", "no_icons");
                int y = this.getHeight() / 2;
                int x = (this.getWidth() / 2) - (int) (g.getFontMetrics().getStringBounds(cad, g).getWidth() / 2);
                g.drawString(cad, x, y);
            }

        }
    }

    public Object[] getIcons() {
        return icons.toArray();
    }

    public Object[] getSelectedIcons() {
        return currIcons.toArray();
    }

    public void addItemsSelectedListener(ActionListener al) {
        itemsSelected.add(al);
    }

    public void removeItemsSelectedListener(ActionListener al) {
        itemsSelected.remove(al);
    }

    public void addItemsAddedListener(ActionListener al) {
        itemsAdded.add(al);
    }

    public void removeItemsAddedListener(ActionListener al) {
        itemsAdded.remove(al);
    }

    public void addItemsRemovedListener(ActionListener al) {
        itemsRemoved.add(al);
    }

    public void removeItemsRemovedListener(ActionListener al) {
        itemsRemoved.remove(al);
    }

    private void fireItemsRemovedAction() {
        Iterator ite = itemsRemoved.iterator();
        ActionEvent ae = new ActionEvent(this, 0, "");
        while (ite.hasNext()) {
            ((ActionListener) (ite.next())).actionPerformed(ae);
        }

    }

    private void fireItemsAddedAction() {
        Iterator ite = itemsRemoved.iterator();
        ActionEvent ae = new ActionEvent(this, 0, "");
        while (ite.hasNext()) {
            ((ActionListener) (ite.next())).actionPerformed(ae);
        }

    }

    private int getEffectiveWidth() {
        return getWidth() - scroll.getVerticalScrollBar().getWidth() - 4;
    }

    private void arrangeIcons() {

        rowCapa = (int) Math.floor((double) getEffectiveWidth() / Icon.THUMB_WIDTH);
        int rows;
        if (icons != null && isEnabled()) {
            rows = (int) Math.ceil(((double) icons.size()) / rowCapa);
        } else {
            rows = 1;
        }

        Dimension d = new Dimension(Icon.THUMB_WIDTH * rowCapa, Icon.THUMB_HEIGHT * rows);
        panel.setPreferredSize(d);
        panel.revalidate();
        panel.repaint();
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        scroll = new javax.swing.JScrollPane();
        panel = new JPanel(){
            public void paint(Graphics g){
                draw(g);
            }
        };

        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });
        setLayout(new java.awt.BorderLayout());

        scroll.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scroll.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        panel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                panelMouseReleased(evt);
            }
        });
        panel.setLayout(null);
        scroll.setViewportView(panel);

        add(scroll, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents
    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
        arrangeIcons();
    }//GEN-LAST:event_formComponentResized

    private void panelMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_panelMouseReleased

        if (evt.getX() > Icon.THUMB_WIDTH * rowCapa) {
            return;
        }

        int currIndex = (int) ((Math.floor(evt.getY() / Icon.THUMB_HEIGHT) * rowCapa) + Math.ceil(evt.getX() / Icon.THUMB_WIDTH));
        if (evt.getModifiers() == MouseEvent.SHIFT_MASK + MouseEvent.BUTTON1_MASK && isMultipleSelectionAllowed()) {
            Icon ic = (Icon) icons.get(currIndex);
            if (currIcons.contains(ic)) {
                currIcons.remove(ic);
            } else {
                currIcons.add(ic);
            }

        } else {
            currIcons.clear();
            currIcons.add(icons.get(currIndex));
        }

        fireListener(itemsSelected);

        repaint();
        
}//GEN-LAST:event_panelMouseReleased

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel panel;
    private javax.swing.JScrollPane scroll;
    // End of variables declaration//GEN-END:variables
    public boolean isMultipleSelectionAllowed() {
        return multipleSelectionAllowed;
    }

    public void setMultipleSelectionAllowed(boolean multipleSelectionAllowed) {
        this.multipleSelectionAllowed = multipleSelectionAllowed;
    }
}
