package windows.servicetest;

import org.atlas.exceptions.HTTPRequestException;
import org.atlas.hibernatedata.Service;
import java.awt.Color;
import java.awt.Component;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.net.URLDecoder;
import java.net.UnknownHostException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import org.atlas.util.ConnectionPropertiesHb;
import org.atlas.util.Cryptography;
import org.atlas.util.DataBaseUtils;
import org.atlas.util.HibernateUtil;
import org.atlas.i18n.LocaleUtil;
import org.atlas.i18n.StringProvider;
import java.awt.Rectangle;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.List;
import javax.swing.SwingUtilities;
import org.atlas.util.ServiceConnection;
import org.atlas.util.WindowUtils;
import org.atlas.util.FrmWait;
import org.atlas.util.NetUtils;
import windows.service.FrmServiceProps;

public class FrmServerTest extends javax.swing.JDialog {

    private ServiceConnection service;
    private ConnectionPropertiesHb cp;
    private HibernateUtil hibernate;
    private boolean userCancel;
    private DefaultTableModel model;
    private boolean error;
    private boolean warning;
    private StringProvider prov = new StringProvider("windows/servicetest/test");
    private StringProvider mes = new StringProvider("windows/servicetest/problems");
    private List pbs;
    private String pass = LocaleUtil.getString("windows/servicetest/test", "passed");
    private String fail = LocaleUtil.getString("windows/servicetest/test", "failed");
    private String test = LocaleUtil.getString("windows/servicetest/test", "testing");
    private String warn = LocaleUtil.getString("windows/servicetest/test", "warning");
    private String testing = prov.getString("testing");//Testing
    private String test1 = prov.getString("test");//Test
    private String result = prov.getString("result");//Result
    private String url_val = prov.getString("url_val");//URL validation for
    private String tcp_val = prov.getString("tcp_val");//TCP/IP connection with
    private String port_txt = prov.getString("port");//  port
    private String http_val = prov.getString("http");//HTTP Server at
    private String atlas_val = prov.getString("atlas");//ATLAS Server at
    private String access = prov.getString("access");//Service Access
    private String servlet_db = prov.getString("servlet_db");//Servlet - Database connection
    private String admin_db = prov.getString("admin_db");//Administrator - Database connection
    private String tabl_struct = prov.getString("tabl_struct");//Tables Structure
    private String wms_meta = prov.getString("wms_meta");//WMS Service Metadata
    private String postgis = prov.getString("postgis");//Postgis
    public FrmServerTest(ServiceConnection service) {
        super((JFrame) null, true);
        initComponents();
        this.service = service;
        org.atlas.util.WindowUtils.center(this);
        org.atlas.util.WindowUtils.setFrameIcon(this, "/media/appicon.png");
        test();
    }

    private void prepareTable() {
        model = new DefaultTableModel() {

            @Override
            public boolean isCellEditable(int row, int column) {
                return false;
            }
        };
        tblTest.setDefaultRenderer(Object.class, new DefaultTableCellRenderer() {

            @Override
            public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
                Component cell = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                cell.setForeground(new Color(0, 0, 0));
                if (value.toString().equals(fail)) {
                    cell.setForeground(new Color(255, 0, 0));
                }
                if (value.toString().equals(pass)) {
                    cell.setForeground(new Color(0, 0, 255));
                }
                if (value.toString().equals(warn)) {
                    cell.setForeground(new Color(255, 150, 30));
                }
                return cell;
            }
        });
        tblTest.setModel(model);
        model.addColumn(prov.getString("test"));
        model.addColumn(prov.getString("result"));
        int width = 80;
        tblTest.getColumnModel().getColumn(1).setPreferredWidth(width);
        tblTest.getColumnModel().getColumn(1).setMaxWidth(width + 10);
        tblTest.getColumnModel().getColumn(1).setMinWidth(width - 10);
    }

    private void setTestResult(String res) {
        model.setValueAt(res, model.getRowCount() - 1, 1);
        if (res.equals(fail)) {
            error = true;
        }
        if (res.equals(warn)) {
            warning = true;
        }
    }

    private void addRow(String name) {
        model.addRow(new Object[]{name, test});
        showLastRow();
    }

    private int testAddress() {
        //test for a valid URL
        String urlcad = service.getWmsUrl();
        URL url = null;
        int testi = 0;
        if (!isError()) {
            try {
                addRow(url_val + " " + urlcad);
                testi++;
                url = new URL(urlcad);
                if (!url.getProtocol().toLowerCase().equals("http")) {
                    pbs.add(new Problem(1, Problem.TYPE_ERROR, mes));
                    setTestResult(fail);
                } else {
                    setTestResult(pass);
                }
            } catch (MalformedURLException ex) {
                setTestResult(fail);
                ex.printStackTrace();
            }
        }

        //test TCP/IP with server
        if (!isError()) {
            addRow(tcp_val + " " + url.getHost() + " " + port_txt + " " + getPort(url));
            testi++;
            try {
                TCPIPTest(url, 5000);
                setTestResult(pass);
            } catch (IOException ex) {
                ex.printStackTrace();
                pbs.add(new Problem(2, Problem.TYPE_ERROR, mes));
                setTestResult(fail);
            }
        }

        //looking for HTTP Server
        if (!isError()) {
            addRow(http_val + " " + url.getHost() + " " + port_txt + " " + getPort(url));
            testi++;
            try {
                HTTPTest(url);
                setTestResult(pass);
            } catch (IOException ex) {
                ex.printStackTrace();
                pbs.add(new Problem(3, Problem.TYPE_ERROR, mes));
                setTestResult(fail);
            }
        }

        //looking for atlas server
        if (!isError()) {
            addRow(atlas_val + " " + urlcad);
            testi++;
            try {
                boolean isAtlasServer;
                isAtlasServer = ATLASServerTest(service);
                if (!isAtlasServer) {
                    setTestResult(fail);
                    pbs.add(new Problem(9, Problem.TYPE_ERROR, mes));
                } else {
                    setTestResult(pass);
                }
            } catch (HTTPRequestException ex) {
                setTestResult(fail);
                pbs.add(new Problem(9, ex.getStatusCode() + " " + ex.getMessage(), Problem.TYPE_ERROR, mes));
                ex.printStackTrace();
            } catch (Exception ex) {
                setTestResult(fail);
                pbs.add(new Problem(9, ex.getMessage(), Problem.TYPE_ERROR, mes));
                ex.printStackTrace();
            }
        }
        return testi;
    }

    private void test() {
        if (hibernate != null) {
            hibernate.terminate();
        }
        btnConnect.setEnabled(false);
        btnCancel.setEnabled(false);
        pnlLog.setText("");
        final Component parent = this;
        Thread t = new Thread(new Runnable() {

            public void run() {
                long milis = System.currentTimeMillis();
                pbs = new ArrayList();
                prepareTable();
                error = false;
                warning = false;
                testAddress();

                //servlet database test
                if (!isError()) {
                    addRow(servlet_db);
                    try {
                        String res = new String(service.getWmsAdminInfo(service.getRequestBytes("GetTest")));
                        String[] parts = res.split("&");

                        String sta = parts[0].split("=")[1];

                        if (sta.equals("true")) {
                            setTestResult(pass);
                        } else {
                            setTestResult(fail);
                            String cla = URLDecoder.decode(parts[1].split("=")[1], "UTF-8");
                            String msg = URLDecoder.decode(parts[2].split("=")[1], "UTF-8");
                            String fin = "<br>" + prov.getString("class_name") + ": " + cla + "<br>" + prov.getString("error_mess") + ": " + msg;
                            pbs.add(new Problem(4, fin, Problem.TYPE_ERROR, mes));
                        }
                    } catch (Exception ex) {
                        setTestResult(fail);
                        WindowUtils.msgError(parent, prov.getString("unex_error"));
                        ex.printStackTrace();
                    }
                }

                //admin data base connection test
                if (!isError()) {
                    addRow(admin_db);
                    try {
                        byte[] res = service.getWmsAdminInfo(service.getRequestBytes("GetConnection"));
                        String[] parts = Cryptography.decrypthAES(res).split("g");
                        String driver = new String(Cryptography.toBytes(parts[0]));
                        String data = new String(Cryptography.toBytes(parts[1]));
                        String host = new String(Cryptography.toBytes(parts[2]));
                        String passw = new String(Cryptography.toBytes(parts[3]));
                        String port = new String(Cryptography.toBytes(parts[4]));
                        String user = new String(Cryptography.toBytes(parts[5]));

                        InetAddress db = InetAddress.getByName(host);

                        if (!NetUtils.isMyMachine(new URL(service.getAppUrl()).getHost())) {
                            if (db.isLoopbackAddress()) {
                                host = new URL(service.getAppUrl()).getHost();
                            }
                        }

                        cp = new ConnectionPropertiesHb(driver, host, port, data, user, passw);

                        FrmWait frm = new FrmWait() {

                            public void action() {
                                try {
                                    hibernate = new HibernateUtil(cp);
                                    setTestResult(pass);
                                } catch (Exception ex) {
                                    pbs.add(new Problem(5, ex.getMessage(), Problem.TYPE_ERROR, mes));
                                    ex.printStackTrace();
                                    setTestResult(fail);
                                }
                            }
                        };

                        frm.setMessage(prov.getString("load_hiber"));
                        frm.setVisible(true);

                    } catch (Exception ex) {
                        setTestResult(fail);
                        WindowUtils.msgError(parent, prov.getString("unex_error"));
                        ex.printStackTrace();
                    }
                }

                //table structure
                if (!isError()) {
                    addRow(tabl_struct);
                    try {
                        getHibernate().validateSchema();
                        setTestResult(pass);
                    } catch (Exception ex) {
                        FrmWait wait = new FrmWait() {

                            public void action() {
                                if (WindowUtils.msgYesNo(parent, prov.getString("req_tables"))) {
                                    try {
                                        getHibernate().dropSchema();
                                        getHibernate().createSchema();
                                        DataBaseUtils.executeFile("/sql/ref_sys.sql", getHibernate().getConnection());
                                        setTestResult(pass);
                                    } catch (Exception ex1) {
                                        pbs.add(new Problem(6, ex1.getMessage(), Problem.TYPE_ERROR, mes));
                                        setTestResult(fail);
                                        ex1.printStackTrace();
                                    }
                                } else {
                                    pbs.add(new Problem(7, Problem.TYPE_ERROR, mes));
                                    setTestResult(fail);
                                }
                            }
                        };
                        wait.setMessage(prov.getString("install_tables"));
                        wait.setVisible(true);

                    }
                }

                //postgis test

                if (!error) {
                    addRow(postgis);
                    Connection con = null;
                    Statement sta = null;
                    try {
                        con = hibernate.getConnection();
                        sta = con.createStatement();
                        if (org.atlas.util.PostGis.isPostgisInstalled(sta)) {
                            if (org.atlas.util.PostGis.isScriptPresent(sta)) {
                                setTestResult(pass);
                            } else {
                                setTestResult(warn);
                            }
                        } else {
                            setTestResult(warn);
                        }
                    } catch (Exception ex) {
                        ex.printStackTrace();
                        WindowUtils.msgError(parent, prov.getString("unex_error"));
                        setTestResult(fail);
                    } finally {
                        try {
                            sta.close();
                            con.close();
                        } catch (SQLException ex) {
                            ex.printStackTrace();
                        }
                    }

                }

                //wms metadata test;
                Service wmsService = null;
                if (!error) {
                    addRow(wms_meta);
                    try {
                        wmsService = Service.getService(hibernate);
                        if (wmsService == null) {
                            WindowUtils.msgInfo(parent, prov.getString("req_metadata"));
                            FrmServiceProps frm = new FrmServiceProps(hibernate, service);
                            frm.setVisible(true);
                            wmsService = Service.getService(hibernate);
                            if (wmsService == null) {
                                pbs.add(new Problem(8, Problem.TYPE_ERROR, mes));
                                setTestResult(fail);
                            } else {
                                setTestResult(pass);
                            }
                        } else {
                            setTestResult(pass);
                        }
                    } catch (Exception ex) {
                        WindowUtils.msgError(parent, prov.getString("unex_error"));
                        setTestResult(fail);
                        ex.printStackTrace();
                    }
                }

                if (!error) {
                    addRow(access);
                    boolean warnb = false;
                    try {
                        String host = new URL(wmsService.getServiceURL()).getHost().toLowerCase();
                        try {
                            InetAddress ia = InetAddress.getByName(host);
                            if (ia.isLoopbackAddress()) {
                                pbs.add(new Problem(10, Problem.TYPE_WARN, mes));
                                warnb = true;
                            }

                            if (!wmsService.isPublicaccess()) {
                                pbs.add(new Problem(11, Problem.TYPE_WARN, mes));
                                warnb = true;
                            }

                            if (warnb) {
                                setTestResult(warn);
                            } else {
                                setTestResult(pass);
                            }

                        } catch (UnknownHostException ex) {
                            ex.printStackTrace();
                        }
                    } catch (MalformedURLException ex) {
                        ex.printStackTrace();
                    }
                }

                String doc = "<font face=\"dialog\" size=\"3\"><br>";

                if (!isError()) {
                    btnContinue.setEnabled(true);
                    doc += (prov.getString("succ") + "<br />");
                }

                milis = System.currentTimeMillis() - milis;
                doc += (prov.getString("test_took") + " " + milis + " ms<br>");
                btnConnect.setEnabled(true);

                for (int i = 0; i < pbs.size(); i++) {
                    Problem p = (Problem) pbs.get(i);
                    doc += "<font color=\"Red\"><b>" + p.getDescription() + "</b></font><br><br>";
                    doc += "<b>" + prov.getString("cau") + " </b>" + p.getCause() + "<br><br>";
                    doc += "<b>" + prov.getString("sol") + " </b>" + p.getSolution() + "<br><br>";
                    if (p.getMessage() != null) {
                        doc += "<b>" + prov.getString("mes") + " </b>" + p.getMessage() + "<br><br>";
                    }

                }
                pnlLog.setText(doc);
                btnCancel.setEnabled(true);
            }
        });
        t.start();
    }

    private void showLastRow() {
        SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                Rectangle r = tblTest.getCellRect(tblTest.getRowCount(), 0, true);
                tblTest.scrollRectToVisible(r);
            }
        });
    }

    private void TCPIPTest(URL url, int timeOut) throws IOException {
        Socket socket = null;
        try {
            int port = url.getPort();
            String host = url.getHost();
            if (port <= 0) {
                port = url.getDefaultPort();
            }
            socket = new Socket();
            socket.connect(new InetSocketAddress(host, port), timeOut);
        } finally {
            try {
                if (socket != null) {
                    socket.close();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }

    private void HTTPTest(URL url) throws MalformedURLException, IOException {
        int port = url.getPort();
        if (port <= 0) {
            port = url.getDefaultPort();
        }
        URL basURL = new URL(url.getProtocol() + "://" + url.getHost() + ":" + port);
        HttpURLConnection urlCon = (HttpURLConnection) basURL.openConnection();
        urlCon.getResponseCode();
    }

    private boolean ATLASServerTest(ServiceConnection conn) throws HTTPRequestException, Exception {
        byte[] bytes = conn.getWmsAdminInfo(conn.getRequestBytes("GetSignature"));
        if (new String(bytes).equals(Service.SIGNATURE)) {
            return true;
        }
        return false;
    }

    private int getPort(URL url) {
        int port = url.getPort();
        if (port <= 0) {
            port = url.getDefaultPort();
        }
        return port;
    }

    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jPanel23 = new javax.swing.JPanel();
        btnCancel = new javax.swing.JButton();
        btnConnect = new javax.swing.JButton();
        btnContinue = new javax.swing.JButton();
        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel1 = new javax.swing.JPanel();
        tblTestSc = new javax.swing.JScrollPane();
        tblTest = new javax.swing.JTable();
        jScrollPane1 = new javax.swing.JScrollPane();
        pnlLog = new javax.swing.JEditorPane();
        jLabel2 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle(java.util.ResourceBundle.getBundle("windows/service/messages",LocaleUtil.getLocale()).getString("server_dig"));
        jPanel23.setLayout(new java.awt.GridLayout(1, 0, 5, 5));

        btnCancel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/process-stop.png")));
        btnCancel.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("Cancel"));
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        jPanel23.add(btnCancel);

        btnConnect.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/test.png")));
        btnConnect.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("test"));
        btnConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnConnectActionPerformed(evt);
            }
        });

        jPanel23.add(btnConnect);

        btnContinue.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/go-next.png")));
        btnContinue.setText(java.util.ResourceBundle.getBundle("windows/connections/messages",LocaleUtil.getLocale()).getString("continue"));
        btnContinue.setEnabled(false);
        btnContinue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnContinueActionPerformed(evt);
            }
        });

        jPanel23.add(btnContinue);

        jSplitPane1.setDividerLocation(250);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        tblTest.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        tblTestSc.setViewportView(tblTest);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(tblTestSc, javax.swing.GroupLayout.DEFAULT_SIZE, 463, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(tblTestSc, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 249, Short.MAX_VALUE)
        );
        jSplitPane1.setLeftComponent(jPanel1);

        pnlLog.setEditable(false);
        pnlLog.setContentType("text/html");
        jScrollPane1.setViewportView(pnlLog);

        jSplitPane1.setRightComponent(jScrollPane1);

        jLabel2.setText("Tests");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 465, Short.MAX_VALUE)
                            .addComponent(jPanel23, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addContainerGap())
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel2, javax.swing.GroupLayout.DEFAULT_SIZE, 463, Short.MAX_VALUE)
                        .addGap(14, 14, 14))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 345, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel23, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents
    private void btnContinueActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnContinueActionPerformed
        this.userCancel = false;
        this.dispose();
    }//GEN-LAST:event_btnContinueActionPerformed

    private void btnConnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnConnectActionPerformed
        this.test();
    }//GEN-LAST:event_btnConnectActionPerformed

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        this.userCancel = true;
        this.dispose();
    }//GEN-LAST:event_btnCancelActionPerformed

    public boolean isUserCancel() {
        return userCancel;
    }

    public boolean isError() {
        return error;
    }

    public boolean isWarning() {
        return warning;
    }

    public HibernateUtil getHibernate() {
        return hibernate;
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnConnect;
    private javax.swing.JButton btnContinue;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel23;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JEditorPane pnlLog;
    private javax.swing.JTable tblTest;
    private javax.swing.JScrollPane tblTestSc;
    // End of variables declaration//GEN-END:variables
}
