package windows.sources;

import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.Source;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.table.AbstractTableModel;
import org.atlas.util.HibernateUtil;
import org.atlas.i18n.LocaleUtil;
import org.atlas.i18n.StringProvider;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import javax.swing.ListSelectionModel;
import org.hibernate.classic.Session;
import org.atlas.util.PostGis;
import org.atlas.util.ServiceConnection;
import org.atlas.util.WindowUtils;
import org.atlas.util.FrmWait;

public class FrmAddPostgis extends javax.swing.JDialog {

    private HibernateUtil hibernate;
    private ServiceConnection serviceConnection;
    private StringProvider src_provider = new org.atlas.i18n.StringProvider("windows/sources/messages");
    private StringProvider provider = new org.atlas.i18n.StringProvider("windows/sources/messages");
    private GeometryColumn[] columns;
    private boolean userCancel;
    private Project project;

    public FrmAddPostgis(Project project, HibernateUtil hibernate, ServiceConnection serviceConnection) throws Exception {
        super((JFrame) null, true);
        this.project = project;
        this.hibernate = hibernate;
        this.serviceConnection = serviceConnection;
        initComponents();
        WindowUtils.center(this);
        WindowUtils.setFrameIcon(this, "/media/appicon.png");
        this.cargarTabla();
        this.tblSources.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    }

    private void cargarTabla() throws Exception {

        Connection con = null;
        Statement st = null;
        ResultSet rs = null;

        try {
            con = hibernate.getConnection();
            st = con.createStatement();
            boolean postgis = false;
            
            if (PostGis.isPostgisInstalled(st)) {
                if (PostGis.isScriptPresent(st)) {
                    postgis = true;
                }
            }
            
            if (!postgis) {
                throw new IllegalStateException("Postgis is not intalled");
            }
                        
            rs = st.executeQuery("SELECT f_table_catalog, f_table_schema, f_table_name, f_geometry_column, srid, \"type\" FROM geometry_columns;");

            ArrayList<GeometryColumn> l = new ArrayList<GeometryColumn>();

            while (rs.next()) {
                GeometryColumn g = new GeometryColumn();
                g.setCatalog(rs.getString("f_table_catalog"));
                g.setSchema(rs.getString("f_table_schema"));
                g.setTable_name(rs.getString("f_table_name"));
                g.setGeometry_column(rs.getString("f_geometry_column"));
                g.setSrid(rs.getInt("srid"));
                g.setType(rs.getString("type"));
                l.add(g);
            }
            columns = new GeometryColumn[l.size()];
            l.toArray(columns);
            tblSources.setModel(new ColumnsTableModel(columns));
        } catch (IllegalStateException ex) {
            Logger.getLogger(FrmAddPostgis.class.getName()).log(Level.WARNING, null, ex);
            WindowUtils.msgError(rootPane, src_provider.getString("no_postgis"));
        } catch (Exception ex) {
            Logger.getLogger(FrmAddPostgis.class.getName()).log(Level.SEVERE, null, ex);
            WindowUtils.msgError(rootPane, src_provider.getString("unex_error"));
            throw new Exception("Unexpected Error");
        } finally {
            try {
                rs.close();
                st.close();
                con.close();
            } catch (SQLException ex) {
                Logger.getLogger(FrmAddPostgis.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public boolean isUserCancel() {
        return userCancel;
    }

    public void setUserCancel(boolean userCancel) {
        this.userCancel = userCancel;
    }

    private class GeometryColumn {

        private String catalog;
        private String schema;
        private String table_name;
        private String geometry_column;
        private int srid;
        private String type;

        public String getCatalog() {
            return catalog;
        }

        public void setCatalog(String catalog) {
            this.catalog = catalog;
        }

        public String getSchema() {
            return schema;
        }

        public void setSchema(String schema) {
            this.schema = schema;
        }

        public String getTable_name() {
            return table_name;
        }

        public void setTable_name(String table_name) {
            this.table_name = table_name;
        }

        public String getGeometry_column() {
            return geometry_column;
        }

        public void setGeometry_column(String geometry_column) {
            this.geometry_column = geometry_column;
        }

        public int getSrid() {
            return srid;
        }

        public void setSrid(int srid) {
            this.srid = srid;
        }

        public String getType() {
            return type;
        }

        public void setType(String type) {
            this.type = type;
        }
    }

    private class ColumnsTableModel extends AbstractTableModel {

        private final GeometryColumn[] columns;
        String[] titulos = {src_provider.getString("schema"), src_provider.getString("table"), src_provider.getString("column"), src_provider.getString("srid"), src_provider.getString("type")};

        public ColumnsTableModel(GeometryColumn[] columns) {
            this.columns = columns;
        }

        @Override
        public boolean isCellEditable(int row, int column) {
            return false;
        }

        @Override
        public String getColumnName(int column) {
            return titulos[column];
        }

        public int getRowCount() {
            return getColumns().length;
        }

        public int getColumnCount() {
            return titulos.length;
        }

        public Object getValueAt(int row, int column) {
            Object obj = null;
            switch (column) {
                case 0:
                    obj = getColumns()[row].getSchema();
                    break;
                case 1:
                    obj = getColumns()[row].getTable_name();
                    break;
                case 2:
                    obj = getColumns()[row].getGeometry_column();
                    break;
                case 3:
                    obj = getColumns()[row].getSrid();
                    break;
                case 4:
                    obj = getColumns()[row].getType();
                    break;
                }
            return obj;
        }

        private GeometryColumn[] getColumns() {
            return columns;
        }
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        tblSources = new javax.swing.JTable();
        btnContinue = new javax.swing.JButton();
        btnCancel = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/sources/messages",LocaleUtil.getLocale()); // NOI18N
        setTitle(bundle.getString("frm_postgis")); // NOI18N
        setLocationByPlatform(true);

        tblSources.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(tblSources);

        btnContinue.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/go-next.png"))); // NOI18N
        btnContinue.setText(bundle.getString("continue")); // NOI18N
        btnContinue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnContinueActionPerformed(evt);
            }
        });

        btnCancel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/process-stop.png"))); // NOI18N
        btnCancel.setText(bundle.getString("cancel")); // NOI18N
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        java.util.ResourceBundle bundle1 = java.util.ResourceBundle.getBundle("windows/sources/messages"); // NOI18N
        jLabel1.setText(bundle1.getString("geom_columns")); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 589, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(btnCancel, javax.swing.GroupLayout.PREFERRED_SIZE, 166, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(5, 5, 5)
                        .addComponent(btnContinue, javax.swing.GroupLayout.PREFERRED_SIZE, 166, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jLabel1, javax.swing.GroupLayout.DEFAULT_SIZE, 589, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 217, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(btnCancel)
                    .addComponent(btnContinue))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private GeometryColumn getSelectedColumn() {
        if (tblSources.getSelectedRow() >= 0) {
            return columns[tblSources.getSelectedRow()];
        }
        return null;
    }

    private void btnContinueActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnContinueActionPerformed
        if (getSelectedColumn() == null) {
            WindowUtils.msgInfo(this, provider.getString("select_column"));
            return;
        }

        Source[] sources = new Source[project.getSources().size()];
        project.getSources().toArray(sources);
        boolean already = false;
        for (int i = 0; i < sources.length && !already; i++) {
            Source source = sources[i];
            if (source.getTablsour().toLowerCase().equals(getSelectedColumn().getTable_name().toLowerCase())) {
                if (source.getFielsour().toLowerCase().equals(getSelectedColumn().getGeometry_column().toLowerCase())) {
                    already = true;
                }

            }
        }

        if (already) {
            WindowUtils.msgInfo(this, provider.getString("source_already"));
            return;
        }

        try {
            PostGis.getGeometryClass(getSelectedColumn().getType());
        } catch (IllegalArgumentException illegalArgumentException) {
            WindowUtils.msgError(this, provider.getString("unsupp_type"));
        }

        if (project.getSpatial_ref_sys().getSrid() != getSelectedColumn().getSrid()) {
            if (!WindowUtils.msgYesNo(rootPane, provider.getString("srid_no_match"))) {
                return;
            }
        }


        FrmWait frm = new FrmWait() {
            @Override
            public void action() {
                try {
                    Connection con = hibernate.getConnection();                                        
                    if(!PostGis.isValidForImport(getSelectedColumn().getTable_name(), con)){
                        String err = java.util.ResourceBundle.getBundle("windows/sources/messages",LocaleUtil.getLocale()).getString("pk_req");
                        WindowUtils.msgError(this, err);
                        con.close();    
                        return;
                    }
                    con.close();
                    Session sess = hibernate.begin();
                    Source src = new Source();
                    src.setTablsour(getSelectedColumn().getTable_name());
                    src.setFielsour(getSelectedColumn().getGeometry_column());
                    src.setGeomType(PostGis.getGeometryClass(getSelectedColumn().getType()));
                    sess.update(project);
                    sess.save(src);
                    
                    project.getSources().add(src);
                    src.setProject(project);
                    
                    con = hibernate.getConnection();
                    src.loadFeatures(con);
                    con.close();
                    src.calcularDimensiones();
                    hibernate.commitAndClose(sess);
                } catch (Exception ex) {
                    Logger.getLogger(FrmAddPostgis.class.getName()).log(Level.SEVERE, null, ex);
                    WindowUtils.msgError(this, provider.getString("error"));
                    dispose();
                }
            }
        };
        frm.setMessage(provider.getString("wait"));
        frm.setVisible(true);
        dispose();
    }//GEN-LAST:event_btnContinueActionPerformed

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        this.setUserCancel(true);
        this.setVisible(false);
    }//GEN-LAST:event_btnCancelActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnContinue;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable tblSources;
    // End of variables declaration//GEN-END:variables
}
