package windows.sources;

import org.atlas.hibernatedata.Icon;
import org.atlas.hibernatedata.Layer;
import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.Rule;
import org.atlas.hibernatedata.Service;
import org.atlas.hibernatedata.Source;
import org.atlas.hibernatedata.Symbology;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.table.AbstractTableModel;
import org.atlas.util.HibernateUtil;
import org.atlas.i18n.LocaleUtil;
import org.atlas.i18n.StringProvider;
import java.io.ByteArrayOutputStream;
import java.util.Arrays;
import java.util.Comparator;
import javax.imageio.ImageIO;
import javax.swing.ListSelectionModel;
import org.hibernate.classic.Session;
import org.atlas.util.ServiceConnection;
import org.atlas.util.WindowUtils;
import org.atlas.util.FrmWait;
import org.atlas.util.FrmWait;
import org.atlas.util.FrmWait;
import windows.symbology.FixedRuleGenerator;

public class FrmLayerSrc extends javax.swing.JDialog {

    private HibernateUtil hibernate;
    private Layer layer;
    private ServiceConnection serviceConnection;
    private StringProvider src_provider = new org.atlas.i18n.StringProvider("windows/sources/messages");
    private StringProvider provider = new org.atlas.i18n.StringProvider("windows/sources/messages");
    private Source[] sources;
    private boolean userCancel;

    public FrmLayerSrc(Layer layer, HibernateUtil hibernate, ServiceConnection serviceConnection) {
        super((JFrame) null, true);
        this.hibernate = hibernate;
        this.layer = layer;
        this.serviceConnection = serviceConnection;
        initComponents();
        WindowUtils.center(this);
        WindowUtils.setFrameIcon(this, "/media/appicon.png");
        this.cargarTabla();
        this.tblSources.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        if (layer.getSource() == null) {
            tblSources.getSelectionModel().setSelectionInterval(0, 0);
        } else {
            for (int i = 0; i < sources.length; i++) {
                if (sources[i].getCodesour() == layer.getSource().getCodesour()) {
                    tblSources.getSelectionModel().setSelectionInterval(i + 1, i + 1);
                    break;
                }
            }
        }
    }

    private void cargarTabla() {
        Session sess = null;
        sources = new Source[0];

        try {
            Project project = layer.getProject();
            sess = hibernate.begin();
            sess.update(project);
            sources = new Source[project.getSources().size()];
            project.getSources().toArray(sources);

            Arrays.sort(sources, new Comparator() {

                public int compare(Object o1, Object o2) {
                    return ((Source) o1).getTablsour().compareTo(((Source) o2).getTablsour());
                }
            });

            for (int i = 0; i < sources.length; i++) {
                sess.update(sources[i]);
                sources[i].getLayers().size();
            }
            tblSources.setModel(new SourcesTableModel(sources));
        } catch (Exception ex) {
            ex.printStackTrace();
            return;
        } finally {
            hibernate.close(sess);
        }
    }

    public boolean isUserCancel() {
        return userCancel;
    }

    public void setUserCancel(boolean userCancel) {
        this.userCancel = userCancel;
    }

    private class SourcesTableModel extends AbstractTableModel {

        private final Source[] sources;
        String assig = src_provider.getString("assig");
        String no_assig = src_provider.getString("no_assig");

        public SourcesTableModel(Source[] sources) {
            this.sources = sources;
        }

        @Override
        public boolean isCellEditable(int row, int column) {
            return false;
        }

        @Override
        public String getColumnName(int column) {
            String[] titulos = {src_provider.getString("table"), src_provider.getString("column"), src_provider.getString("type")};
            return titulos[column];
        }

        public int getRowCount() {
            return getSources().length + 1;
        }

        public int getColumnCount() {
            return 3;
        }

        public Object getValueAt(int row, int column) {
            Object obj;
            if (row > 0) {
                switch (column) {
                    case 0:
                        obj = getSources()[row - 1].getTablsour();
                        break;
                    case 1:
                        obj = getSources()[row - 1].getFielsour();
                        break;
                    case 2:
                        obj = getSources()[row - 1].getGeomClass().getSimpleName().toUpperCase();
                        break;
                    default:
                        obj = null;
                }
            } else {
                return provider.getString("none");
            }
            return obj;
        }

        public Source[] getSources() {
            return sources;
        }
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        tblSources = new javax.swing.JTable();
        btnContinue = new javax.swing.JButton();
        btnCancel = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/sources/messages",LocaleUtil.getLocale()); // NOI18N
        setTitle(bundle.getString("frm_attrib")); // NOI18N
        setResizable(false);

        tblSources.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(tblSources);

        btnContinue.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/go-next.png"))); // NOI18N
        btnContinue.setText(bundle.getString("continue")); // NOI18N
        btnContinue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnContinueActionPerformed(evt);
            }
        });

        btnCancel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/process-stop.png"))); // NOI18N
        btnCancel.setText(bundle.getString("cancel")); // NOI18N
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 510, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(172, 172, 172)
                        .addComponent(btnCancel, javax.swing.GroupLayout.PREFERRED_SIZE, 166, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnContinue, javax.swing.GroupLayout.PREFERRED_SIZE, 166, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 181, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 11, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnContinue)
                    .addComponent(btnCancel))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private Source getSelectedSource() {
        if (tblSources.getSelectedRow() > 0) {
            return sources[tblSources.getSelectedRow() - 1];
        }
        return null;
    }

    private void btnContinueActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnContinueActionPerformed
        if (getSelectedSource() == null) {
            if (layer.getSource() == null) {
                WindowUtils.msgInfo(this, provider.getString("no_change"));
                return;
            }
        } else {
            if (layer.getSource() != null) {
                if (getSelectedSource().getCodesour() == layer.getSource().getCodesour()) {
                    WindowUtils.msgInfo(this, provider.getString("no_change"));
                    return;
                }
            }
        }

        FrmWait frm = new FrmWait() {

            @Override
            public void action() {
                try {
                    Session sess = hibernate.begin();
                    sess.update(layer);
                    Symbology symb = layer.getSymbology();
                    if (symb != null) {
                        symb.delete(sess);
                        hibernate.commit(sess);
                        hibernate.begin(sess);
                    }

                    layer.setSource(getSelectedSource());
                    
                    if (getSelectedSource() != null) {
                        symb = new Symbology();
                        layer.setSymbology(symb);
                        symb.setLayer(layer);

                        Icon defaultIcon = (Icon) sess.createQuery("FROM Icon WHERE descicon = 'default.svg'").list().get(0);
                        FixedRuleGenerator.attachFixedSymbology(symb, defaultIcon);
                        Symbology.attachDefaulFont(symb);
                        ByteArrayOutputStream baos = new ByteArrayOutputStream();
                        Rule rule = (Rule) symb.getRules().toArray()[0];
                        ImageIO.write(rule.getStyle().getIconImage(), "png", baos);
                        rule.setBinarule(baos.toByteArray());
                        symb.save(sess);
                    }else{
                        layer.setSymbology(null);
                    }
                    
                    hibernate.commitAndClose(sess);
                    
                    Service service = Service.getService(hibernate);
                    service.updateCapabilities(hibernate, serviceConnection);
                    this.dispose();
                } catch (Exception ex) {
                    Logger.getLogger(FrmLayerSrc.class.getName()).log(Level.SEVERE, null, ex);
                    WindowUtils.msgError(this, provider.getString("error"));
                }
            }
        };

        frm.setMessage(provider.getString("wait"));
        frm.setVisible(true);
        dispose();
    }//GEN-LAST:event_btnContinueActionPerformed

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        this.setUserCancel(true);
        this.setVisible(false);
    }//GEN-LAST:event_btnCancelActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnContinue;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable tblSources;
    // End of variables declaration//GEN-END:variables
}
