package windows.sources;

import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.Source;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.event.ListSelectionEvent;
import org.atlas.util.HibernateUtil;
import org.atlas.i18n.LocaleUtil;
import java.io.File;
import java.util.Arrays;
import java.util.Comparator;
import javax.swing.JFileChooser;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import org.atlas.geocoding.Geocoder;
import org.atlas.hibernatedata.GeocoderConfig;
import org.hibernate.classic.Session;
import org.atlas.util.Configuration;
import org.atlas.util.ServiceConnection;
import org.atlas.util.WindowUtils;
import org.atlas.util.FrmWait;
import windows.sources.FrmAddShapeFile;

public class FrmSources extends javax.swing.JDialog {

    private HibernateUtil hibernate;
    private Project project;
    private org.atlas.i18n.StringProvider provider;
    private ServiceConnection con;
    private JFileChooser file;
    private Source[] sources;
    private boolean[] uses;

    public FrmSources(Project project, HibernateUtil hu, ServiceConnection con) {
        super((JFrame) null, true);
        try {
            initComponents();
            this.hibernate = hu;
            this.project = project;
            this.provider = new org.atlas.i18n.StringProvider("windows/sources/messages");
            this.con = con;
            this.jComboBox1.addItem(provider.getString("select_source_type"));
            this.jComboBox1.addItem("Postgis");
            this.jComboBox1.addItem("Shape File");

            file = new JFileChooser();
            file.setFileSelectionMode(JFileChooser.FILES_ONLY);
            file.setAcceptAllFileFilterUsed(false);
            file.setFileFilter(new FileFilter() {

                public String getDescription() {
                    return "Shape File";
                }

                public boolean accept(File f) {
                    return f.isDirectory() || f.getAbsolutePath().toLowerCase().endsWith("shp");
                }
            });

            tblSources.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            tblSources.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

                public void valueChanged(ListSelectionEvent e) {
                    if (tblSources.getSelectedRow() == -1) {
                        btnRemSrc.setEnabled(false);
                        return;
                    }
                    btnRemSrc.setEnabled(true);
                }
            });
            cargarTabla();
            WindowUtils.center(this);
            WindowUtils.setFrameIcon(this, "/media/appicon.png");
        } catch (Exception ex) {
            WindowUtils.msgError(this, "Unexpected error " + ex.getMessage());
            dispose();
        }
    }

    private void cargarTabla() throws Exception {



        Session sess = null;
        sources = new Source[0];
        try {
            sess = hibernate.begin();
            sess.update(project);

            GeocoderConfig[] cfgs = new GeocoderConfig[project.getGeocoderConfigs().size()];
            project.getGeocoderConfigs().toArray(cfgs);

            try {
                project.loadGeocoders();
            } catch (Exception ex) {
                WindowUtils.msgError(this, "Unexpected error: " + ex.getMessage());
                return;
            }

            Geocoder[] geocs = project.getGeocoders();

            sources = new Source[project.getSources().size()];
            uses = new boolean[project.getSources().size()];

            project.getSources().toArray(sources);

            Arrays.sort(sources, new Comparator() {

                public int compare(Object o1, Object o2) {
                    return ((Source) o1).getTablsour().compareTo(((Source) o2).getTablsour());
                }
            });

            for (int i = 0; i < sources.length; i++) {
                sess.update(sources[i]);
                sess.refresh(sources[i]);
                uses[i] = false;
                if (sources[i].getLayers().size() > 0) {
                    uses[i] = true;
                }

                for (int j = 0; j < geocs.length; j++) {
                    Geocoder geocoder = geocs[j];
                    uses[i] = uses[i] || geocoder.isUsingSource(sources[i].getCodesour());
                }

            }
            tblSources.setModel(new SourcesTableModel());
        } catch (Exception ex) {
            ex.printStackTrace();
            return;
        } finally {
            try {
                hibernate.close(sess);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jToggleButton1 = new javax.swing.JToggleButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        tblSources = new javax.swing.JTable();
        btnClose = new javax.swing.JButton();
        btnAddSrc = new javax.swing.JButton();
        btnRemSrc = new javax.swing.JButton();
        jComboBox1 = new javax.swing.JComboBox();

        jToggleButton1.setText("jToggleButton1");

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/sources/messages",LocaleUtil.getLocale()); // NOI18N
        setTitle(bundle.getString("source_man")); // NOI18N
        setResizable(false);

        tblSources.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(tblSources);

        btnClose.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/volver.png"))); // NOI18N
        btnClose.setText(bundle.getString("close")); // NOI18N
        btnClose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCloseActionPerformed(evt);
            }
        });

        btnAddSrc.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/plus.png"))); // NOI18N
        btnAddSrc.setToolTipText("");
        btnAddSrc.setEnabled(false);
        btnAddSrc.setIconTextGap(0);
        btnAddSrc.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnAddSrc.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddSrcActionPerformed(evt);
            }
        });

        btnRemSrc.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/trash.png"))); // NOI18N
        btnRemSrc.setToolTipText("");
        btnRemSrc.setDefaultCapable(false);
        btnRemSrc.setEnabled(false);
        btnRemSrc.setIconTextGap(0);
        btnRemSrc.setMargin(new java.awt.Insets(0, 0, 0, 0));
        btnRemSrc.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRemSrcActionPerformed(evt);
            }
        });

        jComboBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox1ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 442, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(jComboBox1, javax.swing.GroupLayout.PREFERRED_SIZE, 201, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnAddSrc)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnRemSrc))
                    .addComponent(btnClose, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 167, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jComboBox1, javax.swing.GroupLayout.DEFAULT_SIZE, 29, Short.MAX_VALUE)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addComponent(btnAddSrc, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(btnRemSrc, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 160, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnClose)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private void btnCloseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCloseActionPerformed
        this.setVisible(false);
    }//GEN-LAST:event_btnCloseActionPerformed

    private void btnAddSrcActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddSrcActionPerformed
        try {
            if (jComboBox1.getSelectedIndex() == 2) {

                Configuration cfg = Configuration.grantReadConfiguration("config.xml");
                String dir = cfg.getLastGeomDirectory();

                if (!dir.equals("")) {
                    file.setCurrentDirectory(new File(dir));
                }

                int res = file.showOpenDialog(null);
                if (res == JFileChooser.APPROVE_OPTION) {
                    File arch = file.getSelectedFile();
                    try {
                        cfg.setLastGeomDirectory(arch.getCanonicalPath());
                        cfg.writeConfiguration("config.xml");
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                    FrmAddShapeFile frmImport = new FrmAddShapeFile(project, arch, hibernate, con);
                    frmImport.setVisible(true);
                }
            } else {
                try {
                    FrmAddPostgis frm = new FrmAddPostgis(project, hibernate, con);
                    frm.setVisible(true);
                } catch (Exception ex) {
                    Logger.getLogger(FrmSources.class.getName()).log(Level.SEVERE, null, ex);
                }
            }

            cargarTabla();
        } catch (Exception ex) {
            Logger.getLogger(FrmSources.class.getName()).log(Level.SEVERE, null, ex);
        }
}//GEN-LAST:event_btnAddSrcActionPerformed

    public Source getSelectedSource() {
        int index = tblSources.getSelectedRow();
        if (index < 0) {
            return null;
        }
        return sources[index];
    }

    public boolean isSelectedUsed() {
        int index = tblSources.getSelectedRow();
        if (index < 0) {
            return false;
        }
        return uses[index];
    }

    private void btnRemSrcActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRemSrcActionPerformed
        if (getSelectedSource() == null) {
            WindowUtils.msgInfo(file, provider.getString("no_src"));
            return;
        }

        if (isSelectedUsed()) {
            WindowUtils.msgInfo(file, provider.getString("still_active"));
            return;
        }


        FrmWait frm = new FrmWait() {

            @Override
            public void action() {
                try {
                    Session sess = hibernate.begin();
                    sess.update(project);
                    project.getSources().remove(getSelectedSource());
                    sess.delete(getSelectedSource());
                    hibernate.commitAndClose(sess);
                    cargarTabla();
                } catch (Exception ex) {
                    Logger.getLogger(FrmSources.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        };
        frm.setVisible(true);
}//GEN-LAST:event_btnRemSrcActionPerformed

    private void jComboBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox1ActionPerformed
        if (jComboBox1.getSelectedIndex() == 0) {
            btnAddSrc.setEnabled(false);
            return;
        }
        btnAddSrc.setEnabled(true);
    }//GEN-LAST:event_jComboBox1ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAddSrc;
    private javax.swing.JButton btnClose;
    private javax.swing.JButton btnRemSrc;
    private javax.swing.JComboBox jComboBox1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JToggleButton jToggleButton1;
    private javax.swing.JTable tblSources;
    // End of variables declaration//GEN-END:variables
    private class SourcesTableModel extends AbstractTableModel {

        String assig = provider.getString("assig");
        String no_assig = provider.getString("no_assig");

        @Override
        public boolean isCellEditable(int row, int column) {
            return false;
        }

        @Override
        public String getColumnName(int column) {
            String[] titulos = {provider.getString("table"), provider.getString("column"), provider.getString("type"), provider.getString("state")};
            return titulos[column];
        }

        public int getRowCount() {
            return sources.length;
        }

        public int getColumnCount() {
            return 4;
        }

        public Object getValueAt(int row, int column) {
            Object obj;
            switch (column) {
                case 0:
                    obj = sources[row].getTablsour();
                    break;
                case 1:
                    obj = sources[row].getFielsour();
                    break;
                case 2:
                    obj = sources[row].getGeomClass().getSimpleName().toUpperCase();
                    break;
                case 3:
                    if (uses[row]) {
                        obj = assig;
                    } else {
                        obj = no_assig;
                    }
                    break;
                default:
                    obj = null;
            }
            return obj;
        }
    }
}
