/*
 * Created on 18 de mayo de 2007, 11:44 AM
 */
package windows.spatial_ref_sys;

import java.io.IOException;
import java.util.Enumeration;
import java.util.List;
import javax.swing.JFrame;
import javax.swing.JTree;
import javax.swing.text.AbstractDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.atlas.i18n.LocaleUtil;
import org.atlas.util.WindowUtils;

public class FrmSelectRefSys extends javax.swing.JDialog {

    private class sysProy {

        int srid;
        String name;

        public sysProy(int srid, String name) {
            this.srid = srid;
            this.name = name;
        }

        @Override
        public String toString() {
            return name;
        }
    }
    /** Creates new form FrmSelectRefSys */
    private int srid;
    private String spatialRefenceSystemName;
    private DefaultMutableTreeNode root = new DefaultMutableTreeNode("Spacial Reference Systems");
    private DefaultTreeModel treeModel = new DefaultTreeModel(root);

    public FrmSelectRefSys(String filter) {
        super((JFrame) null, true);
        initComponents();
        WindowUtils.center(this);
        WindowUtils.setFrameIcon(this, "/media/appicon.png");
        this.loadTree(filter);

        jTextField1.setDocument(new PlainDocument() {

            @Override
            public void insertString(int i, String string, AttributeSet attributeSet) throws BadLocationException {
                super.insertString(i, string, attributeSet);
                loadTree(jTextField1.getText().trim());
            }

            @Override
            protected void postRemoveUpdate(AbstractDocument.DefaultDocumentEvent defaultDocumentEvent) {
                super.postRemoveUpdate(defaultDocumentEvent);
                loadTree(jTextField1.getText().trim());
            }
        });

        jTextField1.setText(filter);        
    }

    public void expandAll(JTree tree, boolean expand) {
        TreeNode rootP = (TreeNode) tree.getModel().getRoot();
        expandAll(tree, new TreePath(rootP), expand);
    }

    private void expandAll(JTree tree, TreePath parent, boolean expand) {

        TreeNode node = (TreeNode) parent.getLastPathComponent();
        if (node.getChildCount() >= 0) {
            for (Enumeration e = node.children(); e.hasMoreElements();) {
                TreeNode n = (TreeNode) e.nextElement();
                TreePath path = parent.pathByAddingChild(n);
                expandAll(tree, path, expand);
            }
        }

        if (expand) {
            tree.expandPath(parent);
        } else {
            tree.collapsePath(parent);
        }
    }

    public void loadTree(String filter) {

        root.removeAllChildren();

        DefaultMutableTreeNode gcs = new DefaultMutableTreeNode("Geographic Coordinate Systems");
        DefaultMutableTreeNode pcs = new DefaultMutableTreeNode("Projected Coordinate Systems");

        this.root.add(gcs);
        this.root.add(pcs);

        try {
            SAXBuilder parser = new SAXBuilder();
            Document doc = parser.build(this.getClass().getResourceAsStream("/sql/spatial_ref_sys.xml"));
            Element rootX = doc.getRootElement();
            Element geoc = rootX.getChild("geogcs");
            List l = null;
            filter = filter.toLowerCase();
            String param[] = filter.split(" ");
            int allpara;

            l = geoc.getChildren();
            for (int i = 0; i < l.size(); i++) {
                Element e = (Element) l.get(i);
                String cad = "EPSG:" + e.getChild("srid").getText() + " " + e.getText().trim();

                allpara = 0;
                for (int j = 0; j < param.length; j++) {
                    if (cad.toLowerCase().contains(param[j])) {
                        allpara++;
                    }
                }

                if (allpara == param.length || filter.equals("")) {
                    sysProy sp = new sysProy(Integer.parseInt(e.getChild("srid").getText()), cad);
                    DefaultMutableTreeNode nodoSys = new DefaultMutableTreeNode(sp);
                    gcs.add(nodoSys);
                }
            }

            Element sysProy = rootX.getChild("projcs");
            l = sysProy.getChildren();
            for (int i = 0; i < l.size(); i++) {
                Element e = (Element) l.get(i);
                DefaultMutableTreeNode nodoSysProy = new DefaultMutableTreeNode(e.getText().trim().replaceAll("_", " "));
                pcs.add(nodoSysProy);
                List l1 = e.getChildren();
                boolean ban = false;

                for (int j = 0; j < l1.size(); j++) {
                    Element ei = (Element) l1.get(j);
                    String cad = "EPSG:" + ei.getChild("srid").getText() + " " + ei.getText().trim();

                    allpara = 0;
                    for (int k = 0; k < param.length; k++) {
                        if (cad.toLowerCase().contains(param[k])) {
                            allpara++;
                        }
                    }

                    if (allpara == param.length || filter.equals("")) {
                        sysProy sp = new sysProy(Integer.parseInt(ei.getChild("srid").getText()), cad);
                        DefaultMutableTreeNode nodoSys = new DefaultMutableTreeNode(sp);
                        nodoSysProy.add(nodoSys);
                        ban = true;
                    }
                }

                if (ban == false) {
                    pcs.remove(nodoSysProy);
                }

            }
            treeModel.reload();
            expandAll(this.jTree1, true);

        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (JDOMException ex) {
            ex.printStackTrace();
        }

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        jTree1 = new JTree(treeModel);
        jPanel1 = new javax.swing.JPanel();
        jButton2 = new javax.swing.JButton();
        jButton1 = new javax.swing.JButton();
        jTextField1 = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();

        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/spatial_ref_sys/messages",LocaleUtil.getLocale()); // NOI18N
        setTitle(bundle.getString("srsys_title")); // NOI18N

        jTree1.setLargeModel(true);
        jTree1.setSelectionModel(jTree1.getSelectionModel());
        jScrollPane1.setViewportView(jTree1);

        jPanel1.setLayout(new java.awt.GridLayout(1, 2, 3, 3));

        jButton2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/process-stop.png"))); // NOI18N
        jButton2.setText(bundle.getString("cancel_key")); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        jButton1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/media/go-next.png"))); // NOI18N
        jButton1.setText(bundle.getString("next_key")); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jTextField1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextField1ActionPerformed(evt);
            }
        });

        jLabel1.setText("Buscar : ");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 592, Short.MAX_VALUE)
                    .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 337, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jTextField1, javax.swing.GroupLayout.DEFAULT_SIZE, 546, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(jButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 167, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 167, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 302, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(14, 14, 14)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel1)
                            .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, 28, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addGap(15, 15, 15)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton1, javax.swing.GroupLayout.PREFERRED_SIZE, 34, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jButton2, javax.swing.GroupLayout.PREFERRED_SIZE, 34, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private void jTextField1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextField1ActionPerformed
        this.loadTree(this.jTextField1.getText().trim());
    }//GEN-LAST:event_jTextField1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        this.setVisible(false);
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        DefaultMutableTreeNode nodo = (DefaultMutableTreeNode) this.jTree1.getLastSelectedPathComponent();

        if (nodo != null) {
            if (nodo.isLeaf()) {
                sysProy sp = (sysProy) nodo.getUserObject();
                this.setSrid(sp.srid);
                this.setSpatialRefenceSystemName(sp.name);
                this.setVisible(false);
            } else {
                WindowUtils.msgError(this, "Por favor, seleccione un sistema de referencia espacial.");
            }
        } else {
            WindowUtils.msgError(this, "Por favor, seleccione un elemento del arbol.");
        }
    }//GEN-LAST:event_jButton1ActionPerformed

    public int getSrid() {
        return srid;
    }

    public void setSrid(int srid) {
        this.srid = srid;
    }

    public String getSpatialRefenceSystemName() {
        return spatialRefenceSystemName;
    }

    public void setSpatialRefenceSystemName(String spatialRefenceSystemName) {
        this.spatialRefenceSystemName = spatialRefenceSystemName;
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTree jTree1;
    // End of variables declaration//GEN-END:variables
}
