package windows.style;

import org.atlas.hibernatedata.AtlasColor;
import org.atlas.hibernatedata.Icon;
import org.atlas.hibernatedata.Style;
import org.atlas.i18n.LocaleUtil;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.atlas.util.ColorsList;

public class PnlEditPointStyle extends javax.swing.JPanel implements StyleEditor {

    private Style style;
    private List<ChangeListener> changeListeners = new ArrayList<ChangeListener>();
    private boolean affectStyle = true;
    private ColorsList colors = new ColorsList();

    public PnlEditPointStyle() {
        initComponents();
        iconsPanel.setMultipleSelectionAllowed(false);

        iconsPanel.addItemsSelectedListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (iconsPanel.getSelectedIcons().length > 0) {
                    Icon ic = (Icon) iconsPanel.getSelectedIcons()[0];
                    style.setIcon(ic);
                    putPicture();
                }
            }
        });

        jSlider2.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                if (affectStyle) {
                    style.getLinestyl().setWidth((float) jSlider2.getValue());
                    putPicture();
                    fireChangeListeners();
                }
            }
        });

        colorBorder.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                style.setLinecolo(colorBorder.getColor());
                putPicture();
            }
        });

        colorFill.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                style.setFillcolo(colorFill.getColor());
                putPicture();
            }
        });
        setEnabled(false);
    }

    private void putPicture() {
        jLabel1.setIcon(new ImageIcon(style.getIconImage()));
    }

    @Override
    public void setEnabled(boolean b) {
        super.setEnabled(b);
        boolean tmp = affectStyle;
        affectStyle = false;

        colorFill.setEnabled(false);
        colorBorder.setEnabled(false);
        jSlider2.setEnabled(b);
        jLabel2.setEnabled(b);
        jLabel1.setEnabled(b);
        jLabel1.setIcon(new ImageIcon(new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB)));
        getIconsPanel().setEnabled(b);
        jSlider2.setValue(0);
        jCheckBox1.setSelected(false);
        jCheckBox1.setEnabled(b);
        iconsPanel.setEnabled(b);
        TitledBorder title2 = new TitledBorder(LocaleUtil.getString("windows/style/messages", "icon"));
        if (!b) {
            title2.setTitleColor(SystemColor.inactiveCaption);
        }
        pnlLine.setBorder(title2);
        affectStyle = tmp;
    }

    public void setStyle(Style style) {
        this.style = style;
        affectStyle = false;
        if (style != null) {
            try {
                setEnabled(true);
                int width = (int) style.getLinestyl().getWidth();
                jSlider2.setValue(width);
                if (style.getLinecolo() != null) {
                    jCheckBox1.setSelected(true);
                    colorFill.setEnabled(true);
                    colorBorder.setEnabled(true);
                    colorBorder.setColor(style.getLinecolo());
                    colorFill.setColor(style.getFillcolo());
                } else {
                    jCheckBox1.setSelected(false);
                    colorFill.setEnabled(false);
                    colorBorder.setEnabled(false);
                }
                iconsPanel.setSelectedIcon(style.getIcon());
                putPicture();
            } catch (Exception ex) {
                Logger.getLogger(PnlEditPointStyle.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            setEnabled(false);
        }
        affectStyle = true;
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        atlasColorChooserOpac2 = new windows.controls.AtlasColorChooserOpac();
        pnlLine = new javax.swing.JPanel();
        iconsPanel = new windows.icons.IconsPanel();
        jPanel1 = new javax.swing.JPanel();
        jCheckBox1 = new javax.swing.JCheckBox();
        colorFill = new windows.controls.AtlasColorChooserOpac();
        colorBorder = new windows.controls.AtlasColorChooserOpac();
        jPanel3 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jSlider2 = new javax.swing.JSlider();
        jPanel2 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();

        setEnabled(false);

        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/style/messages",LocaleUtil.getLocale()); // NOI18N
        pnlLine.setBorder(javax.swing.BorderFactory.createTitledBorder(bundle.getString("icon"))); // NOI18N

        jPanel1.setLayout(new java.awt.GridLayout(4, 1, 5, 5));

        jCheckBox1.setText("Customize"); // NOI18N
        jCheckBox1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jCheckBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBox1ActionPerformed(evt);
            }
        });
        jPanel1.add(jCheckBox1);
        jPanel1.add(colorFill);
        jPanel1.add(colorBorder);

        jLabel2.setText("Size"); // NOI18N

        jSlider2.setMajorTickSpacing(10);
        jSlider2.setMaximum(30);
        jSlider2.setMinorTickSpacing(5);
        jSlider2.setValue(0);

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 52, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSlider2, javax.swing.GroupLayout.DEFAULT_SIZE, 205, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jLabel2, javax.swing.GroupLayout.DEFAULT_SIZE, 34, Short.MAX_VALUE)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(jSlider2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );

        jPanel1.add(jPanel3);

        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jLabel1.setMaximumSize(new java.awt.Dimension(128, 128));
        jLabel1.setMinimumSize(new java.awt.Dimension(128, 128));
        jLabel1.setPreferredSize(new java.awt.Dimension(128, 128));

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 128, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 128, javax.swing.GroupLayout.PREFERRED_SIZE)
        );

        javax.swing.GroupLayout pnlLineLayout = new javax.swing.GroupLayout(pnlLine);
        pnlLine.setLayout(pnlLineLayout);
        pnlLineLayout.setHorizontalGroup(
            pnlLineLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlLineLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(pnlLineLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(iconsPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 405, Short.MAX_VALUE)
                    .addGroup(pnlLineLayout.createSequentialGroup()
                        .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 271, Short.MAX_VALUE)))
                .addContainerGap())
        );
        pnlLineLayout.setVerticalGroup(
            pnlLineLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlLineLayout.createSequentialGroup()
                .addGroup(pnlLineLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jPanel1, 0, 0, Short.MAX_VALUE)
                    .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(18, 18, 18)
                .addComponent(iconsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 266, Short.MAX_VALUE)
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(pnlLine, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(pnlLine, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    private void jCheckBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBox1ActionPerformed
        if (jCheckBox1.isSelected()) {
            if (affectStyle) {
                int i = (int) (Math.random() * 1024);
                AtlasColor at;

                at = new AtlasColor();
                at.setColor(colors.getColor(i));
                style.setFillcolo(at);

                at = new AtlasColor();
                at.setColor(ColorsList.getDarkenColor(colors.getDarkenColor(i)));
                style.setLinecolo(at);
                colorBorder.setColor(style.getLinecolo());
                colorFill.setColor(style.getFillcolo());
                fireChangeListeners();
            }
            colorBorder.setEnabled(true);
            colorFill.setEnabled(true);
        } else {
            colorBorder.setEnabled(false);
            colorFill.setEnabled(false);
            if (affectStyle) {
                style.setLinecolo(null);
                style.setFillcolo(null);
                fireChangeListeners();
            }
        }
        putPicture();
    }//GEN-LAST:event_jCheckBox1ActionPerformed

    public void addStyleChangeListener(ChangeListener cl) {
        changeListeners.add(cl);
    }

    public void removeStyleChangeListener(ChangeListener cl) {
        changeListeners.remove(cl);
    }

    private void fireChangeListeners() {
        Iterator<ChangeListener> it = changeListeners.iterator();
        ChangeEvent ev = new ChangeEvent(this);
        while (it.hasNext()) {
            it.next().stateChanged(ev);
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private windows.controls.AtlasColorChooserOpac atlasColorChooserOpac2;
    private javax.swing.ButtonGroup buttonGroup1;
    private windows.controls.AtlasColorChooserOpac colorBorder;
    private windows.controls.AtlasColorChooserOpac colorFill;
    private windows.icons.IconsPanel iconsPanel;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JSlider jSlider2;
    private javax.swing.JPanel pnlLine;
    // End of variables declaration//GEN-END:variables
    public windows.icons.IconsPanel getIconsPanel() {
        return iconsPanel;
    }
}
