package windows.symbology;

import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.Point;
import org.atlas.hibernatedata.AtlasColor;
import org.atlas.hibernatedata.Icon;
import org.atlas.hibernatedata.Rule;
import org.atlas.hibernatedata.Style;
import org.atlas.hibernatedata.Symbology;
import org.atlas.i18n.LocaleUtil;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import org.atlas.util.ColorsList;
import org.atlas.util.HibernateUtil;
import windows.style.StyleEditor;

public class ClassRuleGenerator extends javax.swing.JPanel implements RuleGenerator {

    private List ruleSelectionChangeListener = new ArrayList();
    private Rule selectedRule;
    private Rule[] rules;
    private Symbology symbology;
    private HibernateUtil hibernate;
    private ColorsList colors = new ColorsList();
    private StyleEditor editor;
    private ClassRuleTableModel crtm;
    private Icon defaultIcon;

    public ClassRuleGenerator() {
        initComponents();

        tableRules.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tableRules.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent le) {
                if (!le.getValueIsAdjusting()) {
                    if (tableRules.getSelectedRow() >= 0) {
                        Rule r = (Rule) crtm.getRules()[tableRules.getSelectedRow()];
                        editor.setStyle(r.getStyle());
                    } else {
                        editor.setStyle(null);
                    }
                }
            }
        });

    }

    public void addRuleSelectionChangeListener(ActionListener al) {
        ruleSelectionChangeListener.add(al);
    }

    public void removeRuleSelectionChangeListener(ActionListener al) {
        ruleSelectionChangeListener.remove(al);
    }

//    private void fireRuleSelectionEvent(){
//        ActionEvent ae = new ActionEvent(this, -1, "");
//        Iterator it = ruleSelectionChangeListener.iterator();
//        while(it.hasNext()){
//            ((ActionListener)it.next()).actionPerformed(ae);
//        }
//    }
    public Rule getSelectedRule() {
        return selectedRule;
    }

    public void setSelectedRule(Rule selectedRule) {
        this.selectedRule = selectedRule;
    }

    public Rule[] getRules() {
        return rules;
    }

    public void setRules(Rule[] rules) {
        this.rules = rules;
    }

    private void loadInterface() {
        editor.setStyle(null);
        List<String> names = symbology.getLayer().getSource().getFieldNames();
        cmbFields.removeAllItems();
        cmbFields.addItem("[SELECT A FIELD]");
        Iterator<String> it = names.iterator();

        cmbFields.setSelectedIndex(0);

        crtm = new ClassRuleTableModel();
        if (getSymbology().getTypeSymb() == Symbology.TYPE_CLASS_SYMB) {
            for (int i = 1; it.hasNext(); i++) {
                String name = it.next();
                cmbFields.addItem(name);
                if (name.equals(symbology.getRuleFieldName())) {
                    cmbFields.setSelectedIndex(i);
                }
            }

            Object[] obs = getSymbology().getRules().toArray();
            Arrays.sort(obs);
            crtm.setRules(obs);
        } else {
            for (int i = 1; it.hasNext(); i++) {
                cmbFields.addItem(it.next());
            }
        }
        tableRules.setDefaultRenderer(Style.class, new StyleCellRenderer());
        tableRules.setModel(crtm);

    }

    public Symbology getSymbology() {
        return symbology;
    }

    public void setSymbology(Symbology symbology) {
        this.symbology = symbology;
        loadInterface();
    }

    public void setDefaultIcon(Icon defaultIcon) {
        this.defaultIcon = defaultIcon;
    }

    private class ClassRuleTableModel extends AbstractTableModel {

        private Object[] rules;
        private String[] names = {"Color", "Descr."};

        @Override
        public Class getColumnClass(int columnIndex) {
            if (columnIndex == 0) {
                return Style.class;
            }
            if (columnIndex == 1) {
                return String.class;
            }
            return null;
        }

        @Override
        public String getColumnName(int column) {
            return names[column];
        }

        public Object getValueAt(int fil, int col) {
            if (col == 0) {
                return ((Rule) getRules()[fil]).getStyle();
            } else if (col == 1) {
                return ((Rule) getRules()[fil]).getDescrule();
            }
            return null;
        }

        public int getRowCount() {
            if (rules != null) {
                return getRules().length;
            }
            return 0;
        }

        public int getColumnCount() {
            return 2;
        }

        public Object[] getRules() {
            return rules;
        }

        public void setRules(Object[] rules) {
            this.rules = rules;
        }
    }

    public HibernateUtil getHibernate() {
        return hibernate;
    }

    public void setHibernate(HibernateUtil hibernate) {
        this.hibernate = hibernate;
    }

    public StyleEditor getEditor() {
        return editor;
    }

    public void setEditor(StyleEditor editor) {
        this.editor = editor;
        editor.addStyleChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent changeEvent) {
                tableRules.repaint();
            }
        });
    }

    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jScrollPane1 = new javax.swing.JScrollPane();
        tableRules = new javax.swing.JTable();
        lblField = new javax.swing.JLabel();
        cmbFields = new javax.swing.JComboBox();
        btnGenerate = new javax.swing.JButton();

        tableRules.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(tableRules);

        lblField.setText(java.util.ResourceBundle.getBundle("windows/symbology/messages",LocaleUtil.getLocale()).getString("fields"));

        cmbFields.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        btnGenerate.setText("Generate");
        btnGenerate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnGenerateActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 415, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(cmbFields, 0, 292, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnGenerate, javax.swing.GroupLayout.PREFERRED_SIZE, 117, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(lblField, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 415, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(lblField)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnGenerate)
                    .addComponent(cmbFields, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 224, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents
    private void btnGenerateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnGenerateActionPerformed
        if (cmbFields.getSelectedIndex() > 0) {
            String tabName = symbology.getLayer().getSource().getTablsour();
            String fldName = cmbFields.getSelectedItem().toString();
            String query = "SELECT DISTINCT " + fldName + " FROM \"" + tabName + "\" ORDER BY " + fldName + " ASC";

            Connection con = null;
            Statement st = null;
            ResultSet rs = null;
            
            try {
                con = getHibernate().getConnection();
                st = con.createStatement();
                rs = st.executeQuery(query);
                symbology.getRules().clear();

                for (int i = 0; rs.next(); i++) {
                    Rule r = new Rule();
                    r.setDescrule(rs.getString(1));
                    r.setValue(rs.getString(1));
                    r.setStyle(Style.getDefaultRandomStyle(defaultIcon));

                    AtlasColor at;
                    
                    at = new AtlasColor();
                    at.setColor(colors.getColor(i));
                    r.getStyle().setFillcolo(at);

                    at = new AtlasColor();
                    
                    Class geom = symbology.getLayer().getSource().getGeomClass();
                    if (geom.equals(Point.class) || geom.equals(MultiPoint.class)) {
                        r.getStyle().getLinestyl().setWidth(Icon.DEFAULT_SIZE);
                        at.setColor(ColorsList.getDarkenColor(colors.getDarkenColor(i)));
                    }else{
                        at.setColor(colors.getDarkenColor(i));    
                    }
                    
                    r.getStyle().setLinecolo(at);
                    symbology.getRules().add(r);
                }

                symbology.setTypeSymb(Symbology.TYPE_CLASS_SYMB);
                symbology.setRuleFieldName(fldName);
                loadInterface();
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                try {
                    rs.close();
                    st.close();
                    con.close();
                } catch (SQLException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }//GEN-LAST:event_btnGenerateActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnGenerate;
    private javax.swing.JComboBox cmbFields;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel lblField;
    private javax.swing.JTable tableRules;
    // End of variables declaration//GEN-END:variables
}

