package windows.symbology;

import org.atlas.hibernatedata.Icon;
import org.atlas.hibernatedata.Layer;
import org.atlas.hibernatedata.Rule;
import org.atlas.hibernatedata.Service;
import org.atlas.hibernatedata.Style;
import org.atlas.hibernatedata.Symbology;
import org.atlas.i18n.LocaleUtil;
import java.io.ByteArrayOutputStream;
import java.sql.Connection;
import java.sql.SQLException;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.hibernate.classic.Session;
import org.atlas.util.HibernateUtil;
import org.atlas.util.WindowUtils;

public class FrmSymbPoint extends javax.swing.JDialog {

    private Layer layer;
    private HibernateUtil hibernate;
    private Symbology clone;

    public FrmSymbPoint(Layer layerp, HibernateUtil hibernate) throws Exception {
        super((JFrame) null, true);
        initComponents();

        fixedRuleGenerator1.setEditor(pointEditor1);
        rangeRuleGenerator1.setEditor(pointEditor2);
        classRuleGenerator1.setEditor(pointEditor3);

        WindowUtils.setFrameIcon(this, "/media/appicon.png");

        layer = layerp;
        this.hibernate = hibernate;

        classRuleGenerator1.setHibernate(hibernate);
        rangeRuleGenerator1.setHibernate(hibernate);
        fixedRuleGenerator1.setHibernate(hibernate);

        Session sess = null;
        Connection con = null;
        try {
            con = hibernate.getConnection();
            layer.getSource().loadFieldMetadata(con);
        } catch (Exception ex) {
            ex.printStackTrace();
            throw ex;
        } finally {
            try {
                con.close();
            } catch (SQLException ex) {
                ex.printStackTrace();
                throw ex;
            }
        }

        try {
            sess = hibernate.begin();            
            Service service = org.atlas.hibernatedata.Service.getService(hibernate);
            sess.update(service);
            Object[] obj = service.getIcons().toArray();
            for (int i = 0; i < obj.length; i++) {
                Icon ic = (Icon)obj[i];
                pointEditor1.getIconsPanel().addIcon(ic);
                pointEditor2.getIconsPanel().addIcon(ic);
                pointEditor3.getIconsPanel().addIcon(ic);
            }
            
            Icon defaultIcon = (Icon) sess.createQuery("FROM Icon WHERE descicon = 'default.svg'").list().get(0);
            classRuleGenerator1.setDefaultIcon(defaultIcon);
            rangeRuleGenerator1.setDefaultIcon(defaultIcon);
            fixedRuleGenerator1.setDefaultIcon(defaultIcon);

            sess.update(layer);
            Symbology sym = layer.getSymbology();
            sess.update(sym);
            Object[] rules = sym.getRules().toArray();
            for (int i = 0; i < rules.length; i++) {
                sess.update((Rule) rules[i]);
                Style st = ((Rule) rules[i]).getStyle();
                sess.update(st);
                Icon ic = st.getIcon();
                System.out.println(ic.getBinaicon().length);
            }

            clone = (Symbology) layer.getSymbology().clone();

            tabPanel.addChangeListener(new ChangeListener() {

                public void stateChanged(ChangeEvent changeEvent) {
                    int index = tabPanel.getSelectedIndex();
                    if (index == 0) {
                        fixedRuleGenerator1.setSymbology(clone);
                    } else if (index == 1) {
                        rangeRuleGenerator1.setSymbology(clone);
                    } else if (index == 2) {
                        classRuleGenerator1.setSymbology(clone);
                    }
                }
            });

            if (sym.getTypeSymb() == Symbology.TYPE_FIXED_SYMB) {
                tabPanel.setSelectedIndex(0);
                fixedRuleGenerator1.setSymbology(clone);
            } else if (sym.getTypeSymb() == Symbology.TYPE_CLASS_SYMB) {
                classRuleGenerator1.setSymbology(clone);
                tabPanel.setSelectedIndex(2);
            } else if (sym.getTypeSymb() == Symbology.TYPE_RANGE_SYMB) {
                rangeRuleGenerator1.setSymbology(clone);
                tabPanel.setSelectedIndex(1);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            throw ex;
        } finally {
            hibernate.close(sess);
        }
    }

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        tabPanel = new javax.swing.JTabbedPane();
        jPanel3 = new javax.swing.JPanel();
        fixedRuleGenerator1 = new windows.symbology.FixedRuleGenerator();
        pointEditor1 = new windows.style.PnlEditPointStyle();
        jPanel2 = new javax.swing.JPanel();
        rangeRuleGenerator1 = new windows.symbology.RangeRuleGenerator();
        pointEditor2 = new windows.style.PnlEditPointStyle();
        jPanel1 = new javax.swing.JPanel();
        classRuleGenerator1 = new windows.symbology.ClassRuleGenerator();
        pointEditor3 = new windows.style.PnlEditPointStyle();
        jPanel4 = new javax.swing.JPanel();
        btnCancel = new javax.swing.JButton();
        btnOk = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("windows/symbology/messages",LocaleUtil.getLocale()); // NOI18N
        setTitle(bundle.getString("frm_poly")); // NOI18N
        setLocationByPlatform(true);
        setModal(true);

        jPanel3.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 50, 1, 50));

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(fixedRuleGenerator1, javax.swing.GroupLayout.PREFERRED_SIZE, 291, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(326, Short.MAX_VALUE))
            .addComponent(pointEditor1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 627, Short.MAX_VALUE)
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(fixedRuleGenerator1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pointEditor1, javax.swing.GroupLayout.DEFAULT_SIZE, 376, Short.MAX_VALUE)
                .addContainerGap())
        );

        tabPanel.addTab("Fixed", jPanel3);

        rangeRuleGenerator1.setPreferredSize(new java.awt.Dimension(339, 411));

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addComponent(rangeRuleGenerator1, javax.swing.GroupLayout.PREFERRED_SIZE, 330, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(pointEditor2, javax.swing.GroupLayout.DEFAULT_SIZE, 377, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(rangeRuleGenerator1, javax.swing.GroupLayout.DEFAULT_SIZE, 440, Short.MAX_VALUE)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGap(11, 11, 11)
                .addComponent(pointEditor2, javax.swing.GroupLayout.DEFAULT_SIZE, 429, Short.MAX_VALUE))
        );

        tabPanel.addTab("Range", jPanel2);

        classRuleGenerator1.setPreferredSize(new java.awt.Dimension(339, 300));

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addComponent(classRuleGenerator1, javax.swing.GroupLayout.DEFAULT_SIZE, 329, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(pointEditor3, javax.swing.GroupLayout.PREFERRED_SIZE, 378, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(12, 12, 12)
                .addComponent(pointEditor3, javax.swing.GroupLayout.DEFAULT_SIZE, 428, Short.MAX_VALUE))
            .addComponent(classRuleGenerator1, javax.swing.GroupLayout.DEFAULT_SIZE, 440, Short.MAX_VALUE)
        );

        tabPanel.addTab("Class", jPanel1);

        jPanel4.setLayout(new java.awt.GridLayout(1, 2, 5, 0));

        btnCancel.setText(bundle.getString("cancel")); // NOI18N
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });
        jPanel4.add(btnCancel);

        btnOk.setText(bundle.getString("Save")); // NOI18N
        btnOk.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOkActionPerformed(evt);
            }
        });
        jPanel4.add(btnOk);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, 313, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(tabPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 732, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(tabPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 468, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private void btnOkActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOkActionPerformed
        Session sess = null;
        try {
            sess = hibernate.begin();
            sess.update(layer);
            layer.getSymbology().delete(sess);
            hibernate.commit(sess);

            hibernate.begin(sess);
            
            Rule[] rules = new Rule[clone.getRules().size()];
            clone.getRules().toArray(rules);
            
            for (int i = 0; i < rules.length; i++) {
                Rule rule = rules[i];
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                ImageIO.write(rule.getStyle().getIconImage(), "png", baos);
                rule.setBinarule(baos.toByteArray());
            }

            
            clone.save(sess);
            layer.setSymbology(clone);
            hibernate.commit(sess);
            dispose();
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            try {
                hibernate.close(sess);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }//GEN-LAST:event_btnOkActionPerformed

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        dispose();
    }//GEN-LAST:event_btnCancelActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnOk;
    private windows.symbology.ClassRuleGenerator classRuleGenerator1;
    private windows.symbology.FixedRuleGenerator fixedRuleGenerator1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private windows.style.PnlEditPointStyle pointEditor1;
    private windows.style.PnlEditPointStyle pointEditor2;
    private windows.style.PnlEditPointStyle pointEditor3;
    private windows.symbology.RangeRuleGenerator rangeRuleGenerator1;
    private javax.swing.JTabbedPane tabPanel;
    // End of variables declaration//GEN-END:variables
}
