package windows.symbology;

import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.Point;
import org.atlas.hibernatedata.AtlasColor;
import org.atlas.hibernatedata.Icon;
import org.atlas.hibernatedata.Rule;
import org.atlas.hibernatedata.Style;
import org.atlas.hibernatedata.Symbology;
import org.atlas.i18n.LocaleUtil;
import java.awt.Color;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import org.atlas.util.ColorsList;
import org.atlas.util.DataBaseUtils;
import org.atlas.util.HibernateUtil;
import windows.style.StyleEditor;

public class RangeRuleGenerator extends javax.swing.JPanel implements RuleGenerator {

    private Symbology symbology;
    private HibernateUtil hibernate;
    private int sugNClasses = -1;
    private int max = -1;
    private int min = -1;
    private int nDatos = -1;
    private DecimalFormat df = new DecimalFormat("#.##");
    private StyleEditor editor;
    private ColorsList colors = new ColorsList();
    private RangeRuleTableModel crtm;
    private Icon defaultIcon;

    public RangeRuleGenerator() {
        initComponents();
        color1.setLabel("Color 1");
        color2.setLabel("Color 2");
        nClasses.setModel(new SpinnerNumberModel(5, 2, 64, 1));

        tableRules.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tableRules.getSelectionModel().addListSelectionListener(new ListSelectionListener() {

            public void valueChanged(ListSelectionEvent le) {
                if (!le.getValueIsAdjusting()) {
                    if (tableRules.getSelectedRow() >= 0) {
                        Rule r = (Rule) crtm.getRules()[tableRules.getSelectedRow()];
                        editor.setStyle(r.getStyle());
                    } else {
                        editor.setStyle(null);
                    }
                }
            }
        });

    }

    private void loadInterface() {
        List<String> names  = symbology.getLayer().getSource().getFieldNames();
        List<Integer> types = symbology.getLayer().getSource().getFieldTypes();
        cmbFields.removeAllItems();
        cmbFields.addItem("[SELECT A FIELD]");
        Iterator<String> it = names.iterator();

        cmbFields.setSelectedIndex(0);

        crtm = new RangeRuleTableModel();
        if (getSymbology().getTypeSymb() == Symbology.TYPE_RANGE_SYMB) {
            for (int i = 1; it.hasNext(); i++) {
                int type = types.get(i - 1).intValue();
                String name = it.next();
                if (DataBaseUtils.isNumeric(type)) {
                    cmbFields.addItem(name);
                    if (name.equals(symbology.getRuleFieldName())) {
                        cmbFields.setSelectedIndex(cmbFields.getItemCount()-1);
                    }
                }
            }

            Object[] obs = getSymbology().getRules().toArray();
            Arrays.sort(obs);
            crtm.setRules(obs);

            Rule r1 = (Rule) crtm.getRules()[0];
            Rule r2 = (Rule) crtm.getRules()[crtm.getRules().length - 1];

            color1.setColor(r1.getStyle().getFillcolo().getColor());
            color2.setColor(r2.getStyle().getFillcolo().getColor());

            nClasses.setValue(crtm.getRules().length);

        } else {

            color1.setColor(colors.getColor((int) (Math.random() * 1024)));
            color2.setColor(colors.getColor((int) (Math.random() * 1024)));

            for (int i = 0; it.hasNext(); i++) {
                int type = types.get(i).intValue();
                String name = it.next();
                if (DataBaseUtils.isNumeric(type)) {
                    cmbFields.addItem(name);
                }
            }
        }

        tableRules.setDefaultRenderer(Style.class, new StyleCellRenderer());
        tableRules.setModel(crtm);
    }

    public Symbology getSymbology() {
        return symbology;
    }

    public void setSymbology(Symbology symbology) {
        if (nDatos == -1) {
            String tabName = symbology.getLayer().getSource().getTablsour();
            String query = "SELECT count(*) as c FROM \"" + tabName + "\"";

            Connection con = null;
            Statement st = null;
            ResultSet rs = null;
            try {
                con = getHibernate().getConnection();
                st = con.createStatement();
                rs = st.executeQuery(query);
                rs.next();
                nDatos = rs.getInt("c");
                sugNClasses = (int) Math.round(1 + (3.3 * Math.log(nDatos)));
                nClasses.setValue(sugNClasses);
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                try {
                    rs.close();
                    st.close();
                    con.close();
                } catch (SQLException ex) {
                    ex.printStackTrace();
                }
            }
        }
        this.symbology = symbology;
        loadInterface();
    }

    public Icon getDefaultIcon() {
        return defaultIcon;
    }

    public void setDefaultIcon(Icon defaultIcon) {
        this.defaultIcon = defaultIcon;
    }

    private class RangeRuleTableModel extends AbstractTableModel {

        private Object[] rules;
        private String[] names = {"Color", "Max.", "Min."};

        @Override
        public Class getColumnClass(int columnIndex) {
            if (columnIndex == 0) {
                return Style.class;
            }
            if (columnIndex == 1) {
                return String.class;
            }
            if (columnIndex == 2) {
                return String.class;
            }
            return null;
        }

        @Override
        public String getColumnName(int column) {
            return names[column];
        }

        public Object getValueAt(int i, int col) {
            if (col == 0) {
                return ((Rule) getRules()[i]).getStyle();
            } else if (col == 1) {
                return df.format(((Rule) getRules()[i]).getMin());
            } else if (col == 2) {
                return df.format(((Rule) getRules()[i]).getMax());
            }
            return null;
        }

        public int getRowCount() {
            if (rules != null) {
                return getRules().length;
            }
            return 0;
        }

        public int getColumnCount() {
            return 3;
        }

        public Object[] getRules() {
            return rules;
        }

        public void setRules(Object[] rules) {
            this.rules = rules;
            Arrays.sort(rules, new Comparator() {

                public int compare(Object o1, Object o2) {
                    return Double.compare(((Rule) o1).getMin(), ((Rule) o2).getMin());
                }
            });

        }
    }

    public HibernateUtil getHibernate() {
        return hibernate;
    }

    public void setHibernate(HibernateUtil hibernate) {
        this.hibernate = hibernate;
    }

    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jScrollPane1 = new javax.swing.JScrollPane();
        tableRules = new javax.swing.JTable();
        lblField = new javax.swing.JLabel();
        cmbFields = new javax.swing.JComboBox();
        btnGenerate = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        color1 = new windows.controls.AtlasColorChooserOpac();
        color2 = new windows.controls.AtlasColorChooserOpac();
        nClasses = new javax.swing.JSpinner();
        jLabel1 = new javax.swing.JLabel();

        tableRules.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(tableRules);

        lblField.setText(java.util.ResourceBundle.getBundle("windows/symbology/messages",LocaleUtil.getLocale()).getString("fields"));

        cmbFields.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        btnGenerate.setText("Generate");
        btnGenerate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnGenerateActionPerformed(evt);
            }
        });

        jPanel1.setLayout(new java.awt.GridLayout(1, 0, 5, 0));

        jPanel1.add(color1);

        jPanel1.add(color2);

        jLabel1.setText(java.util.ResourceBundle.getBundle("windows/symbology/messages",LocaleUtil.getLocale()).getString("n_classes"));

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, 0, 0, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel1, javax.swing.GroupLayout.DEFAULT_SIZE, 135, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(nClasses, javax.swing.GroupLayout.PREFERRED_SIZE, 165, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(lblField, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 312, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.LEADING, 0, 0, Short.MAX_VALUE)
                        .addComponent(cmbFields, javax.swing.GroupLayout.Alignment.LEADING, 0, 312, Short.MAX_VALUE))
                    .addComponent(btnGenerate, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 312, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(lblField)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(cmbFields, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(nClasses, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnGenerate)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 248, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents
    private void btnGenerateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnGenerateActionPerformed
        if (cmbFields.getSelectedIndex() > 0) {
            String tabName = symbology.getLayer().getSource().getTablsour();
            String fldName = cmbFields.getSelectedItem().toString();
            String query = "SELECT min(" + fldName + ") as min , max(" + fldName + ") as max FROM  \"" + tabName + "\"";

            System.out.println(query);

            Connection con = null;
            Statement st = null;
            ResultSet rs = null;
            try {
                con = getHibernate().getConnection();
                st = con.createStatement();
                rs = st.executeQuery(query);
                symbology.getRules().clear();

                rs.next();
                double imin = rs.getInt("min");
                double imax = rs.getInt("max");
                int ngrp = Integer.parseInt(nClasses.getValue().toString());
                double ran = (imax - imin) / ngrp;

                System.out.println(imin + " " + imax);

                double ranr = (color2.getColor().getRed() - color1.getColor().getRed()) / (ngrp - 1);
                double rang = (color2.getColor().getGreen() - color1.getColor().getGreen()) / (ngrp - 1);
                double ranb = (color2.getColor().getBlue() - color1.getColor().getBlue()) / (ngrp - 1);

                double r = color1.getColor().getRed();
                double g = color1.getColor().getGreen();
                double b = color1.getColor().getBlue();

                double acMin = imin;
                for (int i = 0; i < ngrp; i++) {
                    Rule rule = new Rule();
                    rule.setDescrule(df.format(acMin) + " - " + df.format(acMin + ran));
                    rule.setMin(acMin);
                    rule.setMax(acMin + ran);
                    rule.setStyle(Style.getDefaultRandomStyle(defaultIcon));

                    AtlasColor at;
                    at = new AtlasColor();
                    Color c = new Color((int) r, (int) g, (int) b);
                    at.setColor(c);
                    rule.getStyle().setFillcolo(at);

                    at = new AtlasColor();

                    Class geom = symbology.getLayer().getSource().getGeomClass();
                    if (geom.equals(Point.class) || geom.equals(MultiPoint.class)) {
                        rule.getStyle().getLinestyl().setWidth(Icon.DEFAULT_SIZE);
                        at.setColor(ColorsList.getDarkenColor(ColorsList.getDarkenColor(c)));
                    } else {
                        at.setColor(ColorsList.getDarkenColor(c));
                    }

                    rule.getStyle().setLinecolo(at);

                    symbology.getRules().add(rule);
                    acMin = acMin + ran;
                    r += ranr;
                    g += rang;
                    b += ranb;
                }

                symbology.setTypeSymb(Symbology.TYPE_RANGE_SYMB);
                symbology.setRuleFieldName(fldName);
                loadInterface();
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                try {
                    rs.close();
                    st.close();
                    con.close();
                } catch (SQLException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }//GEN-LAST:event_btnGenerateActionPerformed

    public StyleEditor getEditor() {
        return editor;
    }

    public void setEditor(StyleEditor editor) {
        this.editor = editor;
        editor.addStyleChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent changeEvent) {
                tableRules.repaint();
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnGenerate;
    private javax.swing.JComboBox cmbFields;
    private windows.controls.AtlasColorChooserOpac color1;
    private windows.controls.AtlasColorChooserOpac color2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel lblField;
    private javax.swing.JSpinner nClasses;
    private javax.swing.JTable tableRules;
    // End of variables declaration//GEN-END:variables
}
