/* 
 * Created on 15 de noviembre de 2007, 11:31
 */
package windows.wartool;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;
import javax.swing.JDialog;
import javax.swing.JFrame;
import org.atlas.util.Cryptography;
import org.atlas.util.FrmWait;
import org.atlas.util.IOUtils;
import org.atlas.util.PathGeocoder;
import org.atlas.util.WindowUtils;

public class WebArchive {

    private String deploypath;
    private String wmsURL;
    private String geocoderURL;
    private String appUser;
    private char[] appPass;
    private int sizecache;
    private String host;
    private String database;
    private String driver;
    private String port;
    private boolean includeDriver;
    private String databaseUser;
    private char[] databasepass;
    private String wmsServletName;
    private String geocoderServletName;

    public WebArchive() {
        setDeploypath("");
        setURL_WMS("");
        setURL_GEOCODER("");
        setAppUser("");
        setAppPass(null);

        setHost("");
        setDatabase("");
        setDriver("");
        setPort("");
        setIncludeDriver(false);
        setDatabaseUser("");
        setDatabasepass(null);

        setWMSServletName("");
        setGeocoderServletName("");
    }

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public String getWMSServletName() {
        return wmsServletName;
    }

    public void setWMSServletName(String servletName) {
        this.wmsServletName = servletName;
    }

    public String getURL_WMS() {
        return wmsURL;
    }

    public void setURL_WMS(String wmsURL) {
        this.wmsURL = wmsURL;
    }

    public String getURL_GEOCODER() {
        return geocoderURL;
    }

    public void setURL_GEOCODER(String wmsGEOCODER) {
        this.geocoderURL = wmsGEOCODER;
    }

    public String getAppUser() {
        return appUser;
    }

    public void setAppUser(String appUser) {
        this.appUser = appUser;
    }

    public String getDatabase() {
        return database;
    }

    public void setDatabase(String database) {
        this.database = database;
    }

    public String getDriver() {
        return driver;
    }

    public void setDriver(String driver) {
        this.driver = driver;
    }

    public String getPort() {
        return port;
    }

    public void setPort(String port) {
        this.port = port;
    }

    public String getDatabaseUser() {
        return databaseUser;
    }

    public void setDatabaseUser(String databaseUser) {
        this.databaseUser = databaseUser;
    }

    public char[] getServletPass() {
        return appPass;
    }

    public void setAppPass(char[] appPass) {
        this.appPass = appPass;
    }

    public char[] getDatabasepass() {
        return databasepass;
    }

    public void setDatabasepass(char[] databasepass) {
        this.databasepass = databasepass;
    }

    public int getSizecache() {
        return sizecache;
    }

    public void setSizecache(int sizecache) {
        this.sizecache = sizecache;
    }

    public String getGeocoderServletName() {
        return geocoderServletName;
    }

    public void setGeocoderServletName(String geocoderServletName) {
        this.geocoderServletName = geocoderServletName;
    }

    public boolean isIncludeDriver() {
        return includeDriver;
    }

    public void setIncludeDriver(boolean includeDriver) {
        this.includeDriver = includeDriver;
    }

    public String getDeploypath() {
        return deploypath;
    }

    public void setDeploypath(String deploypath) {
        this.deploypath = deploypath;
    }

    public void generateWar(String filepath, String warpath, Vector plugins) throws IOException, FileNotFoundException, Exception {

        Enumeration entries;
        ZipFile zipFile = new ZipFile(filepath + "server.war");
        ZipEntry[] entrys = new ZipEntry[zipFile.size() - 2];
        byte[][] bin = new byte[zipFile.size() - 2][];

        entries = zipFile.entries();
        for (int e = 0; entries.hasMoreElements();) {
            ZipEntry entry = (ZipEntry) entries.nextElement();
            if (!entry.getName().equals("WEB-INF/web.xml") && !entry.getName().equals("META-INF/context.xml")) {
                entrys[e] = entry;
                bin[e] = IOUtils.readFromStream(zipFile.getInputStream(entry));
                e++;
            }

        }
        zipFile.close();

        ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(warpath));
        for (int e = 0; e < entrys.length; e++) {
            zos.putNextEntry(entrys[e]);
            zos.write(bin[e]);
        }

        addWebXML("WEB-INF/web.xml", filepath + "web.xml", zos);
        addContext("META-INF/context.xml", filepath + "context.xml", zos);

        if (isIncludeDriver()) {
            String name = "";
            if (getDriver().equals("org.postgresql.Driver")) {
                name = "postgresql-8.2-504.jdbc3.jar";
            } else {
                name = "mysql-connector-java-5.1.0-bin.jar";
            }
            addEntry("WEB-INF/lib/" + name, filepath + name, zos);
        }

        for (int g = 0; g < plugins.size(); g++) {
            PathGeocoder geo = (PathGeocoder) plugins.get(g);
            addEntry("WEB-INF/lib/" + geo.getName(), geo.getPath(), zos);
        }
        zos.close();
    }

    public void addContext(String entryName, String pathtemplate, ZipOutputStream zos) throws FileNotFoundException, IOException, Exception {
        HashMap<String, String> hash = new HashMap();
        hash.put("@path", getDeploypath());
        addEntry(entryName, pathtemplate, zos, hash);
    }

    public void addWebXML(String entryName, String pathtemplate, ZipOutputStream zos) throws FileNotFoundException, IOException, Exception {
        HashMap<String, String> hash = new HashMap();
        hash.put("@wms_servlet_name", getWMSServletName());
        hash.put("@geocoder_servlet_name", getGeocoderServletName());
        hash.put("@driver", getDriver());
        hash.put("@host", getHost());
        hash.put("@port", getPort());
        hash.put("@database", getDatabase());
        hash.put("@user_db", getDatabaseUser());
        hash.put("@pass_db", Cryptography.toHexadecimal(Cryptography.encrypthAES(new String(getDatabasepass()))));
        hash.put("@user_ser", getAppUser());
        hash.put("@pass_ser", Cryptography.toHexadecimal(Cryptography.getMD5(new String(getServletPass()).getBytes())));
        hash.put("@wms_url", getURL_WMS());
        hash.put("@geocoder_url", getURL_GEOCODER());
        hash.put("@cache", String.valueOf(getSizecache()));
        addEntry(entryName, pathtemplate, zos, hash);
    }

    private static void addEntry(String entryName, String pathFile, ZipOutputStream zos) throws IOException {
        addEntry(entryName, pathFile, zos, null);
    }

    private static void addEntry(String entryName, String pathFile, ZipOutputStream zos, HashMap<String, String> replace) throws IOException {
        File file = new File(pathFile);
        zos.putNextEntry(new ZipEntry(entryName));
        if (replace != null) {
            String txt = new String(org.atlas.util.IOUtils.readFromStream(new FileInputStream(pathFile)));
            String[] keys = new String[replace.size()];
            replace.keySet().toArray(keys);
            for (int i = 0; i < keys.length; i++) {
                txt = txt.replace(keys[i], replace.get(keys[i]));
            }
            zos.write(txt.getBytes());
        } else {
            zos.write(IOUtils.readFromStream(new FileInputStream(file)));
        }
    }

    private static class Dlgs extends FrmWait {

        private WebArchive info = new WebArchive();
        private DialogWMS dialogS;
        private DialogDB dialogD;
        private DialogGeocoder dialogG;
        private JFrame frame;

        public Dlgs(JFrame frm) {
            this.frame = frm;
        }

        @Override
        public void action() {
            info = new WebArchive();
            dialogS = new DialogWMS(frame, true, info);
            dialogD = new DialogDB(frame, true, info);
            dialogG = new DialogGeocoder(frame, true, info);
            WindowUtils.center(dialogS);
            WindowUtils.center(dialogD);
            WindowUtils.center(dialogG);
            dialogS.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
            dialogD.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
            dialogG.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        }

        public WebArchive getInfo() {
            return info;
        }

        public DialogWMS getDialogS() {
            return dialogS;
        }

        public DialogDB getDialogD() {
            return dialogD;
        }

        public DialogGeocoder getDialogG() {
            return dialogG;
        }
    }

    public static void start(JFrame frame) {

        Dlgs dl = new Dlgs(frame);
        dl.setVisible(true);
        try {
            Token t = new Token();
            t.setState(Token.FRM_WMS);

            DialogWMS dialogS = dl.getDialogS();
            DialogDB dialogD = dl.getDialogD();
            DialogGeocoder dialogG = dl.getDialogG();

            dialogS.setToken(t);
            dialogD.setToken(t);
            dialogG.setToken(t);

            boolean cont = true;
            do {
                switch (t.getState()) {
                    case Token.FRM_WMS:
                        dialogS.setVisible(true);
                        break;
                    case Token.FRM_DB:
                        dialogD.setVisible(true);
                        break;
                    case Token.FRM_GEOC:
                        dialogG.setVisible(true);
                        break;
                    case Token.EXIT:
                        cont = false;
                        break;
                }
            } while (cont);

        } catch (Exception ex) {
            Logger.getLogger(WebArchive.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
