package org.atlas.desk.gui;

import org.atlas.clientdata.BBox;
import org.atlas.clientdata.EpsgCRS;
import org.atlas.clientdata.Layer;
import org.atlas.clientdata.Project;
import org.atlas.exceptions.HTTPRequestException;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.ConnectException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JPanel;
import org.atlas.desk.mashup.Marker;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.atlas.util.CoordinateUtils;
import org.atlas.util.IOUtils;

public class MapPanel extends JPanel {

    /**
     * Modo de operación en el que las acciones del usuario son ignradas.
     */
    public static final int NOMODESELECTED = -1;
    /**
     * Modo de operación en el el usuario puede acercarce al mapa haciendo clic en el. 
     */
    public static final int MODEZOOMPLUS = 1;
    /**
     * Modo de operación en el el usuario puede alejarse al mapa haciendo clic en el. 
     */
    public static final int MODEZOOMMINUS = 2;
    /**
     * Modo de operación en el que el usuario puede acercase al mapa arrastrando el mouse
     * para dibujar una caja en encierre el área que desea visualizar.  
     */
    public static final int MODEZOOMBOX = 3;
    /**
     * Modo de operación en el que el usuario puede desplazarse por el mapa 
     * arrastrando el mouse.
     */
    public static final int MODEPAN = 4;
    /**
     * Modo de operación donde al arrastrar el mouse de logra el mismo efecto que 
     * el MODEPAN, pero registra eventos al hacer clic en el mapa y al hacer clic
     * en un marcador.
     */
    public static final int MODEGETCOORD = 5;
    /**
     * Modo de operación en el que el usuario puede obtener información sobre el 
     * contenido el mapa arrastrando el mouse para dibujar un circulo que representa el área 
     * de interés o simplemente haciendo clic en el mapa.
     */
    public static final int MODEFEATUREINFO = 6;
    private boolean adminEntrace;
    private EventManager em = new EventManager(this);
    private Cache cache = new Cache();
    private URL appUrl;
    private URL geocoderUrl;
    private URL wmsUrl;
    private String urlpet;
    private int mode; //modo de operacion actual
    private Point p1; //primer pto para operaciones de dos puntos como panning o zoom punto punto
    private Point2D center;    //map coordinate, center of the viewport
    private double baseScale; //initial scale
    private final int bloqSize = 256;
    private BBox referenceBbox;
    private int lod;
    private double currentScale;
    private double zoomStep = 0.2;
    private int radius; //radio para la peticiÃ³n feature info
    private int zoomBoxWidth;
    private int zoomBoxHeigth;
    private Project[] projects;
    private Project currentProject;
    private Layer[] layers;
    private ArrayList<ActionListener> imageArrivalListeners = new ArrayList<ActionListener>();
    private String[] imageFormats;
    private String[] featureFormats;
    private String currentImageFormat;
    private String currentInfoFormat;
    private Color bgColor;
    //for the mouse wheel
    private Timer timer = new Timer();
    private TimerTask task;
    private int nTicks = 0;
    private boolean enabled = true;
    private boolean drag = false;
    private List markers = new ArrayList();
    private Marker overMarker = null;
    private Marker currentMarker = null;
    private ArrayList<Square> squares = new ArrayList();
    private Layer[] layersFeatureInfo;
    private BufferedImage north;
    private boolean mouseWheel = false;

    /**
     * Construye un Mappanel por defecto, debe llamarse a los métodos de configuración
     * antes de que el control sea útil.
     */
    public MapPanel() {
        try {
            initComponents();
            north = ImageIO.read(this.getClass().getResourceAsStream("/org/atlas/desk/gui/north.png"));
            squares.ensureCapacity(20);
            cache.addImageArrivalListener(new ActionListener() {

                public void actionPerformed(ActionEvent actionEvent) {
                    repaint();
                }
            });
        } catch (IOException ex) {
            Logger.getLogger(MapPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Indica si está o no habilitado el comportamiento que permite hacer zoom con 
     * la rueda el mouse.
     * @return
     */
    public boolean isMouseWheelEnabled() {
        return mouseWheel;
    }

    /**
     * Establece si está o no habilitado el comportamiento que permite hacer zoom con 
     * la rueda el mouse. 
     * @param enabled
     */
    public void setMouseWheelEnabled(boolean enabled) {
        mouseWheel = enabled;
    }

    /**
     * Obtiene el marcador actual.
     * @return
     */
    public Marker getCurrentMaker() {
        return currentMarker;
    }

    /**
     * Establece el marcador actual
     * @param marker
     */
    public void setCurrentMaker(Marker marker) {
        currentMarker = marker;
        em.fireMarkerClickedEvent(marker);
        repaint();

    }

    /**
     * Retorna la imagen con la leyenda de esta capa
     * @param layer 
     * @return
     * @throws java.lang.Exception 
     */
    public BufferedImage getLeyend(Layer layer) throws Exception {
        String txtUrl = wmsUrl + "?request=GetLegend&layers=" + layer.getName();
        if (isAdminEntrace()) {
            txtUrl += "&admin_entrace=true";
        }
        URL dir = new URL(txtUrl);

        HttpURLConnection con = (HttpURLConnection) dir.openConnection();
        if (con.getResponseCode() == HttpURLConnection.HTTP_OK) {
            if (con.getContentType().startsWith("image/")) {
                InputStream is = con.getInputStream();
                BufferedImage img = ImageIO.read(is);
                is.close();
                return img;
            }
        }
        throw new Exception("HTTP error");
    }

    /**
     * Retorna el listado de capas a usarse en peticiones GetFeatureInfo.
     * @return
     */
    public Layer[] getFeatureInfoLayers() {
        return layersFeatureInfo;
    }

    /**
     * Establece el listado de capas a usarse en peticiones GetFeatureInfo.
     * @param layersFeatureInfo
     */
    public void setFeatureInfoLayers(Layer[] layersFeatureInfo) {
        this.layersFeatureInfo = layersFeatureInfo;
    }

    /**
     * Agrega un marcador y actualiza la visualización.
     * @param marker
     */
    public void addMarker(Marker marker) {
        marker.findPosition();
        markers.add(marker);
        currentMarker = marker;
        em.fireMarkerClickedEvent(marker);
        repaint();
    }

    /**
     * Remueve un marcador y actualiza la visualización.
     * @param marker
     */
    public void removeMarker(Marker marker) {
        markers.remove(marker);
        repaint();
    }

    /**
     * Remueve todos los marcadores y actualiza la visualización.
     */
    public void removeAllMarkers() {
        markers.clear();
        repaint();
    }

    /**
     * Obtiene una lista de los marcadores del mapa, la modificación del contenido de
     * esta lista puede tener consecuencias inesperadas.
     * @return
     */
    public List<Marker> getMarkers() {
        return markers;
    }

    /**
     * Obtiene la escala de la visualización actual según el estándar WMS.
     * @return
     */
    public double getWMScale() {
        BBox vis = getVisibleBbox();
        EpsgCRS crs = getCurrentProject().getCrs();
        double k = 0.00028;
        if (crs.getType() == EpsgCRS.UNIT_LINEAR) {
            return ((vis.getWidth() * crs.getConversionFactor()) / getWidth()) / k;
        } else {
            double rad = vis.getWidth() * crs.getConversionFactor();
            double ls = rad * 6378137;
            return (ls / getWidth()) / k;
        }
    }

    private double getCurrentScale() {
        return currentScale;
    }

    private void changeZoom(MouseEvent ev, int deltaLOD, boolean reCenter) {
        if (reCenter) {
            int iCoord = ev.getX();
            int jCoord = ev.getY();
            center = CoordinateUtils.translateIJtoPoint(getVisibleBbox(), getWidth(), getHeight(), iCoord, jCoord, getCurrentProject().getCrs());
        }
        setZoomLevel(getZoomLevel() + deltaLOD);
        placeMarkers();
        em.fireMouseMoveEvent(ev);
    }

    /**
     * Obtiene el bounding box que representa la visualización actual.
     * @return
     */
    public BBox getVisibleBbox() {
        double ancho = this.getWidth() * this.getCurrentScale();
        double alto = this.getHeight() * this.getCurrentScale();
        return new BBox(center.getX() - (ancho / 2), center.getY() - (alto / 2), center.getX() + (ancho / 2), center.getY() + (alto / 2));
    }

    private void findUrlString() {

        if (getCurrentProject() != null && getLayers() != null && getCurrentImageFormat() != null) {
            Layer[] capas = getVisibleLayers();
            urlpet = getURL().toString() + "?version=1.3.0&request=GetMap&styles=&CRS=EPSG:" + getCurrentProject().getCrs().getCode() + "&width=" + this.bloqSize + "&height=" + this.bloqSize;
            urlpet += "&layers=";
            for (int i = 0; i < capas.length; i++) {
                Layer lyr = capas[i];
                urlpet += lyr.getName() + ",";
            }
        } else {
            throw new IllegalStateException("you must select a project, layers and image format first");
        }
    }

    private void findSquares() {
        if (!isEnabled()) {
            return;
        }

        int width = getWidth();
        int height = getHeight();

        double minX = referenceBbox.getMinx();
        double minY = referenceBbox.getMiny();

        double resol = this.getCurrentScale();
        Point2D corner = new Point2D.Double(center.getX() - ((width / 2) * resol), center.getY() - ((height / 2) * resol));

        int hbloq = (int) Math.floor((corner.getX() - minX) / (bloqSize * resol));
        int vbloq = (int) Math.floor((corner.getY() - minY) / (bloqSize * resol));

        Point2D rCorner = new Point2D.Double(minX + ((hbloq) * resol * bloqSize), minY + ((vbloq) * resol * bloqSize));

        final int difx = (int) ((corner.getX() - rCorner.getX()) / resol);
        final int dify = (int) ((corner.getY() - rCorner.getY()) / resol);

        final int nboxH = (int) Math.ceil((double) (width + Math.abs(difx)) / bloqSize);
        final int nboxV = (int) Math.ceil((double) (height + Math.abs(dify)) / bloqSize);

        double offSetx = -difx;
        double offSety = height + dify - (nboxV * (bloqSize));

        double rb = resol * bloqSize;
        ArrayList<Square> delete = (ArrayList<Square>) squares.clone();

        for (int i = 0; i < nboxH; i++) {
            for (int j = 0; j < nboxV; j++) {

                BBox b = new BBox(minX + ((i + hbloq) * rb), minY + ((j + vbloq) * rb), minX + ((i + 1 + hbloq) * rb), minY + ((j + 1 + vbloq) * rb));
                int x = (int) ((i * bloqSize) + offSetx);
                int y = (int) (((nboxV - j - 1) * bloqSize) + offSety);

                boolean noMatch = true;
                for (int k = 0; k < squares.size() && noMatch; k++) {
                    if (squares.get(k).getBbox().equals(b)) {
                        noMatch = false;
                        squares.get(k).setX(x);
                        squares.get(k).setY(y);
                        delete.remove(squares.get(k));
                    }
                }

                if (noMatch) {

                    String txtUrl = urlpet + "&format=" + this.getCurrentImageFormat() + b.getUrlForm();
                    if (isAdminEntrace()) {
                        txtUrl += "&admin_entrace=true";
                    }

                    Square sq = new Square(x, y, txtUrl, b);
                    squares.add(sq);
                    cache.addPending(sq);
                }
            }
        }

        while (delete.size() > 0) {
            squares.remove(delete.get(0));
            delete.remove(0);
        }
    }

    //METODOS DEL CICLO DE VIDA
    /**
     * Establece la URL del servidor Atlas con el que se desea trabajar, y recupera
     * la información correspondiente al contenido disponible en este.
     * 
     * @param appUrl
     * @throws java.net.ConnectException
     * @throws exceptions.HTTPRequestException
     * @throws java.lang.Exception
     */
    public void setURL(URL appUrl) throws ConnectException, HTTPRequestException, Exception {

        String tmp = appUrl.toString();
        if (!tmp.endsWith("/")) {
            tmp += "/";
        }

        this.appUrl = appUrl;
        wmsUrl = new URL(tmp + "wms");
        geocoderUrl = new URL(tmp + "geocoder");

        String textUrl = wmsUrl.toString() + "?request=GetCapabilities&service=WMS&version=1.3.0";

        if (this.isAdminEntrace()) {
            textUrl += "&admin_entrace=true";
        }

        URL urlGetCap = new URL(textUrl);
        HttpURLConnection con = (HttpURLConnection) urlGetCap.openConnection();

        if (con.getResponseCode() != HttpURLConnection.HTTP_OK) {
            System.out.println(textUrl);
            throw (new HTTPRequestException(con));
            
        }
        
        
        
        SAXBuilder builder = new SAXBuilder();
        Document doc = null;
        doc = builder.build(con.getInputStream());
        con.disconnect();

        Element root = doc.getRootElement();
        Namespace ns = root.getNamespace();

        List proyects = root.getChild("Capability", ns).getChild("Layer", ns).getChildren("Layer", ns);
        Iterator ite = proyects.iterator();

        projects = new Project[proyects.size()];

        for (int i = 0; ite.hasNext(); i++) {
            Element pr = (Element) ite.next();
            String crscode = pr.getChildText("CRS", ns).substring(5, pr.getChildText("CRS", ns).length());
            pr.removeChild("CRS", ns);
            pr.addContent(EpsgCRS.getEpsgCRS(crscode, wmsUrl));
            getProjects()[i] = new Project(pr);
        }

        List lImageFormats = root.getChild("Capability", ns).getChild("Request", ns).getChild("GetMap", ns).getChildren("Format", ns);
        List lFeatureFormats = root.getChild("Capability", ns).getChild("Request", ns).getChild("GetFeatureInfo", ns).getChildren("Format", ns);

        imageFormats = new String[lImageFormats.size()];
        for (int i = 0; i < getImageFormats().length; i++) {
            imageFormats[i] = ((Element) lImageFormats.get(i)).getText();
        }

        featureFormats = new String[lFeatureFormats.size()];
        for (int i = 0; i < getFeatureInfoFormats().length; i++) {
            featureFormats[i] = ((Element) lFeatureFormats.get(i)).getText();
        }

        setCurrentProject(null);
        setLayers(null);
    }

    /**
     * Establce el proyecto actual, debe pertenecer al array de proyectos disponibles en el servidor.
     * @param currentProject
     * @throws java.lang.IllegalArgumentException
     * @throws java.lang.IllegalStateException
     */
    public void setCurrentProject(Project currentProject) throws IllegalArgumentException, IllegalStateException {
        if (currentProject != null) {

            if (this.wmsUrl == null) {
                throw new IllegalStateException("The setURL() method must be called first.");
            }
            if (projects.length == 0) {
                throw new IllegalStateException("There is no projects loaded, it is possible that you need to call setURL first,  or the project is empty");
            }

            boolean valid = false;
            for (int i = 0; i < projects.length && !valid; i++) {
                if (currentProject.equals(projects[i])) {
                    valid = true;
                }
            }
            if (!valid) {
                throw new IllegalArgumentException("The currect project must belong to the projects array");
            }

            this.currentProject = currentProject;
            calcularReferencias();
            setCenter(currentProject.getBbox().getCenter());
            setLayers(currentProject.getLayerArray());
            setZoomLevel(0);
        } else {
            this.currentProject = null;
        }
    }

    /**
     * Establece el centro de la visualización en unidades de mapa. Requiere actualizar 
     * la visualización.
     * @param center
     */
    public void setCenter(Point2D center) {
        this.center = (Point2D) center.clone();
    }

    /**
     * Obtiene el centro actual de la visualización en coordenadas de mapa.
     * @return
     */
    public Point2D getCenter() {
        return this.center;
    }

    /**
     * Desplaza el centro del mapa en forma horizontal y vertical, tantas unidades 
     * de pantalla como indiquen las componentes de este punto.
     * @param distpx
     */
    public void moveCenter(Point2D distpx) {
        center.setLocation(center.getX() + (getCurrentScale() * distpx.getX()), center.getY() + (getCurrentScale() * distpx.getY()));
        updateView();
    }

    /**
     * Establece la capas que se usarán para visualización, deben pertenecer al listado de 
     * capas del proyecto actual.
     * @param layers
     */
    public void setLayers(Layer[] layers) {
        this.layers = layers;
        if (layers != null) {
            findUrlString();
        } else {
            urlpet = null;
        }
    }

    /**
     * Establece el formato en que el servidor debe retornar las imagenes
     * debe ser uno de los formatos declarados por el servidor.
     * @param format
     * @throws java.lang.IllegalArgumentException
     * @throws java.lang.IllegalStateException
     */
    public void setCurrentImageFormat(String format) throws IllegalArgumentException, IllegalStateException {

        if (wmsUrl == null) {
            throw new IllegalStateException("The setURL() method must be called first.");
        }
        if (getImageFormats().length == 0) {
            throw new IllegalStateException("There is no formats loaded, it is possible that you need to call setURL first");
        }

        boolean valid = false;
        String[] formats = getImageFormats();

        for (int i = 0; i < formats.length; i++) {
            if (format.equals(formats[i])) {
                valid = true;
            }
        }

        if (!valid) {
            throw new IllegalArgumentException("The format must belong to the image formats array, call getImageFormats()");
        }
        currentImageFormat = format;
    }

    /**
     * Establece el formato en que el servidor debe retornar la información 
     * GetFeatureInfo, debe ser uno de los formatos declarados por el servidor.
     * @param format
     * @throws java.lang.IllegalArgumentException
     * @throws java.lang.IllegalStateException
     */
    public void setCurrentFeatureInfoFormat(String format) throws IllegalArgumentException, IllegalStateException {

        if (wmsUrl == null) {
            throw new IllegalStateException("The setURL() method must be called first.");
        }
        if (getFeatureInfoFormats().length == 0) {
            throw new IllegalStateException("There is no formats loaded, it is possible that you need to call setURL first");
        }

        boolean valid = false;
        for (int i = 0; i < getFeatureInfoFormats().length && !valid; i++) {
            if (format.equals(getFeatureInfoFormats()[i])) {
                valid = true;
            }
        }
        if (!valid) {
            throw new IllegalArgumentException("The format must belong to the features formats array, call getFeatureFormats()");
        }
        currentInfoFormat = format;
    }

    @Override
    public void setEnabled(boolean enabled) throws IllegalStateException {
        if (enabled) {
            if (wmsUrl == null) {
                throw new IllegalStateException("You must call setURL first");
            }
            if (currentProject == null) {
                throw new IllegalStateException("You must call setCurrectProject first");
            }
            if (currentProject.getLayers().size() == 0) {
                throw new IllegalStateException("The project does not cointain any layer");
            }
            if (center == null) {
                throw new IllegalStateException("Set a center first, call setCenter");
            }
        }
        this.enabled = enabled;
    }

    private void placeMarkers() {
        for (int i = 0; i < markers.size(); i++) {
            ((Marker) markers.get(i)).findPosition();
        }
    }

    @Override
    public void paint(Graphics graphics) {
        Graphics2D g2 = (Graphics2D) graphics;
        g2.setBackground(new Color(255, 255, 255));
        g2.clearRect(0, 0, getWidth(), getHeight());



        if (!isEnabled()) {
            return;
        }

        for (int i = 0; i < squares.size(); i++) {
            if (squares.get(i).getImg() != null) {
                squares.get(i).draw(g2);
            }
        }

        Iterator ite = markers.iterator();

        while (ite.hasNext()) {
            Marker m = (Marker) ite.next();

            if (currentMarker == m) {
                int w = m.getCurrent().getWidth() / 2;
                int h = m.getCurrent().getHeight() / 2;
                g2.drawImage(m.getCurrent(), (int) m.getScreen().getX() - w, (int) m.getScreen().getY() - h, null);
            } else if (overMarker == m) {
                int w = m.getOver().getWidth() / 2;
                int h = m.getOver().getHeight() / 2;
                g2.drawImage(m.getOver(), (int) m.getScreen().getX() - w, (int) m.getScreen().getY() - h, null);
            } else {
                int w = m.getNormal().getWidth() / 2;
                int h = m.getNormal().getHeight() / 2;
                g2.drawImage(m.getNormal(), (int) m.getScreen().getX() - w, (int) m.getScreen().getY() - h, null);

            }
        }
        if (p1 != null) {
            if (getMode() == MODEZOOMBOX) {
                int x1 = (int) this.p1.getX();
                int y1 = (int) this.p1.getY();

                int ancho = Math.abs(zoomBoxWidth);
                int alto = Math.abs(zoomBoxHeigth);

                if (zoomBoxWidth < 0) {
                    x1 -= ancho;
                }
                if (zoomBoxHeigth < 0) {
                    y1 -= alto;
                }
                g2.setColor(new Color(255, 255, 255, 128));
                g2.fillRect(x1, y1, ancho, alto);
                g2.setColor(new Color(255, 0, 0, 128));
                g2.setStroke(new BasicStroke(2));
                g2.drawRect(x1, y1, ancho, alto);

                int cx = (int) (x1 + (ancho / 2));
                int cy = (int) (y1 + (alto / 2));
                g2.drawLine(cx, cy - 10, cx, cy + 10);
                g2.drawLine(cx - 10, cy, cx + 10, cy);

            } else if (getMode() == MODEFEATUREINFO) {
                int rm = (int) (radius * 2);
                g2.setColor(new Color(255, 255, 255, 128));
                g2.fillOval((int) p1.getX() - radius, (int) p1.getY() - radius, rm, rm);
                g2.setColor(new Color(255, 0, 0, 128));
                g2.setStroke(new BasicStroke(2));
                g2.drawOval((int) p1.getX() - radius, (int) p1.getY() - radius, rm, rm);
                g2.drawLine(p1.x, p1.y - 10, p1.x, p1.y + 10);
                g2.drawLine(p1.x - 10, p1.y, p1.x + 10, p1.y);
            }
        }
        g2.drawImage(north, getWidth() - 10 - north.getWidth(), 10, null);
    }

    /**
     * Retorna la URL del servidor Atlas actual.
     * @return
     */
    public URL getURL() {
        return wmsUrl;
    }

    /**
     * Retorna el modo del operación actual.
     * @return
     */
    public int getMode() {
        return mode;
    }

    /**
     * Establece el modo de operación actual de acuerdo a las constantes de clase.
     * @param mode
     */
    public void setMode(int mode) {
        this.mode = mode;
    }

    /**
     * Retorna el proyecto actual.
     * @return
     */
    public Project getCurrentProject() {
        return currentProject;
    }

    /**
     * Retorna el listado de proyectos disponibles en el servidor.
     * @return
     */
    public Project[] getProjects() {
        return projects;
    }

    /**
     * Retorna el formato que se seleccionó para que el servidor retorne imagenes.
     * @return
     */
    public String getCurrentImageFormat() {
        return currentImageFormat;
    }

    /**
     * Retorna el formato que se seleccionó para que el servidor retorne información
     * GetFeatureInfo. 
     * @return
     */
    public String getCurrentFeatureInfoFormat() {
        return currentInfoFormat;
    }

    /**
     * Formatos de imagen soportados por el servidor.
     * @return
     */
    public String[] getImageFormats() {
        return imageFormats;
    }

    /**
     * Formatos de GetFeatureInfo soportados por el servidor.
     * @return
     */
    public String[] getFeatureInfoFormats() {
        return featureFormats;
    }

    /**
     * Administrador de eventos asociado a este control.
     * @return
     */
    public EventManager getEventManager() {
        return this.em;
    }

    @Override
    public boolean isEnabled() {
        return enabled;
    }

    /**
     * Retorna el nivel de acercamiento actual como un número entero, positivo para
     * alejase del mapa y negativo para acercarse.
     * @return
     */
    public int getZoomLevel() {
        return lod;
    }

    /**
     * Establece el nivel de acercamiento actual como un número entero, positivo para
     * alejase del mapa y negativo para acercarse.
     * @param lod
     */
    public void setZoomLevel(int lod) {
        if (this.currentProject == null) {
            throw new IllegalStateException("Select a project first");
        }
        this.lod = lod;
        if (getZoomLevel() > 0) {
            currentScale = Math.pow(1 + zoomStep, getZoomLevel()) * baseScale;
        } else if (getZoomLevel() < 0) {
            currentScale = Math.pow(1 - zoomStep, Math.abs(getZoomLevel())) * baseScale;
        } else {
            currentScale = baseScale;
        }
        findUrlString();
        em.fireDisplayEvent();
    }

    /**
     * Retorna el array de capas seleccionadas para solicitar imagenes, la 
     * modificación del contenido de este listado puede tener consecuencias
     * inesperadas.
     * @return
     */
    public Layer[] getLayers() {
        return layers;
    }

    /**
     * Vacia el cache de imagenes del control para que este solicite las versiones
     * más recientes al servidor.
     */
    public void clearCache() {
        squares.clear();
    }

    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        setLayout(null);

        addMouseWheelListener(new java.awt.event.MouseWheelListener() {
            public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) {
                formMouseWheelMoved(evt);
            }
        });
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                formMouseClicked(evt);
            }
            public void mouseExited(java.awt.event.MouseEvent evt) {
                formMouseExited(evt);
            }
            public void mousePressed(java.awt.event.MouseEvent evt) {
                formMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                formMouseReleased(evt);
            }
        });
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });
        addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseDragged(java.awt.event.MouseEvent evt) {
                formMouseDragged(evt);
            }
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                formMouseMoved(evt);
            }
        });

    }// </editor-fold>//GEN-END:initComponents
    private void formMouseWheelMoved(java.awt.event.MouseWheelEvent evt) {//GEN-FIRST:event_formMouseWheelMoved
        if (isEnabled()&&isMouseWheelEnabled()) {
            if (evt.getWheelRotation() < 0) {
                nTicks--;
            } else if (evt.getWheelRotation() > 0) {
                nTicks++;
            }
            final MouseWheelEvent ev = evt;
            timer.cancel();
            timer = new Timer();
            task = new TimerTask() {

                public void run() {
                    changeZoom(ev, nTicks, true);
                    findSquares();
                    cache.activate(0, false);
                    nTicks = 0;
                    em.fireDisplayEvent();
                }
            };
            timer.schedule(task, 250);
        }
    }//GEN-LAST:event_formMouseWheelMoved

    private void formMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseReleased
        if (isEnabled() && drag) {
            if (getMode() == MODEPAN || getMode() == MODEGETCOORD) {
                findSquares();
                placeMarkers();
                em.fireDisplayEvent();
                em.fireMouseMoveEvent(evt);
                repaint();
                cache.deactivate();
                System.gc();
            } else if (getMode() == MODEZOOMBOX) {
                int anchoCaja = (int) Math.abs(p1.getX() - evt.getX());
                int altoCaja = (int) (((double) this.getHeight() / this.getWidth()) * anchoCaja);
                Point tmp = new Point();
                if (evt.getY() > p1.getY()) {
                    tmp.setLocation(evt.getX(), p1.getY() + altoCaja);
                } else {
                    tmp.setLocation(evt.getX(), p1.getY() - altoCaja);
                }

                int centroX = (int) ((p1.getX() + tmp.getX()) / 2);
                int centroY = (int) ((p1.getY() + tmp.getY()) / 2);
                center = CoordinateUtils.translateIJtoPoint(this.getVisibleBbox(), this.getWidth(), this.getHeight(), centroX, centroY, getCurrentProject().getCrs());
                double subScale = (anchoCaja * getCurrentScale()) / getWidth();
                setZoomLevel(-(int) (Math.log(subScale / baseScale) / Math.log(1 - zoomStep)));
                findSquares();
                placeMarkers();
                cache.activate(0, false);
                zoomBoxWidth = 0;
                zoomBoxHeigth = 0;
                em.fireDisplayEvent();
                em.fireMouseMoveEvent(evt);

            } else if (getMode() == MODEFEATUREINFO) {
                sendFeatureInfoRequest(new Point(p1));
            } /*else if (getMode() == MODEGETCOORD) {
        getCoordinate(evt);
        }*/
        }
        drag = false;
        p1 = null;
    }//GEN-LAST:event_formMouseReleased

    private void getCoordinate(java.awt.event.MouseEvent evt) {
        currentMarker = null;
        Iterator ite = markers.iterator();
        while (ite.hasNext()) {
            Marker m = (Marker) ite.next();
            if (m.isPointInside((int) evt.getX(), (int) evt.getY())) {
                currentMarker = m;
            }
        }

        repaint();
        if (currentMarker == null) {
            em.fireMouseClickEvent(evt);
        } else {
            em.fireMarkerClickedEvent(evt, currentMarker);
        }
    }

    /**
     * Envia una petición al geocoder el proyecto actuál, las respuestas deben recibiese
     * programando un escuchador de eventos.
     * @param address
     * @throws java.lang.Exception
     */
    public void sendGeocoderRequest(String address) throws Exception {
        try {
            if (getCurrentProject() == null) {
                throw new IllegalArgumentException("Some project must be selected first");
            }
            String idProj = currentProject.getName().substring(1);
            String request = this.geocoderUrl.toString() + "?project=" + idProj + "&address=" + URLEncoder.encode(address, "UTF8") + "&type=XML";
            if (isAdminEntrace()) {
                request += "&admin_entrace=true";
            }
            em.fireGeocodingEvent(IOUtils.readFromURL(request, true));

        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(MapPanel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void sendFeatureInfoRequest(Point ij) {


        String urlpeti = wmsUrl.toString() + "?type=XML&version=1.3.0&request=GetFeatureInfo&styles=&CRS=EPSG:" + getCurrentProject().getCrs().getCode() + "&width=" + getWidth() + "&height=" + getHeight() + "&format=" + this.getCurrentImageFormat() + "&info_format=" + this.getCurrentFeatureInfoFormat();
        if (isAdminEntrace()) {
            urlpeti += "&admin_entrace=true";
        }
        urlpeti += "&query_layers=";

        Layer[] layersinfo;

        if (getFeatureInfoLayers() != null) {
            layersinfo = getFeatureInfoLayers();
        } else {
            List<Layer> tmp = new ArrayList<Layer>();
            Layer[] lyrs = getVisibleLayers();
            for (int i = 0; i < lyrs.length; i++) {
                Layer layer = lyrs[i];
                if (layer.isQueryable()) {
                    tmp.add(layer);
                }
            }
            layersinfo = new Layer[tmp.size()];
            tmp.toArray(layersinfo);
        }

        for (int i = 0; i < layersinfo.length; i++) {
            urlpeti += ((org.atlas.clientdata.Layer) layersinfo[i]).getName() + ",";
        }

        Layer[] ilayers = currentProject.getLayerArray();
        urlpeti += "&layers=";
        for (int i = 0; i < ilayers.length; i++) {
            urlpeti += (ilayers[i].getName() + ",");
        }
        urlpeti += ("&bbox=" + (getVisibleBbox().getMinx() + "," + getVisibleBbox().getMiny() + "," + getVisibleBbox().getMaxx() + "," + getVisibleBbox().getMaxy()));
        urlpeti += ("&i=" + (int) ij.getX() + "&j=" + (int) ij.getY());
        if (radius > 0) {
            urlpeti += ("&distance=" + radius);
        } else {
            urlpeti += ("&distance=");
        }
        radius = -1;
        repaint();
        try {
            em.fireFeatureInfoEvent(new String(IOUtils.readFromURL(urlpeti, true)), urlpeti);
        } catch (MalformedURLException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    private void formMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMousePressed
        if (isEnabled()) {
            if (getMode() == MODEPAN || getMode() == MODEZOOMBOX || getMode() == MODEFEATUREINFO || getMode() == MODEGETCOORD) {
                p1 = getMousePosition();
                if (getMode() == MODEPAN || getMode() == MODEGETCOORD) {
                    cache.activate(600, true);
                }
            }
        }
    }//GEN-LAST:event_formMousePressed

    private void formMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseClicked
        if (evt.getClickCount() == 1) {
            if (isEnabled()) {
                if (getMode() == MODEZOOMPLUS) {
                    changeZoom(evt, -1, false);
                    squares.clear();
                    findSquares();
                    placeMarkers();
                    cache.activate(0, false);
                    em.fireDisplayEvent();
                } else if (getMode() == MODEZOOMMINUS) {
                    changeZoom(evt, +1, false);
                    squares.clear();
                    findSquares();
                    placeMarkers();
                    cache.activate(0, false);
                    em.fireDisplayEvent();
                } else if (getMode() == MODEFEATUREINFO) {
                    radius = -1;
                    sendFeatureInfoRequest(evt.getPoint());
                } else if (getMode() == MODEGETCOORD) {
                    getCoordinate(evt);
                }
            }
        }
    }//GEN-LAST:event_formMouseClicked

    private void formMouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseMoved
        if (isEnabled()) {
            overMarker = null;
            Iterator ite = markers.iterator();
            while (ite.hasNext()) {
                Marker m = (Marker) ite.next();
                int x = (int) evt.getX();
                int y = (int) evt.getY();
                if (m.isPointInside(x, y)) {
                    overMarker = m;
                }

            }
            repaint();
            em.fireMouseMoveEvent(evt);
        }
    }//GEN-LAST:event_formMouseMoved

    private void formMouseExited(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseExited
// TODO add your handling code here:
    }//GEN-LAST:event_formMouseExited

    private void formMouseDragged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseDragged
        if (isEnabled()) {
            drag = true;
            if (getMode() == MODEZOOMBOX && p1 != null) {
                double cte = (double) getHeight() / getWidth();
                zoomBoxWidth = Math.abs((int) (evt.getX() - p1.getX()));
                zoomBoxHeigth = (int) (zoomBoxWidth * cte);
                if (evt.getX() - p1.getX() < 0) {
                    zoomBoxWidth *= -1;
                }
                if (evt.getY() - p1.getY() < 0) {
                    zoomBoxHeigth *= -1;
                }
                em.fireMouseMoveEvent(evt);
                repaint();
            } else if (getMode() == MODEPAN || getMode() == MODEGETCOORD) {
                double sca = getCurrentScale();
                double dx = ((p1.getX() - evt.getX()) * sca);
                double dy = ((p1.getY() - evt.getY()) * sca);

                center.setLocation(center.getX() + dx, center.getY() - dy);
                p1 = evt.getPoint();
                findSquares();
                placeMarkers();
                repaint();
            } else if (getMode() == MODEFEATUREINFO) {
                radius = (int) Math.sqrt(Math.pow(p1.getX() - evt.getX(), 2) + Math.pow(p1.getY() - evt.getY(), 2));
                em.fireMouseMoveEvent(evt);
                repaint();
            }
        }
    }//GEN-LAST:event_formMouseDragged

    /**
     * Actualiza la visualización para reflejar cambios en los datos.
     */
    public void updateView() {
        findSquares();
        placeMarkers();
        cache.activate(0, false);
        repaint();
    }

    /**
     * Incrementa el acercamiento y actualiza la visualización.
     */
    public void ZoomIn() {
        setZoomLevel(getZoomLevel() - 1);
        updateView();
    }

    /**
     * Decrementa el acercamiento y actualiza la visualización. 
     */
    public void ZoomOut() {
        setZoomLevel(getZoomLevel() + 1);
        updateView();
    }

    private void calcularReferencias() {
        referenceBbox = getCurrentProject().getBbox();
        double hz = (double) getWidth() / getCurrentProject().getBbox().getWidth();
        double vr = (double) getHeight() / getCurrentProject().getBbox().getHeight();

        if (hz < vr) {//escalar al ancho
            baseScale = getCurrentProject().getBbox().getWidth() / getWidth();
        } else {    //escalar al alto
            baseScale = getCurrentProject().getBbox().getHeight() / getHeight();
        }
    }

    /**
     * Ajusta el nivel de acercamiento y centro de tal modo que se visualize todo 
     * el proyecto.
     * @throws java.lang.IllegalStateException
     */
    public void zoomToProject() throws IllegalStateException {
        if (wmsUrl == null) {
            throw new IllegalStateException("You must call setURL first");
        }
        if (currentProject == null) {
            throw new IllegalStateException("You must call setCurrectProject first");
        }
        if (currentProject.getLayers().size() == 0) {
            throw new IllegalStateException("The project does not cointain any layer");
        }

        if (getCurrentImageFormat() == null) {
            throw new IllegalStateException("You must select some image format first");
        }

        setCenter(getCurrentProject().getBbox().getCenter());
        setZoomLevel(0);
        em.fireDisplayEvent();
        updateView();
    }

    /**
     * Ajusta el nivel de acercamiento y centro de tal modo que se visualize la capa.
     * @param layer
     * @throws java.lang.IllegalStateException
     */
    public void zoomToLayer(Layer layer) throws IllegalStateException {
        if (wmsUrl == null) {
            throw new IllegalStateException("You must call setURL first");
        }
        if (currentProject == null) {
            throw new IllegalStateException("You must call setCurrectProject first");
        }
        if (currentProject.getLayers().size() == 0) {
            throw new IllegalStateException("The project does not cointain any layer");
        }
        if (getCurrentImageFormat() == null) {
            throw new IllegalStateException("You must select some image format first");
        }

//        double hz = (double) getWidth() / getCurrentProject().getBbox().getWidth();
//        double vr = (double) getHeight() / getCurrentProject().getBbox().getHeight();
//
//        if (hz < vr) {//escalar al ancho
//            baseScale = getCurrentProject().getBbox().getWidth() / getWidth();
//        } else {    //escalar al alto
//            baseScale = getCurrentProject().getBbox().getHeight() / getHeight();
//        }

        setCenter(layer.getBoundigBox().getCenter());
        em.fireDisplayEvent();
    }

    /**
     * Listado de las capas que son visibles al nivel de acercamiento actual.
     * @return
     * @throws java.lang.IllegalStateException
     */
    public Layer[] getVisibleLayers() throws IllegalStateException {
        List<Layer> rta = new ArrayList<Layer>();

        Layer[] capas = getLayers();
        for (int i = 0; i < capas.length; i++) {
            Layer lyr = capas[i];
            boolean vis = true;
            double sca = this.getWMScale();

            if (lyr.getMaxscaledeno() != Double.NaN) {
                if (sca >= lyr.getMaxscaledeno()) {
                    vis = false;
                }
            }

            if (lyr.getMinscaledeno() != Double.NaN) {
                if (sca < lyr.getMinscaledeno()) {
                    vis = false;
                }
            }

            if (vis) {
                rta.add(lyr);
            }

        }
        Layer[] lyrs = new Layer[rta.size()];
        rta.toArray(lyrs);
        return lyrs;
    }

    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
        if (isEnabled()) {
            calcularReferencias();
            findSquares();
            cache.activate(0, false);
            placeMarkers();
            repaint();
            em.fireDisplayEvent();
        }
    }//GEN-LAST:event_formComponentResized
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    /**
     * 
     * @param admin
     */
    public void setAdminEntrace(boolean admin) {
        adminEntrace = admin;
    }

    /**
     * 
     * @return
     */
    public boolean isAdminEntrace() {
        return adminEntrace;
    }
}
