package org.atlas.desk.gui;

import org.atlas.clientdata.BBox;
import org.atlas.clientdata.EpsgCRS;
import org.atlas.clientdata.Layer;
import org.atlas.clientdata.Project;
import org.atlas.exceptions.HTTPRequestException;
import java.awt.Color;
import java.awt.Point;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.net.ConnectException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.atlas.desk.mashup.Marker;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.atlas.util.CoordinateUtils;

public class MapPanelTest{

    public static final int NOMODESELECTED = -1;
    public static final int MODEZOOMPLUS = 1;
    public static final int MODEZOOMMINUS = 2;
    public static final int MODEZOOMPOINT = 3;
    public static final int MODEPAN = 4;
    public static final int MODEGETCOORD = 5;
    public static final int MODEINFO = 6;
    //private EventManager em = new EventManager(this);
    private URL appUrl;
    private URL geocoderUrl;
    private URL wmsUrl;
    private String urlpet;
    private int mode; //modo de operacion actual
    private Point p1; //primer pto para operaciones de dos puntos como panning o zoom punto punto
    private Point2D center;    //map coordinate, center of the viewport
    private double baseScale; //initial scale
    private final int bloqSize = 256;
    private BBox referenceBbox;
    private int lod;
    private double currentScale;
    private double zoomStep = 0.2;
    private int radius; //radio para la peticiÃ³n feature info
    private int zoomBoxWidth;
    private int zoomBoxHeigth;
    private Object[] projects;
    private Project currentProject;
    private Object[] currentLayers;
    private ArrayList<ActionListener> imageArrivalListeners = new ArrayList<ActionListener>();
    private String[] imageFormats;
    private String[] featureFormats;
    private String currentImageFormat;
    private String currentInfoFormat;
    private Color bgColor;
    //for the mouse wheel
    private int nTicks = 0;
    private boolean enabled;
    private boolean drag = false;
    private List markers = new ArrayList();
    private Marker overMarker = null;
    private Marker currentMarker = null;
    public ArrayList<Square> squares = new ArrayList();
    private Layer[] layersFeatureInfo;

    public MapPanelTest() {
        squares.ensureCapacity(20);
    }

    public int getWidth(){
        return 640;
    }
    
    public int getHeight(){
        return 480;
    }
    
    public Layer[] getLayersFeatureInfo() {
        return layersFeatureInfo;
    }

    public void setLayersFeatureInfo(Layer[] layersFeatureInfo) {
        this.layersFeatureInfo = layersFeatureInfo;
    }

    public void addMarker(Marker marker) {
        marker.findPosition();
        markers.add(marker);
    }

    public void removeMarker(Marker marker) {
        markers.remove(marker);
    }

    public void clearMarkers() {
        markers.clear();
    }

    public double getWmsScale() {
        BBox vis = getVisibleBbox();
        EpsgCRS crs = getCurrentProject().getCrs();
        double k = 0.00028;
        if (crs.getType() == EpsgCRS.UNIT_LINEAR) {
            return ((vis.getWidth() * crs.getConversionFactor()) / getWidth()) / k;
        } else {
            double rad = vis.getWidth() * crs.getConversionFactor();
            double ls = rad * 6378137;
            return (ls / getWidth()) / k;
        }
    }

    private double getCurrentScale() {
        return currentScale;
    }

    private void changeZoom(MouseEvent ev, int deltaLOD, boolean reCenter) {
        if (reCenter) {
            int iCoord = ev.getX();
            int jCoord = ev.getY();
            center = CoordinateUtils.translateIJtoPoint(getVisibleBbox(), getWidth(), getHeight(), iCoord, jCoord, getCurrentProject().getCrs());
        }
        setLod(getLod() + deltaLOD);
//        em.fireMouseMoveEvent(ev);
    }

    public BBox getVisibleBbox() {
        double ancho = this.getWidth() * this.getCurrentScale();
        double alto = this.getHeight() * this.getCurrentScale();
        return new BBox(center.getX() - (ancho / 2), center.getY() - (alto / 2), center.getX() + (ancho / 2), center.getY() + (alto / 2));
    }

    private void findUrlString() {
        if (getCurrentProject() != null && getCurrentLayers() != null && getCurrentImageFormat() != null) {
            urlpet = getUrl().toString() + "?version=1.3.0&request=GetMap&styles=&CRS=EPSG:" + getCurrentProject().getCrs().getCode() + "&width=" + this.bloqSize + "&height=" + this.bloqSize + "&format=" + this.getCurrentImageFormat();
            urlpet += "&layers=";
            Object[] layers = getCurrentLayers();
            for (int i = 0; i < layers.length; i++) {
                org.atlas.clientdata.Layer lyr = (org.atlas.clientdata.Layer) layers[i];
                boolean vis = true;
                double sca = this.getWmsScale();

                if (lyr.getMaxscaledeno() != Double.NaN) {
                    if (sca >= lyr.getMaxscaledeno()) {
                        vis = false;
                    }
                }

                if (lyr.getMinscaledeno() != Double.NaN) {
                    if (sca < lyr.getMinscaledeno()) {
                        vis = false;
                    }
                }

                if (vis) {
                    urlpet += lyr.getName() + ",";
                }
            }
        } else {
            urlpet = "";
        }
    }

    public void findSquares() {
        if (!isEnabled()) {
            return;
        }

        int width = 640;
        int height = 480;

        double minX = referenceBbox.getMinx();
        double minY = referenceBbox.getMiny();

        double resol = this.getCurrentScale();
        Point2D corner = new Point2D.Double(center.getX() - ((width / 2) * resol), center.getY() - ((height / 2) * resol));

        int hbloq = (int) Math.floor((corner.getX() - minX) / (bloqSize * resol));
        int vbloq = (int) Math.floor((corner.getY() - minY) / (bloqSize * resol));

        Point2D rCorner = new Point2D.Double(minX + ((hbloq) * resol * bloqSize), minY + ((vbloq) * resol * bloqSize));

        final int difx = (int) ((corner.getX() - rCorner.getX()) / resol);
        final int dify = (int) ((corner.getY() - rCorner.getY()) / resol);

        final int nboxH = (int) Math.ceil((double) (width + Math.abs(difx)) / bloqSize);
        final int nboxV = (int) Math.ceil((double) (height + Math.abs(dify)) / bloqSize);

        double offSetx = -difx;
        double offSety = height + dify - (nboxV * (bloqSize));

        double rb = resol * bloqSize;

        squares.clear();

        for (int i = 0; i < nboxH; i++) {
            for (int j = 0; j < nboxV; j++) {
                BBox b = new BBox(minX + ((i + hbloq) * rb), minY + ((j + vbloq) * rb), minX + ((i + 1 + hbloq) * rb), minY + ((j + 1 + vbloq) * rb));
                int x = (int) ((i * bloqSize) + offSetx);
                int y = (int) (((nboxV - j - 1) * bloqSize) + offSety);
                Square sq = new Square(x, y, urlpet + b.getUrlForm(), b);
                squares.add(sq);
            }
        }
    }

    //METODOS DEL CICLO DE VIDA
    public void setURL(URL appUrl) throws ConnectException, HTTPRequestException, Exception {

        String tmp = appUrl.toString();
        if (!tmp.endsWith("/")) {
            tmp += "/";
        }

        this.appUrl = appUrl;
        wmsUrl = new URL(tmp + "wms");
        geocoderUrl = new URL(tmp + "geocoder");

        String textUrl = wmsUrl.toString() + "?request=GetCapabilities&service=WMS&version=1.3.0";
        URL urlGetCap = new URL(textUrl);
        HttpURLConnection con = (HttpURLConnection) urlGetCap.openConnection();

        if (con.getResponseCode() != HttpURLConnection.HTTP_OK) {
            throw (new HTTPRequestException(con));
        }

        SAXBuilder builder = new SAXBuilder();
        Document doc = null;
        doc = builder.build(con.getInputStream());
        con.disconnect();

        Element root = doc.getRootElement();
        Namespace ns = root.getNamespace();

        List proyects = root.getChild("Capability", ns).getChild("Layer", ns).getChildren("Layer", ns);
        Iterator ite = proyects.iterator();

        projects = new Project[proyects.size()];

        for (int i = 0; ite.hasNext(); i++) {
            Element pr = (Element) ite.next();
            String crscode = pr.getChildText("CRS", ns).substring(5, pr.getChildText("CRS", ns).length());
            pr.removeChild("CRS", ns);
            pr.addContent(EpsgCRS.getEpsgCRS(crscode, wmsUrl));
            getProjects()[i] = new Project(pr);
        }

        List lImageFormats = root.getChild("Capability", ns).getChild("Request", ns).getChild("GetMap", ns).getChildren("Format", ns);
        List lFeatureFormats = root.getChild("Capability", ns).getChild("Request", ns).getChild("GetFeatureInfo", ns).getChildren("Format", ns);

        imageFormats = new String[lImageFormats.size()];
        for (int i = 0; i < getImageFormats().length; i++) {
            imageFormats[i] = ((Element) lImageFormats.get(i)).getText();
        }

        featureFormats = new String[lFeatureFormats.size()];
        for (int i = 0; i < getFeatureFormats().length; i++) {
            featureFormats[i] = ((Element) lFeatureFormats.get(i)).getText();
        }

        this.currentProject = null;
        this.setCurrentLayers(null);
    }

    public void setCurrentProject(Project currentProject) {
        setCurrentProject(currentProject, true);
    }

    public void setCurrentProject(Project currentProject, boolean reCenter) throws IllegalArgumentException, IllegalStateException {
        if (this.wmsUrl == null) {
            throw new IllegalStateException("The setURL() method must be called first.");
        }
        if (projects.length == 0) {
            throw new IllegalStateException("There is no projects loaded, it is possible that you need to call setURL first,  or the project is empty");
        }
        boolean valid = false;
        for (int i = 0; i < projects.length && !valid; i++) {
            if (currentProject.equals(projects[i])) {
                valid = true;
            }
        }
        if (!valid) {
            throw new IllegalArgumentException("The currect project must belong to the projects array");
        }
        this.currentProject = currentProject;
        if (reCenter) {
            if (this.getCurrentProject().getLayers().size() > 0) {
                setZoomToProject();
            }
        }
    }

    public void setCenter(Point2D center) {
        this.center = (Point2D) center.clone();
    }

    public void setCurrentLayers(Object[] currentLayers) {
        this.currentLayers = currentLayers;
        findUrlString();
    }

    public void setCurrentImageFormat(String format) throws IllegalArgumentException, IllegalStateException {

        if (wmsUrl == null) {
            throw new IllegalStateException("The setURL() method must be called first.");
        }
        if (getImageFormats().length == 0) {
            throw new IllegalStateException("There is no formats loaded, it is possible that you need to call setURL first");
        }

        boolean valid = false;
        String[] formats = getImageFormats();

        for (int i = 0; i < formats.length; i++) {
            if (format.equals(formats[i])) {
                valid = true;
            }
        }

        if (!valid) {
            throw new IllegalArgumentException("The format must belong to the image formats array, call getImageFormats()");
        }
        currentImageFormat = format;
        findUrlString();
    }

    public void setCurrentFeatureFormat(String format) throws IllegalArgumentException, IllegalStateException {

        if (wmsUrl == null) {
            throw new IllegalStateException("The setURL() method must be called first.");
        }
        if (getFeatureFormats().length == 0) {
            throw new IllegalStateException("There is no formats loaded, it is possible that you need to call setURL first");
        }

        boolean valid = false;
        for (int i = 0; i < getFeatureFormats().length && !valid; i++) {
            if (format.equals(getFeatureFormats()[i])) {
                valid = true;
            }
        }
        if (!valid) {
            throw new IllegalArgumentException("The format must belong to the features formats array, call getFeatureFormats()");
        }
        currentInfoFormat = format;
    }

    public void setZoomToProject() throws IllegalStateException {
        if (wmsUrl == null) {
            throw new IllegalStateException("You must call setURL first");
        }
        if (currentProject == null) {
            throw new IllegalStateException("You must call setCurrectProject first");
        }
        if (currentProject.getLayers().size() == 0) {
            throw new IllegalStateException("The project does not cointain any layer");
        }
        if (getCurrentImageFormat() == null) {
            throw new IllegalStateException("You must select some image format first");
        }

        referenceBbox = getCurrentProject().getBbox();
        double hz = (double) getWidth() / getCurrentProject().getBbox().getWidth();
        double vr = (double) getHeight() / getCurrentProject().getBbox().getHeight();

        if (hz < vr) {//escalar al ancho
            baseScale = getCurrentProject().getBbox().getWidth() / getWidth();
        } else {    //escalar al alto
            baseScale = getCurrentProject().getBbox().getHeight() / getHeight();
        }
        this.center = getCurrentProject().getBbox().getCenter();
        setLod(0);
//        em.fireDisplayEvent();
    }


    public void setEnabled(boolean enabled) throws IllegalStateException {
        if (enabled) {
            if (wmsUrl == null) {
                throw new IllegalStateException("You must call setURL first");
            }
            if (currentProject == null) {
                throw new IllegalStateException("You must call setCurrectProject first");
            }
            if (currentProject.getLayers().size() == 0) {
                throw new IllegalStateException("The project does not cointain any layer");
            }
            if (center == null) {
                throw new IllegalStateException("Set a center first, call setCenter");
            }
        }
        this.enabled = enabled;
    }

    private void placeMarkers() {
        //System.out.println("Place Markers");
        for (int i = 0; i < markers.size(); i++) {
            ((Marker) markers.get(i)).findPosition();
           // System.out.println("--Place");
        }
    }

    public URL getUrl() {
        return wmsUrl;
    }

    public int getMode() {
        return mode;
    }

    public void setMode(int mode) {
        this.mode = mode;
    }

    public Project getCurrentProject() {
        return currentProject;
    }

    public Object[] getProjects() {
        return projects;
    }

    public String getCurrentImageFormat() {
        return currentImageFormat;
    }

    public String getCurrentInfoFormat() {
        return currentInfoFormat;
    }

    public String[] getImageFormats() {
        return imageFormats;
    }

    public String[] getFeatureFormats() {
        return featureFormats;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public int getLod() {
        return lod;
    }

    public void setLod(int lod) {
        if (this.currentProject == null) {
            throw new IllegalStateException("Select a project first");
        }
        this.lod = lod;
        if (getLod() > 0) {
            currentScale = Math.pow(1 + zoomStep, getLod()) * baseScale;
        } else if (getLod() < 0) {
            currentScale = Math.pow(1 - zoomStep, Math.abs(getLod())) * baseScale;
        } else {
            currentScale = baseScale;
        }
        findUrlString();
    }

    public Object[] getCurrentLayers() {
        return currentLayers;
    }

    public void clearCache() {
        squares.clear();
    }
   


    public void refresh() {
        findSquares();
        placeMarkers();
    }
    
    public void setRandomCenter(){
        double r1 = Math.random();
        double r2 = Math.random();
        double x = (referenceBbox.getMinx()*(1-r1))+(referenceBbox.getMaxx()*r1);
        double y = (referenceBbox.getMiny()*(1-r2))+(referenceBbox.getMaxy()*r2);
        setCenter(new Point2D.Double(x, y));
    }

    public void setRandomLod(){
        setLod((int)(Math.random()*(-15)));
    }
    
    public static void main(String[] args) throws MalformedURLException, ConnectException, Exception {
        MapPanelTest mpt = new MapPanelTest();
        mpt.setURL(new URL("http://192.168.1.2:8080/server/"));
        mpt.setCurrentImageFormat("image/png");
        Project p = (Project)mpt.getProjects()[0];
        mpt.setCurrentProject(p);
        Layer[] lyrs = new Layer[p.getLayers().size()];
        p.getLayers().toArray(lyrs);
        mpt.setCurrentLayers(lyrs);
        mpt.setRandomCenter();
        mpt.setRandomLod();
        mpt.setEnabled(true);
        mpt.refresh();
        for (int i = 0; i < mpt.squares.size(); i++) {
            Square s = mpt.squares.get(i);
            System.out.println(s.getUrl());
        }
    }
}
