
package org.atlas.desk.mashup;

import org.atlas.desk.gui.MapPanel;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import org.atlas.util.CoordinateUtils;

/**
 * Representa un punto que puede ubicarse sobre el mapa, usualmente simboliza
 * información propia del domino de la aplicación final, logrando de este modo
 * el mashup con la cartografía del servidor.
 * @author Mario
 */
public class Marker{
    
    private Point2D  coordinate;
    private Point2D  screen;
    private BufferedImage    normal;
    private BufferedImage    over;
    private BufferedImage    current;
    private MapPanel map;
    
    
    /**
     * Crea un marcador para un MapPanel, ubicandolo en la coordenada indicada, 
     * y representandolo con la imagen especificada.
     * 
     * @param map
     * @param coordinate
     * @param normal
     */
    public Marker(MapPanel map, Point2D coordinate, BufferedImage normal){
        this.setCoordinate(coordinate);
        this.map = map;
        this.normal  = normal;
        this.over    = normal;
        this.current = normal;
    }
            
    /**
     * Calcula las coordenadas en pixeles que corresponden a este marcador dado el 
     * estado del MapPanel.
     */
    public void findPosition(){
        this.screen = CoordinateUtils.translateXYtoIJ(map.getCurrentProject().getCrs(), map.getVisibleBbox(), map.getWidth(), map.getHeight(), getCoordinate().getX(), getCoordinate().getY());
    }
    
    /**
     * Obtiene las coordenadas en pantalla del marcador en el estado actual del
     * MapPanel.
     * @return
     */
    public Point2D getScreen() {
        return screen;
    }
    
    /**
     * Obtiene las coordenadas del marcador respecto al mapa.
     * @return
     */
    public Point2D getCoordinate() {
        return coordinate;
    }
    
    /**
     * Establece las coordenadas del marcador respecto al mapa.
     * @param coordinate
     */
    public void setCoordinate(Point2D coordinate) {
        this.coordinate = coordinate;
//        findPosition();
    }
    
    /**
     * Obtiene el objeto MapPanel en el que se encuentra el marcador.
     * @return
     */
    public MapPanel getMap() {
        return map;
    }
        
    
    /**
     * Indica si un pixel con coordenadas x, y toca al marcador teniendo en cuenta
     * su posición actual en la pantalla.
     * @param x
     * @param y
     * @return
     */
    public boolean isPointInside(int x, int y){
        int h = (int)(current.getWidth()/2);
        int v = (int)(current.getHeight()/2);
        int sx = (int)getScreen().getX();
        int sy = (int)getScreen().getY();
        if(x-h <= sx && sx <= x+h && y-v <= sy && sy <= y+v){
            return true;
        }
        return false;
    }

    /**
     * Obtiene la imagen que representa al marcador en su estado normal.
     * @return
     */
    public BufferedImage getNormal() {
        return normal;
    }

    /**
     * Establece la imagen que representa al marcador en su estado normal. 
     * @param normal
     */
    public void setNormal(BufferedImage normal) {
        this.normal = normal;
    }

    /**
     * Obtiene la imagen que representa al marcador cuando el mouse pasa sobre él.
     * @return
     */
    public BufferedImage getOver() {
        return over;
    }

    /**
     * Establece la imagen que representa al marcador cuando el mouse pasa sobre él. 
     * @param over
     */
    public void setOver(BufferedImage over) {
        this.over = over;
    }

    /**
     * Establece la imagen que representa al marcador cuando es el actual.
     * @return
     */
    public BufferedImage getCurrent() {
        return current;
    }

    /**
     * Establece la imagen que representa al marcador cuando es el actual.  
     * @param current
     */
    public void setCurrent(BufferedImage current) {
        this.current = current;
    }
    
}
