package org.atlas.clientdata;
import java.awt.geom.Point2D;
import org.jdom.Element;

/**
 * Una representación ligera de un boundig box para ser usada en los componentes de desarrollo.
 * Un bounding box encierra una región del espacio 2D usando dos coordenadas.
 * Su interpretación depende del sistema de coordenadas de referencia que define las unidades en que se expresan las coordenadas y la orientación de los ejes
 * @author Proyecto Atlas 
 */
public class BBox implements Cloneable{
        
    private double minx;
    private double miny;
    private double maxx;
    private double maxy;
    
    /**
     * Para construir desde un elemento XML compatible con WMS
     * @param bBox
     */
    public BBox(Element bBox){
        setMinx(Double.valueOf(bBox.getAttributeValue("minx")));
        setMiny(Double.valueOf(bBox.getAttributeValue("miny")));
        setMaxx(Double.valueOf(bBox.getAttributeValue("maxx")));
        setMaxy(Double.valueOf(bBox.getAttributeValue("maxy")));
    }
    
    /**
     * Para construir dados valores maximos y minimos
     * @param minx
     * @param miny
     * @param maxx
     * @param maxy
     */
    public BBox(double minx, double miny, double maxx, double maxy) {
        this.setMinx(minx);
        this.setMiny(miny);
        this.setMaxx(maxx);
        this.setMaxy(maxy);
    }
        
    /**
     * Obtener mínimo valor del primer eje
     * @return
     */
    public double getMinx() {
        return minx;
    }
    
    
    /**
     * Establecer el minimo valor del primer eje
     * @param minx
     */
    public void setMinx(double minx) {
        this.minx = minx;
    }
    
    /**
     * Obtener mínimo valor del segundo eje
     * @return
     */
    public double getMiny() {
        return miny;
    }
    
    /**
     * Establecer el minimo valor del segundo eje
     * @param miny
     */
    public void setMiny(double miny) {
        this.miny = miny;
    }
    
    /**
     * Obtener el maximo valor del primer eje
     * @return
     */    
    public double getMaxx() {
        return maxx;
    }
    
    /**
     * Establecer el maximo valor del primer eje
     * @param maxx
     */    
    public void setMaxx(double maxx) {
        this.maxx = maxx;
    }
    
    /**
     * Obtener el maximo valor del segundo eje
     * @return
     */    
    public double getMaxy() {
        return maxy;
    }
    
    /**
     * Establecer el maximo valor del segundo eje
     * @param maxy
     */    
    public void setMaxy(double maxy) {
        this.maxy = maxy;
    }
    
    /**
     * Obtener ancho del área delimitada
     * @return
     */        
    public double getWidth(){
        return maxx-minx;
    }
     
    /**
     * Obtener alto del área delimitada
     * @return
     */        
    public double getHeight(){
        return maxy-miny;
    }
    
    /**
     * El centro geométrico del bounding box
     * @return un punto en el espacio
     */
    public Point2D getCenter(){
        return new Point2D.Double((maxx+minx)/2, (maxy+miny)/2);
    }
    
    /**
     * El bounding box en forma de kvp para ser usado en peticiones
     * @return la parte de una url que corresponde al bounding box
     */
    public String getUrlForm(){
        return "&bbox="+minx+","+miny+","+maxx+","+maxy;
    }
        
    @Override
    public Object clone(){
        BBox obj=null;
        try{
            obj=(BBox)super.clone();
        }catch(CloneNotSupportedException ex){
            ex.printStackTrace();
        }
        return obj;
    }


    @Override
    public boolean equals(Object object) {
        BBox b = (BBox)object;
        return b.getMaxx() == getMaxx() && b.getMinx() == getMinx() && b.getMaxy() == getMaxy() && b.getMiny() == getMiny();
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 37 * hash + (int) (Double.doubleToLongBits(this.minx) ^ (Double.doubleToLongBits(this.minx) >>> 32));
        hash = 37 * hash + (int) (Double.doubleToLongBits(this.miny) ^ (Double.doubleToLongBits(this.miny) >>> 32));
        hash = 37 * hash + (int) (Double.doubleToLongBits(this.maxx) ^ (Double.doubleToLongBits(this.maxx) >>> 32));
        hash = 37 * hash + (int) (Double.doubleToLongBits(this.maxy) ^ (Double.doubleToLongBits(this.maxy) >>> 32));
        return hash;
    }
}
