package org.atlas.clientdata;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

/**
 * Representa un sistema de coordenadas de referencia del EPSG para uso en los componentes de desarrollo
 * @author Mario
 */
public class EpsgCRS {
    
    /**
     * Indica que el sistema tiene unidad lineal que se pueden expresan en terminos de metros
     */
    public static final int UNIT_LINEAR  = 1;
    
    /**
     * Indica que el sistema tiene unidad angular que se pueden expresan en terminos de grados
     */
    public static final int UNIT_ANGULAR = 2;
    
    private int code;
    private String name;
    private String axis1;
    private String axis2;
    private int type;
    private double conversionFactor;
    
    /** Creates a new instance of EpsgCRS */
    public EpsgCRS() {
    }
    
    /**
     * Construye desde el elemento xml que entrega el servidor como respuesta a una petición GetEPSGCodeInfo
     * @param crs
     */
    public EpsgCRS(Element crs) {
        setCode(Integer.valueOf(crs.getAttributeValue("id")));
        setName(crs.getAttributeValue("name"));
        setAxis1(crs.getChild("axis1").getAttributeValue("name"));
        setAxis2(crs.getChild("axis2").getAttributeValue("name"));
        setConversionFactor(Double.parseDouble(crs.getChild("unit").getAttributeValue("factor")));
        if(crs.getChild("unit").getAttributeValue("type").equals("linear")){
            setType(EpsgCRS.UNIT_LINEAR);
        }else{
            setType(EpsgCRS.UNIT_ANGULAR);
        }
    }
    
    /**
     * Retorna el código EPSG del sistema
     * @return
     */
    public int getCode() {
        return code;
    }
    
    /**
     * Establece el código EPSG del sistema 
     * @param code
     */
    public void setCode(int code) {
        this.code = code;
    }
    
    /**
     * Retorna el nombre EPSG del sistema
     * @return
     */
    public String getName() {
        return name;
    }
    
    /**
     * Establece el nombre EPSG del sistema
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * Retorna el nombre del primer eje
     * @return
     */
    public String getAxis1() {
        return axis1;
    }
    
    /**
     * Establece el nombre del primer eje
     * @param axis1
     */
    public void setAxis1(String axis1) {
        this.axis1 = axis1;
    }
    
    /**
     * Retorna el nombre del segundo eje
     * @return
     */
    public String getAxis2() {
        return axis2;
    }
    
    /**
     * Establece el nombre del segundo eje
     * @param axis2
     */
    public void setAxis2(String axis2) {
        this.axis2 = axis2;
    }
    
    /**
     * Retorna un Element producto de una solicitud GetEPSGCodeInfo sobre el servidor indicado
     * @param code
     * @param serviceURL
     * @return
     */
    public static Element getEpsgCRS(String code, URL serviceURL){
        String crsURL = serviceURL.toString() + "?request=GetEPSGCodeInfo&epsg_code=" + code+"&admin_entrace=true";
        System.out.println(crsURL);
        SAXBuilder builder =  new SAXBuilder();
        
        try {
            URL url = new URL(crsURL);
            Document doc = builder.build(url.openStream());
            return doc.detachRootElement();
        } catch (MalformedURLException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (JDOMException ex) {
            ex.printStackTrace();
        }        
        return null;
    }

    /**
     * Retorna el tipo del sistema según las constantes de clase
     * @return
     */
    public int getType() {
        return type;
    }

    /**
     * Establece el tipo del sistema según las constantes de clase
     * @param type
     */
    public void setType(int type) {
        this.type = type;
    }

    /**
     * Retorna el factor por que deben multiplicarse las medidas para convertirlas a la unidad base de sistema
     * @return
     */
    public double getConversionFactor() {
        return conversionFactor;
    }

    /**
     * Establece el factor por que deben multiplicarse las medidas para convertirlas a la unidad base de sistema
     * @param conversionFactor
     */
    public void setConversionFactor(double conversionFactor) {
        this.conversionFactor = conversionFactor;
    }
    
}
