/*
 * Created on 6 de agosto de 2007, 05:40 PM
 */
package org.atlas.clientdata;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.atlas.util.BoundingBox;
import org.jdom.Element;
import org.jdom.Namespace;

/**
 * Representa una capa para su uso en los componentes de desarrollo
 */
public class Layer {

    private String name;
    private String title;
    private String summary;
    private List keywordList = new ArrayList();
    private EpsgCRS CRS;
    private Ex_BoundingBox exBoundingBox;
    private BoundingBox boundigBox;
    private String atribution;
    private double minscaledeno;
    private double maxscaledeno;
    private boolean queryable;
    private Project proyect;

    public Layer() {

    }

    /**
     * Construye un layer desde un elemento XML y lo anexa a un proyecto
     * @param layer
     * @param proyect
     */
    public Layer(Element layer, Project proyect) {
        Namespace ns = layer.getNamespace();
        String iName = layer.getChildText("Name", ns);
        setName(iName);
        setTitle(layer.getChildText("Title", ns));
        setSummary(layer.getChildText("Abstract", ns));

        Iterator<Element> ite = layer.getChild("KeywordList", ns).getChildren().iterator();
        while (ite.hasNext()) {
            getKeywordList().add(ite.next().getText());
        }

        setCRS(proyect.getCrs());
        setEx_BoundingBox(new Ex_BoundingBox(layer.getChild("EX_GeographicBoundingBox", layer.getNamespace())));
        setBoundigBox(new BoundingBox(layer.getChild("BoundingBox", ns)));

        if (layer.getChild("Attribution", ns) != null) {
            setAtribution(layer.getChild("Attribution", ns).getChildText("Title", ns));
        } else {
            setAtribution("");
        }

        if (layer.getChild("MinScaleDenominator", layer.getNamespace()) != null) {
            setMinscaledeno(Double.valueOf(layer.getChildText("MinScaleDenominator", layer.getNamespace())));
        } else {
            setMinscaledeno(Double.NaN);
        }

        if (layer.getChild("MaxScaleDenominator", layer.getNamespace()) != null) {
            setMaxscaledeno(Double.valueOf(layer.getChildText("MaxScaleDenominator", layer.getNamespace())));
        } else {
            setMaxscaledeno(Double.NaN);
        }

        int iq = Integer.parseInt(layer.getAttributeValue("queryable"));
        if(iq==1){
            setQueryable(true);
        }else{
            setQueryable(false);
        }        
        setProyect(proyect);

    }

    /**
     * De conformidad al estandar WMS, retorna descripción breve para mostrar
     * @return 
     */
    public String getTitle() {
        return title;
    }

    /**
     * De conformidad al estandar WMS , establece descripción breve para mostrar
     * @param title 
     */
    public void setTitle(String title) {
        this.title = title;
    }

    /**
     * De conformidad al estandar WMS, resumen de la capa
     * @return 
     */
    public String getSummary() {
        return summary;
    }

    /**
     * De conformidad al estandar WMS, resumen de la capa
     * @param summary 
     */
    public void setSummary(String summary) {
        this.summary = summary;
    }

    /**
     * De conformidad al estandar WMS, palabras clave
     * @return 
     */
    public List getKeywordList() {
        return keywordList;
    }

    /**
     * De conformidad al estandar WMS, palabras clave
     * @param keywordList 
     */
    public void setKeywordList(List keywordList) {
        this.keywordList = keywordList;
    }

    /**
     * Retorna el sistema de coordenadas de referencia para la capa
     * @return 
     */
    public EpsgCRS getCRS() {
        return CRS;
    }

    /**
     * Establece el sistema de coordenadas de referencia para la capa
     * @param CRS 
     */
    public void setCRS(EpsgCRS CRS) {
        this.CRS = CRS;
    }

    /**
     * De conformidad al estandar WMS, bounding box respecto a WGS 84
     * @return 
     */
    public Ex_BoundingBox getEx_BoundingBox() {
        return exBoundingBox;
    }

    /**
     * De conformidad al estandar WMS, bounding box respecto a WGS 84
     * @param exBoundingBox 
     */
    public void setEx_BoundingBox(Ex_BoundingBox exBoundingBox) {
        this.exBoundingBox = exBoundingBox;
    }

    /**
     * De conformidad al estandar WMS, bounding box en el sistema de coordenadas de referencia del proyecto
     * @return 
     */
    public BoundingBox getBoundigBox() {
        return boundigBox;
    }

    /**
     * De conformidad al estandar WMS, bounding box en el sistema de coordenadas de referencia del proyecto
     * @param boundigBox 
     */
    public void setBoundigBox(BoundingBox boundigBox) {
        this.boundigBox = boundigBox;
    }

    /**
     * De conformidad al estandar WMS, reponsable de la capa
     * @return 
     */
    public String getAtribution() {
        return atribution;
    }

    /**
     * De conformidad al estandar WMS, responsable de la capa
     * @param atribution 
     */
    public void setAtribution(String atribution) {
        this.atribution = atribution;
    }

    /**
     * De conformidad al estandar WMS, retorna el denominador mínimo de escala
     * @return 
     */
    public double getMinscaledeno() {
        return minscaledeno;
    }

    /**
     * De conformidad al estandar WMS, establece el denominador mínimo de escala
     * @param minscaledeno 
     */
    public void setMinscaledeno(double minscaledeno) {
        this.minscaledeno = minscaledeno;
    }

    /**
     * De conformidad al estandar WMS, retorna el denominador máximo de escala
     * @return 
     */
    public double getMaxscaledeno() {
        return maxscaledeno;
    }

    /**
     * De conformidad al estandar WMS, establece el denominador máximo de escala
     * @param maxscaledeno 
     */
    public void setMaxscaledeno(double maxscaledeno) {
        this.maxscaledeno = maxscaledeno;
    }

    /**
     * Indica si la capa puede ser consultada por una operación GetFeatureInfo
     * @return 
     */
    public boolean isQueryable() {
        return queryable;
    }

    /**
     * Establece si la capa puede ser consultada por una operación GetFeatureInfo
     * @param queryable 
     */
    public void setQueryable(boolean queryable) {
        this.queryable = queryable;
    }

    /**
     * Retorna el proyecto al que pertenece la capa
     * @return 
     */
    public Project getProyect() {
        return proyect;
    }

    /**
     * Establece el proyecto al que pertenece la capa
     * @param proyect 
     */
    public void setProyect(Project proyect) {
        this.proyect = proyect;
    }

    @Override
    public String toString() {
        return this.getTitle();
    }

    /**
     * De conformidad al estandar WMS, bounding box respecto a WGS 84
     */
    public class Ex_BoundingBox {

        private double westBoundLongitude;
        private double eastBoundLongitude;
        private double southBoundLatitude;
        private double northBoundLatitude;

        public Ex_BoundingBox() {
        }

        public Ex_BoundingBox(Element exBoundingBox) {
            setWestBoundLongitude(Double.valueOf(exBoundingBox.getChildText("westBoundLongitude", exBoundingBox.getNamespace())));
            setEastBoundLongitude(Double.valueOf(exBoundingBox.getChildText("eastBoundLongitude", exBoundingBox.getNamespace())));
            setSouthBoundLatitude(Double.valueOf(exBoundingBox.getChildText("southBoundLatitude", exBoundingBox.getNamespace())));
            setNorthBoundLatitude(Double.valueOf(exBoundingBox.getChildText("northBoundLatitude", exBoundingBox.getNamespace())));
        }

        public double getWestBoundLongitude() {
            return westBoundLongitude;
        }

        public void setWestBoundLongitude(double westBoundLongitude) {
            this.westBoundLongitude = westBoundLongitude;
        }

        public double getEastBoundLongitude() {
            return eastBoundLongitude;
        }

        public void setEastBoundLongitude(double eastBoundLongitude) {
            this.eastBoundLongitude = eastBoundLongitude;
        }

        public double getSouthBoundLatitude() {
            return southBoundLatitude;
        }

        public void setSouthBoundLatitude(double southBoundLatitude) {
            this.southBoundLatitude = southBoundLatitude;
        }

        public double getNorthBoundLatitude() {
            return northBoundLatitude;
        }

        public void setNorthBoundLatitude(double northBoundLatitude) {
            this.northBoundLatitude = northBoundLatitude;
        }
    }

    /**
     * De conformidad al estandar WMS, identificador de la capa
     * @return 
     */
    public String getName() {
        return name;
    }

    /**
     * De conformidad al estandar WMS, identificador de la capa
     * @param name 
     */
    public void setName(String name) {
        this.name = name;
    }
}
