/*
 * Created on 6 de agosto de 2007, 05:26 PM
 */

package org.atlas.clientdata;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.jdom.Element;

/**
 * Representa un proyecto para su uso en los componentes de desarrollo
 */
public class Project {
    
    private String name;
    private String title;
    private EpsgCRS crs;
    private BBox    bbox;
    private List    layers = new ArrayList();
    
    /**
     * Construye un proyecto con los campos vacios
     */
    public Project() {
    }
    
    /**
     * Construye desde un elemento XML de un documento GetCapabilities
     * @param project
     */
    public Project(Element project){
        setName(project.getChildText("Name",project.getNamespace()));
        setTile(project.getChildText("Title",project.getNamespace()));
        setCrs(new EpsgCRS(project.getChild("crs")));
        setBbox(new BBox(project.getChild("BoundingBox",project.getNamespace())));
        
        List list = project.getChildren("Layer",project.getNamespace());
        Iterator<Element> ite = list.iterator();
        while(ite.hasNext()){
            getLayers().add(new Layer(ite.next(),this));
        }
    }
    
    /**
     * De conformidad al estandar WMS, retorna descripción breve para mostrar 
     * @return
     */
    public String getTitle() {
        return title;
    }
    
    /**
     * De conformidad al estandar WMS, establace descripción breve para mostrar
     * @param title
     */
    public void setTile(String title) {
        this.title = title;
    }
    
    
    /**
     * De conformidad al estandar WMS, bounding box en el sistema de coordenadas de referencia del proyecto
     * @return
     */
    public BBox getBbox() {
        return bbox;
    }
    
    /**
     * De conformidad al estandar WMS, bounding box en el sistema de coordenadas de referencia del proyecto
     * @param bbox
     */
    public void setBbox(BBox bbox) {
        this.bbox = bbox;
    }
    
    /**
     * Retorna el listado de capas del proyecto
     * @return
     */
    public List getLayers() {
        return layers;
    }
    
    /**
     * Establece el listado de capas del proyecto
     * @param layers
     */
    public void setLayers(List layers) {
        this.layers = layers;
    }
    
    /**
     * Retorna el sistema de coordenadas de referncia
     * @return
     */
    public EpsgCRS getCrs() {
        return crs;
    }
    
    /**
     * Establece el sistema de coordenadas de referncia 
     * @param crs
     */
    public void setCrs(EpsgCRS crs) {
        this.crs = crs;
    }

    @Override
    public String toString() {
        return this.getTitle();
    }
    
    /**
     * De conformidad al estandar WMS, identificador del proyecto
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     * De conformidad al estandar WMS, identificador del proyecto
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }
    
    
    /**
     * Retorna la capas del proyecto como un array.
     * @return
     */
    public Layer[] getLayerArray(){
        Layer[] lys = new Layer[layers.size()];
        layers.toArray(lys);
        return lys;
    }
}
