package org.atlas.geocoding;

import javax.swing.JDialog;
import javax.swing.JFrame;
import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.GeocoderConfig;
import org.atlas.util.HibernateUtil;

/**
 * Super clase para los dialogos de configuración de geocoders.
 * Es usada por el gestór de plugins en el panel de control.
 * Los métodos set de esta clase nomalmente deben llamarse desde la implementación
 * del método getConfigurationDialog del la clase Geocoder.
 * @author Mario
 */
public abstract class ConfigurationDialog extends JDialog {

    private Project project;
    private GeocoderConfig config;
    private HibernateUtil hibernate;
    private boolean cancelled = true;
    /**
     * Crea un dialogo en blanco, a este punto aún no está listo para ser visible
     */
    public ConfigurationDialog(){
        super((JFrame)null, true);
    }
    
    /**
     * Retorna el proyecto para el que se va a configurar el geocoder.
     * @return
     */
    public Project getProject() {
        return project;
    }

    /**
     * Establece el proyecto para el que se va a configurar el geocoder.
     * @param project
     */
    public void setProject(Project project) {
        this.project = project;
    }

    /**
     * Retorna el objeto de configuración del geocoder para un proyecto en particular
     * @return
     */
    public GeocoderConfig getConfig() {
        return config;
    }

    /**
     * Establece el objeto de configuración del geocoder para un proyecto en particular.
     * @param config
     */
    public void setConfig(GeocoderConfig config) {
        this.config = config;
    }

    /**
     * Retorna el objeto de conexión hibernate
     * @return
     */
    public HibernateUtil getHibernate() {
        return hibernate;
    }

    /**
     * Establece el objeto de conexión hibernate
     * @param hibernate
     */
    public void setHibernate(HibernateUtil hibernate) {
        this.hibernate = hibernate;
    }

    public boolean isCancelled() {
        return cancelled;
    }

    public void setCancelled(boolean cancelled) {
        this.cancelled = cancelled;
    }
}
