package org.atlas.geocoding;

import java.awt.geom.Point2D;
import org.jdom.Attribute;
import org.jdom.Element;

/**
 * Indica una posible ubicación para una dirección en una operación de geocoding.
 * @author Mario
 */
public class Result {

    private String address;
    private Point2D location;
    private String processInfo;

    /**
     * Construye dados una dirección y una ubicación.
     * @param address
     * @param location
     */
    public Result(String address, Point2D location) {
        this.address = address;
        this.location = location;
    }

    /**
     * Retorna el resultado como un elemento XML para se incluido en la respuesta.
     * @return
     */
    public Element getAsElement() {
        Element r = new Element("coordinate");
        r.setAttribute(new Attribute("x", location.getX() + ""));
        r.setAttribute(new Attribute("y", location.getY() + ""));
        r.setAttribute(new Attribute("address", address));
        return r;
    }

    /**
     * Retorna el resultado como un elemento JSON para se incluido en la respuesta. 
     * @return
     */
    public String getAsJSON() {
        StringBuilder res = new StringBuilder("{");
        res.append(" x : ");
        res.append(location.getX());
        res.append(",");
        res.append(" y : ");
        res.append(location.getY());
        res.append(",");
        res.append(" address : \"");
        res.append(address);
        res.append("\",");
        res.append(" getY : function(){ return this.y; }, ");
        res.append(" getX : function(){ return this.x; }, ");
        res.append(" getAddress : function(){ return this.address; }, ");
        res.append(" setX : function(x){ this.x = x; }, ");
        res.append(" setY : function(y){ this.y = y; }, ");
        res.append(" setAddress : function(address){ this.address = address; } ");
        res.append("}");
        return res.toString();
    }

    /**
     * Retorna la descripción textual de la ubicación tal como el geocoder la interpretó. 
     * @return
     */
    public String getAddress() {
        return address;
    }

    /**
     * Establece la descripción textual de la ubicación tal como el geocoder la interpretó. 
     * @param address
     */
    public void setAddress(String address) {
        this.address = address;
    }

    /**
     * Retorna la ubicación en el sistema de coordenadas de referencia del proyecto al que corresponde. 
     * @return 
     */
    public Point2D getLocation() {
        return location;
    }

    /**
     * Establece la ubicación en el sistema de coordenadas de referencia del proyecto al que corresponde. 
     * @param location
     */
    public void setLocation(Point2D location) {
        this.location = location;
    }

    public String getProcessInfo() {
        return processInfo;
    }

    public void setProcessInfo(String processInfo) {
        this.processInfo = processInfo;
    }
}
