/*
 * Created on 28 de mayo de 2007, 06:09 PM
 */

package org.atlas.hibernatedata;

import java.awt.Color;
import org.hibernate.classic.Session;
 
/**
 * Representa un color en formato argb. Soporta persistencia.
 * @author Mario
 */
public class AtlasColor implements Cloneable{
    
    private int codecolo;
    private int red;
    private int green;
    private int blue;
    private int alfa;
          
    /**
     * Construye en color vacio
     */
    public AtlasColor() {
    }
    
    /**
     * Retorna un objeto color de java con el rgb de este objeto
     * @return
     */
    public Color getColor(){
        return new Color(this.getRed(), this.getGreen(), this.getBlue(), this.getAlfa());        
    }

    /**
     * Código del color, para pesistencia.
     * @return
     */
    public int getCodecolo() {
        return codecolo;
    }

    /**
     * Código del color, para pesistencia. 
     * @param codecolo
     */
    public void setCodecolo(int codecolo) {
        this.codecolo = codecolo;
    }

    
    /**
     * La componente roja del color
     * @return
     */
    public int getRed() {
        return red;
    }

    /**
     * La componente roja del color 
     * @param red
     */
    public void setRed(int red) {
        this.red = red;
    }

    /**
     * La componente verde del color
     * @return
     */
    public int getGreen() {
        return green;
    }

    /**
     * La componente verde del color 
     * @param green
     */
    public void setGreen(int green) {
        this.green = green;
    }

    /**
     * La componente azul del color
     * @return
     */
    public int getBlue() {
        return blue;
    }

    /**
     * La componente azul del color 
     * @param blue
     */
    public void setBlue(int blue) {
        this.blue = blue;
    }

    /**
     * La componente alfa del color
     * @return
     */
    public int getAlfa() {
        return alfa;
    }

    /**
     * La componente alfa del color 
     * @param alfa
     */
    public void setAlfa(int alfa) {
        this.alfa = alfa;
    }
    
    
    /**
     * Establece los valores rgba para este color
     * @param r
     * @param g
     * @param b
     * @param a
     */
    public void setRGBA(int r, int g, int b, int a){
        this.setRed(r);
        this.setGreen(g);
        this.setBlue(b);
        this.setAlfa(a);
    }
    
    
    
    /**
     * Establece los valores rgba de este color
     * @param color
     */
    public void setColor(Color color){
        this.red   = color.getRed();
        this.green = color.getGreen();
        this.blue  = color.getBlue();
        this.alfa  = color.getAlpha();
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        AtlasColor color = new AtlasColor();
        color.setAlfa(getAlfa());
        color.setBlue(getBlue());
        color.setGreen(getGreen());
        color.setRed(getRed());
        return color;
    }
    
    /**
     * Guarda este objeto en una sesión de hibernate
     * @param sess
     * @throws java.lang.Exception
     */
    public void save(Session sess) throws Exception{
        sess.save(this);
    }
    
    /**
     * Borra este objeto en una sesión de hibernate
     * @param sess
     * @throws java.lang.Exception
     */
    public void delete(Session sess) throws Exception{
        sess.delete(this);
    }
    
}
