/*
 * AtlasStroke.java
 * Created on 28 de mayo de 2007, 11:24 AM
 */

package org.atlas.hibernatedata;

import java.awt.BasicStroke;
import org.hibernate.classic.Session;

/**
 * Representa un estilo de linea, soporta pesistencia.
 * @author Mario Raúl Eraso
 */

public class AtlasStroke implements Cloneable{
    
    private int codestro;
    private float width;
    private int cap;
    private int join;
    private float miterlimit;
    private String dashString;
    private float[] dash;
    private float dashphase;
    
    /**
     * Crea un estilo de linea sin valores.
     */
    public AtlasStroke() {
    }
    
    /**
     * Retorna un BasicStroke con base en este estilo de linea después de multiplicar su ancho 
     * por el factor indicado
     * @param factor
     * @return
     */
    public BasicStroke getBasicStroke(double factor){
        if(this.getDash().length == 0){
            if(this.getWidth() > 0){
                return new BasicStroke((float)(this.getWidth()*factor));
            }else{
                return null;
            }
        }else{
            return new BasicStroke((float)(this.getWidth()*factor),this.getCap(), this.getJoin(), this.getMiterlimit(), this.getDash(getWidth()), this.getDashphase());
        }
        
    }
    
    /**
     * Retorna un BasicStroke por defecto, de 1 pixel de ancho
     * @return
     */
    public BasicStroke getBasicStroke(){
        return getBasicStroke(1.0d);
    }
    
    /**
     * Establece los atributos del objeto para que iguale al basicStroke indicado
     * @param basicStroke
     */
    public void setBasicStroke(BasicStroke basicStroke){
        this.setWidth(basicStroke.getLineWidth());
        this.setCap(basicStroke.getEndCap());
        this.setJoin(basicStroke.getLineJoin());
        this.setMiterlimit(basicStroke.getMiterLimit());
        this.setDash(basicStroke.getDashArray());
        this.setDashphase(basicStroke.getDashPhase());
    }
    
    /**
     * Código del estilo de linea en el sistema de persistencia 
     * @return
     */
    public int getCodestro() {
        return codestro;
    }
    
    /**
     * Código del estilo de linea en el sistema de persistencia 
     * @param codestro
     */
    public void setCodestro(int codestro) {
        this.codestro = codestro;
    }
    
    /**
     * Ancho del estilo de linea
     * @return
     */
    public float getWidth() {
        return width;
    }
    
    /**
     * Ancho del estilo de linea 
     * @param width
     */
    public void setWidth(float width) {
        this.width = width;
    }
    
    /**
     * Cap del estilo de linea, forma de las terminales
     * @return
     */
    public int getCap() {
        return cap;
    }
    
    /**
     * Cap del estilo de linea, forma de las terminales 
     * @param cap
     */
    public void setCap(int cap) {
        this.cap = cap;
    }
    
    /**
     * Join del estilo de linea, decoración de la intersecciones
     * @return
     */
    public int getJoin() {
        return join;
    }
    
    /**
     * Join del estilo de linea, decoración de la intersecciones
     * @param join
     */
    public void setJoin(int join) {
        this.join = join;
    }
    
    /**
     * Miter limit del estilo de linea
     * @return
     */
    public float getMiterlimit() {
        return miterlimit;
    }
    
    /**
     * Miter limit del estilo de linea 
     * @param miterlimit
     */
    public void setMiterlimit(float miterlimit) {
        this.miterlimit = miterlimit;
    }
    
    /**
     * El patrón de punteado de la linea como cadena, para ser almacenado en la persistencia
     * @return
     */
    public String getDashString() {
        return dashString;
    }
    
    /**
     * El patrón de punteado de la linea como cadena, para ser almacenado en la persistencia 
     * @param dashString
     */
    public void setDashString(String dashString) {
        this.dashString = dashString;
    }
    
    /**
     * El patrón de punteado de la linea como un array obtenido desde la reprentación en cadena
     * @return
     */
    public float[] getDash() {
        if(!getDashString().equals("")){
            String[] arr = this.getDashString().split(";");
            this.dash = new float[arr.length];
            for (int i = 0; i < arr.length; i++) {
                this.dash[i] = Float.parseFloat(arr[i]);
            }
            return dash;
        }else{
            return new float[]{};
        }
    }

    
    public float[] getDash(float delta) {
        if(!getDashString().equals("")){
            String[] arr = this.getDashString().split(";");
            this.dash = new float[arr.length];
            for (int i = 0; i < arr.length; i++) {
                this.dash[i] = Float.parseFloat(arr[i])+ delta;
            }
            return dash;
        }else{
            return new float[]{};
        }
    }
    
    
    /**
     * El patrón de punteado de la linea como un array obtenido desde la reprentación en cadena 
     * @param dash
     */
    public void setDash(float[] dash) {
        this.dash = dash;
        this.setDashString("");
        if(dash != null){
            if(dash.length>0){
                for (int i = 0; i < dash.length; i++) {
                    this.setDashString(this.getDashString() + ";" + dash[i]);
                }
                this.setDashString(this.getDashString().substring(1,this.getDashString().length()));
            }
        }
    }
    
    /**
     * Desplazamiento en el patrón de punteado
     * @return
     */
    public float getDashphase() {
        return dashphase;
    }
    
    /**
     * Desplazamiento en el patrón de punteado 
     * @param dashphase
     */
    public void setDashphase(float dashphase) {
        this.dashphase = dashphase;
    }
    
    @Override
    protected Object clone() throws CloneNotSupportedException {
        AtlasStroke as = new AtlasStroke();
        as.setCap(getCap());
        //TODO Revisar si clona el dash
        //as.setDash(getDash());
        as.setDashString(getDashString());
        as.setJoin(getJoin());
        as.setMiterlimit(getMiterlimit());
        as.setWidth(getWidth());
        return as;
    }
    
    /**
     * Envia este objeto al almacenamiento persistente
     * @param sess
     * @throws java.lang.Exception
     */
    public void save(Session sess) throws Exception{
        sess.save(this);
    }
    
    /**
     * Remueve este objeto del almacenamiento persistente 
     * @param sess
     * @throws java.lang.Exception
     */
    public void delete(Session sess) throws Exception{
        sess.delete(this);
    }
    
}
