package org.atlas.hibernatedata;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import org.atlas.svg.SimpleSvgReader;

/**
 * Representa un icono para ser aplicado sobre una geometría tipo punto
 * @author Mario
 */
public class Icon {

    /**
     * Ancho inicial por defecto para los iconos en el sistema
     */
    public static int DEFAULT_SIZE = 10;
    private int codeicon;
    private String descicon;
    private byte[] binaicon;
    private String typeicon;
    private BufferedImage thumb;
    private static final int gap = 8;
    private static final int left = 16;
    private static final int right = 16;
    private static final int top = 16;
    private static final int bottom = 32;
    private static final int ancho = 56;
    private static final int alto = 56;
    public static final int THUMB_WIDTH = left + right + ancho + (gap * 2);
    public static final int THUMB_HEIGHT = top + bottom + alto + (gap * 2);

    /**
     * Crea un objeto con campos vacios
     */
    public Icon() {

    }

    /**
     * Crea un icono con los parametros dados. Binary se refiere al contenido en bytes
     * del documento SVG que define el icono.
     * @param binary
     * @param mime
     * @param description
     * @throws java.lang.Exception
     */
    public Icon(byte[] binary, String mime, String description) throws Exception {
        this.binaicon = binary;
        this.typeicon = mime;
        this.descicon = description;
        createThumb();
    }

    /**
     * Código del objeto en el sistema de persistencia 
     * @return
     */
    public int getCodeicon() {
        return codeicon;
    }

    /**
     * Código del objeto en el sistema de persistencia 
     * @param codeicon
     */
    public void setCodeicon(int codeicon) {
        this.codeicon = codeicon;
    }

    /**
     * Descripción breve del icono
     * @return
     */
    public String getDescicon() {
        return descicon;
    }

    /**
     * Descripción breve del icono 
     * @param descicon
     */
    public void setDescicon(String descicon) {
        this.descicon = descicon;
    }

    /**
     * Contenido binario del icono, normalmente de la cadena del documento SVG que lo
     * representa.
     * @return
     */
    public byte[] getBinaicon() {
        return binaicon;
    }

    /**
     * Contenido binario del icono, normalmente de la cadena del documento SVG que lo
     * representa.
     * @param binaicon
     */
    public void setBinaicon(byte[] binaicon) {
        this.binaicon = binaicon;
    }

    /**
     * Tipo mime del icono
     * @return
     */
    public String getTypeicon() {
        return typeicon;
    }

    /**
     * Tipo mime del icono 
     * @param typeicon
     */
    public void setTypeicon(String typeicon) {
        this.typeicon = typeicon;
    }

    /**
     * Vista en miniatura del icono
     * @return
     * @throws java.lang.Exception
     */
    public BufferedImage getThumb() throws Exception {
        return thumb;
    }

    /**
     * Crea la vista en miniatura del icono, debe llamarse antes de llamar a getThumb
     * @throws java.lang.Exception
     */
    public void createThumb() throws Exception {

        thumb = new BufferedImage(Icon.THUMB_WIDTH, Icon.THUMB_HEIGHT, BufferedImage.TYPE_INT_ARGB);
        Graphics g = thumb.getGraphics();

        g.setColor(new Color(183, 183, 183));
        g.fillRect(left, top, ancho + (2 * gap), alto + (2 * gap));
        g.setColor(new Color(132, 132, 132));

        for (int i = 0; i < (ancho + (2 * gap)) / 8; i++) {
            for (int j = 0; j < (alto + (2 * gap)) / 8; j++) {
                if ((i % 2 == 0 && j % 2 == 0) || (i % 2 != 0 && j % 2 != 0)) {
                    g.fillRect((i * 8) + left, (j * 8) + top, 8, 8);
                }
            }
        }

        BufferedImage icon;
        if (this.typeicon.equals("image/svg")) {
            icon = this.getIcon(Icon.ancho, Icon.alto);
            int x = left + (ancho - icon.getWidth()) / 2;
            int y = top + (alto - icon.getHeight()) / 2;
            g.setColor(new Color(0, 0, 0));
            g.drawImage(icon, x + gap, y + gap, null);

            String txt = descicon;
            if (g.getFontMetrics().getStringBounds(getDescicon(), g).getWidth() > ancho + (2 * gap)) {
                do {
                    txt = txt.substring(0, txt.length() - 1);
                } while (g.getFontMetrics().getStringBounds(txt + "...", g).getWidth() > ancho + (2 * gap));
                txt = txt + "...";
            }
            g.drawString(txt, left, gap + gap + top + alto + (g.getFontMetrics().getHeight() * 1) + 3);
        }
    }

    /**
     * 
     * @param maxWidth
     * @param maxHeight
     * @return
     * @throws java.lang.Exception
     */
    public BufferedImage getIcon(int maxWidth, int maxHeight) throws Exception {
        SimpleSvgReader reader = new SimpleSvgReader(new ByteArrayInputStream(this.binaicon));
        return reader.drawGraphics(true, null, null, maxWidth, maxHeight);
    }
}
