/*
 * Created on 27 de mayo de 2007, 11:48 AM
 */
package org.atlas.hibernatedata;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import org.atlas.service.Feature;
import org.geotools.geometry.GeneralDirectPosition;
import org.hibernate.classic.Session;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.opengis.spatialschema.geometry.DirectPosition;
import org.atlas.util.CoordinateUtils;
import org.atlas.util.HibernateUtil;

/**
 * Una capa del sistema, es empleada por el panel de control y el servidor, soporta persistencia y 
 * los campos requeridos por el estándar WMS
 * @author Mario
 */
public class Layer implements Comparable, Cloneable {

    private int codelaye;        //Name
    private String desclaye;        //Titile
    private String summary;          //Abstract
    private String keywordList;     //KeywordList
    private String atribution;      //Atribution
    private double minscaledeno;    //MinScaleDenominador
    private double maxscaledeno;    //MaxScaleDenominador
    private boolean queryable;      //Queryable
    private int zorder;
    private Project project;
    private String[] labelValues = new String[0];
    private String[] classValues = new String[0];
    private double[] rangeValues = new double[0];
    private Symbology symbology;
    private Source source;
    private byte[] binalegend;

    /**
     * Contruye un objeto sin valores
     */
    public Layer() {
    }

    /**
     * Retorna una imagen con la leyenda de la capa
     * @return
     */
    private void generateLegend() {
        try {
            if (getSource() != null) {
                BufferedImage legend = null;

                int space = 5;
                int tamx = 30;
                int width = space + tamx + space + 160 + space; //space icon space string space
                int height;
                int py = 0;

                if (getSymbology().getRules().size() > 0) {

                    Rule[] rules = new Rule[getSymbology().getRules().size()];
                    getSymbology().getRules().toArray(rules);
                    Arrays.sort(rules);

                    height = ((space + tamx) * rules.length);
                    legend = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
                    Graphics2D gra = (Graphics2D) legend.getGraphics();
                    gra.setColor(Color.white); //Color.white
                    gra.fillRect(0, 0, width, height);
                    gra.setColor(Color.black);

                    if (getSource().getGeomClass() == Point.class || getSource().getGeomClass() == MultiPoint.class) {
                        for (int i = 0; i < rules.length; i++) {
                            Rule rule = rules[i];
                            int cx = space + (tamx / 2) - (rule.getStyle().getIconImage().getWidth() / 2);
                            int cy = py + ((tamx + space) / 2) - (rule.getStyle().getIconImage().getHeight() / 2);
                            int sy = py + ((tamx + space) / 2) + (int) (gra.getFontMetrics().getLineMetrics(rule.getDescrule(), gra).getHeight() / 4);
                            gra.drawImage(rule.getStyle().getIconImage(), cx, cy, null);
                            gra.drawString(rule.getDescrule(), ((space * 2) + tamx), sy);
                            py = (i + 1) * (tamx + space);
                        }
                    }

                    if (getSource().getGeomClass() == LineString.class || getSource().getGeomClass() == MultiLineString.class) {
                        for (int i = 0; i < rules.length; i++) {
                            Rule rule = rules[i];
                            gra.setColor(Color.black);
                            gra.setStroke(new BasicStroke());
                            gra.drawRect(space, (space / 2) + py, tamx, tamx);

                            int cy = py + ((space + tamx) / 2);
                            int sy = py + ((space + tamx) / 2) + (int) (gra.getFontMetrics().getLineMetrics(rule.getDescrule(), gra).getHeight() / 4);
                            gra.drawString(rule.getDescrule(), ((space * 2) + tamx), sy);

                            if (rule.getStyle().getLinestyl().getBasicStroke(1) != null) {
                                gra.setColor(rule.getStyle().getFillcolo().getColor());
                                gra.setStroke(rule.getStyle().getLinestyl().getBasicStroke(1.7));
                                gra.drawLine(space + 3, cy, tamx + space - 1, cy);
                            }
                            py = (i + 1) * (tamx + space);
                        }
                    }

                    if (getSource().getGeomClass() == Polygon.class || getSource().getGeomClass() == MultiPolygon.class) {
                        for (int i = 0; i < rules.length; i++) {
                            Rule rule = rules[i];
                            gra.setColor(rule.getStyle().getFillcolo().getColor());
                            gra.fillRect(space, (space / 2) + py, tamx, tamx);

                            if (rule.getStyle().getLinestyl().getBasicStroke(1) != null) {
                                gra.setColor(rule.getStyle().getLinecolo().getColor());
                                gra.setStroke(rule.getStyle().getLinestyl().getBasicStroke(1.7));
                                gra.drawRect(space, (space / 2) + py, tamx, tamx);
                            }

                            int cy = py + ((space + tamx) / 2);
                            int sy = py + ((space + tamx) / 2) + (int) (gra.getFontMetrics().getLineMetrics(rule.getDescrule(), gra).getHeight() / 4);
                            gra.setColor(Color.black);
                            gra.setStroke(new BasicStroke());
                            gra.drawString(rule.getDescrule(), ((space * 2) + tamx), sy);
                            py = (i + 1) * (tamx + space);
                        }
                    }

                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    ImageIO.write(legend, "png", baos);
                    baos.close();
                    this.setBinalegend(baos.toByteArray());
                }
            }
        } catch (IOException ex) {
            Logger.getLogger(Layer.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Área que ocupa la capa según su origen de datos.
     * @return
     */
    public double getArea() {
        return (this.getXmax() - this.getXmin()) * (this.getYmax() - this.getYmin());
    }

    /**
     * Código del objeto en el sistema de persistencia  
     * @return
     */
    public int getCodelaye() {
        return codelaye;
    }

    /**
     * Código del objeto en el sistema de persistencia   
     * @param codelaye
     */
    public void setCodelaye(int codelaye) {
        this.codelaye = codelaye;
    }

    /**
     * De conformidad al estandar WMS, descripción breve para mostrar
     * @return
     */
    public String getDesclaye() {
        return desclaye;
    }

    /**
     * De conformidad al estandar WMS, descripción breve para mostrar
     * @param desclaye
     */
    public void setDesclaye(String desclaye) {
        this.desclaye = desclaye;
    }

    /**
     * Del bounding box del origen de datos de la capa
     * @return
     */
    public double getXmin() {
        return this.source.getXmin();
    }

    /**
     * Del bounding box del origen de datos de la capa 
     * @return
     */
    public double getXmax() {
        return this.source.getXmax();
    }

    /**
     * Del bounding box del origen de datos de la capa 
     * @return
     */
    public double getYmin() {
        return this.source.getYmin();
    }

    /**
     * Del bounding box del origen de datos de la capa  
     * @return
     */
    public double getYmax() {
        return this.source.getYmax();
    }

    /**
     * Indicador del orden de la capa dentro del proyecto
     * @return
     */
    public int getZorder() {
        return zorder;
    }

    /**
     * Indicador del orden de la capa dentro del proyecto 
     * @param zorder
     */
    public void setZorder(int zorder) {
        this.zorder = zorder;
    }

    /**
     * Proyecto al que pertenece la capa
     * @return
     */
    public Project getProject() {
        return project;
    }

    /**
     * Proyecto al que pertenece la capa 
     * @param project
     */
    public void setProject(Project project) {
        this.project = project;
    }

    /**
     * De conformidad al estandar WMS, resumen de la capa
     * @return
     */
    public String getSummary() {
        return summary;
    }

    /**
     * De conformidad al estandar WMS, resumen de la capa 
     * @param summary
     */
    public void setSummary(String summary) {
        this.summary = summary;
    }

    /**
     * De conformidad al estandar WMS, palabras clave 
     * @return
     */
    public String getKeywordList() {
        return keywordList;
    }

    /**
     * De conformidad al estandar WMS, palabras clave 
     * @param keywordList
     */
    public void setKeywordList(String keywordList) {
        this.keywordList = keywordList;
    }

    /**
     * De conformidad al estandar WMS, reponsable de la capa 
     * @return
     */
    public String getAtribution() {
        return atribution;
    }

    /**
     * De conformidad al estandar WMS, reponsable de la capa
     * @param atribution
     */
    public void setAtribution(String atribution) {
        this.atribution = atribution;
    }

    /**
     * De conformidad al estandar WMS, denominador mínimo de escala 
     * @return
     */
    public double getMinscaledeno() {
        return minscaledeno;
    }

    /**
     * De conformidad al estandar WMS, denominador mínimo de escala 
     * @param minscaledeno
     */
    public void setMinscaledeno(double minscaledeno) {
        this.minscaledeno = minscaledeno;
    }

    /**
     * De conformidad al estandar WMS, denominador mínimo de escala 
     * @return
     */
    public double getMaxscaledeno() {
        return maxscaledeno;
    }

    /**
     * De conformidad al estandar WMS, denominador máximo de escala  
     * @param maxscaledeno
     */
    public void setMaxscaledeno(double maxscaledeno) {
        this.maxscaledeno = maxscaledeno;
    }

    /**
     * Indica si la capa puede ser consultada por una operación GetFeatureInfo 
     * @return
     */
    public boolean isQueryable() {
        return queryable;
    }

    /**
     * Indica si la capa puede ser consultada por una operación GetFeatureInfo 
     * @param queryable
     */
    public void setQueryable(boolean queryable) {
        this.queryable = queryable;
    }

    /**
     * De conformidad al estandar WMS, identificador de la capa 
     * @return
     */
    public String getNameLaye() {
        return "l" + getCodelaye();
    }

    @Override
    public String toString() {
        return this.getDesclaye();
    }

    public int compareTo(Object o) {
        Integer i = new Integer(this.zorder);
        return i.compareTo(new Integer(((Layer) o).getZorder()));
    }

    /**
     * Carga llama al metodo de cargar geometrias del source y carga los valores
     * para simbologias de clase y rango si es necesario.
     * @param con
     * @throws java.lang.Exception
     */
    public void loadFeatures(Connection con) throws Exception {
        if (this.getSource() != null) {
            source.loadFeatures(con);
            Statement sta = null;
            ResultSet res = null;

            String ruleFieldName = getSymbology().getRuleFieldName();
            String labelFieldName = getSymbology().getLabelFieldName();

            if (!labelFieldName.equals("") && getSymbology().isLabeled()) {
                try {
                    String query = "SELECT count(*) as c FROM \"" + source.getTablsour() + "\"";
                    sta =
                            con.createStatement();
                    res =
                            sta.executeQuery(query);
                    res.next();
                    int capa = res.getInt("c");

                    query =
                            "SELECT atlas27_id ";
                    query +=
                            (", " + labelFieldName);
                    query +=
                            (" FROM \"" + source.getTablsour() + "\"");

                    sta =
                            con.createStatement();
                    res =
                            sta.executeQuery(query);

                    setLabelValues(new String[capa]);
                    for (int i = 0; res.next(); i++) {
                        getLabelValues()[i] = res.getString(labelFieldName);
                    }

                } catch (Exception ex) {
                    ex.printStackTrace();
                    throw ex;
                } finally {
                    try {
                        sta.close();
                        res.close();
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }

                }
            }

            if (!ruleFieldName.equals("")) {

                try {

                    String query = "SELECT count(*) as c FROM \"" + source.getTablsour() + "\"";
                    sta =
                            con.createStatement();
                    res =
                            sta.executeQuery(query);
                    res.next();
                    int capa = res.getInt("c");


                    query =
                            "SELECT atlas27_id ";
                    query +=
                            (", " + this.getSymbology().getRuleFieldName());
                    query +=
                            (" FROM \"" + source.getTablsour() + "\"");

                    sta =
                            con.createStatement();
                    res =
                            sta.executeQuery(query);


                    switch (this.getSymbology().getTypeSymb()) {
                        case Symbology.TYPE_CLASS_SYMB:
                            classValues = new String[capa];
                            for (int i = 0; res.next(); i++) {
                                classValues[i] = res.getString(ruleFieldName);
                            }

                            break;
                        case Symbology.TYPE_RANGE_SYMB:
                            rangeValues = new double[capa];
                            for (int i = 0; res.next(); i++) {
                                rangeValues[i] = res.getDouble(ruleFieldName);
                            }

                            break;
                    }

                } catch (Exception ex) {
                    ex.printStackTrace();
                    throw ex;
                } finally {
                    try {
                        sta.close();
                        res.close();
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }

                }
            }
        }
    }

    /**
     * Remueve este objeto del almacenamiento persistente
     * @param hibernate
     * @throws java.lang.Exception
     */
    public void remove(HibernateUtil hibernate) throws Exception {
        Session sess;
        sess =
                hibernate.begin();
        sess.update(this);
        if (getSymbology() != null) {
            getSymbology().delete(sess);
            sess.update(project);
        }

        project.getLayers().remove(this);
        sess.delete(this);
        hibernate.commitAndClose(sess);
    }

    /**
     * Retorna un elemento XML que describe la capa en la version WMS indicada.
     * @param schema
     * @param version
     * @return
     */
    public Element getXMLCapability(
            InputStream schema, int version) {

        Document doc = null;
        SAXBuilder builder = new SAXBuilder();

        try {
            doc = builder.build(schema);
            Element layer = doc.getRootElement();
            Namespace ns = layer.getNamespace();

            if (this.isQueryable()) {
                layer.setAttribute("queryable", "1");
            } else {
                layer.setAttribute("queryable", "0");
            }

            layer.getChild("Name", ns).setText(getNameLaye());
            layer.getChild("Title", ns).setText(this.getDesclaye());

            layer.getChild("Abstract", ns).setText(this.getSummary());

            String CRS = "EPSG:" + this.getProject().getSpatial_ref_sys().getSrid();
            String keywords[] = this.getKeywordList().split(",");
            for (int i = 0; i <
                    keywords.length; i++) {
                layer.getChild("KeywordList", ns).addContent(new Element("Keyword", ns).setText(keywords[i]));
            }

            String srText = getProject().getSpatial_ref_sys().getSrtext();

            DirectPosition minPosition = CoordinateUtils.tranformsCoordinatetoWGS84(srText, new GeneralDirectPosition(this.getXmin(), this.getYmin()));
            DirectPosition maxPosition = CoordinateUtils.tranformsCoordinatetoWGS84(srText, new GeneralDirectPosition(this.getXmax(), this.getYmax()));

            if (!this.getAtribution().equals("")) {
                layer.getChild("Attribution", ns).getChild("Title", ns).setText(this.getAtribution());
            } else {
                layer.removeChildren("Attribution", ns);
            }

            layer.getChild("BoundingBox", ns).setAttribute("minx", String.valueOf(this.getXmin()));
            layer.getChild("BoundingBox", ns).setAttribute("miny", String.valueOf(this.getYmin()));
            layer.getChild("BoundingBox", ns).setAttribute("maxx", String.valueOf(this.getXmax()));
            layer.getChild("BoundingBox", ns).setAttribute("maxy", String.valueOf(this.getYmax()));

            switch (version) {
                case Service.VERSION_1_1_0:
                    layer.getChild("SRS", ns).setText(CRS);
                    layer.getChild("LatLonBoundingBox", ns).setAttribute("minx", String.valueOf(minPosition.getOrdinate(0)));
                    layer.getChild("LatLonBoundingBox", ns).setAttribute("miny", String.valueOf(minPosition.getOrdinate(1)));
                    layer.getChild("LatLonBoundingBox", ns).setAttribute("maxx", String.valueOf(maxPosition.getOrdinate(0)));
                    layer.getChild("LatLonBoundingBox", ns).setAttribute("maxy", String.valueOf(maxPosition.getOrdinate(1)));
                    layer.getChild("BoundingBox", ns).setAttribute("SRS", CRS);
                    break;
                case Service.VERSION_1_1_1:
                    layer.getChild("SRS", ns).setText(CRS);
                    layer.getChild("LatLonBoundingBox", ns).setAttribute("minx", String.valueOf(minPosition.getOrdinate(0)));
                    layer.getChild("LatLonBoundingBox", ns).setAttribute("miny", String.valueOf(minPosition.getOrdinate(1)));
                    layer.getChild("LatLonBoundingBox", ns).setAttribute("maxx", String.valueOf(maxPosition.getOrdinate(0)));
                    layer.getChild("LatLonBoundingBox", ns).setAttribute("maxy", String.valueOf(maxPosition.getOrdinate(1)));
                    layer.getChild("BoundingBox", ns).setAttribute("SRS", CRS);
                    break;
                case Service.VERSION_1_3_0:
                    layer.getChild("CRS", ns).setText(CRS);
                    // La teoria dice que retorna lan/Log la libreria retorna Log/Lat
                    layer.getChild("EX_GeographicBoundingBox", ns).getChild("westBoundLongitude", ns).setText(String.valueOf(minPosition.getOrdinate(0)));
                    layer.getChild("EX_GeographicBoundingBox", ns).getChild("eastBoundLongitude", ns).setText(String.valueOf(maxPosition.getOrdinate(0)));

                    layer.getChild("EX_GeographicBoundingBox", ns).getChild("southBoundLatitude", ns).setText(String.valueOf(minPosition.getOrdinate(1)));
                    layer.getChild("EX_GeographicBoundingBox", ns).getChild("northBoundLatitude", ns).setText(String.valueOf(maxPosition.getOrdinate(1)));

                    layer.getChild("BoundingBox", ns).setAttribute("CRS", CRS);

                    if (this.getMinscaledeno() > 0) {
                        layer.getChild("MinScaleDenominator", ns).setText(String.valueOf(this.getMinscaledeno()));
                    } else {
                        layer.removeChildren("MinScaleDenominator", ns);
                    }

                    if (this.getMaxscaledeno() > 0) {
                        layer.getChild("MaxScaleDenominator", ns).setText(String.valueOf(this.getMaxscaledeno()));
                    } else {
                        layer.removeChildren("MaxScaleDenominator", ns);
                    }

                    break;
            }

            return doc.detachRootElement();
        } catch (JDOMException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        }

        return null;
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        Object retValue;
        retValue =
                super.clone();
        return retValue;
    }

    /**
     * Entrega una respuesta featureInfo que contiene los features indicados en el formato
     * solicitado
     * @param conn
     * @param features
     * @param INFO_FORMAT
     * @param crs
     * @return
     */
    public StringBuilder getFormatFeatureInfo(
            Connection conn, ArrayList features, String INFO_FORMAT, int crs) { //TODO crs capturar el de la capa

        try {
            String[][] res = getFeaturesInfo(conn, features);
            StringBuilder format = new StringBuilder();

            if (INFO_FORMAT.toLowerCase().equals("text/plain")) {
                if (this.isQueryable()) {

                    format.append(nl());
                    format.append("Layer: ");
                    format.append(this.getDesclaye() + nl() + nl());

                    int h = res.length;
                    int v = res[0].length;

                    for (int j = 0; j <
                            h; j++) {

                        for (int k = 0; k <
                                v; k++) {
                            format.append(res[j][k] + tab());
                        }

                        format.append(nl());
                    }

                    format.append(nl());
                } else {
                    format.append(nl() + "Layer: " + this.getDesclaye() + " not Queryable" + nl() + nl());
                }

            }

            if (INFO_FORMAT.toLowerCase().equals("text/html")) {
                format.append("<table style='margin-top:20px;' cellpadding='5'border='2'>");
                format.append("<tr><td bgcolor='#4977a7' align='center' colspan ='" + res[0].length + "' ><font color='#ffffff'><b>LAYER " + this.getDesclaye().toUpperCase() + "</b></font></td></tr>");

                if (this.isQueryable()) {
                    if (features.size() > 0) {
                        for (int j = 0; j <
                                res.length; j++) {
                            format.append("<tr>");
                            for (int k = 0; k <
                                    res[0].length; k++) {
                                if (!res[j][k].trim().equals("")) {
                                    if (j == 0) {
                                        format.append("<td align='center'><b>" + res[j][k] + "<b></td>");
                                    } else {
                                        if (j % 2 != 0) {
                                            format.append("<td bgcolor='#e6e4e4' align='right'>" + res[j][k] + "</td>");
                                        } else {
                                            format.append("<td align='right'>" + res[j][k] + "</td>");
                                        }

                                    }
                                } else {
                                    if (j % 2 != 0) {
                                        format.append("<td bgcolor='#e6e4e4'>&nbsp;</td>");
                                    } else {
                                        format.append("<td>&nbsp;</td>");
                                    }

                                }
                            }
                            format.append("</tr>");
                        }

                    } else {
                        format.append("<tr><td align='center'>not features found</td></tr>");
                    }

                } else {
                    format.append("<tr><td align='center'><font color='#e91e1e'>not queryable</font></td></tr>");
                }

                format.append("</table>");
            }

            if (INFO_FORMAT.toLowerCase().equals("application/vnd.ogc.gml")) {
                String desLayer = this.getDesclaye().replace(" ", "_");
                if (this.isQueryable()) {

                    if (features.size() > 0) {

                        String[] names = res[0];

                        for (int j = 1; j <
                                res.length; j++) {

                            format.append(nl(1) + "<featureMember>");

                            format.append(nl(2) + "<atlas:" + desLayer + " id='" + ((Feature) features.get(j - 1)).getId() + "'>");
                            format.append(nl(3) + "<the_geom>");

                            //Puntos

                            if (getSource().getGeomClass() == Point.class) {
                                Point point = (Point) ((Feature) features.get(j - 1)).getGeom();

                                format.append(nl(
                                        4) + "<Point srsName='EPSG:" + crs + "'>");
                                format.append(nl(5) + "<coordinates decimal='.' cs=',' ts=' '>" + nl(6));
                                format.append(point.getX() + "," + point.getY());
                                format.append(nl(5) + "</coordinates>");
                                format.append(nl(4) + "</Point>");
                            }

                            if (getSource().getGeomClass() == MultiPoint.class) {
                                MultiPoint multi = (MultiPoint) ((Feature) features.get(j - 1)).getGeom();

                                format.append(nl(
                                        4) + "<MultiPoint>");
                                format.append(nl(5) + "<pointMember>");


                                for (int i = 0;
                                        i < multi.getNumGeometries();
                                        i++) {
                                    Point point = (Point) multi.getGeometryN(i);
                                    format.append(nl(6) + "<Point srsName='EPSG:" + crs + "'>");
                                    format.append(nl(7) + "<coordinates decimal='.' cs=',' ts=' '>" + nl(8));
                                    format.append(point.getX() + "," + point.getY());
                                    format.append(nl(7) + "</coordinates>");
                                    format.append(nl(6) + "</Point>");
                                }

                                format.append(nl(
                                        5) + "</pointMember>");
                                format.append(nl(4) + "</MultiPoint>");
                            }


                            //Lineas
                            if (getSource().getGeomClass() == LineString.class) {
                                format.append(nl(4) + "<LineString srsName='EPSG:" + crs + "'>");
                                LineString line = (LineString) ((Feature) features.get(j - 1)).getGeom();

                                format.append(nl(5) + "<coordinates decimal='.' cs=',' ts=' '>" + nl(6));
                                int k;

                                for (k = 0;
                                        k < line.getNumPoints() - 1; k++) {
                                    format.append(line.getPointN(k).getX() + "," + line.getPointN(k).getY() + " ");
                                }

                                format.append(line.getPointN(k).getX() + "," + line.getPointN(k).getY());
                                format.append(nl(5) + "</coordinates>");
                                format.append(nl(4) + "</LineString>");
                            }

                            if (getSource().getGeomClass() == MultiLineString.class) {
                                MultiLineString multi = (MultiLineString) ((Feature) features.get(j - 1)).getGeom();

                                format.append(nl(
                                        4) + "<MultiLineString>");
                                format.append(nl(5) + "<lineStringMember>");


                                for (int i = 0;
                                        i < multi.getNumGeometries();
                                        i++) {
                                    format.append(nl(6) + "<LineString srsName='EPSG:" + crs + "'>");
                                    LineString line = (LineString) multi.getGeometryN(i);

                                    format.append(nl(7) + "<coordinates decimal='.' cs=',' ts=' '>" + nl(8));
                                    int k;
                                    for (k = 0; k < line.getNumPoints() - 1; k++) {
                                        format.append(line.getPointN(k).getX() + "," + line.getPointN(k).getY() + " ");
                                    }
                                    format.append(line.getPointN(k).getX() + "," + line.getPointN(k).getY());
                                    format.append(nl(7) + "</coordinates>");

                                    format.append(nl(6) + "</LineString>");
                                }

                                format.append(nl(
                                        5) + "</lineStringMember>");
                                format.append(nl(4) + "</MultiLineString>");
                            }

                            //Polygonos
                            if (getSource().getGeomClass() == Polygon.class) {
                                format.append(nl(4) + "<Polygon srsName='EPSG:" + crs + "'>");
                                Polygon poly = (Polygon) ((Feature) features.get(j - 1)).getGeom();
                                int k;

                                format.append(nl(
                                        5) + "<outerBoundaryls>");
                                LineString extRing = poly.getExteriorRing();
                                format.append(nl(6) + "<LinearRing>");
                                format.append(nl(7) + "<coordinates decimal='.' cs=',' ts=' '>" + nl(8));

                                for (k = 0;
                                        k < extRing.getNumPoints() - 1; k++) {
                                    format.append(extRing.getPointN(k).getX() + "," + extRing.getPointN(k).getY() + " ");
                                }

                                format.append(extRing.getPointN(k).getX() + "," + extRing.getPointN(k).getY());
                                format.append(nl(7) + "</coordinates>");
                                format.append(nl(6) + "</LinearRing>");
                                format.append(nl(5) + "</outerBoundaryls>");




                                if (poly.getNumInteriorRing() > 0) {
                                    for (k = 0; k < poly.getNumInteriorRing(); k++) {
                                        format.append(nl(5) + "<innerBoundaryls>");
                                        LineString intRing = poly.getInteriorRingN(k);
                                        format.append(nl(6) + "<LinearRing>");
                                        format.append(nl(7) + "<coordinates decimal='.' cs=',' ts=' '>" + nl(8));
                                        int l;
                                        for (l = 0; l < intRing.getNumPoints() - 1; l++) {
                                            format.append(intRing.getPointN(l).getX() + "," + intRing.getPointN(l).getY() + " ");
                                        }
                                        format.append(intRing.getPointN(l).getX() + "," + intRing.getPointN(l).getY());
                                        format.append(nl(7) + "</coordinates>");
                                        format.append(nl(6) + "</LinearRing>");
                                        format.append(nl(5) + "</innerBoundaryls>");
                                    }
                                }

                                format.append(nl(4) + "</Polygon>");
                            }

                            if (getSource().getGeomClass() == MultiPolygon.class) {
                                MultiPolygon multi = (MultiPolygon) ((Feature) features.get(j - 1)).getGeom();

                                format.append(nl(
                                        4) + "<Multipolygon>");
                                format.append(nl(5) + "<polygonMember>");


                                for (int i = 0;
                                        i < multi.getNumGeometries();
                                        i++) {
                                    format.append(nl(6) + "<Polygon srsName='EPSG:" + crs + "'>");
                                    Polygon poly = (Polygon) multi.getGeometryN(i);

                                    int k;
                                    format.append(nl(7) + "<outerBoundaryls>");
                                    LineString extRing = poly.getExteriorRing();
                                    format.append(nl(8) + "<LinearRing>");
                                    format.append(nl(9) + "<coordinates decimal='.' cs=',' ts=' '>" + nl(10));

                                    for (k = 0; k < extRing.getNumPoints() - 1; k++) {
                                        format.append(extRing.getPointN(k).getX() + "," + extRing.getPointN(k).getY() + " ");
                                    }
                                    format.append(extRing.getPointN(k).getX() + "," + extRing.getPointN(k).getY());
                                    format.append(nl(9) + "</coordinates>");
                                    format.append(nl(8) + "</LinearRing>");
                                    format.append(nl(7) + "</outerBoundaryls>");

                                    if (poly.getNumInteriorRing() > 0) {
                                        for (k = 0; k < poly.getNumInteriorRing(); k++) {
                                            format.append(nl(7) + "<inBoundaryls>");
                                            LineString intRing = poly.getInteriorRingN(k);
                                            format.append(nl(8) + "<LinearRing>");
                                            format.append(nl(9) + "<coordinates decimal='.' cs=',' ts=' '>" + nl(10));
                                            int l;
                                            for (l = 0; l < intRing.getNumPoints() - 1; l++) {
                                                format.append(intRing.getPointN(l).getX() + "," + intRing.getPointN(l).getY() + " ");
                                            }
                                            format.append(intRing.getPointN(l).getX() + "," + intRing.getPointN(l).getY());
                                            format.append(nl(9) + "</coordinates>");
                                            format.append(nl(8) + "</LinearRing>");
                                            format.append(nl(7) + "</inBoundaryls>");
                                        }
                                    }
                                    format.append(nl(6) + "</Polygon>");
                                }

                                format.append(nl(
                                        5) + "</polygonMember>");
                                format.append(nl(4) + "</Multipolygon>");
                            }

                            format.append(nl(3) + "</the_geom>");

                            for (int k = 0; k <
                                    res[j].length; k++) {
                                format.append(nl(3) + "<atlas:" + names[k] + ">" + res[j][k] + "</atlas:" + names[k] + ">");
                            }

                            format.append(nl(2) + "</atlas:" + desLayer + ">");
                            format.append(nl(1) + "</featureMember>");
                        }

                    } else {
                        format.append(nl(1) + "<Message>" + nl(2) + "Layer " + desLayer + " not features found" + nl(1) + "</Message>");
                    }

                } else {
                    format.append(nl(1) + "<Message>" + nl(2) + "Layer " + desLayer + " not queryable" + nl(1) + "</Message>");

                }

            }

            return format;
        } catch (SQLException ex) {
            ex.printStackTrace();
            return new StringBuilder("");
        }

    }

    private String[][] getFeaturesInfo(Connection conn, ArrayList features) throws SQLException {

        if (this.isQueryable()) {

            String query = "SELECT ";
            List fields = source.getFieldNames();
            List types = source.getFieldTypes();
            for (int i = 0; i < fields.size(); i++) {
                if(((Integer)types.get(i)).intValue() != java.sql.Types.BINARY){
                    query += fields.get(i) + ", ";
                }else{
                    fields.remove(i);
                }                
            }

            query = query.substring(0, query.length() - 2);
            query += (" FROM \"" + source.getTablsour() + "\" WHERE atlas27_id = ?");
            query += " ORDER BY atlas27_id";

            PreparedStatement ps = conn.prepareStatement(query);
            String[][] res = new String[features.size() + 1][];
            String[] vec = new String[fields.size()];
            
            for (int k = 0; k < fields.size(); k++) {
                vec[k] = fields.get(k).toString();
            }

            res[0] = vec;

            if (features.size() > 0) {
                ResultSet resq = null;
                for (int j = 0; j < features.size(); j++) {
                    ps.setInt(1, ((Feature) features.get(j)).getId());
                    resq = ps.executeQuery();
                    resq.next();
                    vec = new String[fields.size()];
                    for (int l = 0; l < fields.size(); l++) {
                        vec[l] = resq.getString(l + 1);
                    }

                    res[j + 1] = vec;
                    resq.close();
                }

                ps.close();
            }

            return res;
        } else {
            String[][] res = new String[1][0];
            res[0] = new String[0];
            return res;

        }

    }

    /**
     * 
     * @param INFO_FORMAT
     * @param crs
     * @param envelope
     * @param atlasNS
     * @return
     */
    public static String getHeaderFormat(
            String INFO_FORMAT, int crs, Envelope envelope, URL atlasNS) {

        if (INFO_FORMAT.toLowerCase().equals("text/html")) {
            return "<html>" + nl(1) + "<head>" + nl(2) + "<title>Get Feature Info Atlas</title>" + nl(1) + "</head>" + nl(1) + "<body>" + nl(2) + "<div align='center'>";
        }

        if (INFO_FORMAT.toLowerCase().equals("application/vnd.ogc.gml")) {
            String header = "";

            header +=
                    "<FeatureCollection xmlns='http://www.opengis.net/gml' xmlns:atlas='" + atlasNS + "' xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' xsi:schemaLocation='http://www.opengis.net/gml http://schemas.opengis.net/gml/3.0.0/base/feature.xsd'>";
            header +=
                    nl(1) + "<boundedBy>";
            header +=
                    nl(2) + "<Envelope srsName='EPSG:" + crs + "'>";
            header +=
                    nl(3) + "<coordinates decimal='.' cs=',' ts=' '>";
            header +=
                    nl(4) + envelope.getMinX() + "," + envelope.getMinY() + " " + envelope.getMaxX() + "," + envelope.getMaxY();
            header +=
                    nl(3) + "</coordinates>";
            header +=
                    nl(2) + "</Envelope>";
            header +=
                    nl(1) + "</boundedBy>";
            return header;
        }

        return "";
    }

    /**
     * 
     * @param INFO_FORMAT
     * @return
     */
    public static String getFooterFormat(
            String INFO_FORMAT) {

        if (INFO_FORMAT.toLowerCase().equals("text/html")) {
            return nl(2) + "<div>" + nl(1) + "</body>" + nl() + "</html>";
        }

        if (INFO_FORMAT.toLowerCase().equals("application/vnd.ogc.gml")) {
            return nl() + "</FeatureCollection>";
        }

        return "";
    }

    private static String tab(int tab) {
        String ttab = "";
        for (int i = 0; i <
                tab; i++) {
            ttab += "\t";
        }

        return ttab;
    }

    private static String tab() {
        return tab(1);
    }

    private static String nl(int tab) {
        return "\n" + tab(tab);
    }

    private static String nl() {
        return nl(0);
    }

    /**
     * Simbología de capa 
     * @return
     */
    public Symbology getSymbology() {
        return symbology;
    }

    /**
     * Simbología de capa
     * @param symbology
     */
    public void setSymbology(Symbology symbology) {
        this.symbology = symbology;
        this.generateLegend();
    }

    /**
     * Array que contiene un elemento por cada geometría en el origen de datos.
     * su contenido se usa para determinar la simbología apropiada en caso de que 
     * esta sea de tipo clase
     * @return
     */
    public String[] getClassValues() {
        return classValues;
    }

    /**
     * Array que contiene un elemento por cada geometría en el origen de datos.
     * su contenido se usa para determinar la simbología apropiada en caso de que 
     * esta sea de tipo rango
     * @return
     */
    public double[] getRangeValues() {
        return rangeValues;
    }

    /**
     * El origen de datos asociado a la capa 
     * @return
     */
    public Source getSource() {
        return source;
    }

    /**
     * El origen de datos asociado a la capa
     * @param source
     */
    public void setSource(Source source) {
        this.source = source;
    }

    public byte[] getBinalegend() {
        return binalegend;
    }

    public void setBinalegend(byte[] binalegend) {
        this.binalegend = binalegend;
    }

    public String[] getLabelValues() {
        return labelValues;
    }

    public void setLabelValues(String[] labelValues) {
        this.labelValues = labelValues;
    }
}
