package org.atlas.hibernatedata;

/**
 * Define una regla para aplicar un estilo a gráfico a un tupla en particular
 * de una capa. Soporta persistencia
 * @author Mario
 */
public class Rule implements Comparable {

    private int coderule;
    private Symbology symbology;
    private Style style;
    private String value;
    private double max;
    private double min;
    private byte[] binarule;
    private String descrule;

    /**
     * Construye un objeto en blanco
     */
    public Rule() {

    }

    /**
     * Código del objeto en el sistema de persistencia
     * @return
     */
    public int getCoderule() {
        return coderule;
    }

    /**
     * Código del objeto en el sistema de persistencia 
     * @param coderule
     */
    public void setCoderule(int coderule) {
        this.coderule = coderule;
    }

    /**
     * Estilo que debe aplicarse a la tupla en caso de ser afectada por la regla
     * @return
     */
    public Style getStyle() {
        return style;
    }

    /**
     * Estilo que debe aplicarse a la tupla en caso de ser afectada por la regla
     * @param style
     */
    public void setStyle(Style style) {
        this.style = style;
    }

    /**
     * Simbología a la que pertenece la regla
     * @return
     */
    public Symbology getSymbology() {
        return symbology;
    }

    /**
     * Simbología a la que pertenece la regla 
     * @param symbology
     */
    public void setSymbology(Symbology symbology) {
        this.symbology = symbology;
    }

    /**
     * Si la regla pertenece a un modelo de rangos, representa el valor máximo del intervalo
     * @return
     */
    public double getMax() {
        return max;
    }

    /**
     * Si la regla pertenece a un modelo de rangos, representa el valor máximo del intervalo 
     * @param max
     */
    public void setMax(double max) {
        this.max = max;
    }

    /**
     * Si la regla pertenece a una simbología de rangos, representa el valor mínimo del intervalo
     * @return
     */
    public double getMin() {
        return min;
    }

    /**
     * Si la regla pertenece a una simbología de rangos, representa el valor mínimo del intervalo 
     * @param min
     */
    public void setMin(double min) {
        this.min = min;
    }

    /**
     * Una descripción para la regla
     * @return
     */
    public String getDescrule() {
        return descrule;
    }

    /**
     * Una descripción para la regla 
     * @param descrule
     */
    public void setDescrule(String descrule) {
        this.descrule = descrule;
    }

    /**
     * El valor en caso de que la regla pertenezca a una simbología de clases
     * @return
     */
    public String getValue() {
        return value;
    }

    /**
     * El valor en caso de que la regla pertenezca a una simbología de clases 
     * @param value
     */
    public void setValue(String value) {
        this.value = value;
    }

    @Override
    public String toString() {
        return descrule;
    }

    public int compareTo(Object obj) {
        return this.toString().compareTo(obj.toString());
    }

    @Override
    protected Object clone() throws CloneNotSupportedException {
        Rule r = new Rule();
        r.setDescrule(getDescrule());
        r.setMax(getMax());
        r.setMin(getMin());
        r.setValue(getValue());
        r.setStyle((Style) getStyle().clone());
        return r;
    }

    /**
     * Campo binario que contiene la imagen que deben usarse en caso de simbologías
     * para capas de puntos
     * @return
     */
    public byte[] getBinarule() {
        return binarule;
    }

    /**
     * Campo binario que contiene la imagen que deben usarse en caso de simbologías
     * para capas de puntos 
     * @param binarule
     */
    public void setBinarule(byte[] binarule) {
        this.binarule = binarule;
    }

}
