/*
 * Created on 16 de julio de 2007, 08:45 AM
 */
package org.atlas.hibernatedata;

import java.io.IOException;
import java.io.InputStream;
import java.net.ConnectException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import javax.units.SI;
import javax.units.Unit;
import org.atlas.exceptions.HTTPRequestException;
import org.hibernate.classic.Session;
import org.jdom.DocType;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.atlas.util.HibernateUtil;
import org.atlas.util.ServiceConnection;

/**
 * Representa un servicio WMS
 * @author Mario
 */
public class Service {

    private int codeserv;
    private String title;
    private String summary;
    private String keywordList;
    private String providerURL;
    private String contactPerson;
    private String contactOrganization;
    private String contactElectronicMailAddress;
    private String layerLimit;
    private String maxWidth;
    private String maxHeight;
    private boolean publicaccess;
    private String serviceURL;
    private String WMSCapabilities;
    private Set projects = new HashSet();
    private Set icons = new HashSet();
    private String WMSGC1_1_0;
    private String WMSGC1_1_1;
    private String WMSGC1_3_0;
    private String WMSJSON;
    public static final int VERSION_1_1_0 = 1;
    public static final int VERSION_1_1_1 = 2;
    public static final int VERSION_1_3_0 = 3;
    public static final int VERSION_JSON = 4;
    public static final String SIGNATURE = "ae985d71e72a4a4c4e1a34df689de48c";

    /**
     * Construye un objeto en blanco
     */
    public Service() {
    }

    /**
     * Retorna el objeto service del sistema desde una conexión hibernate
     * @param hibernate
     * @return 
     * @throws java.lang.Exception
     */
    public static Service getService(HibernateUtil hibernate) throws Exception {
        Session sess = null;
        try {
            sess = hibernate.begin();
            List services = sess.createQuery("from Service").list();
            if (services.size() == 1) {
                Service rta = (Service) services.get(0);
                hibernate.close(sess);
                return rta;
            }
            hibernate.close(sess);
        } catch (Exception ex) {
            try {
                hibernate.close(sess);
            } catch (Exception ex1) {
                ex1.printStackTrace();
            }
            throw ex;
        }
        return null;
    }

    /**
     * Indentificador del objeto en el sistema de persistencia
     * @return
     */
    public int getCodeserv() {
        return codeserv;
    }

    /**
     * Indentificador del objeto en el sistema de persistencia 
     * @param codeserv
     */
    public void setCodeserv(int codeserv) {
        this.codeserv = codeserv;
    }

    /**
     * Descripción del breve del servicio. De conformidad con el estándar WMS
     * @return
     */
    public String getTitle() {
        return title;
    }

    /**
     * Descripción del breve del servicio. De conformidad con el estándar WMS 
     * @param title
     */
    public void setTitle(String title) {
        this.title = title;
    }

    /**
     * Descripción del servicio. De conformidad con el estándar WMS 
     * @return
     */
    public String getSummary() {
        return summary;
    }

    /**
     * Descripción del servicio. De conformidad con el estándar WMS  
     * @param summary
     */
    public void setSummary(String summary) {
        this.summary = summary;
    }

    /**
     * Palabras clave. De conformidad con el estándar WMS 
     * @return
     */
    public String getKeywordList() {
        return keywordList;
    }

    /**
     * Palabras clave. De conformidad con el estándar WMS  
     * @param keywordList
     */
    public void setKeywordList(String keywordList) {
        this.keywordList = keywordList;
    }

    /**
     * URL con información del servicio. De conformidad con el estándar WMS
     * @return
     */
    public String getProviderURL() {
        return providerURL;
    }

    /**
     * URL con información del servicio. De conformidad con el estándar WMS
     * @param providerURL
     */
    public void setProviderURL(String providerURL) {
        this.providerURL = providerURL;
    }

    /**
     * Persona encargada del servicio. De conformidad con el estándar WMS
     * @return
     */
    public String getContactPerson() {
        return contactPerson;
    }

    /**
     * Persona encargada del servicio. De conformidad con el estándar WMS 
     * @param contactPerson
     */
    public void setContactPerson(String contactPerson) {
        this.contactPerson = contactPerson;
    }

    /**
     * Organización encargada del servicio. De conformidad con el estándar WMS 
     * @return
     */
    public String getContactOrganization() {
        return contactOrganization;
    }

    /**
     * Organización encargada del servicio. De conformidad con el estándar WMS  
     * @param contactOrganization
     */
    public void setContactOrganization(String contactOrganization) {
        this.contactOrganization = contactOrganization;
    }

    /**
     * Correo electronico del encargado del servicio. De conformidad con el estándar WMS
     * @return
     */
    public String getContactElectronicMailAddress() {
        return contactElectronicMailAddress;
    }

    /**
     * Correo electronico del encargado del servicio. De conformidad con el estándar WMS 
     * @param contactElectronicMailAddress
     */
    public void setContactElectronicMailAddress(String contactElectronicMailAddress) {
        this.contactElectronicMailAddress = contactElectronicMailAddress;
    }

    /**
     * Número máximo de capas por petición. De conformidad con el estándar WMS
     * @return
     */
    public String getLayerLimit() {
        return layerLimit;
    }

    /**
     * Número máximo de capas por petición. De conformidad con el estándar WMS 
     * @param layerLimit
     */
    public void setLayerLimit(String layerLimit) {
        this.layerLimit = layerLimit;
    }

    /**
     * Ancho máximo de la imagen por petición. De conformidad con el estándar WMS 
     * @return
     */
    public String getMaxWidth() {
        return maxWidth;
    }

    /**
     * Ancho máximo de la imagen por petición. De conformidad con el estándar WMS  
     * @param maxWidth
     */
    public void setMaxWidth(String maxWidth) {
        this.maxWidth = maxWidth;
    }

    /**
     * Alto máximo de la imagen por petición. De conformidad con el estándar WMS  
     * @return
     */
    public String getMaxHeight() {
        return maxHeight;
    }

    /**
     * Alto máximo de la imagen por petición. De conformidad con el estándar WMS   
     * @param maxHeight
     */
    public void setMaxHeight(String maxHeight) {
        this.maxHeight = maxHeight;
    }

    /**
     * URL del servicio WMS. De conformidad con el estándar WMS
     * @return
     */
    public String getServiceURL() {
        return serviceURL;
    }

    /**
     * URL del servicio WMS. De conformidad con el estándar WMS 
     * @param serviceURL
     */
    public void setServiceURL(String serviceURL) {
        this.serviceURL = serviceURL;
    }

    /**
     * Obtiene el documento de capabilities del sistema según la versión
     * @param version
     * @return
     */
    public String getWMSCapabilities(int version) {
        switch (version) {
            case VERSION_1_1_0:
                return this.getWMSGC1_1_0();
            case VERSION_1_1_1:
                return this.getWMSGC1_1_1();
            case VERSION_1_3_0:
                return this.getWMSGC1_3_0();
            case VERSION_JSON:
                return this.getWMSJSON();
        }
        return null;
    }

    /**
     * Establece el documento de capabilities del sistema según la versión
     * @param WMSCapabilities
     * @param version
     */
    public void setWMSCapabilities(String WMSCapabilities, int version) {
        switch (version) {
            case VERSION_1_1_0:
                setWMSGC1_1_0(WMSCapabilities);
            case VERSION_1_1_1:
                setWMSGC1_1_1(WMSCapabilities);
            case VERSION_1_3_0:
                setWMSGC1_3_0(WMSCapabilities);
            case VERSION_JSON:
                setWMSJSON(WMSCapabilities);
        }
    }

    /**
     * Conjunto de proyectos del servicio
     * @return
     */
    public Set getProjects() {
        return projects;
    }

    /**
     * Conjunto de proyectos del servicio 
     * @param projects
     */
    public void setProjects(Set projects) {
        this.projects = projects;
    }

    /**
     * Conjunto de iconos del servicio 
     * @return
     */
    public Set getIcons() {
        return icons;
    }

    /**
     * Conjunto de iconos del servicio 
     * @param icons
     */
    public void setIcons(Set icons) {
        this.icons = icons;
    }

    /**
     * Determina si el servicio tiene acceso al público
     * @return
     */
    public boolean isPublicaccess() {
        return publicaccess;
    }

    /**
     * Determina si el servicio tiene acceso al público 
     * @param publicaccess
     */
    public void setPublicaccess(boolean publicaccess) {
        this.publicaccess = publicaccess;
    }

    private String getWMSGC1_1_0() {
        return WMSGC1_1_0;
    }

    private void setWMSGC1_1_0(String WMSGC1_1_0) {
        this.WMSGC1_1_0 = WMSGC1_1_0;
    }

    private String getWMSGC1_1_1() {
        return WMSGC1_1_1;
    }

    private void setWMSGC1_1_1(String WMSGC1_1_1) {
        this.WMSGC1_1_1 = WMSGC1_1_1;
    }

    private String getWMSGC1_3_0() {
        return WMSGC1_3_0;
    }

    private void setWMSGC1_3_0(String WMSGC1_3_0) {
        this.WMSGC1_3_0 = WMSGC1_3_0;
    }

    /**
     * Obtiene el documento de capabilities en formato JSON
     * @return
     */
    public String getWMSJSON() {
        return WMSJSON;
    }

    /**
     * Obtiene el documento de capabilities en formato JSON 
     * @param WMSJSON
     */
    public void setWMSJSON(String WMSJSON) {
        this.WMSJSON = WMSJSON;
    }

    private String getXMLCapabilities(int version, InputStream eschema) throws IOException, Exception {
        Document doc = null;
        SAXBuilder builder = new SAXBuilder();
        XMLOutputter outp = new XMLOutputter(Format.getPrettyFormat());
        try {
            doc = builder.build(eschema);
            DocType doctype = new DocType("WMT_MS_Capabilities");
            switch (version) {
                case VERSION_1_1_0:
                    doctype.setSystemID(this.getServiceURL() + "?request=GetSchema&version=1.1.0");
                    doc.setDocType(doctype);
                    break;
                case VERSION_1_1_1:
                    doctype.setSystemID(this.getServiceURL() + "?request=GetSchema&version=1.1.1");
                    doc.setDocType(doctype);
                    break;
                case VERSION_1_3_0:
                    //TODO Validar para internet explored;
                    break;
            }
            this.getXMLService(doc.getRootElement(), version);
            this.getXMLCapability(doc.getRootElement(), version);
        } catch (JDOMException ex) {
            ex.printStackTrace();
        }

        return outp.outputString(doc);
    }

    private void getXMLCapability(Element root, int version) throws Exception {
        Namespace ns = root.getNamespace();
        Namespace xlinkns = null;
        Element getCapability = root.getChild("Capability", ns).getChild("Request", ns).getChild("GetCapabilities", ns);
        Element getMap = root.getChild("Capability", ns).getChild("Request", ns).getChild("GetMap", ns);
        Element getFeatureInfo = root.getChild("Capability", ns).getChild("Request", ns).getChild("GetFeatureInfo", ns);

        switch (version) {
            case VERSION_1_1_0:
                xlinkns = getCapability.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).getNamespace("xlink");
                getCapability.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).setAttribute("href", this.getServiceURL(), xlinkns);
                getMap.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).setAttribute("href", this.getServiceURL(), xlinkns);
                getFeatureInfo.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).setAttribute("href", this.getServiceURL(), xlinkns);
                break;

            case VERSION_1_1_1:
                xlinkns = getCapability.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).getNamespace("xlink");
                getCapability.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).setAttribute("href", this.getServiceURL(), xlinkns);
                getMap.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).setAttribute("href", this.getServiceURL(), xlinkns);
                getFeatureInfo.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).setAttribute("href", this.getServiceURL(), xlinkns);
                break;

            case VERSION_1_3_0:
                xlinkns = root.getNamespace("xlink");
                getCapability.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).setAttribute("href", this.getServiceURL(), xlinkns);
                getMap.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).setAttribute("href", this.getServiceURL(), xlinkns);
                getFeatureInfo.getChild("DCPType", ns).getChild("HTTP", ns).getChild("Get", ns).getChild("OnlineResource", ns).setAttribute("href", this.getServiceURL(), xlinkns);
                break;
        }

        Iterator<Project> ite = this.getProjects().iterator();
        while (ite.hasNext()) {
            Project proyect = ite.next();
            root.getChild("Capability", ns).getChild("Layer", ns).addContent(proyect.getXMLCapability(root, version));
        }
    }

    private void getXMLService(Element root, int version) {

        Namespace ns = root.getNamespace();
        Namespace xlinkns = null;

        Element service = root.getChild("Service", ns);
        service.getChild("Title", ns).setText(this.getTitle());
        service.getChild("Abstract", ns).setText(this.getSummary());

        String keywords[] = this.getKeywordList().split(",");
        for (int i = 0; i < keywords.length; i++) {
            service.getChild("KeywordList", ns).addContent(new Element("Keyword", ns).setText(keywords[i]));
        }

        service.getChild("ContactInformation", ns).getChild("ContactPersonPrimary", ns).getChild("ContactPerson", ns).setText(this.getContactPerson());
        service.getChild("ContactInformation", ns).getChild("ContactPersonPrimary", ns).getChild("ContactOrganization", ns).setText(this.getContactOrganization());
        service.getChild("ContactInformation", ns).getChild("ContactElectronicMailAddress", ns).setText(this.getContactElectronicMailAddress());

        switch (version) {
            case VERSION_1_1_0:
                xlinkns = service.getChild("OnlineResource", ns).getNamespace("xlink");
                service.getChild("OnlineResource", ns).setAttribute("href", this.getProviderURL(), xlinkns);
                break;

            case VERSION_1_1_1:
                xlinkns = service.getChild("OnlineResource", ns).getNamespace("xlink");
                service.getChild("OnlineResource", ns).setAttribute("href", this.getProviderURL(), xlinkns);
                break;

            case VERSION_1_3_0:


                xlinkns = root.getNamespace("xlink");
                service.getChild("OnlineResource", ns).setAttribute("href", this.getProviderURL(), xlinkns);

                //TODO validar los de mas campos opcionales

                if (!getLayerLimit().equals("")) {
                    service.getChild("LayerLimit", ns).setText(this.getLayerLimit());
                } else {
                    service.removeChild("LayerLimit", ns);
                }

                if (!getMaxWidth().equals("")) {
                    service.getChild("MaxWidth", ns).setText(this.getMaxWidth());
                } else {
                    service.removeChild("MaxWidth", ns);
                }

                if (!getMaxWidth().equals("")) {
                    service.getChild("MaxHeight", ns).setText(this.getMaxWidth());
                } else {
                    service.removeChild("MaxHeight", ns);
                }
                break;
        }
    }

    private void saveCapabalities(HibernateUtil hibernate, String XML110, String XML111, String XML130, String JSON) throws Exception {

        Session session = null;
        try {
            session = hibernate.begin();
            session.update(this);
            this.setWMSCapabilities(XML110, VERSION_1_1_0);
            this.setWMSCapabilities(XML111, VERSION_1_1_1);
            this.setWMSCapabilities(XML130, VERSION_1_3_0);
            this.setWMSJSON(JSON);
            hibernate.commitAndClose(session);
        } catch (Exception ex) {
            ex.printStackTrace();
            try {
                hibernate.close(session);
            } catch (Exception ex1) {
                ex1.printStackTrace();
            }
            throw ex;
        }
    }

    /**
     * Envia al servidor la señal que indica que deben recargase todos los buffers
     * desde la base de datos
     * @param sc
     * @throws java.net.ConnectException
     * @throws exceptions.HTTPRequestException
     * @throws java.lang.Exception
     */
    public void sendWMSReload(ServiceConnection sc) throws ConnectException, HTTPRequestException, Exception {
        sc.sendWMSReload();
    }

    private String getJSONCapabilities() throws Exception {

        if (this.getProjects().size() > 0) {
            StringBuilder json = new StringBuilder("[");
            Iterator<Project> itep = this.getProjects().iterator();
            while (itep.hasNext()) {
                Project project = itep.next();
                json.append("{");

                json.append("name : ");
                json.append("\"");
                json.append(project.getNameProj());
                json.append("\",");
                json.append("getName : function(){ return this.name; },");

                json.append("title : ");
                json.append("\"");
                json.append(project.getDescproj());
                json.append("\",");
                json.append("getTitle : function(){ return this.title; },");

                json.append("bbox  : { crs :");
                json.append("\"");
                json.append("EPSG:");
                json.append(project.getSpatial_ref_sys().getSrid());
                json.append("\",");
                json.append("minx : ");
                json.append(project.getBoundingBox().getMinx());
                json.append(",");
                json.append("miny : ");
                json.append(project.getBoundingBox().getMiny());
                json.append(",");
                json.append("maxx : ");
                json.append(project.getBoundingBox().getMaxx());
                json.append(",");
                json.append("maxy : ");
                json.append(project.getBoundingBox().getMaxy());
                json.append(",");
                json.append("getCRS : function(){ return this.crs; },");
                json.append("getMinX : function(){ return this.minx; },");
                json.append("getMinY : function(){ return this.miny; },");
                json.append("getMaxX : function(){ return this.maxx; },");
                json.append("getMaxY : function(){ return this.maxy; },");
                json.append("getWidth : function(){ return this.maxx-this.minx; },");
                json.append("getHeight : function(){ return this.maxy-this.miny; }");
                json.append("},");
                json.append("getBoundingBox : function(){ return this. bbox; },");

                json.append("getEpsgCrs : function(){ return { ");
                try {
                    CoordinateReferenceSystem gtcrs = project.getSpatial_ref_sys().getCRS();

                    int code = project.getSpatial_ref_sys().getSrid();
                    String name = gtcrs.getName().getCode();
                    String axis1 = gtcrs.getCoordinateSystem().getAxis(0).getDirection().name().toLowerCase();
                    String axis2 = gtcrs.getCoordinateSystem().getAxis(1).getDirection().name().toLowerCase();

                    Unit un = gtcrs.getCoordinateSystem().getAxis(0).getUnit();
                    double fac = 0;
                    String type = "";
                    Unit sis = un.getSystemUnit();
                    if (sis.equals(SI.RADIAN)) {
                        type = "angular";
                        fac = un.getConverterTo(SI.RADIAN).convert(1.0d);
                    }
                    if (sis.equals(SI.METER)) {
                        type = "linear";
                        fac = un.getConverterTo(SI.METER).convert(1.0d);
                    }

                    json.append(" getCode : function() { return ");
                    json.append("\"");
                    json.append(code);
                    json.append("\"; ");
                    json.append("},");
                    json.append(" getName : function() { return ");
                    json.append("\"");
                    json.append(name);
                    json.append("\"; ");
                    json.append("},");
                    json.append(" getAxis1 : function(){ return ");
                    json.append("\"");
                    json.append(axis1);
                    json.append("\"; ");
                    json.append("},");
                    json.append(" getAxis2 : function(){ return ");
                    json.append("\"");
                    json.append(axis2);
                    json.append("\"; ");
                    json.append("},");
                    json.append(" getUnitType : function(){ return ");
                    json.append("\"");
                    json.append(type);
                    json.append("\"; ");
                    json.append("},");
                    json.append(" getConvertionFactor : function(){ return ");
                    json.append(fac);
                    json.append("; }");

                } catch (IndexOutOfBoundsException ex) {
                    ex.printStackTrace();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
                json.append(" } },");

                json.append("layers : [");
                Iterator<Layer> itel = project.getSortedLayers().iterator();
                while (itel.hasNext()) {
                    Layer layer = itel.next();

                    if (layer.getSource() != null) {
                        json.append("{");
                        json.append("name : ");
                        json.append("\"");
                        json.append(layer.getNameLaye());
                        json.append("\",");
                        json.append("title : ");
                        json.append("\"");
                        json.append(layer.getDesclaye());
                        json.append("\",");

                        json.append("maxscaledeno : ");
                        json.append("\"");
                        json.append(layer.getMaxscaledeno());
                        json.append("\",");

                        json.append("minscaledeno : ");
                        json.append("\"");
                        json.append(layer.getMinscaledeno());
                        json.append("\",");

                        json.append("getName : function(){ return this.name; },");
                        json.append("getTitle : function(){ return this.title; },");
                        json.append("getMaxscaledeno : function(){ return this.maxscaledeno; },");
                        json.append("getMinscaledeno : function(){ return this.minscaledeno; }");

                        json.append("},");
                    }
                }

                json.setLength(json.length() - 1);
                json.append("],");
                json.append("getLayers : function(){ return this.layers; }");
                json.append("},");

            }
            json.setLength(json.length() - 1);
            json.append("]");
            return json.toString();
        }

        return null;
    }

    /**
     * Actualiza los capabilities del servicio en todas sus versiones
     * @param hibernate
     * @param sc
     * @throws java.net.ConnectException
     * @throws exceptions.HTTPRequestException
     * @throws java.lang.Exception
     */
    public void updateCapabilities(HibernateUtil hibernate, ServiceConnection sc) throws ConnectException, HTTPRequestException, Exception {
        loadTree(hibernate);
        String XML110 = getXMLCapabilities(VERSION_1_1_0, this.getClass().getResourceAsStream("/xml/GetCapabilities_1.1.0.xml"));
        String XML111 = getXMLCapabilities(VERSION_1_1_1, this.getClass().getResourceAsStream("/xml/GetCapabilities_1.1.1.xml"));
        String XML130 = getXMLCapabilities(VERSION_1_3_0, this.getClass().getResourceAsStream("/xml/GetCapabilities_1.3.0.xml"));
        String JSON = getJSONCapabilities();
        saveCapabalities(hibernate, XML110, XML111, XML130, JSON);
        sendWMSReload(sc);
    }

    private void loadTree(HibernateUtil hibernate) throws Exception {
        Session sess = null;
        sess = hibernate.begin();
        sess.update(this);
        Object[] projectst = this.getProjects().toArray();
        for (int i = 0; i < projectst.length; i++) {
            Project pr = (Project) projectst[i];
            sess.update(pr);
            String tmp = pr.getSpatial_ref_sys().getSrtext();
            sess.update(pr);
            Object[] layers = pr.getLayers().toArray();
            for (int j = 0; j < layers.length; j++) {
                Layer ly = (Layer) layers[j];
                sess.update(ly);
                if (ly.getSource() != null) {
                    sess.update(ly.getSource());
                    tmp = ly.getSource().getGeomType();
                }
            }
        }
        hibernate.commitAndClose(sess);
    }
}
