/*
 * Created on 29 de mayo de 2007, 09:12 AM
 */
package org.atlas.hibernatedata;

import java.awt.BasicStroke;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.atlas.svg.SimpleSvgReader;


/**
 * Define un estilo que se aplica al momento de renderizar una geometría en caso
 * de que cumpla una regla en particular
 * @author Mario
 */
public class Style implements Cloneable {

    private int codestyl;
    private Icon icon;
    private AtlasStroke linestyl;
    private AtlasColor linecolo;
    private AtlasColor fillcolo;

    /**
     * Crea un objeto sin valores
     */
    public Style() {
    }

    /**
     * El icono del estilo
     * @return
     */
    public Icon getIcon() {
        return icon;
    }

    /**
     * El icono del estilo 
     * @param icon
     */
    public void setIcon(Icon icon) {
        this.icon = icon;
    }

    /**
     * El estilo de linea
     * @return
     */
    public AtlasStroke getLinestyl() {
        return linestyl;
    }

    /**
     * El estilo de linea 
     * @param linestyl
     */
    public void setLinestyl(AtlasStroke linestyl) {
        this.linestyl = linestyl;
    }

    /**
     * Color de linea
     * @return
     */
    public AtlasColor getLinecolo() {
        return linecolo;
    }

    /**
     * Color de linea 
     * @param linecolo
     */
    public void setLinecolo(AtlasColor linecolo) {
        this.linecolo = linecolo;
    }

    /**
     * Color de relleno para la geometría 
     * @return
     */
    public AtlasColor getFillcolo() {
        return fillcolo;
    }

    /**
     * Color de relleno para la geometría 
     * @param fillcolo
     */
    public void setFillcolo(AtlasColor fillcolo) {
        this.fillcolo = fillcolo;
    }

    /**
     * Genera un estilo aleatorio que llevan las capas por defecto
     * @param icon
     * @return
     */
    public static Style getDefaultRandomStyle(Icon icon) {
        int max = 32;
        Style style = new Style();
        int r = (int) (Math.random() * max);
        int g = (int) (Math.random() * max);
        int b = (int) (Math.random() * max);

        AtlasColor fill = new AtlasColor();
        fill.setRGBA(r + (255 - max), g + (255 - max), b + (255 - max), 255);

        AtlasColor line = new AtlasColor();
        line.setRGBA(r + (164 - max), g + (164 - max), b + (164 - max), 255);

        AtlasStroke stroke = new AtlasStroke();
        stroke.setBasicStroke(new BasicStroke(1));

        style.setFillcolo(fill);
        style.setLinecolo(line);
        style.setLinestyl(stroke);
        style.setIcon(icon);
        
        return style;
    }

    /**
     * Código del objeto en el sistema de persistencia
     * @return
     */
    public int getCodestyl() {
        return codestyl;
    }

    /**
     * Código del objeto en el sistema de persistencia 
     * @param codestyl
     */
    public void setCodestyl(int codestyl) {
        this.codestyl = codestyl;
    }

    /**
     * Renderizar la imagen del icono del estilo
     * @return
     */
    public BufferedImage getIconImage() {
        BufferedImage img = null;
        try {
            int width = (int) getLinestyl().getWidth();
            SimpleSvgReader reader = new SimpleSvgReader(new ByteArrayInputStream(getIcon().getBinaicon()));
            if (getFillcolo() == null) {
                img = reader.drawGraphics(true, null, null, width, width);
            } else {
                img = reader.drawGraphics(true, getFillcolo().getColor(), getLinecolo().getColor(), width, width);
            }

        } catch (Exception ex) {
            Logger.getLogger(Style.class.getName()).log(Level.SEVERE, null, ex);
        }
        return img;
    }

    @Override
    protected Object clone() throws CloneNotSupportedException {
        Style styl = new Style();
        
        if(getFillcolo()!=null){
            styl.setFillcolo((AtlasColor) getFillcolo().clone());    
        }
        
        if(getLinecolo()!=null){
            styl.setLinecolo((AtlasColor) getLinecolo().clone());    
        }
        
        styl.setIcon(getIcon());
        styl.setLinestyl((AtlasStroke) getLinestyl().clone());
        return styl;
    }
}
