package org.atlas.service;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.io.WKTReader;
import org.atlas.hibernatedata.Layer;
import org.atlas.hibernatedata.Rule;
import org.atlas.hibernatedata.Style;
import org.atlas.hibernatedata.Symbology;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import org.jfree.text.TextUtilities;
import org.jfree.ui.TextAnchor;

public class BasicRender {

    //sobre la conversión de espacio de coordenadas
    private boolean swap;
    private double minI;
    private double minJ;
    private double factI;
    private double factJ;
    private boolean invertI;
    private boolean invertJ;
    private int width;
    private int heigth;
    //objeto grafico
    private Graphics2D gra;
    //limites del area de dibujo
    private Envelope envelop;
    private Polygon limits;
    private Color border;
    private Color fill;
    private BasicStroke stroke;
    private BufferedImage icon;
    private int offx,  offy; // for placing icons
    private Layer layer;
    private Feature[] iterFeat;
    private Envelope[] iterBBox;
    private String[] iterClass;
    private double[] iterRange;
    private String[] iterLabel;
    private Rule[] rules;
    private Style currentStyle;
    private Rule currentRule;

    public BufferedImage wmsRender(Layer[] layers, CRS crs, double xmin, double ymin, double xmax, double ymax, int width, int heigth, boolean transparent, Color BgColor) throws Exception {

        this.heigth = heigth;
        this.width = width;

        String axis1 = crs.getAxis1();
        String axis2 = crs.getAxis2();

        if (axis1.endsWith("west") || axis1.endsWith("east")) {
            swap = false;
            factI = width / (xmax - xmin);
            factJ = heigth / (ymax - ymin);
            minI = (xmin * factI);
            minJ = (ymin * factJ);
            if (axis1.equals("east")) {
                invertI = false;
            } else {
                invertI = true;
            }

            if (axis2.equals("south")) {
                invertJ = false;
            } else {
                invertJ = true;
            }
        } else {
            swap = true;
            factI = width / (ymax - ymin);
            factJ = heigth / (xmax - xmin);
            minI = (ymin * factI);
            minJ = (xmin * factJ);

            if (axis2.equals("east")) {
                invertJ = false;
            } else {
                invertJ = true;
            }
            if (axis1.equals("south")) {
                invertI = false;
            } else {
                invertI = true;
            }

        }

        BufferedImage img;

        if (transparent) {
            img = new BufferedImage(width, heigth, BufferedImage.TYPE_INT_ARGB);
            gra = (Graphics2D) img.getGraphics();
        } else {
            img = new BufferedImage(width, heigth, BufferedImage.TYPE_INT_RGB);
            gra = (Graphics2D) img.getGraphics();
            if (BgColor != null) {
                gra.setBackground(BgColor);
                gra.fillRect(0, 0, width, heigth);
            } else {
                gra.setBackground(new Color(255, 255, 255)); //TODO modificame
                gra.fillRect(0, 0, width, heigth);
            }
        }

        //gra.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        Font fonts[] = new Font[layers.length];
        ArrayList<Label>[] labels = new ArrayList[layers.length];


        int ix = (int) ((xmax - xmin) * 0.3);
        int iy = (int) ((ymax - ymin) * 0.3);

        envelop = new Envelope(xmin - ix, xmax + ix, ymin - iy, ymax + iy);
        limits = getGeometry(envelop);


        for (int k = 0; k < layers.length; k++) {
            try {
                layer = layers[k];
                boolean labeled = false;
//                int fs = (int) ((width / (xmax - xmin)) * layer.getSymbology().getFontMapSize());
//                if (layer.getSymbology().isLabeled()) {
//                    if (fs >= layer.getSymbology().getFontMinSize()) {
//                        fonts[k] = new Font(layer.getSymbology().getFontName(), layer.getSymbology().getFontStyle(), fs);
//                        labeled = true;
//                    }
//                }


                if (layer.getSymbology().isLabeled()) {
                    int fs = layer.getSymbology().getFontSize();
                    fonts[k] = new Font(layer.getSymbology().getFontName(), layer.getSymbology().getFontStyle(), fs);
                    gra.setFont(fonts[k]);
                    labeled = true;
                }

                labels[k] = new ArrayList<Label>();

                iterFeat = layer.getSource().getFeatures();
                iterBBox = layer.getSource().getBboxes();
                iterClass = layer.getClassValues();
                iterRange = layer.getRangeValues();
                iterLabel = layer.getLabelValues();

                currentRule = null;
                currentStyle = null;

                rules = new Rule[layer.getSymbology().getRules().size()];
                layer.getSymbology().getRules().toArray(rules);

                if (layer.getSource().getGeomClass() == Point.class) {
                    readIcon(rules[0]);
                    for (int l = 0; l < iterFeat.length; l++) {
                        findIcon(l);
                        Point geom = (Point) iterFeat[l].getGeom();
                        int[] res = toIJ(geom.getX(), geom.getY());
                        int i = res[0];
                        int j = res[1];
                        gra.drawImage(icon, i - offx, j - offy, null);
                        if (labeled) {
                            Rectangle2D r2 = gra.getFontMetrics().getStringBounds(iterLabel[l], gra);
                            int w = (int) r2.getWidth();
                            int h = (int) r2.getHeight();
                            Envelope e = new Envelope(i - (w / 2) + 1, i + (w / 2) - 1, j - (h / 2) + offy + 5 + 2, j + (h / 2) + offy + 5 - 2);
                            Label lab = new Label(iterLabel[l], 0, TextAnchor.CENTER, TextAnchor.CENTER, new Point2D.Double((double) i, (double) j + offy + 5), e);
                            labels[k].add(lab);
                        }
                    }
                } else if (layer.getSource().getGeomClass() == MultiPoint.class) {


                    readIcon(rules[0]);
                    for (int l = 0; l < iterFeat.length; l++) {
                        findIcon(l);
                        MultiPoint multi = (MultiPoint) iterFeat[l].getGeom();
                        Envelope env = iterBBox[l];

                        if (env.intersects(envelop)) {
                            for (int m = 0; m < multi.getNumGeometries(); m++) {
                                Point geom = (Point) multi.getGeometryN(m);
                                int[] res = toIJ(geom.getX(), geom.getY());
                                int i = res[0];
                                int j = res[1];
                                gra.drawImage(icon, i - offx, j - offy, null);
                                if (labeled) {
                                    Rectangle2D r2 = gra.getFontMetrics().getStringBounds(iterLabel[l], gra);
                                    int w = (int) r2.getWidth();
                                    int h = (int) r2.getHeight();
                                    Envelope e = new Envelope(i - (w / 2) + 1, i + (w / 2) - 1, j - (h / 2) + offy + 5 + 2, j + (h / 2) + offy + 5 - 2);
                                    Label lab = new Label(iterLabel[l], 0, TextAnchor.CENTER, TextAnchor.CENTER, new Point2D.Double((double) i, (double) j + offy + 5), e);
                                    labels[k].add(lab);
                                }
                            }
                        }
                    }



                } else if (layer.getSource().getGeomClass() == LineString.class) {

                } else if (layer.getSource().getGeomClass() == MultiLineString.class) {
                    readStyle(rules[0].getStyle());
                    for (int l = 0; l < iterFeat.length; l++) {
                        findStyle(l);
                        gra.setColor(border);
                        if (stroke != null) {
                            gra.setStroke(stroke);
                        }
                        MultiLineString multi = (MultiLineString) iterFeat[l].getGeom();
                        Envelope env = iterBBox[l];

                        if (env.intersects(envelop)) {
                            double w;
                            if (labeled) {
                                w = Math.pow(gra.getFontMetrics().getStringBounds(iterLabel[l], gra).getWidth(), 2);
                            } else {
                                w = Integer.MAX_VALUE;
                            }

                            //int p1lx = 0, p1ly = 0, p2lx = 0, p2ly = 0;
                            //double maxDist = 0;

                            for (int i = 0; i < multi.getNumGeometries(); i++) {
                                LineString line = (LineString) multi.getGeometryN(i);
                                if (line.intersects(limits)) {
                                    Geometry g = line;
                                    LineString ls = (LineString) g;
                                    int[] res = toIJ(ls.getPointN(0).getX(), ls.getPointN(0).getY());
                                    int p1x = res[0];
                                    int p1y = res[1];

                                    for (int j = 1; j < ls.getNumPoints(); j++) {
                                        res = toIJ(ls.getPointN(j).getX(), ls.getPointN(j).getY());
                                        int p2x = res[0];
                                        int p2y = res[1];
                                        if (stroke != null) {
                                            gra.drawLine(p1x, p1y, p2x, p2y);
                                        }
                                        if (labeled) {
                                            double l1 = Math.pow(p1x - p2x, 2) + Math.pow(p1y - p2y, 2);
//                                            if (l1 > maxDist) {
//                                                maxDist = l1;
//                                                p1lx = p1x;
//                                                p1ly = p1y;
//                                                p2lx = p2x;
//                                                p2ly = p2y;
//                                            }

                                            if (l1 > w) {
                                                double ang;
                                                try {
                                                    ang = Math.atan((double) (p2y - p1y) / (p2x - p1x));
                                                } catch (Exception ex) {
                                                    ang = 0;
                                                }
                                                Rectangle2D r2 = gra.getFontMetrics().getStringBounds(iterLabel[l], gra);
                                                int wb = (int) r2.getWidth();
                                                int hb = (int) r2.getHeight();

                                                Point2D pt = new Point2D.Double((double) (p1x + p2x) / 2, (double) (p1y + p2y) / 2);
                                                int xmaxb = (int) ((wb / 2) * Math.cos(ang));
                                                int ymaxb = (int) ((wb / 2) * Math.sin(ang));

                                                int xminb = (int) ((xmaxb * -1) + pt.getX());
                                                int yminb = (int) ((ymaxb * -1) + pt.getY());

                                                xmaxb += pt.getX();
                                                ymaxb += pt.getY();

                                                int hb4 = hb / 4;

                                                Envelope e = new Envelope(Math.min(xmaxb, xminb) - hb4 + 1, Math.max(xmaxb, xminb) + hb4 - 1, Math.min(ymaxb, yminb) - hb4 + 2, Math.max(ymaxb, yminb) + hb4 - 2);
                                                Label lab = new Label(iterLabel[l], ang, TextAnchor.CENTER, TextAnchor.CENTER, pt, e);
                                                labels[k].add(lab);
                                            }
                                        }
                                        p1x = p2x;
                                        p1y = p2y;
                                    }
                                }
                            }
                        }
                    }
                } else if (layer.getSource().getGeomClass() == com.vividsolutions.jts.geom.Polygon.class) {

                } else if (layer.getSource().getGeomClass() == MultiPolygon.class) {
                    Polygon poly;
                    LineString ring;
                    int[] iPts;
                    int[] jPts;

                    readStyle(rules[0].getStyle());

                    for (int m = 0; m < iterFeat.length; m++) {
                        MultiPolygon multi = (MultiPolygon) iterFeat[m].getGeom();
                        Envelope env = iterBBox[m];
                        if (env.intersects(envelop)) {

                            if (labeled) {
                                Point p2 = multi.getInteriorPoint();
                                int[] res = toIJ(p2.getX(), p2.getY());

                                Rectangle2D r2 = gra.getFontMetrics().getStringBounds(iterLabel[m], gra);
                                int w = (int) r2.getWidth();
                                int h = (int) r2.getHeight();
                                Envelope e = new Envelope(res[0] - (w / 2) + 1, res[0] + (w / 2) - 1, res[1] - (h / 2) + 2, res[1] + (h / 2) - 2);
                                Label lab = new Label(iterLabel[m], 0, TextAnchor.CENTER, TextAnchor.CENTER, new Point2D.Double((double) res[0], (double) res[1]), e);
                                labels[k].add(lab);
                            }

                            findStyle(m);
                            for (int i = 0; i < multi.getNumGeometries(); i++) {
                                poly = (Polygon) multi.getGeometryN(i);
                                ring = poly.getExteriorRing();
                                iPts = new int[ring.getNumPoints()];
                                jPts = new int[ring.getNumPoints()];
                                for (int j = 0; j < ring.getNumPoints(); j++) {
                                    int[] res = toIJ(ring.getPointN(j).getX(), ring.getPointN(j).getY());
                                    iPts[j] = res[0];
                                    jPts[j] = res[1];
                                }

                                java.awt.Polygon poli = new java.awt.Polygon(iPts, jPts, iPts.length);

                                if (poly.getNumInteriorRing() > 0) {
                                    Area apol = new Area(poli);
                                    for (int j = 0; j < poly.getNumInteriorRing(); j++) {
                                        ring = poly.getInteriorRingN(j);
                                        iPts = new int[ring.getNumPoints()];
                                        jPts = new int[ring.getNumPoints()];
                                        for (int l = 0; l < ring.getNumPoints(); l++) {
                                            int[] res = toIJ(ring.getPointN(l).getX(), ring.getPointN(l).getY());
                                            iPts[l] = res[0];
                                            jPts[l] = res[1];
                                        }
                                        apol.subtract(new Area(new java.awt.Polygon(iPts, jPts, iPts.length)));
                                    }
                                    gra.setColor(fill);
                                    gra.fill(apol);
                                    if (stroke != null) {
                                        gra.setColor(border);
                                        gra.setStroke(stroke);
                                        gra.draw(apol);
                                    }
                                } else {
                                    gra.setColor(fill);
                                    gra.fillPolygon(poli);
                                    if (stroke != null) {
                                        gra.setColor(border);
                                        gra.setStroke(stroke);
                                        gra.drawPolygon(poli);
                                    }
                                }

                            }
                        }
                    }
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                throw ex;
            }
        }

        //colission detection

        for (int i = 0; i < labels.length; i++) {
            for (int j = 0; j < labels[i].size(); j++) {
                Label lab = labels[i].get(j);
                if (lab.isActive()) {
                    Envelope lim = lab.getEnvolope();
                    for (int k = 0; k < labels.length; k++) {
                        for (int l = 0; l < labels[k].size(); l++) {
                            Label lab1 = labels[k].get(l);
                            if (lab1.isActive()) {
                                Envelope lim1 = lab1.getEnvolope();
                                //if (i != k && j != l) {
                                if (!lab1.equals(lab)) {
                                    if (lim.intersects(lim1)) {

                                        int prior = layers[i].getSymbology().getPriority();
                                        int prior1 = layers[k].getSymbology().getPriority();

                                        if (prior > prior1) {
                                            lab.setActive(false);
                                        } else {
                                            lab1.setActive(false);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        gra.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        gra.setColor(new Color(50, 50, 50));

        for (int i = 0; i < layers.length; i++) {
            ArrayList<Label> lbls = labels[i];
            if (lbls.size() > 0) {
                gra.setFont(fonts[i]);
                for (int j = 0; j < lbls.size(); j++) {
                    if (lbls.get(j).isActive()) {
                        gra.setColor(new Color(50, 50, 50));
                        float cx = (float) labels[i].get(j).getBasePoint().getX();
                        float cy = (float) labels[i].get(j).getBasePoint().getY();
                        float ang = (float) labels[i].get(j).getAngle();
                        TextUtilities.drawRotatedString(labels[i].get(j).getText(), gra, cx, cy, labels[i].get(j).getAnchor1(), ang, labels[i].get(j).getAnchor2());
                    //Envelope lim = labels[i].get(j).getEnvolope();
                    //gra.drawRect((int) lim.getMinX(), (int) lim.getMinY(), (int) (lim.getMaxX() - lim.getMinX()), (int) (lim.getMaxY() - lim.getMinY()));
                    } /*else {
                gra.setColor(new Color(255, 0, 0));
                float cx = (float) labels[i].get(j).getBasePoint().getX();
                float cy = (float) labels[i].get(j).getBasePoint().getY();
                float ang = (float) labels[i].get(j).getAngle();
                TextUtilities.drawRotatedString(labels[i].get(j).getText(), gra, cx, cy, labels[i].get(j).getAnchor1(), ang, labels[i].get(j).getAnchor2());
                Envelope lim = labels[i].get(j).getEnvolope();
                gra.drawRect((int) lim.getMinX(), (int) lim.getMinY(), (int) (lim.getMaxX() - lim.getMinX()), (int) (lim.getMaxY() - lim.getMinY()));
                }*/
                }
            }
        }
        return img;
    }

    private void findStyle(int m) {

        if (layer.getSymbology().getTypeSymb() == Symbology.TYPE_CLASS_SYMB) {
            for (int i = 0; i < rules.length; i++) {
                Rule r = rules[i];
                if (r.getValue().equals(iterClass[m])) {
                    readStyle(rules[i].getStyle());
                    break;
                }
            }
        } else if (layer.getSymbology().getTypeSymb() == Symbology.TYPE_RANGE_SYMB) {
            double n = iterRange[m];
            int i;
            for (i = 0; i < rules.length - 1; i++) {
                if (rules[i].getMin() <= n && n < rules[i].getMax()) {
                    readStyle(rules[i].getStyle());
                    break;
                }
            }

            if (rules[i].getMin() <= n && n <= rules[i].getMax()) {
                readStyle(rules[i].getStyle());
            }
        }
    }

    private void readStyle(Style stl) {
        if (stl.equals(currentStyle)) {
            return;
        }
        border = stl.getLinecolo().getColor();
        fill = stl.getFillcolo().getColor();
        stroke = stl.getLinestyl().getBasicStroke();
        currentStyle = stl;
    }

    private void findIcon(int m) throws Exception {

        if (layer.getSymbology().getTypeSymb() == Symbology.TYPE_CLASS_SYMB) {
            for (int i = 0; i < rules.length; i++) {
                if (rules[i].getValue().equals(iterClass[m])) {
                    readIcon(rules[i]);
                    break;
                }
            }
        } else if (layer.getSymbology().getTypeSymb() == Symbology.TYPE_RANGE_SYMB) {
            double n = iterRange[m];
            int i;
            for (i = 0; i < rules.length - 1; i++) {
                if (rules[i].getMin() <= n && n < rules[i].getMax()) {
                    readIcon(rules[i]);
                    break;
                }
            }

            if (rules[i].getMin() <= n && n <= rules[i].getMax()) {
                readIcon(rules[i]);
            }
        }
    }

    private void readIcon(Rule r) throws Exception {
        if (r.equals(currentRule)) {
            return;
        }
        try {
            icon = ImageIO.read(new ByteArrayInputStream(r.getBinarule()));
            offx = (int) (icon.getWidth() / 2);
            offy = (int) (icon.getHeight() / 2);
        } catch (Exception ex) {
            Logger.getLogger(BasicRender.class.getName()).log(Level.SEVERE, null, ex);
        }
        currentRule = r;
    }

    private int[] toIJ(double x, double y) {
        int[] res = new int[2];
        if (swap) {
            if (invertI) {
                res[0] = (int) (width + minI - (y * factI));
            } else {
                res[0] = (int) ((y * factI) - minI);
            }

            if (invertJ) {
                res[1] = (int) (heigth + minJ - (x * factJ));
            } else {
                res[1] = (int) ((x * factJ) - minJ);
            }
        } else {
            if (invertI) {
                res[0] = (int) (width + minI - (x * factI));
            } else {
                res[0] = (int) ((x * factI) - minI);
            }

            if (invertJ) {
                res[1] = (int) (heigth + minJ - (y * factJ));
            } else {
                res[1] = (int) ((y * factJ) - minJ);
            }
        }
        return res;
    }

    private void drawEnvelope(Envelope env) {

        int[] p1 = toIJ(env.getMinX(), env.getMinY());
        int[] p2 = toIJ(env.getMaxX(), env.getMaxY());

        int minIB = Math.min(p1[0], p2[0]);
        int minJB = Math.min(p1[1], p2[1]);

        int anchoB = Math.abs(p1[0] - p2[0]);
        int altoB = Math.abs(p1[1] - p2[1]);

        if (anchoB < 1 || altoB < 1) {
            gra.setColor(new Color(255, 0, 0));
        } else {
            gra.setColor(new Color(0, 0, 255));
        }
        gra.drawRect(minIB, minJB, anchoB, altoB);
    }

    private Polygon getGeometry(Envelope env) throws Exception {
        String wkt = "POLYGON((" + env.getMinX() + " " + env.getMinY() + "," + env.getMaxX() + " " + env.getMinY() + "," + env.getMaxX() + " " + env.getMaxY() + "," + env.getMinX() + " " + env.getMaxY() + "," + env.getMinX() + " " + env.getMinY() + "))";
        return (Polygon) new WKTReader().read(wkt);
    }

    public void drawVisibleLine(LineString ls, List labels, List points, List angles) {


    }
}


