/*
 * Created on 4 de agosto de 2007, 10:54 AM
 */
package org.atlas.service;

import org.atlas.hibernatedata.AtlasColor;
import org.atlas.hibernatedata.AtlasStroke;
import org.atlas.hibernatedata.Icon;
import org.atlas.hibernatedata.Layer;
import org.atlas.hibernatedata.Rule;
import org.atlas.hibernatedata.Source;
import org.atlas.hibernatedata.Style;
import org.atlas.hibernatedata.Symbology;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.units.SI;
import javax.units.Unit;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.atlas.util.ConnectionProperties;
import org.atlas.util.DataBaseUtils;

public class ServerBuffer {

    private Connection conn;
    private byte[] wmsgc110;
    private byte[] wmsgc111;
    private byte[] wmsgc130;
    private byte[] wmsjson;
    private List layers;
    private List sources;
    private List crss;
    private List icons;
    private Object capaLock = new Object();
    private Object geomLock = new Object();
    private Object dataLock = new Object();
    private Object crssLock = new Object();
    private ConnectionProperties cp;
    private Boolean serviceEnabled;
    

    public ServerBuffer(ConnectionProperties connectionProperties) {

        wmsgc110 = null;
        wmsgc111 = null;
        wmsgc130 = null;
        wmsjson = null;

        conn = null;
        cp = connectionProperties;
        serviceEnabled = null;
        crss = new ArrayList();
        layers = new ArrayList();
        sources = new ArrayList();
        icons = new ArrayList();
    }

    public byte[] getCapabilities(String version) throws Exception {
        synchronized (capaLock) {
            readServiceInfo(version);
            if (version.equals("1.1.0")) {
                return wmsgc110;
            }
            if (version.equals("1.1.1")) {
                return wmsgc111;
            }
            if (version.equals("1.3.0")) {
                return wmsgc130;
            }
            if (version.equals("json")) {
                return wmsjson;
            }
            return null;

        }
    }

    public Boolean isServiceEnabled() throws Exception {
        synchronized (capaLock) {
            linkDataBase();
            Statement sta = conn.createStatement();
            ResultSet rta = sta.executeQuery("SELECT publicaccess FROM atlas27_service");
            if (rta.next()) {
                serviceEnabled = new Boolean(rta.getBoolean(1));
                rta.close();
                sta.close();
            }
            return serviceEnabled;
        }
    }

    public Icon getIcon(int code) throws Exception {
        Iterator<Icon> ite = icons.iterator();
        while (ite.hasNext()) {
            Icon icon = ite.next();
            if (icon.getCodeicon() == code) {
                return icon;
            }
        }

        String sql = "SELECT descicon, typeicon, binaicon FROM atlas27_icon WHERE codeicon = " + code;
        Statement sta = conn.createStatement();
        ResultSet res = sta.executeQuery(sql.toString());

        Icon icon = null;

        if (res.next()) {
            icon = new Icon();
            icon.setCodeicon(code);
            icon.setDescicon(res.getString("descicon"));
            icon.setTypeicon(res.getString("typeicon"));
            icon.setBinaicon(res.getBytes("binaicon"));
            icons.add(icon);
        }

        res.close();
        sta.close();

        if (icon != null) {
            return icon;
        } else {
            throw new Exception("The resquested Icon does not exist.");
        }
    }

    public Source getSource(int codesour) throws Exception {
        for (int i = 0; i < sources.size(); i++) {
            Source source = ((Source) sources.get(i));
            if (source.getCodesour() == codesour) {
                return source;
            }
        }

        synchronized (dataLock) {
            linkDataBase();
            StringBuilder sql = new StringBuilder();

            sql.append("SELECT codesour, tablsour, fielsour, geomtype, xminlaye, xmaxlaye, yminlaye, ymaxlaye, codeproj ");
            sql.append("FROM atlas27_source WHERE codesour = " + codesour);

            Statement sta = conn.createStatement();
            ResultSet res = sta.executeQuery(sql.toString());

            if (res.next()) {
                Source source = new Source();
                source.setCodesour(codesour);
                source.setFielsour(res.getString("fielsour"));
                source.setGeomType(res.getString("geomtype"));
                source.setTablsour(res.getString("tablsour"));
                source.setXmax(res.getDouble("xmaxlaye"));
                source.setXmin(res.getDouble("xminlaye"));
                source.setYmax(res.getDouble("ymaxlaye"));
                source.setYmin(res.getDouble("yminlaye"));
                source.loadFieldMetadata(conn);
                source.loadFeatures(conn);
                res.close();
                sta.close();
                return source;
            } else {
                res.close();
                sta.close();
                throw new Exception("The resquested source does not exist.");
            }
        }
    }

    public Layer getLayer(int code) throws Exception {
        synchronized (geomLock) {
            for (int i = 0; i < layers.size(); i++) {
                Layer layer = ((Layer) layers.get(i));
                if (layer.getCodelaye() == code) {
                    return (Layer) layer;
                }
            }

            synchronized (dataLock) {
                linkDataBase();
                StringBuilder sql = new StringBuilder();
                sql.append("SELECT rulefieldname, labelfieldname, codesour, desclaye, queryable, zorder, codesymb, typesymb, binalegend, fontname, fontstyle, fontsize, maxscale, priority, labeled");
                sql.append(" FROM atlas27_layer lay");
                sql.append(" INNER JOIN atlas27_symbology sym ON sym.codelaye = lay.codelaye");
                sql.append(" WHERE lay.codelaye = " + code);

                
                
                Statement sta = conn.createStatement();
                ResultSet res = sta.executeQuery(sql.toString());

                if (res.next()) {
                    Symbology sym = new Symbology();
                    sym.setCodesymb(res.getInt("codesymb"));
                    sym.setTypeSymb(res.getInt("typesymb"));
                    sym.setRuleFieldName(res.getString("rulefieldname"));
                    sym.setLabelFieldName(res.getString("labelfieldname"));
                    sym.setFontSize(res.getInt("fontsize"));
                    sym.setMaxScale(res.getInt("maxscale"));
                    sym.setPriority(res.getInt("priority"));
                    sym.setFontName(res.getString("fontname"));
                    sym.setFontStyle(res.getInt("fontstyle"));
                    sym.setLabeled(res.getBoolean("labeled"));
                    
                    Layer layer = new Layer();
                    layer.setCodelaye(code);

                    layer.setDesclaye(res.getString("desclaye"));
                    layer.setQueryable(res.getBoolean("queryable"));

                    layer.setZorder(res.getInt("zorder"));
                    layer.setSymbology(sym);
                    layer.setSource(getSource(res.getInt("codesour")));
                    layer.setBinalegend(res.getBytes("binalegend"));

                    res.close();

                    sql = new StringBuilder();

                    sql.append("SELECT coderule, binarule, value, max, min, descrule,");
                    sql.append(" sty.codestyl, sty.codestro, sty.codeicon, codelinecolo, codefillcolo,");
                    sql.append(" width, cap, joinstro, miterlimit, dashstring, dashphase,");
                    sql.append(" lin.red as redlin, lin.green as greelin, lin.blue as bluelin, lin.alfa as alfalin,");
                    sql.append(" fil.red as redfil, fil.green as greefil, fil.blue as bluefil, fil.alfa as alfafil");
                    sql.append(" FROM atlas27_rule rul");
                    sql.append(" INNER JOIN atlas27_style sty  ON rul.codestyl = sty.codestyl");
                    sql.append(" INNER JOIN atlas27_stroke str ON sty.codestro = str.codestro");
                    sql.append(" LEFT JOIN atlas27_color lin  ON sty.codelinecolo = lin.codecolo");
                    sql.append(" LEFT JOIN atlas27_color fil  ON sty.codefillcolo = fil.codecolo");
                    sql.append(" WHERE codesymb = " + sym.getCodesymb());

                    res = sta.executeQuery(sql.toString());
                    while (res.next()) {

                        AtlasColor line = null;
                        try {
                            line = new AtlasColor();
                            line.setCodecolo(res.getInt("codelinecolo"));
                            line.setRed(res.getInt("redlin"));
                            line.setGreen(res.getInt("greelin"));
                            line.setBlue(res.getInt("bluelin"));
                            line.setAlfa(res.getInt("alfalin"));
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }

                        AtlasColor fill = null;
                        try {
                            fill = new AtlasColor();
                            fill.setCodecolo(res.getInt("codefillcolo"));
                            fill.setRed(res.getInt("redfil"));
                            fill.setGreen(res.getInt("greefil"));
                            fill.setBlue(res.getInt("bluefil"));
                            fill.setAlfa(res.getInt("alfafil"));
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }

                        AtlasStroke str = null;
                        try {
                            str = new AtlasStroke();
                            str.setCodestro(res.getInt("codestro"));
                            str.setCap(res.getInt("cap"));
                            str.setDashString(res.getString("dashstring"));
                            str.setDashphase(res.getFloat("dashphase"));
                            str.setJoin(res.getInt("joinstro"));
                            str.setMiterlimit(res.getFloat("miterlimit"));
                            str.setWidth(res.getFloat("width"));
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }
                        
                        Style sty = new Style();
                        sty.setCodestyl(res.getInt("codestyl"));
                        
                        if (fill != null) {
                            sty.setFillcolo(fill);
                        }
                        if (line != null) {
                            sty.setLinecolo(line);
                        }
                        if (str != null) {
                            sty.setLinestyl(str);
                        }
                        
                        Rule rule = new Rule();

                        try {
                            rule.setBinarule(res.getBytes("binarule"));
                            rule.setCoderule(res.getInt("coderule"));
                            rule.setMax(res.getDouble("max"));
                            rule.setMin(res.getDouble("min"));
                            rule.setValue(res.getString("value"));
                            rule.setDescrule(res.getString("descrule"));
                        } catch (Exception ex) {
                            ex.printStackTrace();
                        }

                        sym.getRules().add(rule);
                        rule.setSymbology(sym);
                        rule.setStyle(sty);

                    }

                    res.close();
                    sta.close();

                    layer.loadFeatures(conn);
                    layers.add(layer);
                    return layer;

                } else {
                    res.close();
                    sta.close();
                    throw new Exception("The resquested layer does not exist.");
                }
            }
        }
    }

    public CRS getCRS(int EPSGCode) throws Exception {
        synchronized (crssLock) {
            for (int i = 0; i < crss.size(); i++) {
                CRS crs = ((CRS) crss.get(i));
                if (crs.getCode() == EPSGCode) {
                    return crs;
                }
            }

            synchronized (dataLock) {
                String sql = "SELECT srtext FROM atlas27_spatial_ref_sys WHERE srid = " + EPSGCode;

                Statement sta = null;
                ResultSet res = null;

                try {
                    linkDataBase();
                    sta = conn.createStatement();
                    res = sta.executeQuery(sql);
                    if (res.next()) {
                        //TODO duplicado en getJSONCapabilities()
                        CoordinateReferenceSystem gtcrs = org.geotools.referencing.CRS.parseWKT(res.getString("srtext"));
                        Unit un = gtcrs.getCoordinateSystem().getAxis(0).getUnit();

                        double fac = 0;
                        String type = "";
                        Unit sis = un.getSystemUnit();

                        if (sis.equals(SI.RADIAN)) {
                            type = "angular";
                            fac = un.getConverterTo(SI.RADIAN).convert(1.0d);
                        }

                        if (sis.equals(SI.METER)) {
                            type = "linear";
                            fac = un.getConverterTo(SI.METER).convert(1.0d);
                        }

                        String a1 = gtcrs.getCoordinateSystem().getAxis(0).getDirection().name().toLowerCase();
                        String a2 = gtcrs.getCoordinateSystem().getAxis(1).getDirection().name().toLowerCase();
                        CRS crs = new CRS(EPSGCode, gtcrs.getName().getCode(), a1, a2, un.toString(), fac, type);
                        crss.add(crs);
                        return crs;
                    } else {
                        throw new Exception("The resquested EPSG code does not exist.");
                    }
                } finally {
                    try {
                        sta.close();
                        res.close();
                    } catch (SQLException ex) {
                        ex.printStackTrace();
                    }
                }
            }
        }
    }

    private void linkDataBase() throws Exception {
        if (!DataBaseUtils.isValidConnection(conn)) {
            try {
                conn = DataBaseUtils.createConnection(cp);
            } catch (Exception ex) {
                throw ex;
            }
        }
    }

    public void invalidate() {
        synchronized (capaLock) {
            synchronized (geomLock) {
                wmsgc110 = null;
                wmsgc111 = null;
                wmsgc130 = null;
                wmsjson = null;
                serviceEnabled = null;
                layers = new ArrayList();
                sources = new ArrayList();
            }
        }
    }

    private void readServiceInfo(String version) throws Exception {
        synchronized (capaLock) {

            byte[] document = null;
            String sql = "";
            if (version.equals("1.1.0")) {
                document = wmsgc110;
                sql = "SELECT wmsgc1_1_0, publicaccess FROM atlas27_service";
            }

            if (version.equals("1.1.1")) {
                document = wmsgc111;
                sql = "SELECT wmsgc1_1_1, publicaccess FROM atlas27_service";
            }

            if (version.equals("1.3.0")) {
                document = wmsgc130;
                sql = "SELECT wmsgc1_3_0, publicaccess FROM atlas27_service";
            }

            if (version.equals("json")) {
                document = wmsjson;
                sql = "SELECT wmsjson, publicaccess FROM atlas27_service";
            }

            if (document == null) {
                synchronized (dataLock) {
                    linkDataBase();
                    try {

                        Statement sta = conn.createStatement();
                        ResultSet rta = sta.executeQuery(sql);
                        if (rta.next()) {
                            try {
                                document = rta.getString(1).getBytes();
                            } catch (Exception ex) {
                                document = null;
                            }

                            serviceEnabled = new Boolean(rta.getBoolean(2));
                            rta.close();
                            sta.close();
                        } else {
                            if (version.equals("1.1.0")) {
                                wmsgc110 = null;
                            }

                            if (version.equals("1.1.1")) {
                                wmsgc111 = null;
                            }

                            if (version.equals("1.3.0")) {
                                wmsgc130 = null;
                            }

                            if (version.equals("json")) {
                                wmsjson = null;
                            }

                            serviceEnabled = null;
                            throw new Exception("WMS Service information not set");
                        }

                        if (version.equals("1.1.0")) {
                            wmsgc110 = document;
                        }

                        if (version.equals("1.1.1")) {
                            wmsgc111 = document;
                        }

                        if (version.equals("1.3.0")) {
                            wmsgc130 = document;
                        }

                        if (version.equals("json")) {
                            wmsjson = document;
                        }

                    } catch (Exception ex) {
                        ex.printStackTrace();
                        if (version.equals("1.1.0")) {
                            wmsgc110 = null;
                        }

                        if (version.equals("1.1.1")) {
                            wmsgc111 = null;
                        }

                        if (version.equals("1.3.0")) {
                            wmsgc130 = null;
                        }

                        if (version.equals("json")) {
                            wmsjson = null;
                        }

                        serviceEnabled = null;
                        throw new Exception("Unexpected Error while reading WMS Service Information: " + ex.getMessage());
                    }

                }
            }
        }
    }

    public boolean isDataBaseReady() throws Exception {
        synchronized (dataLock) {
            try {
                linkDataBase();
                return true;
            } catch (Exception ex) {
                throw ex;
            }
        }
    }
}
