package org.atlas.svg;

import java.awt.Color;
import java.awt.geom.PathIterator;
import org.jdom.Element;

public class SvgStyle{
    
    private boolean fillNone  = false;
    private boolean strokeNone = false;
    
    private Color   fill;
    private Color   stroke;
    private Integer fill_rule;
    private Double  stroke_width;
    
    public Color getFill() {
        return fill;
    }
    
    public void setFill(Color fill) {
        this.fill = fill;
    }
    
    public Color getStroke() {
        return stroke;
    }
    
    public void setStroke(Color stroke) {
        this.stroke = stroke;
    }
    
    public Integer getFill_rule() {
        return fill_rule;
    }
    
    public void setFill_rule(Integer fill_rule) {
        this.fill_rule = fill_rule;
    }
    
    public Double getStroke_width() {
        return stroke_width;
    }
    
    public void setStroke_width(Double stroke_width) {
        this.stroke_width = stroke_width;
    }
    
    public boolean isFillNone() {
        return fillNone;
    }
    
    public void setFillNone(boolean fillNone) {
        this.fillNone = fillNone;
    }
    
    public boolean isStrokeNone() {
        return strokeNone;
    }
    
    public void setStrokeNone(boolean strokeNone) {
        this.strokeNone = strokeNone;
    }
    
    private static Color getColor(String part){
        part = part.trim().toLowerCase();
        if(part.startsWith("#")){
            String ac = part.substring(1,3);
            String bc = part.substring(3,5);
            String cc = part.substring(5,7);
            int r = Integer.decode("0x"+ac).intValue();
            int g = Integer.decode("0x"+bc).intValue();
            int b = Integer.decode("0x"+cc).intValue();
            return new Color(r,g,b);
        } else if(part.startsWith("rgb")){
            part = part.replaceAll("[^0-9,]","");
            String[] cols = part.split(",");
            int r = Integer.parseInt(cols[0]);
            int g = Integer.parseInt(cols[1]);
            int b = Integer.parseInt(cols[2]);
            return new Color(r,g,b);
        } else {
            Color c = SvgStyle.getSVGColor(part);
            if(c!=null)return c;
            System.out.println("unkwon color "+part);
            return c;
        }
    }
    
    public static void getStyle(Element e, SvgStyle stl){
        SvgStyle tmp = SvgStyle.readStyle(e);
        
        if(stl.getFill()==null&&!stl.isFillNone()){
            if(tmp.getFill()!=null||tmp.isFillNone()){
                stl.setFill(tmp.getFill());
                stl.setFillNone(tmp.isFillNone());
            }
        }
        
        
        if(stl.getStroke()==null&&!stl.isStrokeNone()){
            if(tmp.getStroke()!=null||tmp.isStrokeNone()){
                stl.setStroke(tmp.getStroke());
                stl.setStrokeNone(tmp.isStrokeNone());
            }
        }
        
        
        
        if(tmp.getStroke()!=null&&stl.getStroke()==null&&!stl.isStrokeNone()){
            stl.setStroke(tmp.getStroke());
            stl.setStrokeNone(tmp.isStrokeNone());
        }
        
        if(tmp.getFill_rule()!=null&&stl.getFill_rule()==null)
            stl.setFill_rule(tmp.getFill_rule());
        
        if(tmp.getStroke_width()!=null&&stl.getStroke_width()==null)
            stl.setStroke_width(tmp.getStroke_width());
        
        if(e.getParentElement()!=null){
            getStyle(e.getParentElement(), stl);
        }
    }
    
    public static SvgStyle readStyle(Element e){
        String stroke_width = null;
        String fill_rule    = null;
        String fill         = null;
        String stroke       = null;
        
        SvgStyle stl     = new SvgStyle();
        
        //attribute reading
        
        if(e.getAttributeValue("fill")!=null){
            fill = e.getAttributeValue("fill");
        }
        
        if(e.getAttributeValue("stroke")!=null){
            stroke       = e.getAttributeValue("stroke");
        }
        
        if(e.getAttributeValue("fill-rule")!=null){
            fill_rule    = e.getAttributeValue("fill-rule");
        }
        
        if(e.getAttributeValue("stroke-width")!=null){
            stroke_width = e.getAttributeValue("stroke-width");
        }
        
        //style reading
        
        String style = e.getAttributeValue("style");
        
        if(style!=null){
            style = style.replaceAll("[ ]{1,}","");
            if(!style.equals("")){
                String prts[] = style.split(";");
                for (int i = 0; i < prts.length; i++) {
                    String[] prt = prts[i].split(":");
                    if(prt[0].equals("fill")){
                        fill = prt[1];
                    }else if(prt[0].equals("stroke")){
                        stroke = prt[1];
                    }else if(prt[0].equals("fill-rule")){
                        fill_rule = prt[1];
                    }else if(prt[0].equals("stroke-width")){
                        stroke_width = prt[1];
                    }
                }
            }
        }
                
        //assignament
        
        if(fill != null){
            fill = fill.toLowerCase().trim();
            if(fill.equals("none")){
                stl.fillNone = true;
            }else{
                stl.setFill(SvgStyle.getColor(fill));
            }
        }
        
        
        if(stroke != null){
            stroke = stroke.toLowerCase().trim();
            if(stroke.equals("none")){
                stl.strokeNone = true;
            }else{
                stl.setStroke(SvgStyle.getColor(stroke));
            }
        }
        
        
        if(fill_rule!=null){
            if(fill_rule.equals("nonzero")){
                stl.setFill_rule(new Integer(PathIterator.WIND_NON_ZERO));
            }else if(fill_rule.equals("evenodd")){
                stl.setFill_rule(new Integer(PathIterator.WIND_EVEN_ODD));
            }
        }
        
        if(stroke_width!=null){
            stl.setStroke_width(new Double(Double.parseDouble((stroke_width.replaceAll("[^0-9.,]","")))));
        }
        return stl;
    }
    
    private static Color getSVGColor(String color){
        color = color.toLowerCase().trim();
        if(color.equals("aliceblue")){return new Color(240,248,255);} else if(color.equals("antiquewhite")){return new Color(250,235,215);} else if(color.equals("aqua")){return new Color(0,255,255);} else if(color.equals("aquamarine")){return new Color(127,255,212);} else if(color.equals("azure")){return new Color(240,255,255);} else if(color.equals("beige")){return new Color(245,245,220);} else if(color.equals("bisque")){return new Color(255,228,196);} else if(color.equals("black")){return new Color(0,0,0);} else if(color.equals("blanchedalmond")){return new Color(255,235,205);} else if(color.equals("blue")){return new Color(0,0,255);} else if(color.equals("blueviolet")){return new Color(138,43,226);} else if(color.equals("brown")){return new Color(165,42,42);} else if(color.equals("burlywood")){return new Color(222,184,135);} else if(color.equals("cadetblue")){return new Color(95,158,160);} else if(color.equals("chartreuse")){return new Color(127,255,0);} else if(color.equals("chocolate")){return new Color(210,105,30);} else if(color.equals("coral")){return new Color(255,127,80);} else if(color.equals("cornflowerblue")){return new Color(100,149,237);} else if(color.equals("cornsilk")){return new Color(255,248,220);} else if(color.equals("crimson")){return new Color(220,20,60);} else if(color.equals("cyan")){return new Color(0,255,255);} else if(color.equals("darkblue")){return new Color(0,0,139);} else if(color.equals("darkcyan")){return new Color(0,139,139);} else if(color.equals("darkgoldenrod")){return new Color(184,134,11);} else if(color.equals("darkgray")){return new Color(169,169,169);} else if(color.equals("darkgreen")){return new Color(0,100,0);} else if(color.equals("darkgrey")){return new Color(169,169,169);} else if(color.equals("darkkhaki")){return new Color(189,183,107);} else if(color.equals("darkmagenta")){return new Color(139,0,139);} else if(color.equals("darkolivegreen")){return new Color(85,107,47);} else if(color.equals("darkorange")){return new Color(255,140,0);} else if(color.equals("darkorchid")){return new Color(153,50,204);} else if(color.equals("darkred")){return new Color(139,0,0);} else if(color.equals("darksalmon")){return new Color(233,150,122);} else if(color.equals("darkseagreen")){return new Color(143,188,143);} else if(color.equals("darkslateblue")){return new Color(72,61,139);} else if(color.equals("darkslategray")){return new Color(47,79,79);} else if(color.equals("darkslategrey")){return new Color(47,79,79);} else if(color.equals("darkturquoise")){return new Color(0,206,209);} else if(color.equals("darkviolet")){return new Color(148,0,211);} else if(color.equals("deeppink")){return new Color(255,20,147);} else if(color.equals("deepskyblue")){return new Color(0,191,255);} else if(color.equals("dimgray")){return new Color(105,105,105);} else if(color.equals("dimgrey")){return new Color(105,105,105);} else if(color.equals("dodgerblue")){return new Color(30,144,255);} else if(color.equals("firebrick")){return new Color(178,34,34);} else if(color.equals("floralwhite")){return new Color(255,250,240);} else if(color.equals("forestgreen")){return new Color(34,139,34);} else if(color.equals("fuchsia")){return new Color(255,0,255);} else if(color.equals("gainsboro")){return new Color(220,220,220);} else if(color.equals("ghostwhite")){return new Color(248,248,255);} else if(color.equals("gold")){return new Color(255,215,0);} else if(color.equals("goldenrod")){return new Color(218,165,32);} else if(color.equals("gray")){return new Color(128,128,128);} else if(color.equals("green")){return new Color(0,128,0);} else if(color.equals("greenyellow")){return new Color(173,255,47);} else if(color.equals("grey")){return new Color(128,128,128);} else if(color.equals("honeydew")){return new Color(240,255,240);} else if(color.equals("hotpink")){return new Color(255,105,180);} else if(color.equals("indianred")){return new Color(205,92,92);} else if(color.equals("indigo")){return new Color(75,0,130);} else if(color.equals("ivory")){return new Color(255,255,240);} else if(color.equals("khaki")){return new Color(240,230,140);} else if(color.equals("lavender")){return new Color(230,230,250);} else if(color.equals("lavenderblush")){return new Color(255,240,245);} else if(color.equals("lawngreen")){return new Color(124,252,0);} else if(color.equals("lemonchiffon")){return new Color(255,250,205);} else if(color.equals("lightblue")){return new Color(173,216,230);} else if(color.equals("lightcoral")){return new Color(240,128,128);} else if(color.equals("lightcyan")){return new Color(224,255,255);} else if(color.equals("lightgoldenrodyellow")){return new Color(250,250,210);} else if(color.equals("lightgray")){return new Color(211,211,211);} else if(color.equals("lightgreen")){return new Color(144,238,144);} else if(color.equals("lightgrey")){return new Color(211,211,211);} else if(color.equals("lightpink")){return new Color(255,182,193);} else if(color.equals("lightsalmon")){return new Color(255,160,122);} else if(color.equals("lightseagreen")){return new Color(32,178,170);} else if(color.equals("lightskyblue")){return new Color(135,206,250);} else if(color.equals("lightslategray")){return new Color(119,136,153);} else if(color.equals("lightslategrey")){return new Color(119,136,153);} else if(color.equals("lightsteelblue")){return new Color(176,196,222);} else if(color.equals("lightyellow")){return new Color(255,255,224);} else if(color.equals("lime")){return new Color(0,255,0);} else if(color.equals("limegreen")){return new Color(50,205,50);} else if(color.equals("linen")){return new Color(250,240,230);} else if(color.equals("magenta")){return new Color(255,0,255);} else if(color.equals("maroon")){return new Color(128,0,0);} else if(color.equals("mediumaquamarine")){return new Color(102,205,170);} else if(color.equals("mediumblue")){return new Color(0,0,205);} else if(color.equals("mediumorchid")){return new Color(186,85,211);} else if(color.equals("mediumpurple")){return new Color(147,112,219);} else if(color.equals("mediumseagreen")){return new Color(60,179,113);} else if(color.equals("mediumslateblue")){return new Color(123,104,238);} else if(color.equals("mediumspringgreen")){return new Color(0,250,154);} else if(color.equals("mediumturquoise")){return new Color(72,209,204);} else if(color.equals("mediumvioletred")){return new Color(199,21,133);} else if(color.equals("midnightblue")){return new Color(25,25,112);} else if(color.equals("mintcream")){return new Color(245,255,250);} else if(color.equals("mistyrose")){return new Color(255,228,225);} else if(color.equals("moccasin")){return new Color(255,228,181);} else if(color.equals("navajowhite")){return new Color(255,222,173);} else if(color.equals("navy")){return new Color(0,0,128);} else if(color.equals("oldlace")){return new Color(253,245,230);} else if(color.equals("olive")){return new Color(128,128,0);} else if(color.equals("olivedrab")){return new Color(107,142,35);} else if(color.equals("orange")){return new Color(255,165,0);} else if(color.equals("orangered")){return new Color(255,69,0);} else if(color.equals("orchid")){return new Color(218,112,214);} else if(color.equals("palegoldenrod")){return new Color(238,232,170);} else if(color.equals("palegreen")){return new Color(152,251,152);} else if(color.equals("paleturquoise")){return new Color(175,238,238);} else if(color.equals("palevioletred")){return new Color(219,112,147);} else if(color.equals("papayawhip")){return new Color(255,239,213);} else if(color.equals("peachpuff")){return new Color(255,218,185);} else if(color.equals("peru")){return new Color(205,133,63);} else if(color.equals("pink")){return new Color(255,192,203);} else if(color.equals("plum")){return new Color(221,160,221);} else if(color.equals("powderblue")){return new Color(176,224,230);} else if(color.equals("purple")){return new Color(128,0,128);} else if(color.equals("red")){return new Color(255,0,0);} else if(color.equals("rosybrown")){return new Color(188,143,143);} else if(color.equals("royalblue")){return new Color(65,105,225);} else if(color.equals("saddlebrown")){return new Color(139,69,19);} else if(color.equals("salmon")){return new Color(250,128,114);} else if(color.equals("sandybrown")){return new Color(244,164,96);} else if(color.equals("seagreen")){return new Color(46,139,87);} else if(color.equals("seashell")){return new Color(255,245,238);} else if(color.equals("sienna")){return new Color(160,82,45);} else if(color.equals("silver")){return new Color(192,192,192);} else if(color.equals("skyblue")){return new Color(135,206,235);} else if(color.equals("slateblue")){return new Color(106,90,205);} else if(color.equals("slategray")){return new Color(112,128,144);} else if(color.equals("slategrey")){return new Color(112,128,144);} else if(color.equals("snow")){return new Color(255,250,250);} else if(color.equals("springgreen")){return new Color(0,255,127);} else if(color.equals("steelblue")){return new Color(70,130,180);} else if(color.equals("tan")){return new Color(210,180,140);} else if(color.equals("teal")){return new Color(0,128,128);} else if(color.equals("thistle")){return new Color(216,191,216);} else if(color.equals("tomato")){return new Color(255,99,71);} else if(color.equals("turquoise")){return new Color(64,224,208);} else if(color.equals("violet")){return new Color(238,130,238);} else if(color.equals("wheat")){return new Color(245,222,179);} else if(color.equals("white")){return new Color(255,255,255);} else if(color.equals("whitesmoke")){return new Color(245,245,245);} else if(color.equals("yellow")){return new Color(255,255,0);} else if(color.equals("yellowgreen")){return new Color(154,205,50);}
        return null;
    }
}
