/*
 * Created on 21 de julio de 2007, 11:18 AM
 */

package org.atlas.util;

import org.atlas.hibernatedata.Service;
import java.awt.geom.Point2D;
import org.jdom.Element;
import org.jdom.Namespace;

public class BoundingBox implements Cloneable{
    
    private int CRS;
    private double minx;
    private double miny;
    private double maxx;
    private double maxy;
    
    public BoundingBox(Element bBox){
        this(bBox, Service.VERSION_1_3_0);
    }
    
    public BoundingBox(Element bBox, int version){
        switch(version){
            case Service.VERSION_1_1_0:
                break;
            case Service.VERSION_1_1_1:
                setCRS(Integer.valueOf( bBox.getAttributeValue("SRS").substring(5,bBox.getAttributeValue("SRS").length()) ));
                break;
            case Service.VERSION_1_3_0:
                setCRS(Integer.valueOf( bBox.getAttributeValue("CRS").substring(5,bBox.getAttributeValue("CRS").length()) ));
                break;
        }
        
        setMinx(Double.valueOf(bBox.getAttributeValue("minx")));
        setMiny(Double.valueOf(bBox.getAttributeValue("miny")));
        setMaxx(Double.valueOf(bBox.getAttributeValue("maxx")));
        setMaxy(Double.valueOf(bBox.getAttributeValue("maxy")));
    }
    
    public BoundingBox(double minx, double miny, double maxx, double maxy) {
        this.setMinx(minx);
        this.setMiny(miny);
        this.setMaxx(maxx);
        this.setMaxy(maxy);
    }
    
    public BoundingBox(double minx, double miny, double maxx, double maxy, int CRS) {
        this.setCRS(CRS);
        this.setMinx(minx);
        this.setMiny(miny);
        this.setMaxx(maxx);
        this.setMaxy(maxy);
    }
    
    public double getMinx() {
        return minx;
    }
    
    public void setMinx(double minx) {
        this.minx = minx;
    }
    
    public double getMiny() {
        return miny;
    }
    
    public void setMiny(double miny) {
        this.miny = miny;
    }
    
    public double getMaxx() {
        return maxx;
    }
    
    public void setMaxx(double maxx) {
        this.maxx = maxx;
    }
    
    public double getMaxy() {
        return maxy;
    }
    
    public void setMaxy(double maxy) {
        this.maxy = maxy;
    }
    
    public int getCRS() {
        return CRS;
    }
    
    public void setCRS(int CRS) {
        this.CRS = CRS;
    }
          
    public double getArea(){
        return (getMaxx()-getMinx())*(getMaxy()-getMiny());
    }
    
    public Element getXMLCapability(Element root, String CRS, int version){
        
        Namespace ns = root.getNamespace();
        
        Element bBox = new Element("BoundingBox",ns);
         switch(version){
            case Service.VERSION_1_1_0:
                bBox.setAttribute("SRS",CRS);
                break;
            case Service.VERSION_1_1_1:
                bBox.setAttribute("SRS",CRS);
                break;
            case Service.VERSION_1_3_0:
                bBox.setAttribute("CRS",CRS);
                break;
        }
        
        bBox.setAttribute("minx",String.valueOf(this.getMinx()));
        bBox.setAttribute("miny",String.valueOf(this.getMiny()));
        bBox.setAttribute("maxx",String.valueOf(this.getMaxx()));
        bBox.setAttribute("maxy",String.valueOf(this.getMaxy()));
        
        return bBox;
    }
    
    public static BoundingBox getInitialBBox(String axis1, String axis2, double xmin, double ymin,double xmax, double ymax, int anchoI, int altoI){
        
        double anchoImg = (double)anchoI;
        double altoImg  = (double)altoI;
        double mapaI;
        double mapaJ;
        double centroI;
        double centroJ;
        boolean invert;
        
        if(axis1.equals("east")||axis1.equals("west")){
            invert = false;
            mapaI = (xmax-xmin);
            mapaJ = (ymax-ymin);
            centroI    = (xmax+xmin)/2;
            centroJ    = (ymax+ymin)/2;
        }else{
            invert = true;
            mapaI = (ymax-ymin);
            mapaJ = (xmax-xmin);
            centroI    = (ymax+ymin)/2;
            centroJ    = (xmax+xmin)/2;
        }
        
        double p1 = anchoImg/mapaI;
        double p2 = altoImg /mapaJ;
        
        if(p1>p2){
            //ajustar en ancho, mostrar todo el alto
            double mapaIC = (anchoImg/altoImg)*mapaJ;
            if(!invert){
                return new BoundingBox(centroI-(mapaIC/2),ymin,centroI+(mapaIC/2),ymax);
            }else{
                return new BoundingBox(xmin, centroI-(mapaIC/2),xmax, centroI+(mapaIC/2));
            }
        }else{
            //ajustar en alto, mostrar todo el ancho
            double mapaJC = (altoImg/anchoImg)*mapaI;
            if(!invert){
                return new BoundingBox(xmin, centroJ-(mapaJC/2),xmax, centroJ+(mapaJC/2));
            }else{
                return new BoundingBox(centroJ-(mapaJC/2),ymin,centroI+(mapaJC/2),ymax);
            }
        }
    }
    
    @Override
    public Object clone(){
        BoundingBox obj=null;
        try{
            obj=(BoundingBox)super.clone();
        }catch(CloneNotSupportedException ex){
            ex.printStackTrace();
        }
        return obj;
    }
    
     /**
     * El centro geométrico del bounding box
     * @return un punto en el espacio
     */
    public Point2D getCenter(){
        return new Point2D.Double((maxx+minx)/2, (maxy+miny)/2);
    }
}
