package org.atlas.util;

import java.awt.Color;
import java.util.Arrays;
import java.util.Comparator;

public class ColorsList {
    
    private Color[] colors;
    
    public ColorsList() {
        
        int min = 180;
        int max = 240;
        int n   = 5;
        
        int[] vals = new int[n];
        
        vals[0] = min;
        double esp = (max-min)/(n-1);
        for (int i = 1; i < vals.length; i++) {
            vals[i] = (int)(vals[i-1]+esp);
        }
        
        colors = new Color[(int)Math.pow(vals.length,3)];
        int i = 0;
        for (int r = 0; r < vals.length; r++) {
            for (int g = 0; g < vals.length; g++) {
                for (int b = 0; b < vals.length; b++, i++) {
                    colors[i] = new Color(vals[r], vals[g], vals[b]);
                }
            }
        }
        
        Arrays.sort(colors, new Comparator(){
            public int compare(Object o1, Object o2) {
                Color c1 = (Color)o1;
                Color c2 = (Color)o2;
                
                double  l1 =  (c1.getRed()+c1.getGreen()+c1.getBlue())/3;
                double  l2 =  (c2.getRed()+c2.getGreen()+c2.getBlue())/3;
                
                return Double.compare(l2,l1);

            }
        });
    }
    
    public Color getColor(int i){
        int in = (int)(i-(Math.floor(i/colors.length)*colors.length));
        return colors[in];
    }
    
    public Color getDarkenColor(int i){
        int in = (int)(i-(Math.floor(i/colors.length)*colors.length));
        return getDarkenColor(colors[in]);
    }
    
    public static Color getDarkenColor(Color c){
        double k = 0.85;
        int rb = (int) Math.min((double) c.getRed() * k, 255.0d);
        int gb = (int) Math.min((double) c.getGreen() * k, 255.0d);
        int bb = (int) Math.min((double) c.getBlue() * k, 255.0d);
        return new Color(rb, gb, bb);
    }
    
}
