package org.atlas.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import javax.swing.UIManager;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * @author Mario R Eraso
 */
public class Configuration {
    
    private List    connections;
    private String  lookAndFeel;
    private String  lastGeomDirectory;
    private String  lastIconDirectory;
    private Locale  locale;
    private int     currentConnection;
    
    public Configuration() {
        this.setConnections(new ArrayList());
        this.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        this.setLastGeomDirectory("");
        this.setLastIconDirectory("");
        this.setLocale(Locale.ROOT);
        this.setCurrentConnection(-1);
    }
    
    
    public void writeConfiguration(String path) throws FileNotFoundException, IOException  {
        Document document = new Document();
        
        Element  config   = new Element("atlasConfig");
        document.addContent(config);
        
        Element laf = new Element("lookAndFeel");
        laf.setText(String.valueOf(this.getLookAndFeel()));
        config.addContent(laf);
        
        Element lastGeomDir = new Element("lastGeomDir");
        lastGeomDir.setAttribute("path",this.getLastGeomDirectory());
        config.addContent(lastGeomDir);
        
        Element lastIconDir = new Element("lastIconDir");
        lastIconDir.setAttribute("path",this.getLastIconDirectory());
        config.addContent(lastIconDir);
        
        Element ilocale = new Element("locale");
        ilocale.setAttribute("laguaje",this.getLocale().getLanguage());
        ilocale.setAttribute("country",this.getLocale().getCountry());
        config.addContent(ilocale);
        
        Element currentcon = new Element("currentcon");
        currentcon.setText(String.valueOf(this.getCurrentConnection()));
        config.addContent(currentcon);
        
        Element iconnections = new Element("connections");
        
        for (int i = 0; i < this.getConnections().size(); i++) {
            Element connection = new Element("connection");
            ServiceConnection con = (ServiceConnection)this.getConnections().get(i);
            
            Element name       = new Element("name").setText(con.getName());
            connection.addContent(name);
            Element url        = new Element("url").setText(con.getAppUrl());
            connection.addContent(url);
            Element user       = new Element("username").setText(con.getUsername());
            connection.addContent(user);
            Element pass       = new Element("password").setText(con.getPassword());
            connection.addContent(pass);
            
            iconnections.addContent(connection);
        }
        
        config.addContent(iconnections);
        XMLOutputter outp = new XMLOutputter(Format.getPrettyFormat());
        FileOutputStream fos = new FileOutputStream(new File(path));
        outp.output(document,fos);
        fos.close();
    }
    
    
    public static Configuration readConfiguration(File f) throws JDOMException, IOException  {
        Configuration conf = new Configuration();
        
        SAXBuilder builder = new SAXBuilder();
        Document doc = builder.build(f);
        
        Element  config   = doc.getRootElement();
        
        conf.setLookAndFeel(config.getChild("lookAndFeel").getText());
        conf.setLastGeomDirectory(config.getChild("lastGeomDir").getAttribute("path").getValue());
        conf.setLastIconDirectory(config.getChild("lastIconDir").getAttribute("path").getValue());
        
        String len = config.getChild("locale").getAttribute("laguaje").getValue();
        String con = config.getChild("locale").getAttribute("country").getValue();
        
        conf.setLocale(new Locale(len,con));
        conf.setCurrentConnection(Integer.parseInt(config.getChild("currentcon").getText()));
        
        List connections = config.getChild("connections").getChildren();
        
        for (int i = 0; i < connections.size(); i++) {
            Element connection = (Element)connections.get(i);
            String name         = connection.getChild("name").getValue();
            String url          = connection.getChild("url").getValue();
            String user         = connection.getChild("username").getValue();
            String pass         = connection.getChild("password").getValue();
            
            ServiceConnection sc = new ServiceConnection(name, url, user, pass);
            conf.getConnections().add(sc);
        }
        return conf;
    }
    
    public static Configuration grantReadConfiguration(String path){
        Configuration con = null;
        try {
            con = Configuration.readConfiguration(new File(path));
        } catch (Exception ex){
            con = new Configuration();
        }
        return con;
    }
    
    public List getConnections() {
        return connections;
    }
    
    public void setConnections(List connections) {
        this.connections = connections;
    }
    
    public String getLookAndFeel() {
        return lookAndFeel;
    }
    
    public void setLookAndFeel(String lookAndFeel) {
        this.lookAndFeel = lookAndFeel;
    }
    
    public Locale getLocale() {
        return locale;
    }
    
    public void setLocale(Locale locale) {
        this.locale = locale;
    }
    
    public int getCurrentConnection() {
        return currentConnection;
    }
    
    public void setCurrentConnection(int currentConnection) {
        this.currentConnection = currentConnection;
    }

    public String getLastGeomDirectory() {
        return lastGeomDirectory;
    }

    public void setLastGeomDirectory(String lastGeomDirectory) {
        this.lastGeomDirectory = lastGeomDirectory;
    }

    public String getLastIconDirectory() {
        return lastIconDirectory;
    }

    public void setLastIconDirectory(String lastIconDirectory) {
        this.lastIconDirectory = lastIconDirectory;
    }

    public static String getWorkingDirectory(){
        return System.getProperty("user.dir")+File.separator;
    }
}
