/*
 * Created on 31 de julio de 2007, 04:35 PM
 */

package org.atlas.util;

import org.atlas.clientdata.BBox;
import org.atlas.clientdata.EpsgCRS;
import java.awt.geom.Point2D;
import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Set;
import org.geotools.geometry.GeneralDirectPosition;
import org.geotools.referencing.CRS;
import org.geotools.referencing.FactoryFinder;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.CoordinateOperation;
import org.opengis.referencing.operation.CoordinateOperationFactory;
import org.opengis.referencing.operation.MathTransform;
import org.opengis.referencing.operation.OperationNotFoundException;
import org.opengis.referencing.operation.TransformException;
import org.opengis.spatialschema.geometry.DirectPosition;
import org.opengis.spatialschema.geometry.MismatchedDimensionException;

public class CoordinateUtils {
    
    public CoordinateUtils() {
    }
    
    public static DirectPosition tranformsCoordinatetoWGS84(String sourceSrtext, GeneralDirectPosition position){
        String targetSrtext = "GEOGCS[\"WGS 84\",DATUM[\"WGS_1984\",SPHEROID[\"WGS 84\",6378137,298.257223563,AUTHORITY[\"EPSG\",\"7030\"]],TOWGS84[0,0,0,0,0,0,0],AUTHORITY[\"EPSG\",\"6326\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.01745329251994328,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4326\"]]";
        return tranformsCoordinate(sourceSrtext, targetSrtext, position);
    }
    
    public static DirectPosition tranformsCoordinate(String sourceSrtext, String  targetSrtext, GeneralDirectPosition position){
        
        CoordinateOperationFactory coFactory = FactoryFinder.getCoordinateOperationFactory(null);
        CoordinateReferenceSystem source;
        CoordinateReferenceSystem target;
        CoordinateOperation operation;
        
        try {
            source = CRS.parseWKT(sourceSrtext);
            target = CRS.parseWKT(targetSrtext);
            
            operation = coFactory.createOperation(source, target);
            MathTransform transform = operation.getMathTransform();
            position.setCoordinateReferenceSystem(source);
            GeneralDirectPosition res = (GeneralDirectPosition)transform.transform(position, null);
            res.setCoordinateReferenceSystem(target);
            return res;
        } catch (MismatchedDimensionException ex) {
            ex.printStackTrace();
        } catch (OperationNotFoundException ex) {
            ex.printStackTrace();
        } catch (TransformException ex) {
            ex.printStackTrace();
        } catch (FactoryException ex) {
            ex.printStackTrace();
        }
        return null;
    }
    
    public static Point2D translateIJtoPoint(BoundingBox bbox, int dimI, int dimJ, int coordI, int coordJ, String axis1, String axis2){
        double[] p = translateIJtoXY(bbox, dimI, dimJ, coordI, coordJ, axis1, axis2);
        return new Point2D.Double(p[0], p[1]);
    }
    
    public static Point2D translateIJtoPoint(BBox bbox, int dimI, int dimJ, int coordI, int coordJ, EpsgCRS crs){
        double[] p = translateIJtoXY(bbox, dimI, dimJ, coordI, coordJ, crs);
        return new Point2D.Double(p[0], p[1]);
    }
    
    public static double[] translateIJtoXY(BoundingBox bbox, int dimI, int dimJ, int coordI, int coordJ, String axis1, String axis2){
        
        double imin = Double.NaN;
        double jmin = Double.NaN;
        double imax = Double.NaN;
        double jmax = Double.NaN;
        
        double porcI = (double)coordI/dimI;
        double porcJ = (double)coordJ/dimJ;
        
        double x;
        double y;
        if(axis1.equals("east")||axis1.equals("west")){
            
            if(axis1.equals("east")){
                imin = bbox.getMinx();
                imax = bbox.getMaxx();
            }
            
            if(axis1.equals("west")){
                imin = bbox.getMaxx();
                imax = bbox.getMinx();
            }
            
            if(axis2.equals("north")){
                jmin = bbox.getMaxy();
                jmax = bbox.getMiny();
            }
            
            if(axis2.equals("south")){
                jmin = bbox.getMiny();
                jmax = bbox.getMaxy();
            }
            
            x = imin + (imax-imin)*porcI;
            y = jmin + (jmax-jmin)*porcJ;
        }else{
            
            if(axis2.equals("east")){
                imin = bbox.getMiny();
                imax = bbox.getMaxy();
            }
            
            if(axis2.equals("west")){
                imin = bbox.getMaxy();
                imax = bbox.getMiny();
            }
            
            if(axis1.equals("north")){
                jmin = bbox.getMaxx();
                jmax = bbox.getMinx();
            }
            
            if(axis1.equals("south")){
                jmin = bbox.getMinx();
                jmax = bbox.getMaxx();
            }
            
            x = jmin + (jmax-jmin)*porcJ;
            y = imin + (imax-imin)*porcI;
        }
        
        return new double[]{x,y};
    }
    
    public static double[] translateIJtoXY(BBox bbox, int dimI, int dimJ, int coordI, int coordJ, EpsgCRS crs){
        
        double imin = Double.NaN;
        double jmin = Double.NaN;
        double imax = Double.NaN;
        double jmax = Double.NaN;
        
        double porcI = (double)coordI/dimI;
        double porcJ = (double)coordJ/dimJ;
        
        String axis1 = crs.getAxis1();
        String axis2 = crs.getAxis2();
        
        double x;
        double y;
        if(axis1.equals("east")||axis1.equals("west")){
            
            if(axis1.equals("east")){
                imin = bbox.getMinx();
                imax = bbox.getMaxx();
            }
            
            if(axis1.equals("west")){
                imin = bbox.getMaxx();
                imax = bbox.getMinx();
            }
            
            if(axis2.equals("north")){
                jmin = bbox.getMaxy();
                jmax = bbox.getMiny();
            }
            
            if(axis2.equals("south")){
                jmin = bbox.getMiny();
                jmax = bbox.getMaxy();
            }
            
            x = imin + (imax-imin)*porcI;
            y = jmin + (jmax-jmin)*porcJ;
        }else{
            
            if(axis2.equals("east")){
                imin = bbox.getMiny();
                imax = bbox.getMaxy();
            }
            
            if(axis2.equals("west")){
                imin = bbox.getMaxy();
                imax = bbox.getMiny();
            }
            
            if(axis1.equals("north")){
                jmin = bbox.getMaxx();
                jmax = bbox.getMinx();
            }
            
            if(axis1.equals("south")){
                jmin = bbox.getMinx();
                jmax = bbox.getMaxx();
            }
            
            x = jmin + (jmax-jmin)*porcJ;
            y = imin + (imax-imin)*porcI;
        }
        
        return new double[]{x,y};
    }
    
    public static Point2D translateXYtoIJ(org.atlas.clientdata.EpsgCRS crs, org.atlas.clientdata.BBox bbox, int width, int heigth, double x, double y){
        boolean invertI;
        boolean invertJ;
        boolean swap;
        
        String axis1 = crs.getAxis1();
        String axis2 = crs.getAxis2();
        
        double factI;
        double factJ;
        
        double xmin = bbox.getMinx();
        double ymin = bbox.getMiny();
        double xmax = bbox.getMaxx();
        double ymax = bbox.getMaxy();
        
        double minI;
        double minJ;
        
        if(axis1.endsWith("west")||axis1.endsWith("east")){
            swap = false;
            factI = width/(xmax-xmin);
            factJ = heigth/(ymax-ymin);
            minI  = (xmin*factI);
            minJ  = (ymin*factJ);
            if(axis1.equals("east")){
                invertI = false;
            }else{
                invertI = true;
            }
            
            if(axis2.equals("south")){
                invertJ = false;
            }else{
                invertJ = true;
            }
        }else{
            swap = true;
            factI = width/(ymax-ymin);
            factJ = heigth/(xmax-xmin);
            minI  = (ymin*factI);
            minJ  = (xmin*factJ);
            
            if(axis2.equals("east")){
                invertJ = false;
            }else{
                invertJ = true;
            }
            if(axis1.equals("south")){
                invertI = false;
            }else{
                invertI = true;
            }
            
        }
        
        
        int[] res = new int[2];
        if(swap){
            if(invertI){
                res[0] = (int)(width + minI - (y*factI));
            }else{
                res[0] = (int)((y*factI) - minI);
            }
            
            if(invertJ){
                res[1] = (int)(heigth + minJ - (x*factJ));
            }else{
                res[1] = (int)((x*factJ) - minJ);
            }
        }else{
            if(invertI){
                res[0] = (int)(width + minI - (x*factI));
            }else{
                res[0] = (int)((x*factI) - minI);
            }
            
            if(invertJ){
                res[1] = (int)(heigth + minJ - (y*factJ));
            }else{
                res[1] = (int)((y*factJ) - minJ);
            }
        }
        return new Point2D.Double(res[0], res[1]);
    }
    
    public static void main(String[] args) throws Exception{
        
        
        ArrayList units = new ArrayList();
        
        Set s = CRS.getSupportedCodes("EPSG");
        Object[] codes = s.toArray();
        
        File f = new File("codes.txt");
        FileOutputStream fos = new FileOutputStream(f);
        int ok = 0;
        
        for (int i = 0; i < codes.length; i++) {
            try {
                CoordinateReferenceSystem c = CRS.decode("EPSG:"+codes[i]);
                if(c.getCoordinateSystem().getDimension()==2){
                    String dir1 = c.getCoordinateSystem().getAxis(0).getDirection().name().toLowerCase();
                    String dir2 = c.getCoordinateSystem().getAxis(1).getDirection().name().toLowerCase();
                    
                    String wkt = c.toWKT().replaceAll("\n"," ");
                    wkt = wkt.replaceAll("[ ]{1,}"," ");
                    wkt = wkt.replaceAll("[']{1,}"," ");
                    String sql = "INSERT INTO ATLAS27_SPATIAL_REF_SYS VALUES ("+codes[i]+", '"+wkt+"');\n";
                    
                    if(dir1.equals("east")&&dir2.equals("north")){
                        //ok++;
                    }else if(dir1.equals("north")&&dir2.equals("east")){
                        //ok++;
                    }else if(dir1.equals("west")&&dir2.equals("south")){
                        ok++;
                    }else{
                        System.out.println(codes[i]);
                        System.out.println(c.getName().getCode());
                    }
                    
                }
            } catch (Exception ex) {
                //System.out.println(codes[i]+" error: "+ex.getMessage());
            }
        }
        System.out.println(ok);
        
    }
}
