package org.atlas.util;

import java.io.IOException;
import java.net.ConnectException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;

public class DataBaseUtils {

    public static Connection createConnection(ConnectionProperties p) throws IOException, SQLException, ClassNotFoundException, ConnectException {
        Socket socket;
        socket = new Socket();
        socket.connect(new InetSocketAddress(p.getHost(), Integer.parseInt(p.getPort())), 3000);
        socket.close();
        Class.forName(p.getDriver());
        Connection con = DriverManager.getConnection(p.getURL(), p.getUser(), p.getPass());
        return con;
    }

    public static void executeFile(String classpath, Connection con) throws SQLException, IOException {
        String cad = IOUtils.readStringFromClassPath(classpath);
        String[] cads = cad.split("\n");
        Statement st = con.createStatement();
        for (int i = 0; i < cads.length; i++) {
            st.execute(cads[i]);
        }
    }

    public static boolean getTableExists(String tblName, Connection con) throws SQLException {
        ResultSet rs = con.getMetaData().getTables(con.getCatalog(), null, null, null);
        String tbl = tblName.toLowerCase();
        while (rs.next()) {
            if (rs.getString(3).toLowerCase().equals(tbl)) {
                return true;
            }
        }
        return false;
    }

    public static void removeTable(String tblName, Connection con) throws SQLException {
        String sql = "DROP TABLE \"" + tblName + "\"";
        con.createStatement().executeUpdate(sql);
    }

    public static boolean isValidConnection(Connection con) {
        Statement sta = null;
        //System.out.println("Testing Connection....");
        if (con == null) {
            //System.out.println("Null Connection....");
            return false;
        }
        try {
            sta = con.createStatement();
            sta.executeQuery("SELECT 1");
            sta.close();
            //System.out.println("Test Passed");
            return true;
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        //System.out.println("Test Failed");
        return false;
    }

    public static boolean isNumeric(int type) {
        return type == Types.BIGINT || type == Types.DECIMAL || type == Types.DOUBLE || type == Types.FLOAT || type == Types.INTEGER || type == Types.NUMERIC || type == Types.REAL || type == Types.TINYINT;
    }

    public static boolean isText(int type) {
        return type == Types.CHAR || type == Types.CLOB || type == Types.LONGNVARCHAR || type == Types.LONGVARCHAR || type == Types.NVARCHAR || type == Types.VARCHAR;
    }

    public static String getSqlTypeString(int code) {
        if (code == 2003) {
            return "ARRAY";
        }
        if (code == -5) {
            return "BIGINT";
        }
        if (code == -2) {
            return "BINARY";
        }
        if (code == -7) {
            return "BIT";
        }
        if (code == 2004) {
            return "BLOB";
        }
        if (code == 16) {
            return "BOOLEAn ";
        }
        if (code == 1) {
            return "CHAR";
        }
        if (code == 2005) {
            return "CLOB";
        }
        if (code == 70) {
            return "DATALINK";
        }
        if (code == 91) {
            return "DATE";
        }
        if (code == 3) {
            return "DECIMAL";
        }
        if (code == 2001) {
            return "DISTINCT";
        }
        if (code == 8) {
            return "DOUBLE";
        }
        if (code == 6) {
            return "FLOAT";
        }
        if (code == 4) {
            return "INTEGER";
        }
        if (code == 2000) {
            return "JAVA_OBJECT";
        }
        if (code == -16) {
            return "LONGNVARCHAR";
        }
        if (code == -4) {
            return "LONGVARBINARY";
        }
        if (code == -1) {
            return "LONGVARCHAR";
        }
        if (code == -15) {
            return "NCHAR";
        }
        if (code == 2011) {
            return "NCLOB";
        }
        if (code == 0) {
            return "NULL";
        }
        if (code == 2) {
            return "NUMERIC";
        }
        if (code == -9) {
            return "NVARCHAR";
        }
        if (code == 1111) {
            return "OTHER";
        }
        if (code == 7) {
            return "REAL";
        }
        if (code == 2006) {
            return "REF";
        }
        if (code == -8) {
            return "ROWID";
        }
        if (code == 5) {
            return "SMALLINT";
        }
        if (code == 2009) {
            return "SQLXML";
        }
        if (code == 2002) {
            return "STRUCT";
        }
        if (code == 92) {
            return "TIME";
        }
        if (code == 93) {
            return "TIMESTAMP";
        }
        if (code == -6) {
            return "TINYINT";
        }
        if (code == -3) {
            return "VARBINARY";
        }
        if (code == 12) {
            return "VARCHAR";
        }
        return "";
    }
}
