
package org.atlas.util;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.classic.Session;
import org.hibernate.tool.hbm2ddl.SchemaExport;
import org.hibernate.tool.hbm2ddl.SchemaValidator;

public class HibernateUtil {
    
    private SessionFactory       sessions  = null;
    private Configuration        config;
    private ConnectionPropertiesHb cp;
    
    private static final int MAX_RETRY = 15;
    private static final int DELAY     = 500;
    
    private static final int TIME_OUT  = 3000;
    private Object hibernatedata;
        
    public HibernateUtil(ConnectionPropertiesHb cp) throws SQLException, ClassNotFoundException, IOException, Exception{
        this(cp,false,false);
    }
    
    public HibernateUtil(ConnectionPropertiesHb cp, boolean format, boolean showSQL) throws SQLException, ClassNotFoundException, IOException, Exception{
        Socket socket = null;
        try {
            socket = new Socket();
            socket.connect(new InetSocketAddress(cp.getHost(),Integer.parseInt(cp.getPort())), TIME_OUT);
        } finally{
            try {
                socket.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        
        Properties p = new Properties();        
        
        p.setProperty("hibernate.connection.driver_class",cp.getDriver());
        p.setProperty("hibernate.connection.url",cp.getURL());
        p.setProperty("hibernate.dialect",cp.getDialect().getClass().getCanonicalName());
        
        //System.out.println(cp.getDialect().toString());
        p.setProperty("hibernate.connection.username",cp.getUser());
        p.setProperty("hibernate.connection.password",cp.getPass());
        p.setProperty("hibernate.cache.provider_class","org.hibernate.cache.NoCacheProvider");
        
        p.setProperty("hibernate.c3p0.acquire_increment","1");
        p.setProperty("hibernate.c3p0.idle_test_period","100");
        p.setProperty("hibernate.c3p0.timeout","100");
        p.setProperty("hibernate.c3p0.max_size","100");
        p.setProperty("hibernate.c3p0.max_statements","0");
        p.setProperty("hibernate.c3p0.min_size","10");
        
//        p.setProperty("hibernate.c3p0.min_size","1");
//        p.setProperty("hibernate.c3p0.max_size","1");
//        p.setProperty("hibernate.c3p0.timeout","1800");
//        p.setProperty("hibernate.c3p0.max_statements","50");
        
        /*if(format)
            p.setProperty("hibernate.show_sql","true");
        if(showSQL)
            p.setProperty("hibernate.format_sql","true");*/
        
        
        
        //p.setProperty("hibernate.show_sql","true");
       // p.setProperty("hibernate.format_sql","true");
        
        p.setProperty("hibernate.current_session_context_class","thread");
        Configuration c = new Configuration();
        c.setProperties(p);
        c.addClass(org.atlas.hibernatedata.AtlasColor.class);
        c.addClass(org.atlas.hibernatedata.AtlasStroke.class);
        c.addClass(org.atlas.hibernatedata.Icon.class);
        c.addClass(org.atlas.hibernatedata.Layer.class);
        c.addClass(org.atlas.hibernatedata.GeocoderConfig.class);
        c.addClass(org.atlas.hibernatedata.Project.class);
        c.addClass(org.atlas.hibernatedata.Source.class);
        c.addClass(org.atlas.hibernatedata.Service.class);
        c.addClass(org.atlas.hibernatedata.Spatial_ref_sys.class);
        c.addClass(org.atlas.hibernatedata.Style.class);
        c.addClass(org.atlas.hibernatedata.Symbology.class);
        c.addClass(org.atlas.hibernatedata.Rule.class);
        
        this.config = c;
        this.cp = cp;
        this.buildSession();
    }
    
    public void validateSchema() throws Exception{
        SchemaValidator validator = null;
        validator = new SchemaValidator(this.config);
        validator.validate();
    }
    
    public void dropSchema(){
        SchemaExport    export    = new SchemaExport(config);
        export = export.setHaltOnError(false);
        export.execute(false,true,true,false);
        System.out.println(export.getExceptions().size());
        for (int i = 0; i < export.getExceptions().size(); i++) {
            System.out.println(((Exception)export.getExceptions().get(i)).getMessage());
        }
    }
    
    public void createSchema() throws Exception{
        SchemaExport    export    = new SchemaExport(config);
        export.setHaltOnError(true);
        export.execute(false,true,false,true);
        if(export.getExceptions().size()>0){
            throw (Exception)export.getExceptions().get(0);
        }
    }
    
    public Session begin() throws Exception{
        boolean retry = true;
        Session sess = null;
        for (int i = 0; i < HibernateUtil.MAX_RETRY && retry; i++) {
            try{
                System.out.println("Testing Network");
                Socket socket;
                socket = new Socket();
                socket.connect(new InetSocketAddress(cp.getHost(),Integer.parseInt(cp.getPort())), TIME_OUT);
                socket.close();
                System.out.println("Trying to start session");
                sess = sessions.openSession();
                sess.beginTransaction();
                System.out.println("Session begin success");
                retry = false;
            }catch(Exception ex){
                retry = true;
                System.out.println("Session begin failure");
                try {
                    Thread.sleep(250);
                } catch (InterruptedException ex1) {
                    ex1.printStackTrace();
                }
            }
        }
        if(retry){
            throw new Exception("Unable to connect with the database");
        }
        return sess;
    }
    
    public void begin(Session sess) throws Exception{
        sess.beginTransaction();
    }
    
    public void commitAndClose(Session s) throws Exception{
        try {
            s.getTransaction().commit();
            s.close();
        } catch (Exception ex) {
            System.err.println("Unable to commit");
            ex.printStackTrace();
            throw ex;            
        }
    }
    
    public void commit(Session s) throws Exception{
        s.getTransaction().commit();
    }
    
    public void close(Session s){
        s.close();
        System.out.println("Session closed");
    }
    
    public Connection getConnection() throws Exception{
        boolean retry = true;
        Connection con = null;
        for (int i = 0; i < HibernateUtil.MAX_RETRY && retry; i++) {
            try{
                System.out.println("Trying to start connection");
                con = DataBaseUtils.createConnection(cp);
                System.out.println("Connection create success");
                retry = false;
            }catch(Exception ex){
                retry = true;
                System.out.println("Connection create failure");
                try {
                    Thread.sleep(HibernateUtil.DELAY);
                } catch (InterruptedException ex1) {
                    ex1.printStackTrace();
                }
            }
        }
        if(retry){
            throw new Exception("Unable to connect with the database");
        }
        return con;
    }
    
    
    private Connection buildSession() throws Exception{
        boolean retry = true;
        Connection con = null;
        for (int i = 0; i < HibernateUtil.MAX_RETRY && retry; i++) {
            try{
                System.out.println("Building Session Factory: ");
                this.sessions = this.config.buildSessionFactory();
                System.out.print("OK");
                retry = false;
            }catch(Exception ex){
                ex.printStackTrace();
                retry = true;
                System.out.print("Fatal");
                try {
                    Thread.sleep(HibernateUtil.DELAY);
                } catch (InterruptedException ex1) {
                    ex1.printStackTrace();
                }
            }
        }
        if(retry){
            throw new Exception("Unable to build session factory");
        }
        return con;
    }
    
    public void terminate(){
        sessions.close();
    }
    
    public ConnectionPropertiesHb getConnectionProperties(){
        return this.cp;
    }
}