package org.atlas.util;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * 
 * @author Mario
 */
public class IOUtils {

    /**
     * 
     * @param classpath
     * @return
     * @throws java.io.IOException
     */
    public static byte[] readFromClassPath(String classpath) throws IOException {
        InputStream is = System.class.getResourceAsStream(classpath);
        return readFromStream(is);
    }

    /**
     * 
     * @param classpath
     * @return
     * @throws java.io.IOException
     */
    public static String readStringFromClassPath(String classpath) throws IOException {
        return new String(IOUtils.readFromClassPath(classpath));
    }

    /**
     * 
     * @param in
     * @return
     * @throws java.io.IOException
     */
    public static byte[] readFromStream(InputStream in) throws IOException {
        BufferedInputStream bi = new BufferedInputStream(in);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        int i;
        while((i=bi.read())!=-1){
            baos.write(i);
        }
        bi.close();
        return baos.toByteArray();
    }
    
    /**
     * 
     * @param bytes
     * @param os
     * @throws java.io.IOException
     */
    public static void writeToStream(byte[] bytes, OutputStream os) throws IOException{
        BufferedOutputStream bos = new BufferedOutputStream(os);
        for (int i = 0; i < bytes.length; i++) {
            bos.write(bytes[i]);
        }
        bos.close();
    }
    
    /**
     * 
     * @param url
     * @return
     * @throws java.net.MalformedURLException
     * @throws java.io.IOException
     */
    public static byte[] readFromURL(String url) throws MalformedURLException, IOException {
        URL u = new URL(url);
        HttpURLConnection con = (HttpURLConnection) u.openConnection();
        return readFromStream(con.getInputStream());
    }

    /**
     * 
     * @param url
     * @param knowLength
     * @return
     * @throws java.net.MalformedURLException
     * @throws java.io.IOException
     */
    public static byte[] readFromURL(String url, boolean knowLength) throws MalformedURLException, IOException {
        URL u = new URL(url);
        HttpURLConnection con = (HttpURLConnection) u.openConnection();
        if (!knowLength) {
            return readFromStream(con.getInputStream());
        } else {
            InputStream is = con.getInputStream();
            byte[] res = readFromStream(is);
            is.close();
            return res;
        }
    }
}
