package org.atlas.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.jar.Attributes.Name;
import java.util.jar.Manifest;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

public class JarFile {

    public static final int REMOVE_FROM_CLASSPATH = 1;
    public static final int ADD_TO_CLASSPATH      = 2;

    public static void editClassPath(int operation, String jar, String entry) throws ZipException, IOException {

        File file = new File(jar);
        ZipFile zipFile = new ZipFile(file);
        Enumeration<ZipEntry> entries = (Enumeration<ZipEntry>) zipFile.entries();
        ZipEntry[] zipent = new ZipEntry[zipFile.size()];
        byte[][] bin = new byte[zipFile.size()][];

        for (int i = 0; entries.hasMoreElements(); i++) {
            zipent[i] = entries.nextElement();
            InputStream is = zipFile.getInputStream(zipent[i]);
            BufferedInputStream bis = new BufferedInputStream(is);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int n;
            while ((n = bis.read()) != -1) {
                baos.write(n);
            }
            baos.close();
            bis.close();
            is.close();
            bin[i] = baos.toByteArray();
        }
        zipFile.close();

        FileOutputStream fos = new FileOutputStream(file);
        ZipOutputStream zout = new ZipOutputStream(fos);
        for (int i = 0; i < zipent.length; i++) {
            ZipEntry ze = zipent[i];
            if (ze.getName().toLowerCase().endsWith("manifest.mf")) {
                
                Manifest manif = new Manifest(new ByteArrayInputStream(bin[i]));

                String cpath = manif.getMainAttributes().getValue(Name.CLASS_PATH);
                manif.getMainAttributes().remove(Name.CLASS_PATH);
                cpath = cpath.replaceAll(entry, "");
                                
                if (operation == ADD_TO_CLASSPATH) {
                    cpath +=(" " + entry);
                }

                manif.getMainAttributes().put(Name.CLASS_PATH, cpath);
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                manif.write(baos);
                
                zout.putNextEntry(new ZipEntry(ze.getName()));
                zout.write(baos.toByteArray());
            } else {
                zout.putNextEntry(ze);
                zout.write(bin[i]);
            }
        }
        zout.close();
        fos.close();
    }
}
