package org.atlas.util;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class PostGis {

    public static boolean isPostgisInstalled(Statement sta) {
        ResultSet rs = null;
        try {
            rs = sta.executeQuery("select postgis_version();");
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
            return false;
        } finally {
            try {
                rs.close();
            } catch (Exception ex) {
                System.out.println(ex.getMessage());
            }
        }
        return true;
    }

    public static boolean isScriptPresent(Statement sta) {
        ResultSet rs = null;
        try {
            rs = sta.executeQuery("select postgis_scripts_installed();");
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
            return false;
        } finally {
            try {
                rs.close();
            } catch (Exception ex) {
                System.out.println(ex.getMessage());
            }
        }
        return true;
    }

    public static void removeGeometryColumn(Statement sta, String schema_name, String table_name, String col_name) throws SQLException, Exception {
        boolean schema_ok;
        String query;
        ResultSet rs;
        String realSchema;

        if (!schema_name.equals("")) {
            schema_ok = false;
            query = "SELECT nspname FROM pg_namespace WHERE text(nspname) = '" + schema_name + "'";
            rs = sta.executeQuery(query);
            while (rs.next()) {
                schema_ok = true;
            }
            rs.close();

            if (schema_ok != true) {
                System.err.println("Invalid schema name - using current_schema()");
                query = "SELECT current_schema()";
                rs = sta.executeQuery(query);
                rs.next();
                realSchema = rs.getString(1);
                rs.close();
            } else {
                realSchema = schema_name;
            }
        } else {
            query = "SELECT current_schema()";
            rs = sta.executeQuery(query);
            rs.next();
            realSchema = rs.getString(1);
            rs.close();
        }

        try {
            query = "DELETE FROM geometry_columns WHERE f_table_catalog = '' AND f_table_schema  = '%s' ";
            query += "AND f_table_name = '%s' AND f_geometry_column = '%s'";
            query = String.format(query, realSchema, table_name, col_name);
            sta.execute(query);
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

        try {
            query = "ALTER TABLE \"%s\".\"%s\" ALTER COLUMN \"%s\" TYPE bytea";
            query = String.format(query, realSchema, table_name, col_name);
            sta.execute(query);
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

        try {
            query = "ALTER TABLE \"%s\".\"%s\" DROP CONSTRAINT \"enforce_srid_%s\"";
            query = String.format(query, realSchema, table_name, col_name);
            sta.execute(query);
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

        try {
            query = "ALTER TABLE \"%s\".\"%s\" DROP CONSTRAINT \"enforce_dims_%s\"";
            query = String.format(query, realSchema, table_name, col_name);
            sta.execute(query);
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

        try {
            query = "ALTER TABLE \"%s\".\"%s\" DROP CONSTRAINT \"geometry_valid_check\"";
            query = String.format(query, realSchema, table_name);
            sta.execute(query);
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

        try {
            query = "ALTER TABLE \"%s\".\"%s\" DROP CONSTRAINT \"enforce_geotype_%s\"";
            query = String.format(query, realSchema, table_name, col_name);
            sta.execute(query);
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
    }

    public static void addGeometryColumn(Statement sta, String schema_name, String table_name, String col_name, int srid, String new_type) throws SQLException, Exception {
        int dim = 2;
        boolean schema_ok;
        String query;
        ResultSet rs;
        String realSchema;

        if (!((new_type.equals("GEOMETRY")) ||
                (new_type.equals("GEOMETRYCOLLECTION")) ||
                (new_type.equals("POINT")) ||
                (new_type.equals("MULTIPOINT")) ||
                (new_type.equals("POLYGON")) ||
                (new_type.equals("MULTIPOLYGON")) ||
                (new_type.equals("LINESTRING")) ||
                (new_type.equals("MULTILINESTRING")) ||
                (new_type.equals("GEOMETRYCOLLECTIONM")) ||
                (new_type.equals("POINTM")) ||
                (new_type.equals("MULTIPOINTM")) ||
                (new_type.equals("POLYGONM")) ||
                (new_type.equals("MULTIPOLYGONM")) ||
                (new_type.equals("LINESTRINGM")) ||
                (new_type.equals("MULTILINESTRINGM")) ||
                (new_type.equals("CIRCULARSTRING")) ||
                (new_type.equals("CIRCULARSTRINGM")) ||
                (new_type.equals("COMPOUNDCURVE")) ||
                (new_type.equals("COMPOUNDCURVEM")) ||
                (new_type.equals("CURVEPOLYGON")) ||
                (new_type.equals("CURVEPOLYGONM")) ||
                (new_type.equals("MULTICURVE")) ||
                (new_type.equals("MULTICURVEM")) ||
                (new_type.equals("MULTISURFACE")) ||
                (new_type.equals("MULTISURFACEM")))) {
            throw new Exception("Invalid type name");
        }

        if (dim > 4 || dim < 0) {
            throw new Exception("invalid dimension");
        }

        if (new_type.endsWith("M") && dim != 3) {
            throw new Exception("TypeM needs 3 dimensions");
        }

        if (!schema_name.equals("")) {
            schema_ok = false;
            query = "SELECT nspname FROM pg_namespace WHERE text(nspname) = '" + schema_name + "'";
            rs = sta.executeQuery(query);
            while (rs.next()) {
                schema_ok = true;
            }
            rs.close();

            if (schema_ok != true) {
                System.err.println("Invalid schema name - using current_schema()");
                query = "SELECT current_schema()";
                rs = sta.executeQuery(query);
                rs.next();
                realSchema = rs.getString(1);
                rs.close();
            } else {
                realSchema = schema_name;
            }
        } else {
            query = "SELECT current_schema()";
            rs = sta.executeQuery(query);
            rs.next();
            realSchema = rs.getString(1);
            rs.close();
        }

        //query = "ALTER TABLE '"+realSchema+"'.'"+table_name+"' ADD COLUMN '"+col_name+"' geometry";
        //sta.execute(query);

        query = "DELETE FROM geometry_columns WHERE f_table_catalog = '' AND f_table_schema  = '%s' ";
        query += "AND f_table_name = '%s' AND f_geometry_column = '%s'";
        query = String.format(query, realSchema, table_name, col_name);
        sta.execute(query);

        query = "ALTER TABLE \"%s\".\"%s\" ALTER COLUMN \"%s\" TYPE geometry";
        query = String.format(query, realSchema, table_name, col_name);
        sta.execute(query);

        query = "INSERT INTO geometry_columns VALUES ('','%s','%s','%s', %d , %d ,'%s')";
        query = String.format(query, realSchema, table_name, col_name, dim, srid, new_type);
        sta.execute(query);

        query = "ALTER TABLE \"%s\".\"%s\" ADD CONSTRAINT \"enforce_srid_%s\" CHECK (SRID(\"%s\") = %d)";
        query = String.format(query, realSchema, table_name, col_name, col_name, srid);
        sta.execute(query);

        query = "ALTER TABLE \"%s\".\"%s\" ADD CONSTRAINT \"enforce_dims_%s\" CHECK (ndims(\"%s\") = %d)";
        query = String.format(query, realSchema, table_name, col_name, col_name, dim);
        sta.execute(query);

        query = "ALTER TABLE \"%s\".\"%s\" ADD CONSTRAINT \"geometry_valid_check\" CHECK (isvalid(\"%s\"))";
        query = String.format(query, realSchema, table_name, col_name);
        sta.execute(query);

        if (!new_type.equals("GEOMETRY")) {
            query = "ALTER TABLE \"%s\".\"%s\" ADD CONSTRAINT \"enforce_geotype_%s\" CHECK (geometrytype(\"%s\") = '%s' OR (\"%s\") is null)";
            query = String.format(query, realSchema, table_name, col_name, col_name, new_type, col_name);
            sta.execute(query);
        }
    }

    public static String getGeometryClass(String geomType) throws IllegalArgumentException {
        if (geomType.equals("POINT")) {
            return com.vividsolutions.jts.geom.Point.class.getName();
        }

        if (geomType.equals("MULTIPOINT")) {
            return com.vividsolutions.jts.geom.MultiPoint.class.getName();
        }

        if (geomType.equals("POLYGON")) {
            return com.vividsolutions.jts.geom.Polygon.class.getName();
        }

        if (geomType.equals("MULTIPOLYGON")) {
            return com.vividsolutions.jts.geom.MultiPolygon.class.getName();
        }

        if (geomType.equals("LINESTRING")) {
            return com.vividsolutions.jts.geom.LineString.class.getName();
        }

        if (geomType.equals("MULTILINESTRING")) {
            return com.vividsolutions.jts.geom.MultiLineString.class.getName();
        }

        throw new IllegalArgumentException("Unsupported type name");
    }

    public static boolean isValidForImport(String tabName, Connection con) throws Exception {
        DatabaseMetaData dm = con.getMetaData();
        ResultSet rs = dm.getPrimaryKeys(con.getCatalog(), null, tabName);
        while (rs.next()) {
            String col = rs.getString(4).toLowerCase();
            if (col.equals("atlas27_id")) {
                ResultSet rs1 = dm.getColumns(con.getCatalog(), null, tabName, col);
                if (rs1.next()) {
                    if (rs1.getInt(5) == java.sql.Types.INTEGER) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    private PostGis() {
    }
}
