package org.atlas.util;

import org.atlas.exceptions.HTTPRequestException;
import java.io.IOException;
import java.net.ConnectException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

public class ServiceConnection implements Cloneable{
    
    private String name;
    private String appUrl;
    private String username;
    private String password;
        
    public ServiceConnection(){
        
    }
    
    public ServiceConnection(String name, String appUrl, String username, String password){
        this.setName(name);
        this.setAppUrl(appUrl);
        this.setUsername(username);
        this.setPassword(password);
    }
    
    public String getName() {
        return name;
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    public String getAppUrl() {
        return appUrl;
    }
    
    public String getWmsUrl(){
        return appUrl+"wms";
    }
    
    public String getGeocoderUrl(){
        return appUrl+"geocoder";
    }
    
    public void setAppUrl(String appUrl) {
        this.appUrl = appUrl;
        if(!this.appUrl.endsWith("/")){
            this.appUrl+="/";
        }
    }
    
    public String getUsername() {
        return username;
    }
    
    public void setUsername(String username) {
        this.username = username;
    }
    
    public String getPassword() {
        return password;
    }
    
    public void setPassword(String password) {
        this.password = password;
    }
    
    @Override
    public String toString(){
        return this.getName();
    }
    
    public boolean equalParams(ServiceConnection con){
        if(!getName().equals(con.getName()))return false;
        if(!getPassword().equals(con.getPassword()))return false;
        if(!getAppUrl().equals(con.getAppUrl()))return false;
        if(!getUsername().equals(con.getUsername()))return false;
        return true;
    }
    
    public byte[] getRequestBytes(String request) throws Exception{
        try {
            String user = Cryptography.toHexadecimal(getUsername().getBytes());
            String pass = Cryptography.toHexadecimal(Cryptography.getMD5(getPassword().getBytes()));
            String params = "10573b873d2fa5a365d558a45e328e47="+request+"&14c4b06b824ec593239362517f538b29="+user+"&5f4dcc3b5aa765d61d8327deb882cf99="+pass;
            return Cryptography.encrypthAES(params);
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new Exception("Unexpected Encrypt Exception");
        }
    }
    
    public byte[] getWmsAdminInfo(byte[] request) throws IOException, HTTPRequestException{
        URL iurl;
        try {
            iurl = new URL(getWmsUrl());
        } catch (MalformedURLException ex) {
            return null;
        }
        HttpURLConnection urlCon;
        urlCon = (HttpURLConnection)iurl.openConnection();
        urlCon.setDoOutput(true);
        urlCon.setDoInput(true);
        urlCon.setUseCaches(false);
        urlCon.getOutputStream().write(request);
        int status = urlCon.getResponseCode();
        if(status!=HttpURLConnection.HTTP_OK){
            throw new HTTPRequestException(urlCon);
        }else{
            return IOUtils.readFromStream(urlCon.getInputStream());
        }
    }  

    public byte[] getGeocoderAdminInfo(byte[] request) throws IOException, HTTPRequestException{
        URL iurl;
        try {
            iurl = new URL(getGeocoderUrl());
        } catch (MalformedURLException ex) {
            return null;
        }
        HttpURLConnection urlCon;
        urlCon = (HttpURLConnection)iurl.openConnection();
        urlCon.setDoOutput(true);
        urlCon.setDoInput(true);
        urlCon.setUseCaches(false);
        urlCon.getOutputStream().write(request);
        int status = urlCon.getResponseCode();
        if(status!=HttpURLConnection.HTTP_OK){
            throw new HTTPRequestException(urlCon);
        }else{
            return IOUtils.readFromStream(urlCon.getInputStream());
        }
    }  

    public void sendWMSReload() throws ConnectException, HTTPRequestException, Exception {
        getWmsAdminInfo(getRequestBytes("ForceReload"));
    }
    
    public void sendGeocoderReload() throws ConnectException, HTTPRequestException, Exception {
        getGeocoderAdminInfo(getRequestBytes("ForceReload"));
    }
    
    
    @Override
    public Object clone() throws CloneNotSupportedException {
        return super.clone();
    }
}