package org.atlas.util;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.io.WKBWriter;
import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.Source;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureResults;
import org.geotools.data.FeatureSource;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.feature.AttributeType;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureType;
import org.hibernate.classic.Session;
import org.hibernate.dialect.Dialect;

/**
 * @author Mario R Eraso
 */
public class ShapeFileUtils {

    public static AttributeType[] readAttributes(File f) throws Exception {
        try {
            ShapefileDataStore store = new ShapefileDataStore(f.toURI().toURL());
            String name = store.getTypeNames()[0];
            FeatureSource source = store.getFeatureSource(name);
            FeatureType type = source.getSchema();
            return type.getAttributeTypes();
        } catch (MalformedURLException ex) {
            throw ex;
        } catch (IOException ex) {
            throw ex;
        }
    }

    public static void importShapeFile(File f, String tabName, Boolean[] selection, Project project, HibernateUtil hu) throws IOException, MalformedURLException, SQLException, Exception {

        ShapefileDataStore store = new ShapefileDataStore(f.toURI().toURL());
        System.out.println("Creando los lectores");
        FeatureSource source = store.getFeatureSource(store.getTypeNames()[0]);
        FeatureResults results = source.getFeatures();
        FeatureType type = source.getSchema();
        FeatureReader reader = results.reader();
                
        Connection con = hu.getConnection();
        Statement sta = con.createStatement();
        System.out.println("Creando la tabla");
        int srid = project.getSpatial_ref_sys().getSrid();
        ShapeFileUtils.createTable(tabName, type.getAttributeTypes(), selection, srid, hu.getConnectionProperties().getDialect(), sta);
        String ins = ShapeFileUtils.getPreparedInsertStatement(tabName, type.getAttributeTypes(), selection);
        PreparedStatement prepared = con.prepareStatement(ins);

        System.out.println("Poblando la tabla ");

        int nAttr = type.getAttributeCount();
        WKBWriter writer = new WKBWriter();

        Source src = new Source();
        src.setTablsour(tabName);

        
        try {
            for (int x = 0; reader.hasNext(); x++) {
                Feature feature = reader.next();
                prepared.setInt(1, x);
                int attr = 1;
                try {
                    int i = 0;
                    for (i = 0; i < nAttr; i++) {
                        if (selection[i]) {
                            attr++;
                            Class tipo = feature.getAttribute(i).getClass();
                            if (tipo.equals(Integer.class)) {
                                prepared.setInt(attr, (Integer) feature.getAttribute(i));
                            } else if (tipo.equals(Double.class)) {
                                prepared.setDouble(attr, (Double) feature.getAttribute(i));
                            } else if (tipo.equals(Long.class)) {
                                prepared.setLong(attr, (Long) feature.getAttribute(i));
                            } else if (tipo.equals(String.class)) {
                                prepared.setString(attr, (String) feature.getAttribute(i));
                            } else if (tipo.equals(Point.class) || tipo.equals(LineString.class) || tipo.equals(Polygon.class) || tipo.equals(MultiPoint.class) || tipo.equals(MultiLineString.class) || tipo.equals(MultiPolygon.class)) {
                                Coordinate[] coords = feature.getDefaultGeometry().getCoordinates();
                                
                                byte[] bytes = writer.write(feature.getDefaultGeometry());
                                bytes = EWKB.WKTtoEWKT(bytes, srid);
                                prepared.setBytes(attr, bytes);
                                src.setGeomType(tipo.getCanonicalName());
                                src.setFielsour(type.getAttributeTypes()[i].getName());
                            }
                        }
                    }
                    prepared.executeUpdate();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            src.loadFeatures(con);
        } catch (Exception ex) {
            throw ex;
        } finally {
            prepared.close();
            reader.close();
            sta.close();
            System.gc();
        }

        System.out.println("Cargando la nueva geometria");

        System.out.println("Actualizando la capa");
        Session sess = hu.begin();

        try {
            sess.update(project);

            sess.save(src);
            project.getSources().add(src);
            src.setProject(project);
            src.calcularDimensiones();
            hu.commitAndClose(sess);
        } catch (Exception ex) {
            hu.close(sess);
            throw ex;
        } 
    }

    public static void createTable(String tblName, AttributeType[] attrs, Boolean[] selection, int srid, Dialect dialect, Statement sta) throws Exception {

        AttributeType geoType = null;
        boolean postgis = false;
        if (PostGis.isPostgisInstalled(sta)) {
            if (PostGis.isScriptPresent(sta)) {
                postgis = true;
            }
        }

        String sql = "CREATE TABLE \"" + tblName + "\" ( atlas27_id " + dialect.getTypeName(Types.INTEGER) + ", ";

        for (int i = 0; i < attrs.length; i++) {
            Class tipo = attrs[i].getType();
            if (selection[i]) {
                if (tipo.equals(Integer.class)) {
                    sql = sql + attrs[i].getName() + " " + dialect.getTypeName(Types.INTEGER) + ", ";
                } else if (tipo.equals(Long.class)) {
                    sql = sql + attrs[i].getName() + " " + dialect.getTypeName(Types.DOUBLE) + ", ";
                } else if (tipo.equals(Double.class)) {
                    sql = sql + attrs[i].getName() + " " + dialect.getTypeName(Types.DOUBLE) + ", ";
                } else if (tipo.equals(String.class)) {
                    sql = sql + attrs[i].getName() + " " + dialect.getTypeName(Types.CLOB) + ", ";
                } else if (tipo.equals(Point.class) || tipo.equals(LineString.class) || tipo.equals(Polygon.class)) {
                    sql = sql + attrs[i].getName() + " " + dialect.getTypeName(Types.VARBINARY) + ", ";
                    geoType = attrs[i];
                } else if (tipo.equals(MultiPoint.class) || tipo.equals(MultiLineString.class) || tipo.equals(MultiPolygon.class)) {
                    sql = sql + attrs[i].getName() + " " + dialect.getTypeName(Types.VARBINARY) + ", ";
                    geoType = attrs[i];
                } else {
                    throw new Exception("El tipo " + tipo + " encontrado en el archivo no es soportado");
                }
            }
        }
        sql = sql.substring(0, sql.length() - 2);
        sql = sql + ");";

        System.out.println("Creando la tabla");
        System.out.println(sql);
        sta.execute(sql);

        String query = "ALTER TABLE \"%s\" ADD CONSTRAINT \"%s_pkey\" PRIMARY KEY (atlas27_id)";
        query = String.format(query, tblName, tblName);
        System.out.println(query);
        sta.execute(query);

        if (geoType == null) {
            throw new Exception("There is no geometry field");
        }

        if (postgis) {
            System.out.println("Registrando con Postgis");
            PostGis.addGeometryColumn(sta, "", tblName, geoType.getName(), srid, geoType.getType().getSimpleName().toUpperCase());
        }
    }

    public static String getPreparedInsertStatement(String tblName, AttributeType[] attrs, Boolean[] selection) throws Exception {
        String sql = "INSERT INTO \"" + tblName + "\" ( atlas27_id , ";
        String args = "?, ";
        for (int i = 0; i < attrs.length; i++) {
            if (selection[i]) {
                sql = sql + attrs[i].getName() + ", ";
                args = args + "?, ";
            }
        }
        sql = sql.substring(0, sql.length() - 2);
        args = args.substring(0, args.length() - 2);
        sql = sql + ") VALUES (" + args + ");";
        return sql;
    }
}
