package demo;

import org.atlas.mobile.events.MarkerTapEvent;
import org.atlas.mobile.gui.MapPanel;
import org.atlas.mobile.data.Layer;
import org.atlas.mobile.events.GeocodingEvent;
import org.atlas.mobile.data.Project;
import demo.TextMarker;
import org.atlas.mobile.events.FeatureInfoEvent;
import org.atlas.mobile.events.FeatureInfoListener;
import org.atlas.mobile.events.GeocodingListener;
import org.atlas.mobile.events.MarkerTapListener;
import org.atlas.mobile.events.PointerTapEvent;
import org.atlas.mobile.events.PointerTapListener;
import org.atlas.mobile.geocoding.Result;
import java.io.IOException;
import java.util.Vector;
import javax.microedition.io.ConnectionNotFoundException;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.Command;
import org.atlas.mobile.mashup.Marker;
import org.atlas.mobile.util.IO;

public class Atlas extends MIDlet implements CommandListener {

    private Command mode;
    private Command exit;
    private Command okey;
    private Command cancel;
    private Command geocoder;
    private Command legend;
    private TextField tfield;
    private TextField address;
    private TextMarker geocoderMarker;
    private boolean midletPaused = false;
    private MapPanel mapPanel;
    private Form form;
    private Alert alert;
    private Image homeImage;
    private String urlMashap = "http://192.168.1.10/web/mobile.php";
    private String url;


    //<editor-fold defaultstate="collapsed" desc=" Generated Fields ">//GEN-BEGIN:|fields|0|
    //</editor-fold>//GEN-END:|fields|0|
    public Atlas() {
        try {
            mode = new Command("Zoom In", Command.SCREEN, 1);
            exit = new Command("Exit", Command.SCREEN, 1);
            tfield = new TextField("Marker", null, 25, TextField.ANY);
            okey = new Command("Ok", Command.SCREEN, 1);
            cancel = new Command("Cancel", Command.SCREEN, 1);
            geocoder = new Command("Geocoder", Command.SCREEN, 1);
            legend = new Command("Legend", Command.SCREEN, 1);
            address = new TextField("Address", null, 30, TextField.ANY);
            url = null;

            byte[] bytesH = IO.readFromClassPath("/demo/home.png");
            homeImage = Image.createImage(bytesH, 0, bytesH.length);
            geocoderMarker = new TextMarker(null, homeImage);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    public Form getForm() {
        if (form == null) {
            form = new Form("Atlas", new Item[]{getMapPanel()});
            form.addCommand(mode);
            form.addCommand(exit);
            form.addCommand(geocoder);
            form.addCommand(legend);
            form.setCommandListener(this);
        }
        return form;
    }

    public MapPanel getMapPanel() {
        if (mapPanel == null) {
            try {
                mapPanel = new MapPanel(300, 338);
                mapPanel.setMode(MapPanel.MODEZOOMPLUS);
                mapPanel.setURL(url);
                Project project = (Project) getMapPanel().getProjects().elementAt(0);
                mapPanel.setCurrentProject(project);
                mapPanel.setLayers(project.getLayers());

                byte[] bytesN = IO.readFromClassPath("/demo/north.png");
                Image northImage = Image.createImage(bytesN, 0, bytesN.length);
                mapPanel.setCornerImage(northImage, MapPanel.TOP_RIGHT);

                mapPanel.setCache(2000);
                mapPanel.updateView();
            } catch (ConnectionNotFoundException ex) {
                alert = new Alert("Information", "Without connection", null, AlertType.INFO);
                switchDisplayable(alert, getForm());
                ex.printStackTrace();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        return mapPanel;
    }

    private void init(String url) {
        try {

            this.url = url;
            switchDisplayable(null, getForm());

            byte[] bytesD = IO.readFromClassPath("/demo/default.png");
            final Image defaultImage = Image.createImage(bytesD, 0, bytesD.length);
            byte[] bytesS = IO.readFromClassPath("/demo/select.png");
            final Image selectImage = Image.createImage(bytesS, 0, bytesS.length);

            getMapPanel().getEventManager().addFeatureInfoListener(new FeatureInfoListener() {

                public void actionPerformed(FeatureInfoEvent ev) {

                    Form formFI = new Form("Atlas Feature Info");
                    Command back = new Command("Back", Command.BACK, 0);
                    StringItem si = new StringItem("", ev.getFeatureInfo());
                    formFI.addCommand(back);
                    formFI.append(si);

                    switchDisplayable(null, formFI);

                    formFI.setCommandListener(new CommandListener() {

                        public void commandAction(Command comm, Displayable disp) {
                            if (comm.getCommandType() == Command.BACK) {
                                switchDisplayable(null, getForm());
                                getDisplay().setCurrentItem(getMapPanel());
                            }
                        }
                    });
                }
            });

            getMapPanel().getEventManager().addPointerTapListener(new PointerTapListener() {

                public void actionPerformed(PointerTapEvent ev) {

                    TextMarker marker = new TextMarker(ev.getPointMap(), homeImage, homeImage);
                    getMapPanel().setCurrentMarker(marker);
                    getMapPanel().addMarker(marker);
                    getForm().removeCommand(mode);
                    getForm().removeCommand(exit);

                    getForm().append(tfield);
                    getForm().addCommand(okey);
                    getForm().addCommand(cancel);
                    tfield.setString("Marker " + getMapPanel().getMarkers().size());
                    getDisplay().setCurrentItem(tfield);
                }
            });

            //getMapPanel().setMarkers(TextMarker.load(urlMashap, defaultImage, selectImage));
            getMapPanel().getEventManager().addMarkerTapListener(new MarkerTapListener() {

                public void actionPerformed(MarkerTapEvent mte) {
                    Marker marker = mte.getMarker();
                    getMapPanel().setCurrentMarker(marker);
                    alert = new Alert("Tap Marker", ((TextMarker) marker).getLabel(), marker.getCurrentImage(), AlertType.INFO);
                    alert.setTimeout(Alert.FOREVER);
                    switchDisplayable(alert, getForm());
                }
            });


            getMapPanel().getEventManager().addGeocodingListener(new GeocodingListener() {

                public void actionPerformed(GeocodingEvent ev) {
                    try {
                        Result[] res = ev.getAddresses();
                        if (res.length > 0) {

                            MapPanel map = ev.getMapSource();
                            switchDisplayable(null, getForm());

                            map.setZoomLevel(-15);
                            map.setCenter(res[0].getLocation());
                            map.updateView();

                            map.removeMarker(geocoderMarker);
                            geocoderMarker.setLabel(res[0].getAddress());
                            geocoderMarker.setCoordinate(res[0].getLocation());
                            map.addMarker(geocoderMarker);
                        } else {
                            alert = new Alert("Information", "No address found", null, AlertType.INFO);
                            switchDisplayable(alert, getForm());
                        }

                        getDisplay().setCurrentItem(getMapPanel());
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                }
            });


            getDisplay().setCurrentItem(getMapPanel());

        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    //<editor-fold defaultstate="collapsed" desc=" Generated Methods ">//GEN-BEGIN:|methods|0|
    //</editor-fold>//GEN-END:|methods|0|

    //<editor-fold defaultstate="collapsed" desc=" Generated Method: initialize ">//GEN-BEGIN:|0-initialize|0|0-preInitialize
    /**
     * Initilizes the application.
     * It is called only once when the MIDlet is started. The method is called before the <code>startMIDlet</code> method.
     */
    private void initialize() {//GEN-END:|0-initialize|0|0-preInitialize
    // write pre-initialize user code here
//GEN-LINE:|0-initialize|1|0-postInitialize
    // write post-initialize user code here
    }//GEN-BEGIN:|0-initialize|2|
    //</editor-fold>//GEN-END:|0-initialize|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Method: startMIDlet ">//GEN-BEGIN:|3-startMIDlet|0|3-preAction
    /**
     * Performs an action assigned to the Mobile Device - MIDlet Started point.
     */
    public void startMIDlet() {//GEN-END:|3-startMIDlet|0|3-preAction

//GEN-LINE:|3-startMIDlet|1|3-postAction


        Form formC = new Form("Atlas Connection");
        Command okc = new Command("Ok", Command.OK, 0);
        Command exc = new Command("Exit", Command.EXIT, 0);

        formC.addCommand(okc);
        formC.addCommand(exc);
        final TextField tfurl = new TextField("URL Server: ", "http://192.168.1.10:8080/service", 100, TextField.URL);
        formC.append(tfurl);

        formC.setCommandListener(new CommandListener() {

            public void commandAction(Command cmd, Displayable dis) {

                if (cmd.getCommandType() == Command.EXIT) {
                    exitMIDlet();
                } else {

                    final String url = tfurl.getString();

                    new Thread(new Runnable() {

                        public void run() {
                            init(url);
                        }
                    }).start();
                }
            }
        });

        switchDisplayable(null, formC);

    }//GEN-BEGIN:|3-startMIDlet|2|
    //</editor-fold>//GEN-END:|3-startMIDlet|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Method: resumeMIDlet ">//GEN-BEGIN:|4-resumeMIDlet|0|4-preAction
    /**
     * Performs an action assigned to the Mobile Device - MIDlet Resumed point.
     */
    public void resumeMIDlet() {//GEN-END:|4-resumeMIDlet|0|4-preAction
    // write pre-action user code here
//GEN-LINE:|4-resumeMIDlet|1|4-postAction
    // write post-action user code here        
    }//GEN-BEGIN:|4-resumeMIDlet|2|
    //</editor-fold>//GEN-END:|4-resumeMIDlet|2|

    //<editor-fold defaultstate="collapsed" desc=" Generated Method: switchDisplayable ">//GEN-BEGIN:|5-switchDisplayable|0|5-preSwitch
    /**
     * Switches a current displayable in a display. The <code>display</code> instance is taken from <code>getDisplay</code> method. This method is used by all actions in the design for switching displayable.
     * @param alert the Alert which is temporarily set to the display; if <code>null</code>, then <code>nextDisplayable</code> is set immediately
     * @param nextDisplayable the Displayable to be set
     */
    public void switchDisplayable(Alert alert, Displayable nextDisplayable) {//GEN-END:|5-switchDisplayable|0|5-preSwitch

        Display display = getDisplay();//GEN-BEGIN:|5-switchDisplayable|1|5-postSwitch
        if (alert == null) {
            display.setCurrent(nextDisplayable);
        } else {
            display.setCurrent(alert, nextDisplayable);
        }//GEN-END:|5-switchDisplayable|1|5-postSwitch
    
    }//GEN-BEGIN:|5-switchDisplayable|2|
    //</editor-fold>//GEN-END:|5-switchDisplayable|2|
    public Display getDisplay() {
        return Display.getDisplay(this);
    }

    public void exitMIDlet() {
        switchDisplayable(null, null);
        destroyApp(true);
        notifyDestroyed();
    }

    public void startApp() {
        if (midletPaused) {
            resumeMIDlet();
        } else {
            initialize();
            startMIDlet();
        }
        midletPaused = false;
    }

    public void pauseApp() {
        midletPaused = true;
    }

    public void destroyApp(boolean unconditional) {
    }

    public void commandAction(Command cmd, Displayable arg1) {
        if (cmd == exit) {
            exitMIDlet();
        }

        if (cmd == cancel) {
            getMapPanel().removeMarker(getMapPanel().getCurrentMarker());
            getMapPanel().setCurrentMarker(null);
            getForm().removeCommand(okey);
            getForm().removeCommand(cancel);
            getForm().delete(1);
            getForm().addCommand(exit);
            getForm().addCommand(mode);
            getDisplay().setCurrentItem(getMapPanel());
            return;
        }

        if (cmd == okey) {

            TextMarker tmarker = (TextMarker) getMapPanel().getCurrentMarker();
            tmarker.setLabel(tfield.getString());


            getForm().removeCommand(okey);
            getForm().removeCommand(cancel);
            getForm().delete(1);
            getForm().addCommand(exit);
            getForm().addCommand(mode);

            getDisplay().setCurrentItem(getMapPanel());
            return;

        }

        if (cmd == geocoder) {

            Form formGeo = new Form("Atlas Geocoder");
            Command locate = new Command("Locate", Command.OK, 0);
            Command back = new Command("Back", Command.BACK, 0);
            address.setString("");

            formGeo.append(address);
            formGeo.addCommand(back);
            formGeo.addCommand(locate);
            switchDisplayable(null, formGeo);

            formGeo.setCommandListener(new CommandListener() {

                public void commandAction(Command comm, Displayable disp) {
                    if (comm.getCommandType() == Command.OK) {

                        if (!address.getString().equals("")) {
                            try {
                                getMapPanel().sendGeocoderRequest(address.getString());
                            } catch (Exception ex) {
                                ex.printStackTrace();
                            }
                        }
                    }

                    if (comm.getCommandType() == Command.BACK) {
                        switchDisplayable(null, getForm());
                        getDisplay().setCurrentItem(getMapPanel());
                    }

                }
            });

            return;
        }

        if (cmd == legend) {

            Form formLen = new Form("Atlas Lengend");
            Vector layers = getMapPanel().getVisibleLayers();
            if (layers != null) {
                if (layers.size() > 0) {

                    try {
                        for (int i = 0; i < layers.size(); i++) {
                            Layer layer = (Layer) layers.elementAt(i);
                            Image img = getMapPanel().getLegend(layer);
                            ImageItem itemLegend = new ImageItem(layer.getTitle(), img, ImageItem.LAYOUT_DEFAULT, "");
                            formLen.append(itemLegend);
                        }
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                }
            }

            Command back = new Command("Back", Command.BACK, 0);
            formLen.addCommand(back);
            switchDisplayable(null, formLen);
            formLen.setCommandListener(new CommandListener() {

                public void commandAction(Command comm, Displayable disp) {
                    if (comm.getCommandType() == Command.BACK) {
                        switchDisplayable(null, getForm());
                        getDisplay().setCurrentItem(getMapPanel());
                    }
                }
            });

            return;
        }

        getForm().removeCommand(cmd);
        switch (getMapPanel().getMode()) {
            case MapPanel.MODEZOOMPLUS:
                mode = new Command("Zoom Out", Command.SCREEN, 0);
                getMapPanel().setMode(MapPanel.MODEZOOMMINUS);
                break;
            case MapPanel.MODEZOOMMINUS:
                mode = new Command("Zoom Box", Command.SCREEN, 0);
                getMapPanel().setMode(MapPanel.MODEZOOMBOX);
                break;
            case MapPanel.MODEZOOMBOX:
                mode = new Command("Drag", Command.SCREEN, 0);
                getMapPanel().setMode(MapPanel.MODEPAN);
                break;
            case MapPanel.MODEPAN:
                mode = new Command("Info", Command.SCREEN, 0);
                getMapPanel().setMode(MapPanel.MODEFEATUREINFO);
                break;
            case MapPanel.MODEFEATUREINFO:
                mode = new Command("Marker", Command.SCREEN, 0);
                getMapPanel().setMode(MapPanel.MODEGETCOORD);
                break;
            case MapPanel.MODEGETCOORD:
                mode = new Command("Zoom In", Command.SCREEN, 0);
                getMapPanel().setMode(MapPanel.MODEZOOMPLUS);
                break;
        }

        getForm().addCommand(mode);
        getDisplay().setCurrentItem(getMapPanel());

    }
}

