package org.atlas.mobile.data;

import org.kxml.kdom.Element;
import org.atlas.mobile.util.Point;

public class BBox {

    private String crs;
    private double minx;
    private double miny;
    private double maxx;
    private double maxy;

    public BBox(Element bBox) {
        String tmp = bBox.getAttribute("CRS").getValue();
        setCRS(tmp.substring("EPSG:".length(), tmp.length()));
        setMinx(Double.parseDouble(bBox.getAttribute("minx").getValue()));
        setMiny(Double.parseDouble(bBox.getAttribute("miny").getValue()));
        setMaxx(Double.parseDouble(bBox.getAttribute("maxx").getValue()));
        setMaxy(Double.parseDouble(bBox.getAttribute("maxy").getValue()));
    }

    public BBox(double minx, double miny, double maxx, double maxy) {
        this.setMinx(minx);
        this.setMiny(miny);
        this.setMaxx(maxx);
        this.setMaxy(maxy);
    }

    public double getMinx() {
        return minx;
    }

    public void setMinx(double minx) {
        this.minx = minx;
    }

    public double getMiny() {
        return miny;
    }

    public void setMiny(double miny) {
        this.miny = miny;
    }

    public double getMaxx() {
        return maxx;
    }

    public void setMaxx(double maxx) {
        this.maxx = maxx;
    }

    public double getMaxy() {
        return maxy;
    }

    public void setMaxy(double maxy) {
        this.maxy = maxy;
    }

    public double getWidth() {
        return getMaxx() - getMinx();
    }

    public double getHeight() {
        return getMaxy() - getMaxy();
    }

    public Point getCenter() {
        return new Point((getMaxx() + getMinx()) / 2, (getMaxy() + getMiny()) / 2);
    }

    public String getUrlForm() {
        return "&bbox=" + getMinx() + "," + getMiny() + "," + getMaxx() + "," + getMaxy();
    }

    public String getCRS() {
        return crs;
    }

    public void setCRS(String crs) {
        this.crs = crs;
    }

    public boolean equals(BBox bbox) {
        if (this.getMaxx() != bbox.getMaxx()) {
            return false;
        }
        if (this.getMaxy() != bbox.getMaxy()) {
            return false;
        }
        if (this.getMinx() != bbox.getMinx()) {
            return false;
        }
        if (this.getMiny() != bbox.getMiny()) {
            return false;
        }
        return true;
    }

    public boolean intersection(BBox bbox) {
        
        Point[] corners = new Point[4];        
        corners[0] = new Point(bbox.getMinx(), bbox.getMaxy());
        corners[1] = new Point(bbox.getMaxx(), bbox.getMaxy());
        corners[2] = new Point(bbox.getMinx(), bbox.getMiny());
        corners[3] = new Point(bbox.getMaxx(), bbox.getMiny());

        for (int i = 0; i < 4; i++) {
            Point corner = corners[i];
            if ((corner.getX() > this.getMinx()) && (corner.getX() < this.getMaxx())) {
                if ((corner.getY() > this.getMiny()) && (corner.getY() < this.getMaxy())) {
                    return true;
                }
            }
        }
        
        return false;
    }
}
