package org.atlas.mobile.gui;

import org.atlas.mobile.data.BBox;
import org.atlas.mobile.data.CRS;
import org.atlas.mobile.data.Layer;
import org.atlas.mobile.data.Project;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.io.ConnectionNotFoundException;
import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import javax.microedition.lcdui.CustomItem;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import org.atlas.mobile.mashup.Marker;
import org.kxml.kdom.Document;
import org.kxml.kdom.Element;
import org.kxml.parser.XmlParser;
import org.atlas.mobile.util.Cache;
import org.atlas.mobile.util.Point;
import org.atlas.mobile.util.Square;
import org.atlas.mobile.util.Mathe;
import org.atlas.mobile.util.Coordinate;
import org.atlas.mobile.util.EventManager;
import org.atlas.mobile.util.IO;
import org.atlas.mobile.util.Network;

public class MapPanel extends CustomItem {

    private String appUrl;
    private String geocoderUrl;
    private String wmsUrl;
    private Vector projects;
    private Project currentProjetc;
    private String currentImageFormat;
    private String currentInfoFormat;
    private Vector imageFormats;
    private Vector featuresInfoFormats;
    private Point center;
    private Vector layers;
    private Vector layersFeatureInfo;
    private BBox referenceBbox;
    private double baseScale;
    private int zoom;
    private double zoomStep;
    private int bloqSize;
    private Vector squares;
    private String urlpet;
    private int mode;
    private boolean drag;
    private Point startdrag;
    private Point startauxdrag;
    private Point enddrag;
    private int radius;
    private Cache cache;
    private Image bufferedImage;
    private EventManager eventM;
    private Vector markers;
    private Marker currentMarker;
    private int width;
    private int height;
    private Image imageCorner;
    private int positionCornerImage;
    private long lastKey = 0;
    public static final int NOMODESELECTED = 0;
    public static final int MODEZOOMPLUS = 1;
    public static final int MODEZOOMMINUS = 2;
    public static final int MODEPAN = 3;
    public static final int MODEZOOMBOX = 4;
    public static final int MODEFEATUREINFO = 5;
    public static final int MODEGETCOORD = 6;
    public static final int TOP_RIGHT = 1;
    public static final int TOP_LEFT = 2;
    public static final int BOTTOM_RIGHT = 3;
    public static final int BOTTOM_LEFT = 4;

    public MapPanel(int width, int height) {
        super("");
        setCurrentImageFormat("image/png");
        setCurrentInfoFormat("text/plain");
        setZoomLevel(0);
        setZoomStep(0.2);
        setBloqSize(128);
        setDrag(false);
        setStartdrag(null);
        setCurrentMarker(null);
        setRadius(-1);
        setWidth(width);
        setHeight(height);

        projects = new Vector();
        imageFormats = new Vector();
        squares = new Vector();
        markers = new Vector();
        featuresInfoFormats = new Vector();
        bufferedImage = Image.createImage(getWidth(), getHeight());
        startauxdrag = null;
        eventM = new EventManager(this);
        imageCorner = null;
        positionCornerImage = -1;
    }

    protected void keyReleased(int keycode) {
        
        long tmp = System.currentTimeMillis();
        if (tmp - lastKey < 500) {
            return;
        }
        
        lastKey = tmp;
        super.keyPressed(keycode);
        final int key = keycode;

        new Thread(new Runnable() {

            public void run() {                
                double delta = 30;
                try {

                    switch (key) {
                        case -1:
                            pan(new Point(0, -delta));
                            break;
                        case -2:
                            pan(new Point(0, +delta));
                            break;
                        case -4:
                            pan(new Point(-delta, 0));
                            break;
                        case -3:
                            pan(new Point(delta, 0));
                            break;
                    }

                    if (key == -5) {
                        switch (getMode()) {
                            case MODEZOOMPLUS:
                                zoomIn();
                                break;
                            case MODEZOOMMINUS:
                                zoomOut();
                                break;
                            case MODEZOOMBOX:
                                setZoomLevel(0);
                                setCenter(getCurrentProject().getBbox().getCenter());
                                findSquares();
                                break;
                        }
                    }

                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        }).start();
    }

    protected void pointerZoomPlus(final Point point, final int event) {
        new Thread(new Runnable() {

            public void run() {
                switch (event) {
                    case POINTER_PRESS:
                        setStartdrag(point);
                        break;
                    case POINTER_RELEASE:
                        Marker marker = markerInside(point);
                        if (marker == null) {
                            if (getStartdrag().equals(point)) {
                                try {
                                    zoomIn(Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), (int) point.getX(), (int) point.getY(), getCurrentProject().getCrs()));
                                } catch (IOException ex) {
                                    ex.printStackTrace();
                                }
                            }
                        } else {
                            Point map = Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), (int) point.getX(), (int) point.getY(), getCurrentProject().getCrs());
                            getEventManager().fireMarkerTapEvent(marker, point, map);
                        }
                        break;
                }
            }
        }).start();
    }

    protected void pointerZoomMinus(final Point point, final int event) {
        new Thread(new Runnable() {

            public void run() {
                switch (event) {
                    case POINTER_PRESS:
                        setStartdrag(point);
                        break;
                    case POINTER_RELEASE:
                        Marker marker = markerInside(point);
                        if (marker == null) {
                            if (getStartdrag().equals(point)) {
                                try {
                                    zoomOut(Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), (int) point.getX(), (int) point.getY(), getCurrentProject().getCrs()));
                                } catch (IOException ex) {
                                    ex.printStackTrace();
                                }
                            }
                        } else {
                            Point map = Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), (int) point.getX(), (int) point.getY(), getCurrentProject().getCrs());
                            getEventManager().fireMarkerTapEvent(marker, point, map);
                        }
                        break;
                }
            }
        }).start();
    }

    protected void pointerPan(Point point, int event) { // eran final
        switch (event) {
            case POINTER_PRESS:
                setDrag(true);
                setStartdrag(point);
                startauxdrag = new Point(point.getX(), point.getY());
                break;
            case POINTER_DRAG:
                if (isDrag()) {
                    moveAllsquare(getStartdrag(), point);
                    setStartdrag(point);
                    repaint();
                }
                break;
            case POINTER_RELEASE:
                int ax = (int) (startauxdrag.getX() - point.getX());
                int ay = (int) (point.getY() - startauxdrag.getY());
                setDrag(false);
                setStartdrag(null);
                startauxdrag = null;

                if (ax == 0 && ay == 0) {
                    Marker marker = markerInside(point);
                    if (marker != null) {
                        Point map = Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), (int) point.getX(), (int) point.getY(), getCurrentProject().getCrs());
                        getEventManager().fireMarkerTapEvent(marker, point, map);
                    }
                } else {
                    try {
                        pan(new Point(ax, ay));
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                }

                break;
        }
    }

    protected void launchEvent(Point point, int event) {
        switch (event) {
            case POINTER_PRESS:
                setDrag(true);
                setStartdrag(point);
                startauxdrag = new Point(point.getX(), point.getY());
                break;
            case POINTER_DRAG:
                if (isDrag()) {
                    moveAllsquare(getStartdrag(), point);
                    setStartdrag(point);
                    repaint();
                }
                break;
            case POINTER_RELEASE:
                int ax = (int) (startauxdrag.getX() - point.getX());
                int ay = (int) (point.getY() - startauxdrag.getY());

                setDrag(false);
                setStartdrag(null);
                setEnddrag(null);
                startauxdrag = null;

                try {
                    pan(new Point(ax, ay));
                } catch (IOException ex) {
                    ex.printStackTrace();
                }

                if (ax == 0 && ay == 0) {
                    Marker marker = markerInside(point);
                    if (marker == null) {
                        Point coord = Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), (int) point.getX(), (int) point.getY(), getCurrentProject().getCrs());
                        getEventManager().firePointerTapEvent(point, coord);
                    } else {
                        Point map = Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), (int) point.getX(), (int) point.getY(), getCurrentProject().getCrs());
                        getEventManager().fireMarkerTapEvent(marker, point, map);
                    }
                }

                break;
        }
    }

    protected Marker markerInside(Point point) {
        Vector markersT = getMarkers();
        for (int k = 0; k < markers.size(); k++) {
            Marker cmarker = (Marker) markersT.elementAt(k);
            if (cmarker.isPointInside(point)) {
                return cmarker;
            }
        }
        return null;
    }

    protected void pointerZoomBox(final Point point, final int event) {
        new Thread(new Runnable() {

            public void run() {
                switch (event) {
                    case POINTER_PRESS:
                        setDrag(true);
                        setStartdrag(point);
                        break;
                    case POINTER_DRAG:
                        if (isDrag()) {
                            setEnddrag(point);
                            repaint();
                        }
                        break;
                    case POINTER_RELEASE:
                        setDrag(false);

                        Point p1 = getStartdrag();
                        Point p2 = point;

                        if (p1.equals(p2)) {
                            Marker marker = markerInside(point);
                            if (marker != null) {
                                Point map = Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), (int) point.getX(), (int) point.getY(), getCurrentProject().getCrs());
                                getEventManager().fireMarkerTapEvent(marker, point, map);
                            }
                            return;
                        }

                        int anchoCaja = (int) Math.abs(p1.getX() - p2.getX());
                        int altoCaja = (int) (((double) getHeight() / getWidth()) * anchoCaja);

                        Point tmp = new Point();
                        if (p2.getY() > p1.getY()) {
                            tmp.setLocation(p2.getX(), p1.getY() + altoCaja);
                        } else {
                            tmp.setLocation(p2.getX(), p1.getY() - altoCaja);
                        }

                        int centroX = (int) ((p1.getX() + tmp.getX()) / 2);
                        int centroY = (int) ((p1.getY() + tmp.getY()) / 2);

                        setCenter(Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), centroX, centroY, getCurrentProject().getCrs()));
                        double subScale = (anchoCaja * getCurrentScale()) / getWidth();
                        setZoomLevel(-(int) (Mathe.log(subScale / getBaseScale()) / Mathe.log(1 - getZoomStep())));
                        try {
                            findSquares();
                        } catch (IOException ex) {
                            ex.printStackTrace();
                        }
                        break;
                }
            }
        }).start();
    }

    protected void pointerFeatureInfo(final Point point, final int event) {
        new Thread(new Runnable() {

            public void run() {
                switch (event) {
                    case POINTER_PRESS:
                        setDrag(true);
                        setStartdrag(point);
                        break;
                    case POINTER_DRAG:
                        if (isDrag()) {
                            Point p1 = getStartdrag();
                            setRadius((int) Math.sqrt(Mathe.pow(p1.getX() - point.getX(), 2) + Mathe.pow(p1.getY() - point.getY(), 2)));
                            repaint();
                        }
                        break;
                    case POINTER_RELEASE:
                        setDrag(false);
                        Point p1 = getStartdrag();
                        setRadius((int) Math.sqrt(Mathe.pow(p1.getX() - point.getX(), 2) + Mathe.pow(p1.getY() - point.getY(), 2)));

                        if (p1.equals(point)) {
                            Marker marker = markerInside(point);
                            if (marker != null) {
                                Point map = Coordinate.translateIJtoXY(getVisibleBbox(), getWidth(), getHeight(), (int) point.getX(), (int) point.getY(), getCurrentProject().getCrs());
                                getEventManager().fireMarkerTapEvent(marker, point, map);
                                return;
                            }
                        }
                        sendFeatureInfoRequest(p1);
                        break;
                }
            }
        }).start();
    }

    protected void pointerPressed(int i, int j) {
        super.pointerPressed(i, j);
        Point ppres = new Point(i, j);
        switch (getMode()) {
            case MODEPAN:
                pointerPan(ppres, POINTER_PRESS);
                break;
            case MODEZOOMPLUS:
                pointerZoomPlus(ppres, POINTER_PRESS);
                break;
            case MODEZOOMMINUS:
                pointerZoomMinus(ppres, POINTER_PRESS);
                break;
            case MODEZOOMBOX:
                pointerZoomBox(ppres, POINTER_PRESS);
                break;
            case MODEFEATUREINFO:
                pointerFeatureInfo(ppres, POINTER_PRESS);
                break;
            case MODEGETCOORD:
                launchEvent(ppres, POINTER_PRESS);
                break;
            }
    }

    protected void pointerDragged(int i, int j) {
        super.pointerDragged(i, j);
        Point pdrag = new Point(i, j);
        switch (getMode()) {
            case MODEPAN:
                pointerPan(pdrag, POINTER_DRAG);
                break;
            case MODEZOOMBOX:
                pointerZoomBox(pdrag, POINTER_DRAG);
                break;
            case MODEFEATUREINFO:
                pointerFeatureInfo(pdrag, POINTER_DRAG);
                break;
            case MODEGETCOORD:
                launchEvent(pdrag, POINTER_DRAG);
                break;
        }
    }

    protected void pointerReleased(int i, int j) {
        super.pointerReleased(i, j);
        Point prelea = new Point(i, j);

        switch (getMode()) {
            case MODEZOOMPLUS:
                pointerZoomPlus(prelea, POINTER_RELEASE);
                break;
            case MODEZOOMMINUS:
                pointerZoomMinus(prelea, POINTER_RELEASE);
                break;
            case MODEPAN:
                pointerPan(prelea, POINTER_RELEASE);
                break;
            case MODEZOOMBOX:
                pointerZoomBox(prelea, POINTER_RELEASE);
                break;
            case MODEFEATUREINFO:
                pointerFeatureInfo(prelea, POINTER_RELEASE);
                break;
            case MODEGETCOORD:
                launchEvent(prelea, POINTER_RELEASE);
                break;
            }
    }

    private void sendFeatureInfoRequest(Point ij) {

        String urlpeti = getURL_WMS() + "?version=1.3.0&request=GetFeatureInfo&styles=&CRS=EPSG:" + getCurrentProject().getCrs().getCode() + "&width=" + getWidth() + "&height=" + getHeight() + "&format=" + getCurrentImageFormat() + "&info_format=" + this.getCurrentInfoFormat();
        urlpeti += "&query_layers=";

        Vector layersinfo;

        if (getFeatureInfoLayers() != null) {
            layersinfo = getFeatureInfoLayers();
        } else {
            layersinfo = getVisibleLayers();
        }

        for (int i = 0; i < layersinfo.size(); i++) {
            urlpeti += ((Layer) layersinfo.elementAt(i)).getName() + ",";
        }

        urlpeti += "&layers=";
        for (int i = 0; i < getVisibleLayers().size(); i++) {
            urlpeti += ((Layer) getVisibleLayers().elementAt(i)).getName() + ",";
        }

        urlpeti += getVisibleBbox().getUrlForm();
        urlpeti += ("&i=" + (int) ij.getX() + "&j=" + (int) ij.getY());

        if (getRadius() > 0) {
            urlpeti += ("&distance=" + getRadius());
        } else {
            urlpeti += ("&distance=");
        }

        setRadius(-1);
        repaint();
        try {
            getEventManager().fireFeatureInfoEvent(new String(IO.readFromURL(urlpeti)), urlpeti);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    public String getURL() {
        return appUrl;
    }

    private String getURL_WMS() {
        return this.wmsUrl;
    }

    private String getURL_GEOCODER() {
        return this.geocoderUrl;
    }

    public void setCornerImage(Image image, int position) {
        this.imageCorner = image;
        this.positionCornerImage = position;
    }

    protected boolean traverse(int arg0, int arg1, int arg2, int[] arg3) {
        return true;
    }

    public void setCurrentProject(Project project) {
        this.currentProjetc = project;
        this.setReferenceBbox(project.getBbox());
        calculateBaseScale(getWidth(), getHeight());
        setCenter(getReferenceBbox().getCenter());
        setZoomLevel(0);
    }

    private void calculateBaseScale(double width, double height) {

        BBox bb = getReferenceBbox();
        double hz = (double) width / bb.getWidth();
        double vr = (double) height / bb.getHeight();

        if (hz < vr) {//escalar al ancho
            setBaseScale(bb.getWidth() / width);
        } else {    //escalar al alto
            setBaseScale(bb.getHeight() / height);
        }
    }

    public Project getCurrentProject() {
        return this.currentProjetc;
    }

    public void setCenter(Point center) {
        this.center = new Point(center);
    }

    public Point getCenter() {
        return this.center;
    }

    public void moveCenter(Point point) throws IOException {
        this.center.setX(this.center.getX() + (point.getX() * this.getCurrentScale()));
        this.center.setY(this.center.getY() + (point.getY() * this.getCurrentScale()));
        this.updateView();
    }

    public void setURL(String url) throws ConnectionNotFoundException, IOException, ConnectionNotFoundException {

        if (!url.endsWith("/")) {
            url += "/";
        }

        appUrl = url;
        wmsUrl = url + "wms";
        geocoderUrl = url + "geocoder";

        String request = wmsUrl + "?request=GetCapabilities&service=WMS&version=1.3.0";
                
        HttpConnection con = (HttpConnection) Connector.open(request, Connector.READ, true);
        con.setRequestMethod(HttpConnection.GET);
        
        
        Reader reader = new InputStreamReader(con.openInputStream());
        XmlParser parser = new XmlParser(reader);

        Document doc = new Document();
        doc.parse(parser);
        Element root = doc.getRootElement();

        //Formatos de imagen peticion getMap
        Element getMap = root.getElement("Capability").getElement("Request").getElement("GetMap");
        for (int i = 0; i < getMap.getChildCount(); i++) {
            Object ele = getMap.getChild(i);
            if (ele instanceof Element) {
                Element format = (Element) ele;
                if (format.getName().equals("Format")) {
                    getImageFormats().addElement(format.getText());
                }

            }
        }

        //Formatos peticion featureinfo
        Element getInfo = root.getElement("Capability").getElement("Request").getElement("GetFeatureInfo");
        for (int i = 0; i < getInfo.getChildCount(); i++) {
            Object ele = getInfo.getChild(i);
            if (ele instanceof Element) {
                Element format = (Element) ele;
                if (format.getName().equals("Format")) {
                    getFeatureInfoFormats().addElement(format.getText());
                }

            }
        }

        Element proyects = root.getElement("Capability").getElement("Layer");
        for (int i = 0; i < proyects.getChildCount(); i++) {
            Object ele = proyects.getChild(i);
            if (ele instanceof Element) {
                Element project = (Element) ele;
                if (project.getName().equals("Layer")) { //for each project 
                    getProjects().addElement(new Project(project, wmsUrl));
                }

            }
        }

        reader.close();
        con.close();
    }

    public Vector getProjects() {
        return projects;
    }

    public void setProjects(Vector projects) {
        this.projects = projects;
    }

    protected void paint(Graphics g, int arg1, int arg2) {

        Graphics g2 = bufferedImage.getGraphics();
        g2.setColor(255, 255, 255);
        g2.fillRect(0, 0, getWidth(), getHeight());

        Vector tempSqures = getSquares();
        for (int i = 0; i < tempSqures.size(); i++) {
            Square square = (Square) tempSqures.elementAt(i);
            if (square.getImg() != null) {
                square.draw(g2);
            /*if (getCurrentProject().getBbox().intersection(square.getBbox())) {
            g2.setColor(0, 0, 0);
            g2.drawRect(square.getX(), square.getY(), this.getBloqSize(), this.getBloqSize());
            }*/
            }

        }

        Point disp;
        for (int i = 0; i < getMarkers().size(); i++) {
            Marker marker = (Marker) getMarkers().elementAt(i);
            if (getCurrentMarker() == marker) {
                marker.setCurrentImage(marker.getSelectedImage());
            } else {
                marker.setCurrentImage(marker.getDefaultImage());
            }

            if (isDrag()) {
                disp = marker.getPointDisplay();
            } else {
                disp = marker.getCalculatePointDisplay();
            }

            g2.drawImage(marker.getCurrentImage(), (int) disp.getX(), (int) disp.getY(), Graphics.SOLID);
        }

        switch (getMode()) {
            case MODEZOOMBOX:
                if (isDrag()) {

                    Point p1 = getStartdrag();
                    Point p2 = getEnddrag();

                    int x1 = (int) p1.getX();
                    int y1 = (int) p1.getY();

                    double cte = (double) getHeight() / getWidth();
                    int zoomBoxWidth = Math.abs((int) (p2.getX() - p1.getX()));
                    int zoomBoxHeigth = (int) (zoomBoxWidth * cte);

                    if (p2.getX() - p1.getX() < 0) {
                        zoomBoxWidth *= -1;
                    }

                    if (p2.getY() - p1.getY() < 0) {
                        zoomBoxHeigth *= -1;
                    }

                    int ancho = Math.abs(zoomBoxWidth);
                    int alto = Math.abs(zoomBoxHeigth);

                    if (zoomBoxWidth < 0) {
                        x1 -= ancho;
                    }

                    if (zoomBoxHeigth < 0) {
                        y1 -= alto;
                    }

                    g2.setColor(255, 0, 0);
                    g2.setStrokeStyle(Graphics.SOLID);
                    g2.drawRect(x1, y1, ancho, alto);
                }
                break;

            case MODEFEATUREINFO:
                if (isDrag()) {

                    Point p1 = getStartdrag();
                    int diameter = getRadius() * 2;

                    g2.setColor(255, 0, 0);
                    g2.setStrokeStyle(Graphics.SOLID);
                    g2.drawArc((int) (p1.getX() - getRadius()), (int) (p1.getY() - getRadius()), diameter, diameter, 0, 360);

                    g2.drawLine((int) p1.getX(), (int) p1.getY() - 10, (int) p1.getX(), (int) p1.getY() + 10);
                    g2.drawLine((int) p1.getX() - 10, (int) p1.getY(), (int) p1.getX() + 10, (int) p1.getY());


                }
                break;
        }
       
        if (this.imageCorner != null) {
            switch (this.positionCornerImage) {
                case TOP_RIGHT:                    
                    g2.drawImage(this.imageCorner, this.getWidth() - imageCorner.getWidth() - 10, 10, Graphics.SOLID);
                    break;
                case TOP_LEFT:
                    g2.drawImage(this.imageCorner, 10, 10, Graphics.SOLID);
                    break;
                case BOTTOM_RIGHT:
                    g2.drawImage(this.imageCorner, this.getWidth() - imageCorner.getWidth() - 10, this.getHeight() - imageCorner.getHeight() - 10, Graphics.SOLID);			
                    break;
                case BOTTOM_LEFT:
                    g2.drawImage(this.imageCorner, 10, this.getHeight() - imageCorner.getHeight() - 10, Graphics.SOLID);			
                    break;
            }
        }
        
        g.drawImage(bufferedImage, 0, 0, Graphics.SOLID);

    }

    private void findSquares() throws IOException {

        int widtht = getWidth();
        int heightt = getHeight();

        double minX = getReferenceBbox().getMinx();
        double minY = getReferenceBbox().getMiny();

        double resol = this.getCurrentScale();

        Point corner = new Point(getCenter().getX() - ((widtht / 2) * resol), getCenter().getY() - ((heightt / 2) * resol));

        int hbloq = (int) Math.floor((corner.getX() - minX) / (getBloqSize() * resol));
        int vbloq = (int) Math.floor((corner.getY() - minY) / (getBloqSize() * resol));

        Point rCorner = new Point(minX + ((hbloq) * resol * getBloqSize()), minY + ((vbloq) * resol * getBloqSize()));

        final int difx = (int) ((corner.getX() - rCorner.getX()) / resol);
        final int dify = (int) ((corner.getY() - rCorner.getY()) / resol);

        final int nboxH = (int) Math.ceil((double) (widtht + Math.abs(difx)) / getBloqSize());
        final int nboxV = (int) Math.ceil((double) (heightt + Math.abs(dify)) / getBloqSize());

        double offSetx = -difx;
        double offSety = heightt + dify - (nboxV * getBloqSize());

        double rb = resol * getBloqSize();

        Vector delete = new Vector();
        for (int i = 0; i < this.getSquares().size(); i++) {
            Square square = (Square) this.getSquares().elementAt(i);
            delete.addElement(square);
        }

        int countimage = 0;
        Square temp;
        boolean noMatch;

        for (int i = 0; i < nboxH; i++) {
            for (int j = 0; j < nboxV; j++) {

                BBox b = new BBox(minX + ((i + hbloq) * rb), minY + ((j + vbloq) * rb), minX + ((i + 1 + hbloq) * rb), minY + ((j + 1 + vbloq) * rb));
                int x = (int) ((i * getBloqSize()) + offSetx);
                int y = (int) (((nboxV - j - 1) * getBloqSize()) + offSety);

                noMatch = true;
                for (int k = 0; k < getSquares().size() && noMatch; k++) {
                    temp = (Square) getSquares().elementAt(k);
                    if (temp.getBbox().equals(b)) {
                        noMatch = false;
                        temp.setX(x);
                        temp.setY(y);
                        delete.removeElement(temp);
                    }
                }

                if (noMatch) {
                    Square sq = new Square(x, y, getUrlpet() + b.getUrlForm(), b);
                    getSquares().addElement(sq);
                    countimage++;
                }

            }
        }

        repaint();

        while (delete.size() > 0) {
            Square rs = (Square) delete.elementAt(0);
            getSquares().removeElement(rs);
            delete.removeElement(rs);
        }

        int pph = 4;
        int thread = (int) countimage / pph;
        int mod = countimage % pph;
        int last;
        int t = pph;

        if (mod > 2) {
            thread++;
            last = mod;
        } else {
            last = pph + mod;
        }

        Vector loadSquare = new Vector();
        Vector tempSqures = getSquares();
        for (int l = 0; l < tempSqures.size(); l++) {
            Square square = (Square) tempSqures.elementAt(l);
            if (square.getImg() == null) {
                loadSquare.addElement(square);
                if (thread > 1) {
                    if (t > 1) {
                        t--;
                    } else {
                        loadImages(loadSquare);
                        loadSquare = new Vector();
                        t = pph;
                        thread--;
                    }
                } else {
                    if (last > 1) {
                        last--;
                    } else {
                        loadImages(loadSquare);
                    }
                }
            }
        }

        

    }

    public void moveAllsquare(Point p1, Point p2) {
        Square square;

        int x = (int) (p2.getX() - p1.getX());
        int y = (int) (p2.getY() - p1.getY());
        for (int k = 0; k < getSquares().size(); k++) {
            square = (Square) getSquares().elementAt(k);
            square.setX(square.getX() + x);
            square.setY(square.getY() + y);
        }

        for (int i = 0; i < getMarkers().size(); i++) {
            Marker marker = (Marker) getMarkers().elementAt(i);
            Point delta = marker.getPointDisplay();
            marker.setPointDisplay(new Point(delta.getX() + x, delta.getY() + y));
        }

    }

    public void loadImages(final Vector squares) {
        new Thread(new Runnable() {

            public void run() {
                for (int i = 0; i < squares.size(); i++) {
                    Square square = (Square) squares.elementAt(i);
                    try {
                        loadImage(square);
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                }
            }
        }).start();
    }

    public Image getLegend(Layer layer) throws IOException {
        String txtUrl = getURL_WMS() + "?request=GetLegend&layers=" + layer.getName();
        byte[] bytes = IO.readFromURL(txtUrl);
        return Image.createImage(bytes, 0, bytes.length);
    }

    public void loadImage(Square square) throws IOException {
        if (getCurrentProject().getBbox().intersection(square.getBbox())) {
            byte[] bytes;
            if (cache != null) {
                bytes = cache.findImage(square);
                if (bytes == null) {
                    bytes = org.atlas.mobile.util.IO.readFromURL(square.getUrl());
                    cache.add(bytes, square.getBbox());
                }
                square.setImg(Image.createImage(bytes, 0, bytes.length));
            } else {
                bytes = org.atlas.mobile.util.IO.readFromURL(square.getUrl());
                square.setImg(Image.createImage(bytes, 0, bytes.length));
            }
            repaint();
        }
    }

    public Vector getImageFormats() {
        return imageFormats;
    }

    public void setImageFormats(Vector imageFormats) {
        this.imageFormats = imageFormats;
    }

    public String getCurrentImageFormat() {
        return currentImageFormat;

    }

    public void setCurrentImageFormat(String currentImageFormat) {
        this.currentImageFormat = currentImageFormat;
        findUrlString();
    }

    public Vector getLayers() {
        return layers;
    }

    public void setLayers(Vector layers) {
        this.layers = layers;
        findUrlString();
    }

    public Vector getVisibleLayers() {

        Vector visLayers = new Vector();
        boolean vis;
        double sca;

        for (int i = 0; i < getLayers().size(); i++) {
            Layer lyr = (Layer) getLayers().elementAt(i);
            vis = true;
            sca = this.getWMScale();
            if (lyr.getMaxscaledeno() != Double.NaN) {
                if (sca >= lyr.getMaxscaledeno()) {
                    vis = false;
                }
            }
            if (lyr.getMinscaledeno() != Double.NaN) {
                if (sca < lyr.getMinscaledeno()) {
                    vis = false;
                }
            }
            if (vis) {
                visLayers.addElement(lyr);
            }
        }

        return visLayers;
    }

    public void updateView() throws IOException {
        findSquares();
    }

    public double getCurrentScale() {
        if (this.getZoomLevel() > 0) {
            return Mathe.pow((1 + this.getZoomStep()), this.getZoomLevel()) * this.getBaseScale();
        } else if (this.getZoomLevel() < 0) {
            return Mathe.pow((1 - this.getZoomStep()), Math.abs(this.getZoomLevel())) * this.getBaseScale();
        }

        return getBaseScale();
    }

    public double getWMScale() {
        BBox vis = getVisibleBbox();
        CRS crs = getCurrentProject().getCrs();
        double k = 0.00028;
        if (crs.getType() == CRS.UNIT_LINEAR) {
            return ((vis.getWidth() * crs.getConversionFactor()) / getWidth()) / k;
        } else {
            double rad = vis.getWidth() * crs.getConversionFactor();
            double ls = rad * 6378137;
            return (ls / getWidth()) / k;
        }
    }

    private void findUrlString() {
        if (getCurrentProject() != null && getLayers() != null && getCurrentImageFormat() != null) {
            setUrlpet(getURL_WMS() + "?version=1.3.0&request=GetMap&styles=&CRS=EPSG:" + getCurrentProject().getCrs().getCode() + "&width=" + this.getBloqSize() + "&height=" + this.getBloqSize() + "&format=" + this.getCurrentImageFormat());
            setUrlpet(getUrlpet() + "&layers=");

            Vector requestLayers = getVisibleLayers();
            String slyr = "";
            for (int i = 0; i < requestLayers.size(); i++) {
                slyr += ((Layer) requestLayers.elementAt(i)).getName() + ",";
            }

            if (slyr.length() > 0) {
                slyr = slyr.substring(0, slyr.length() - 1);
                setUrlpet(getUrlpet() + slyr);
            } else {
                setUrlpet("");
            }
        }
    }

    public void sendGeocoderRequest(String address) throws Exception {

        if (getCurrentProject() == null) {
            throw new IllegalArgumentException("Some project must be selected first");
        }

        String idProj = getCurrentProject().getName().substring(1);
        String request = getURL_GEOCODER() + "?project=" + idProj + "&address=" + Network.urlEncode(address) + "&type=XML";
        getEventManager().fireGeocodingEvent(request);

    }

    public BBox getReferenceBbox() {
        return referenceBbox;
    }

    public void setReferenceBbox(BBox referenceBbox) {
        this.referenceBbox = referenceBbox;
    }

    public double getBaseScale() {
        return baseScale;
    }

    public void setBaseScale(double baseScale) {
        this.baseScale = baseScale;
    }

    public int getZoomLevel() {
        return this.zoom;
    }

    public void setZoomLevel(int level) {
        this.zoom = level;
        findUrlString();
    }

    public double getZoomStep() {
        return zoomStep;
    }

    public void setZoomStep(double zoomStep) {
        this.zoomStep = zoomStep;
    }

    public int getBloqSize() {
        return bloqSize;
    }

    public void setBloqSize(int bloqSize) {
        this.bloqSize = bloqSize;
    }

    public Vector getSquares() {
        return squares;
    }

    public void setSquares(Vector squares) {
        this.squares = squares;
    }

    public String getUrlpet() {
        return urlpet;
    }

    public void setUrlpet(String urlpet) {
        this.urlpet = urlpet;
    }

    public void zoomToProject() {
        calculateBaseScale(getWidth(), getHeight());
        setCenter(getCurrentProject().getBbox().getCenter());
        setZoomLevel(0);
    }

    public void zoomIn() throws IOException {
        zoomIn(getCenter());
    }

    public void zoomIn(Point point) throws IOException {
        setCenter(point);
        setZoomLevel(getZoomLevel() - 1);
        findSquares();
    }

    public void zoomOut(Point point) throws IOException {
        setCenter(point);
        setZoomLevel(getZoomLevel() + 1);
        findSquares();
    }

    public void zoomOut() throws IOException {
        zoomOut(getCenter());
    }

    private void pan(Point point) throws IOException {
        Point c = getCenter();
        c.setX(c.getX() + (point.getX() * getCurrentScale()));
        c.setY(c.getY() + (point.getY() * getCurrentScale()));
        findSquares();
    }

    public int getMode() {
        return mode;
    }

    public void setMode(int mode) {
        this.mode = mode;
    }

    public BBox getVisibleBbox() {
        double ancho = this.getWidth() * this.getCurrentScale();
        double alto = this.getHeight() * this.getCurrentScale();
        return new BBox(getCenter().getX() - (ancho / 2), getCenter().getY() - (alto / 2), getCenter().getX() + (ancho / 2), getCenter().getY() + (alto / 2));
    }

    public void setCache(int sizeK) {
        this.cache = new Cache(sizeK * 1024);
    }

    public void clearCache() {
        this.cache.clear();
    }

    public boolean isDrag() {
        return drag;
    }

    public void setDrag(boolean drag) {
        this.drag = drag;
    }

    private Point getStartdrag() {
        return startdrag;
    }

    private void setStartdrag(Point startdrag) {
        this.startdrag = startdrag;
    }

    private Point getEnddrag() {
        return enddrag;
    }

    private void setEnddrag(Point enddrag) {
        this.enddrag = enddrag;
    }

    public String getCurrentInfoFormat() {
        return currentInfoFormat;
    }

    public Vector getFeatureInfoFormats() {
        return featuresInfoFormats;
    }

    public void setCurrentInfoFormat(String currentInfoFormat) {
        this.currentInfoFormat = currentInfoFormat;
    }

    public Vector getFeatureInfoLayers() {
        return layersFeatureInfo;
    }

    public void setFeatureInfoLayers(Vector layersFeatureInfo) {
        this.layersFeatureInfo = layersFeatureInfo;
    }

    public int getRadius() {
        return radius;
    }

    public void setRadius(int radius) {
        this.radius = radius;
    }

    public EventManager getEventManager() {
        return this.eventM;
    }

    public Vector getMarkers() {
        return markers;
    }

    public void setMarkers(Vector markers) {
        this.markers.removeAllElements();
        if (markers != null) {
            for (Enumeration e = markers.elements(); e.hasMoreElements();) {
                Marker marker = (Marker) e.nextElement();
                marker.setMapPanel(this);
                getMarkers().addElement(marker);
            }
        }
        repaint();
    }

    public Marker getCurrentMarker() {
        return currentMarker;
    }

    public void setCurrentMarker(Marker currentMarker) {
        this.currentMarker = currentMarker;
    }

    public void addMarker(Marker marker) {
        marker.setMapPanel(this);
        getMarkers().addElement(marker);
        repaint();
    }

    public void removeMarker(Marker marker) {
        if (getMarkers().removeElement(marker)) {
            repaint();
        }
    }

    public void removeAllMarkers() {
        getMarkers().removeAllElements();
        setCurrentMarker(null);
    }

    protected int getMinContentWidth() {
        return this.getWidth();
    }

    protected int getMinContentHeight() {
        return this.getHeight();
    }

    protected int getPrefContentWidth(int height) {
        return this.getWidth();
    }

    protected int getPrefContentHeight(int width) {
        return this.getHeight();
    }

    public int getWidth() {
        return width;
    }

    public void setWidth(int width) {
        if (width > 0) {
            this.width = width;
        }
    }

    public int getHeight() {
        return height;
    }

    public void setHeight(int height) {
        if (height > 0) {
            this.height = height;
        }
    }
}
