package org.atlas.mobile.mashup;

import javax.microedition.lcdui.Image;
import org.atlas.mobile.gui.MapPanel;
import org.atlas.mobile.util.Coordinate;
import org.atlas.mobile.util.Point;

public class Marker {

    private Point coordinate;
    private Point pointDisplay;
    private MapPanel mapPanel;
    private Image defaultImage;
    private Image selectedImage;
    private Image currentImage;
    
    final static int UPPER_LEFT = 0;
    final static int UPPER_RIGHT = 1;
    final static int CENTER_CENTER = 2;

    public Marker(Point coordinate, Image defaultImage) {
        this(coordinate, defaultImage, defaultImage);
    }

    public Marker(Point coordinate, Image defaultImage, Image selectedImage) {
        setCoordinate(coordinate);
        setDefaultImage(defaultImage);
        setSelectedImage(selectedImage);
    }

    public MapPanel getMapPanel() {
        return mapPanel;
    }

    public void setMapPanel(MapPanel mapPanel) {
        this.mapPanel = mapPanel;
    }

    public Image getDefaultImage() {
        return defaultImage;
    }

    public void setDefaultImage(Image defaultImage) {
        this.defaultImage = defaultImage;
    }

    public Image getSelectedImage() {
        return selectedImage;
    }

    public void setSelectedImage(Image selectedImage) {
        this.selectedImage = selectedImage;
    }

    public Image getCurrentImage() {
        return currentImage;
    }

    public void setCurrentImage(Image currentImage) {
        this.currentImage = currentImage;
    }

    public Point getCalculatePointDisplay() {
        return getCalculatePointDisplay(UPPER_LEFT);
    }

    public Point getCalculatePointDisplay(int position) {
        Point p;
        int w = getCurrentImage().getWidth() / 2;
        int h = getCurrentImage().getHeight() / 2;
        p = Coordinate.translateXYtoIJ(getMapPanel().getCurrentProject().getCrs(), getMapPanel().getVisibleBbox(), getMapPanel().getWidth(), getMapPanel().getHeight(), getCoordinate().getX(), getCoordinate().getY());
        switch (position) {
            case UPPER_LEFT:
                p.setLocation(p.getX() - w, p.getY() - h);
                break;
            case UPPER_RIGHT:
                p.setLocation(p.getX() + w, p.getY() - h);
                break;
            case CENTER_CENTER:
                p.setLocation(p.getX(), p.getY());
                break;
        }
        
        setPointDisplay(p);
        return p;
    }
    
    public boolean isPointInside(Point point) {
        int x = (int)point.getX();
        int y = (int)point.getY();
        int h = getCurrentImage().getWidth() / 2;
        int v = getCurrentImage().getHeight() / 2;
        int sx = (int) getCalculatePointDisplay(CENTER_CENTER).getX();
        int sy = (int) getCalculatePointDisplay(CENTER_CENTER).getY();
        if (x - h <= sx && sx <= x + h && y - v <= sy && sy <= y + v) {
            return true;
        }
        return false;
    }

    public Point getPointDisplay() {
        return this.pointDisplay;
    }

    public void setPointDisplay(Point pointDisplay) {
        this.pointDisplay = pointDisplay;
    }

    public Point getCoordinate() {
        return coordinate;
    }

    public void setCoordinate(Point coordinate) {
        this.coordinate = coordinate;
    }
}
