package org.atlas.mobile.util;

import org.atlas.mobile.data.BBox;
import org.atlas.mobile.data.CRS;

public class Coordinate {

    public static Point translateIJtoXY(BBox bbox, int width, int height, int i, int j, CRS crs) {

        double imin = Double.NaN;
        double jmin = Double.NaN;
        double imax = Double.NaN;
        double jmax = Double.NaN;

        double porcI = (double) i / width;
        double porcJ = (double) j / height;

        double x;
        double y;

        String axis1 = crs.getAxis1();
        String axis2 = crs.getAxis2();

        if (axis1.equals("east") || axis1.equals("west")) {

            if (axis1.equals("east")) {
                imin = bbox.getMinx();
                imax = bbox.getMaxx();
            }

            if (axis1.equals("west")) {
                imin = bbox.getMaxx();
                imax = bbox.getMinx();
            }

            if (axis2.equals("north")) {
                jmin = bbox.getMaxy();
                jmax = bbox.getMiny();
            }

            if (axis2.equals("south")) {
                jmin = bbox.getMiny();
                jmax = bbox.getMaxy();
            }

            x = imin + (imax - imin) * porcI;
            y = jmin + (jmax - jmin) * porcJ;
        } else {

            if (axis2.equals("east")) {
                imin = bbox.getMiny();
                imax = bbox.getMaxy();
            }

            if (axis2.equals("west")) {
                imin = bbox.getMaxy();
                imax = bbox.getMiny();
            }

            if (axis1.equals("north")) {
                jmin = bbox.getMaxx();
                jmax = bbox.getMinx();
            }

            if (axis1.equals("south")) {
                jmin = bbox.getMinx();
                jmax = bbox.getMaxx();
            }

            x = jmin + (jmax - jmin) * porcJ;
            y = imin + (imax - imin) * porcI;
        }

        return new Point(x, y);
    }

    public static Point translateXYtoIJ(CRS crs, BBox bbox, int width, int heigth, double x, double y) {
        boolean invertI;
        boolean invertJ;
        boolean swap;

        String axis1 = crs.getAxis1();
        String axis2 = crs.getAxis2();

        double factI;
        double factJ;

        double xmin = bbox.getMinx();
        double ymin = bbox.getMiny();
        double xmax = bbox.getMaxx();
        double ymax = bbox.getMaxy();

        double minI;
        double minJ;

        if (axis1.endsWith("west") || axis1.endsWith("east")) {
            swap = false;
            factI = width / (xmax - xmin);
            factJ = heigth / (ymax - ymin);
            minI = (xmin * factI);
            minJ = (ymin * factJ);
            if (axis1.equals("east")) {
                invertI = false;
            } else {
                invertI = true;
            }

            if (axis2.equals("south")) {
                invertJ = false;
            } else {
                invertJ = true;
            }
        } else {
            swap = true;
            factI = width / (ymax - ymin);
            factJ = heigth / (xmax - xmin);
            minI = (ymin * factI);
            minJ = (xmin * factJ);

            if (axis2.equals("east")) {
                invertJ = false;
            } else {
                invertJ = true;
            }
            if (axis1.equals("south")) {
                invertI = false;
            } else {
                invertI = true;
            }

        }


        int[] res = new int[2];
        if (swap) {
            if (invertI) {
                res[0] = (int) (width + minI - (y * factI));
            } else {
                res[0] = (int) ((y * factI) - minI);
            }

            if (invertJ) {
                res[1] = (int) (heigth + minJ - (x * factJ));
            } else {
                res[1] = (int) ((x * factJ) - minJ);
            }
        } else {
            if (invertI) {
                res[0] = (int) (width + minI - (x * factI));
            } else {
                res[0] = (int) ((x * factI) - minI);
            }

            if (invertJ) {
                res[1] = (int) (heigth + minJ - (y * factJ));
            } else {
                res[1] = (int) ((y * factJ) - minJ);
            }
        }
        return new Point(res[0], res[1]);
    }
}
