package org.atlas.mobile.util;

import org.atlas.mobile.events.FeatureInfoEvent;
import org.atlas.mobile.events.FeatureInfoListener;
import org.atlas.mobile.events.GeocodingEvent;
import org.atlas.mobile.events.GeocodingListener;
import org.atlas.mobile.events.MarkerTapListener;
import org.atlas.mobile.events.PointerTapEvent;
import org.atlas.mobile.events.PointerTapListener;
import org.atlas.mobile.geocoding.Result;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Vector;
import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import org.atlas.mobile.events.MarkerTapEvent;
import org.atlas.mobile.mashup.Marker;
import org.atlas.mobile.gui.MapPanel;
import org.kxml.kdom.Document;
import org.kxml.kdom.Element;
import org.kxml.parser.XmlParser;

public class EventManager {

    private Vector featureInfoListeners;
    private Vector pointerTapListeners;
    private Vector markerTapListeners;
    private Vector geocodingListeners;
    
    private MapPanel mapPanel;

    public EventManager(MapPanel mapPanel) {
        featureInfoListeners = new Vector();
        pointerTapListeners = new Vector();
        markerTapListeners = new Vector();
        geocodingListeners = new Vector();
        setMapPanel(mapPanel);
    }

    private void setMapPanel(MapPanel mapPanel) {
        this.mapPanel = mapPanel;
    }

    private MapPanel getMapPanel() {
        return this.mapPanel;
    }

    public void addFeatureInfoListener(FeatureInfoListener fl) {
        featureInfoListeners.addElement(fl);
    }

    public void addPointerTapListener(PointerTapListener pl) {
        pointerTapListeners.addElement(pl);
    }
    
    public void addMarkerTapListener(MarkerTapListener ml) {
        markerTapListeners.addElement(ml);
    }
    
     public void addGeocodingListener(GeocodingListener gl) {
        geocodingListeners.addElement(gl);
    }

    public void removeFeatureInfoListener(FeatureInfoListener fl) {
        featureInfoListeners.removeElement(fl);
    }
    
    public void removeAllFeatureInfoListener() {
        featureInfoListeners.removeAllElements();        
    }

    public void removePointerTapListener(PointerTapListener pl) {
        pointerTapListeners.removeElement(pl);        
    }
    
    public void removeAllPointerTapListener() {
        pointerTapListeners.removeAllElements();        
    }
    
    public void removeMarkerTapListener(MarkerTapListener ml) {
        markerTapListeners.removeElement(ml);        
    }
    
    public void removeAllMarkerTapListener() {
        markerTapListeners.removeAllElements();        
    }
    
     public void removeAddGeocodingListener(GeocodingListener gl) {
        geocodingListeners.removeElement(gl);
    }
     
    public void removeAllGeocodingListener() {
        geocodingListeners.removeAllElements();        
    }
    
    public void fireFeatureInfoEvent(String info, String url) {
        FeatureInfoEvent f = new FeatureInfoEvent(getMapPanel(), info, url);
        for (int i = 0; i < featureInfoListeners.size(); i++) {
            ((FeatureInfoListener) featureInfoListeners.elementAt(i)).actionPerformed(f);
        }
    }

    public void firePointerTapEvent(Point screen, Point map) {
        PointerTapEvent p = new PointerTapEvent(getMapPanel(), screen, map);
        for (int i = 0; i < pointerTapListeners.size(); i++) {
            ((PointerTapListener) pointerTapListeners.elementAt(i)).actionPerformed(p);
        }
    }
    
    public void fireMarkerTapEvent(Marker marker, Point screen, Point map) {  
        MarkerTapEvent m = new MarkerTapEvent(getMapPanel(), screen, map, marker);
        for (int i = 0; i < markerTapListeners.size(); i++) {
            ((MarkerTapListener) markerTapListeners.elementAt(i)).actionPerformed(m);
        }
    }
    
    public void fireGeocodingEvent(String requestURL) throws IOException  {
        
        HttpConnection con = (HttpConnection) Connector.open(requestURL);
        Reader reader = new InputStreamReader(con.openInputStream());
        XmlParser parser = new XmlParser(reader);        
        Document doc = new Document();
        doc.parse(parser);
        reader.close();
        con.close();
        
        Element response = doc.getRootElement();
        
        String request = response.getAttribute("request").getValue();
        String prjName = response.getAttribute("project").getValue();
        Element coordinates = response.getElement("coordinates");
        
        Result[] rs = new Result[coordinates.getChildCount()];
        
        for (int i = 0; i < coordinates.getChildCount(); i++) {
            Object ele = coordinates.getChild(i);
            if (ele instanceof Element) {
                Element coordinate = (Element) ele;
                if (coordinate.getName().equals("coordinate")) {
                    Point p = new Point(Double.parseDouble(coordinate.getAttribute("x").getValue()), Double.parseDouble(coordinate.getAttribute("y").getValue()));
                    rs[i] = new Result(coordinate.getAttribute("address").getValue(), p);                    
                }
            }
        }
        
        GeocodingEvent ev = new GeocodingEvent(getMapPanel(), prjName, request, rs);
        for (int i = 0; i < geocodingListeners.size(); i++) {
            ((GeocodingListener) geocodingListeners.elementAt(i)).actionPerformed(ev);
        }       
    }
}
