package geocoder;

import org.atlas.geocoding.Geocoder;
import org.atlas.geocoding.Result;
import org.atlas.hibernatedata.GeocoderConfig;
import org.atlas.hibernatedata.Project;
import org.atlas.hibernatedata.Source;
import java.io.*;
import java.net.*;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.*;
import javax.servlet.http.*;
import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;
import org.atlas.util.ConnectionProperties;
import org.atlas.util.Cryptography;
import org.atlas.util.DataBaseUtils;
import org.atlas.util.IOUtils;

/**
 * Servlet encargado de responder a peticiones de geocodificación.
 * @author Mario Eraso
 */
public class service extends HttpServlet {

    private static ConnectionProperties cp;
    private static Connection conn;
    private static String user_ser;
    private static String pass_ser;
    private Project[] projects;

    private void loadProjects() throws Exception {
        String query;
        ArrayList<Project> list = new ArrayList<Project>();
        query = "SELECT codeproj, codeserv, descproj, srid  FROM atlas27_project;";
        linkDataBase();

        Statement stproj = conn.createStatement();
        Statement stgeoc = conn.createStatement();
        Statement stsour = conn.createStatement();

        ResultSet rsproj = stproj.executeQuery(query);

        while (rsproj.next()) {
            Project p = new Project();
            p.setSources(new HashSet());
            p.setGeocoderConfigs(new HashSet());

            p.setCodeproj(rsproj.getInt("codeproj"));
            p.setDescproj(rsproj.getString("descproj"));

            list.add(p);

            query = "SELECT codegeoc, classname, configuration FROM atlas27_geocoder WHERE codeproj  = " + p.getCodeproj();
            stgeoc = conn.createStatement();
            ResultSet rsgeoc = stgeoc.executeQuery(query);

            while (rsgeoc.next()) {
                GeocoderConfig gc = new GeocoderConfig();
                gc.setCodegeoc(rsgeoc.getInt("codegeoc"));
                gc.setClassname(rsgeoc.getString("classname"));
                gc.setConfiguration(rsgeoc.getString("configuration"));
                p.getGeocoderConfigs().add(gc);
            }
            rsgeoc.close();

            query = "SELECT codesour, tablsour, fielsour, geomtype, xminlaye, xmaxlaye, yminlaye, ymaxlaye FROM atlas27_source WHERE codeproj = " + p.getCodeproj();
            stsour = conn.createStatement();
            ResultSet rssour = stsour.executeQuery(query);

            while (rssour.next()) {
                Source source = new Source();
                source.setCodesour(rssour.getInt("codesour"));
                source.setFielsour(rssour.getString("fielsour"));
                source.setGeomType(rssour.getString("geomtype"));
                source.setTablsour(rssour.getString("tablsour"));
                source.setXmax(rssour.getDouble("xmaxlaye"));
                source.setXmin(rssour.getDouble("xminlaye"));
                source.setYmax(rssour.getDouble("ymaxlaye"));
                source.setYmin(rssour.getDouble("yminlaye"));
                p.getSources().add(source);
            }
            rssour.close();
        }
        rsproj.close();

        stproj.close();
        stgeoc.close();
        stsour.close();

        projects = new Project[list.size()];
        list.toArray(projects);
    }

    /**
     * Responde las peticiones realizadas a geocoder mediante el método HTTP GET
     * el contenido de la URL de la petición normalmente es redactado por los componentes
     * de la biblioteca de desarrollo.
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String CODEPROJ = request.getParameter("project");
        String ADDRESS = request.getParameter("address");
        String TYPE = request.getParameter("type");
        String CODEMAP = request.getParameter("idmap");

        int codeproj = Integer.parseInt(CODEPROJ);
        if (TYPE.equals("XML")) {

            Element root = new Element("response");
            Document doc = new Document(root);

            root.setAttribute(new Attribute("request", ADDRESS));
            root.setAttribute(new Attribute("project", "p" + codeproj));

            Element coords = new Element("coordinates");
            root.getChildren().add(coords);

            for (int i = 0; i < projects.length; i++) {
                Project project = projects[i];
                if (project.getCodeproj() == codeproj) {
                    Geocoder[] geos = project.getGeocoders();
                    for (int j = 0; j < geos.length; j++) {
                        try {
                            Geocoder geocoder = geos[j];
                            if (geocoder.isCapableFor(ADDRESS)) {
                                String[] parts = geocoder.parseAdress(geocoder.getStandardizedAddress(ADDRESS));
                                Result[] rs = geocoder.locateAddress(parts);
                                for (int k = 0; k < rs.length; k++) {
                                    Result result = rs[k];
                                    coords.getChildren().add(result.getAsElement());
                                }
                                
                                if(rs.length==0){
                                    System.out.println(ADDRESS + "No results");
                                }
                                
                                break;
                            }
                            
                        } catch (Exception ex) {
                            System.out.println(ADDRESS + "\tError "+ex.getMessage());
                        }
                    }
                    break;
                }
            }
            XMLOutputter outputter = new XMLOutputter();
            outputter.output(doc, response.getOutputStream());
        } else if (TYPE.equals("JSON")) {
            StringBuilder resg = null;
            StringBuilder coords = new StringBuilder("var currectMap = atlasHandler.getMap(\"" + CODEMAP + "\"); ");

            for (int i = 0; i < projects.length; i++) {
                Project project = projects[i];
                if (project.getCodeproj() == codeproj) {
                    Geocoder[] geos = project.getGeocoders();

                    resg = new StringBuilder("{ ");
                    resg.append("request : ");
                    resg.append("\"" + ADDRESS + "\"");
                    resg.append(",");
                    resg.append("project : ");
                    resg.append(CODEPROJ);

                    for (int j = 0; j < geos.length; j++) {
                        try {
                            Geocoder geocoder = geos[j];
                            if (geocoder.isCapableFor(ADDRESS)) {
                                String[] parts = geocoder.parseAdress(geocoder.getStandardizedAddress(ADDRESS));
                                Result[] rs = geocoder.locateAddress(parts);
                                resg.append(",");
                                if (rs.length > 0) {
                                    resg.append("coordinates : [ ");
                                    for (int k = 0; k < rs.length - 1; k++) {
                                        Result result = rs[k];
                                        resg.append(result.getAsJSON());
                                        resg.append(", ");
                                    }
                                    resg.append(rs[rs.length - 1].getAsJSON());
                                    resg.append("]");
                                } else {
                                    resg.append("coordinates : null ");
                                }
                                resg.append(",");
                                resg.append(" getRequest : function(){ return this.request; }, ");
                                resg.append(" getProject : function(){ return this.project; }, ");
                                resg.append(" getCoordinates : function(){ return this.coordinates; } ");
                                break;
                            }
                        } catch (Exception ex) {
                            Logger.getLogger(service.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
                    resg.append("}");
                    break;
                }
            }
            coords.append("currectMap.responsegeocoder = ");
            coords.append(resg);
            coords.append("; ");
            coords.append("currectMap.fireListener(currectMap.geocoderListener);");

            ServletOutputStream out = response.getOutputStream();
            out.write(coords.toString().getBytes());
            out.close();
        }
    }

    /**
     * Responde a las peticiones realizadas al servlet via HTTP POST
     * estás peticiones nomalmente son señales administrativas
     * que indican instrucciones como reiniciar el servlet para
     * adoptar cambios en la configuración o en el material de referencia.
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        byte[] cont = IOUtils.readFromStream(request.getInputStream());
        OutputStream os = null;
        try {
            os = response.getOutputStream();
            String pet = Cryptography.decrypthAES(cont);

            String[] params = pet.split("&");
            String REQUEST = params[0].split("=")[1];

            String user = new String(Cryptography.toBytes(params[1].split("=")[1]));
            String pass = new String(params[2].split("=")[1]);

            if (!user.equals(user_ser) || !pass.equals(pass_ser)) {
                response.sendError(404, "The password and/or username are not valid, check and try again");
                return;
            }

            if (REQUEST.equals("ForceReload")) {
                start();
                response.getOutputStream().write(0);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            response.sendError(500);
        } finally {
            try {
                os.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }

    @Override
    public String getServletInfo() {
        return "Short description";
    }

    /**
     * Método de inicialización llamado por el contenedor de servlets. 
     * Se encarga de verificar la conexión con la base de datos y configurar
     * los geocoder asociados a todos los proyectos con el fin de que estén listos para
     * contestar peticiones.
     * @param config
     * @throws javax.servlet.ServletException
     */
    @Override
    public void init(ServletConfig config) throws ServletException {
        try {
            super.init(config);
            String driver = config.getInitParameter("driver");
            String host = config.getInitParameter("host");
            String port = config.getInitParameter("port");
            String database = config.getInitParameter("database");
            String user_db = config.getInitParameter("user_db");
            String pass_db = null;
            pass_db = new String(Cryptography.decrypthAES(Cryptography.toBytes(config.getInitParameter("pass_db"))));
            user_ser = config.getInitParameter("user_ser");
            pass_ser = config.getInitParameter("pass_ser");
            cp = new ConnectionProperties(driver, host, port, database, user_db, pass_db);
            start();

        } catch (Exception ex) {
            Logger.getLogger(service.class.getName()).log(Level.SEVERE, null, ex);
            throw new ServletException(ex.getMessage());
        }
    }

    private void start() throws Exception {
        linkDataBase();
        loadProjects();
        for (int i = 0; i < projects.length; i++) {
            Project project = projects[i];
            project.loadGeocoders();
            try {
                Geocoder[] geocs = project.getGeocoders();
                for (int j = 0; j < geocs.length; j++) {
                    Geocoder geocoder = geocs[j];
                    geocoder.init(cp);
                }
            } catch (IllegalStateException ex) {
                ex.printStackTrace();
            }

        }
    }

    private void linkDataBase() throws Exception {
        if (!DataBaseUtils.isValidConnection(conn)) {
            try {
                conn = DataBaseUtils.createConnection(cp);
            } catch (Exception ex) {
                ex.printStackTrace();
                throw new Exception("Unable to get a connection with the data base");
            }
        }
    }
}
