package wmsserver;

import org.atlas.service.CRS;
import org.atlas.service.ServerBuffer;
import com.objectplanet.image.PngEncoder;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.ParseException;
import com.vividsolutions.jts.io.WKTReader;
import org.atlas.hibernatedata.Layer;
import org.atlas.hibernatedata.Service;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLEncoder;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Enumeration;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.atlas.service.BasicRender;
import org.atlas.service.Feature;
import org.atlas.util.BoundingBox;
import org.atlas.util.ConnectionProperties;
import org.atlas.util.CoordinateUtils;
import org.atlas.util.Cryptography;
import org.atlas.util.DataBaseUtils;
import org.atlas.util.IOUtils;
import org.atlas.util.JAIUtils;

/**
 * Servlet principal, encargado de responder peticiones administrativas 
 * y de generación de mapas de vital impotacia para el sistema.
 * @author Mario
 */
public class service extends HttpServlet {

    private static ConnectionProperties cp;
    private static ServerBuffer buffer;
    private static Connection conn;
    private static String user_ser;
    private static String pass_ser;
    private long maxCacheSize;
    private long currCacheSize = 0;

    /**
     * Encargado de responder todas las peticiones realizadas mediante HTTP POST
     * reservado para funciones administrativas como:
     * <ul>
     * <li>Reportar conectividad entre el servlet y la base de datos</li>
     * <li>Entregar los datos de conexión a la base de datos al panel de control</li>
     * <li>Re iniciar el servlet a fin de adoptar cambios en las configuración o en el material de georeferencial</li>
     * </ul>
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        byte[] cont = IOUtils.readFromStream(request.getInputStream());
        OutputStream os = null;
        try {
            os = response.getOutputStream();
            String pet = Cryptography.decrypthAES(cont);

            String[] params = pet.split("&");
            String REQUEST = params[0].split("=")[1];

            String user = new String(Cryptography.toBytes(params[1].split("=")[1]));
            String pass = new String(params[2].split("=")[1]);

            if (!user.equals(user_ser) || !pass.equals(pass_ser)) {
                response.sendError(404, "The password and/or username are not valid, check and try again");
                return;
            }

            if (REQUEST.equals("GetConnection")) {
                String info = Cryptography.toHexadecimal(cp.getDriver().getBytes());
                info += ("g" + Cryptography.toHexadecimal(cp.getData().getBytes()));
                info += ("g" + Cryptography.toHexadecimal(cp.getHost().getBytes()));
                info += ("g" + Cryptography.toHexadecimal(cp.getPass().getBytes()));
                info += ("g" + Cryptography.toHexadecimal(cp.getPort().getBytes()));
                info += ("g" + Cryptography.toHexadecimal(cp.getUser().getBytes()));
                byte[] binfo = Cryptography.encrypthAES(info);
                response.getOutputStream().write(binfo);
            } else if (REQUEST.equals("GetTest")) {

                String msg = " ";
                String cla = " ";
                String sta;
                try {
                    buffer.isDataBaseReady();
                    sta = "true";
                } catch (Exception ex) {
                    sta = "false";
                    msg = URLEncoder.encode(ex.getMessage(), "UTF-8");
                    cla = URLEncoder.encode(ex.getClass().getName(), "UTF-8");
                    ex.printStackTrace();
                }
                response.getOutputStream().write(("enabled=" + sta + "&class=" + cla + "&msg=" + msg).getBytes());
            } else if (REQUEST.equals("ForceReload")) {
                buffer.invalidate();
                deleteCache();
                response.getOutputStream().write(0);
            } else if (REQUEST.equals("GetSignature")) {
                response.getOutputStream().write(Service.SIGNATURE.getBytes());
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            response.sendError(500);
        } finally {
            try {
                os.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }

    private int[] parseLayerList(String list) {
        int[] res = new int[0];
        String[] parts = list.split(",");
        for (int i = 0; i < parts.length; i++) {
            String code = parts[i].substring(1);
            int[] tmp = new int[res.length + 1];
            System.arraycopy(res, 0, tmp, 0, res.length);
            tmp[res.length] = Integer.parseInt(code);
            res = tmp;
        }
        return res;
    }

    /**
     * Encargado de responder las peticiones realizadas via HTTP GET, que comprenden
     * las tareas asociadas al servicio WMS, asi como recuperación de leyendas por capa
     * e información sobre sistema de referencia.
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        System.out.println(request.getQueryString());
        //getCapabilities
        String VERSION = "";//version         O
        String SERVICE = "";//WMS             M
        String REQUEST = "";//GetCapabilities M
        String FORMAT = "";//MIME_type        O
        String UPDATESEQUENCE = "";//string   O

        //getMap
        //String VERSION="";//1.3.0             M
        //String REQUEST="";//GetMap            M
        String LAYERS = "";//layer_list           M
        String STYLES = "";//style_list           M
        String CRS = "";//namespace:identifier    M
        String BBOX = "";//minx,miny,maxx,maxy    M
        String WIDTH = "";//output_width          M
        String HEIGHT = "";//output_height        M
        //String FORMAT="";//output_format      M
        String TRANSPARENT = "";//TRUE|FALSE      O
        String BGCOLOR = "";//color_value         O
        String EXCEPTIONS = "";//exception_format O
        String TIME = "";//time                   O
        String ELEVATION = "";//elevation         O
        //String Other sample dimension(s)      O

        //getFeatureInfo
        //String VERSION="";//1.3.0                        M
        //String REQUEST="";//GetFeatureInfo               M
        //String map request part                          M
        String QUERY_LAYERS = "";//layer_list                M
        String INFO_FORMAT = "";//output_format              M
        String FEATURE_COUNT = "";//number                   O
        String I = "";//pixel_column                         M
        String J = "";//pixel_row                            M
        //String EXCEPTIONS="";//exception_format          O
        //atlas
        String ADMIN_ENTRACE = "";
        String EPSG_CODE = "";

        String DISTANCE = "";
        String IDMAP = "";

        Enumeration params = request.getParameterNames();
        while (params.hasMoreElements()) {
            String param = String.valueOf(params.nextElement());
            String upperParam = param.toUpperCase();
            if (upperParam.equals("VERSION")) {
                VERSION = request.getParameter(param);
            } else if (upperParam.equals("SERVICE")) {
                SERVICE = request.getParameter(param);
            } else if (upperParam.equals("REQUEST")) {
                REQUEST = request.getParameter(param);
            } else if (upperParam.equals("FORMAT")) {
                FORMAT = request.getParameter(param);
            } else if (upperParam.equals("UPDATESEQUENCE")) {
                UPDATESEQUENCE = request.getParameter(param);
            } else if (upperParam.equals("LAYERS")) {
                LAYERS = request.getParameter(param);
            } else if (upperParam.equals("STYLES")) {
                STYLES = request.getParameter(param) + "yes";
            } else if (upperParam.equals("CRS") || upperParam.equals("SRS")) {
                CRS = request.getParameter(param);
            } else if (upperParam.equals("BBOX")) {
                BBOX = request.getParameter(param);
            } else if (upperParam.equals("WIDTH")) {
                WIDTH = request.getParameter(param);
            } else if (upperParam.equals("HEIGHT")) {
                HEIGHT = request.getParameter(param);
            } else if (upperParam.equals("TRANSPARENT")) {
                TRANSPARENT = request.getParameter(param);
            } else if (upperParam.equals("BGCOLOR")) {
                BGCOLOR = request.getParameter(param);
            } else if (upperParam.equals("EXCEPTIONS")) {
                EXCEPTIONS = request.getParameter(param);
            } else if (upperParam.equals("TIME")) {
                TIME = request.getParameter(param);
            } else if (upperParam.equals("QUERY_LAYERS")) {
                QUERY_LAYERS = request.getParameter(param);
            } else if (upperParam.equals("INFO_FORMAT")) {
                INFO_FORMAT = request.getParameter(param);
            } else if (upperParam.equals("FEATURE_COUNT")) {
                FEATURE_COUNT = request.getParameter(param);
            } else if (upperParam.equals("I") || upperParam.equals("X")) {
                I = request.getParameter(param);
            } else if (upperParam.equals("J") || upperParam.equals("Y")) {
                J = request.getParameter(param);
            } else if (upperParam.equals("ADMIN_ENTRACE")) {
                ADMIN_ENTRACE = request.getParameter(param);
            } else if (upperParam.equals("EPSG_CODE")) {
                EPSG_CODE = request.getParameter(param);
            } else if (upperParam.equals("DISTANCE")) {
                DISTANCE = request.getParameter(param);
            } else if (upperParam.equals("IDMAP")) {
                IDMAP = request.getParameter(param);
            }
        }

        String[] valipara = {VERSION, LAYERS, STYLES, CRS, BBOX, WIDTH, HEIGHT, QUERY_LAYERS, INFO_FORMAT, FEATURE_COUNT, I, J, FORMAT, DISTANCE};

        if (ADMIN_ENTRACE.equals("")) {
            try {
                if (!buffer.isServiceEnabled().booleanValue()) {
                    response.sendError(503, "The sever is under maintenace, please try again later");
                    return;
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                response.sendError(503, ex.getMessage());
                return;
            }
        }

        if (REQUEST.equals("")) {
            response.sendError(400, "The request parameter is mandatory");
            return;
        }

        if (REQUEST.equals("GetCapabilities") == true) {

            if (!SERVICE.equals("WMS")) {
                response.sendError(400, "The service parameters must be set to WMS");
                return;
            }

            if (VERSION.equals("")) {
                VERSION = "1.1.1";
            } else {
                if (!validateVersion(response, VERSION)) {
                    return;
                }
                String[] pa = VERSION.split("\\.");
                int version = Integer.parseInt(pa[0] + pa[1] + pa[2]);
                if (version >= 130) {
                    VERSION = "1.3.0";
                }
                if (version >= 111 && version < 130) {
                    VERSION = "1.1.1";
                }
                if (version >= 110 && version < 111) {
                    VERSION = "1.1.0";
                }
                if (version < 110) {
                    VERSION = "1.1.0";
                }
            }

            response.setContentType("text/xml;charset=UTF-8");
            response.setCharacterEncoding("UTF-8");

            OutputStream os = response.getOutputStream();

            try {
                os.write(buffer.getCapabilities(VERSION));
            } catch (Exception ex) {
                response.sendError(500, "Unexpeted Error: " + ex.getMessage());
            }
            os.close();
            return;
        }

        if (REQUEST.equals("GetMap") == true) {

            if (!validateRequest(response, REQUEST, valipara)) {
                return;
            }

            if (!(FORMAT.equals("image/png") || FORMAT.equals("image/jpeg"))) {
                response.sendError(400, "Unsupported image format " + FORMAT);
                return;
            }

            int[] codes = parseLayerList(LAYERS);

            String[] bboxs = BBOX.split(",");

            double minx = Double.parseDouble(bboxs[0]);
            double miny = Double.parseDouble(bboxs[1]);
            double maxx = Double.parseDouble(bboxs[2]);
            double maxy = Double.parseDouble(bboxs[3]);

            int width = Integer.parseInt(WIDTH);
            int heigth = Integer.parseInt(HEIGHT);

            String name = "";

            for (int i = 0; i < codes.length - 1; i++) {
                name += String.format("%s,", codes[i]);
            }
            name += String.format("%s", codes[codes.length - 1]);


            if (FORMAT.equals("image/png")) {
                name = String.format("/%s&%f,%f,%f,%f=%d,%d.%s", name, minx, miny, maxx, maxy, width, heigth, ".png");
            } else {
                //FORMAT.equals("image/jpeg"))
                name = String.format("/%s&%f,%f,%f,%f=%d,%d.%s", name, minx, miny, maxx, maxy, width, heigth, ".jpeg");
            }




            String uri = super.getServletContext().getRealPath(name);
            File file = new File(uri);
            if (file.exists()) {
                FileInputStream fis = new FileInputStream(file);
                BufferedInputStream bis = new BufferedInputStream(fis);
                response.setContentType(FORMAT);
                OutputStream os = response.getOutputStream();
                response.addIntHeader("Content-Length", (int) file.length());
                BufferedOutputStream bos = new BufferedOutputStream(os);
                bos.write(IOUtils.readFromStream(bis));
                bos.close();
                os.close();
                bis.close();
                fis.close();
                return;
            } else {
                Layer[] array = new Layer[codes.length];
                for (int i = 0; i < codes.length; i++) {
                    try {
                        Layer l = buffer.getLayer(codes[i]);
                        array[i] = l;
                    } catch (Exception ex) {
                        ex.printStackTrace();
                        response.sendError(500, ex.getMessage());
                        return;
                    }
                }

                boolean transparent = false;

                if (TRANSPARENT.toUpperCase().equals("TRUE")) {
                    transparent = true;
                }


                //          String BGCOLOR="";//color_value         O
                //          String EXCEPTIONS="";//exception_format O

                int icrs = Integer.parseInt(CRS.substring("EPSG:".length(), CRS.length()));
                CRS crs = null;
                try {
                    crs = buffer.getCRS(icrs);
                } catch (Exception ex) {
                    ex.printStackTrace();
                    response.sendError(500);
                    return;
                }

                BasicRender bRender = new BasicRender();
                BufferedImage img;
                try {
                    img = bRender.wmsRender(array, crs, minx, miny, maxx, maxy, width, heigth, transparent, null);
                } catch (Exception ex) {
                    ex.printStackTrace();
                    response.sendError(500);
                    return;
                }

                byte[] bin;
                if (FORMAT.equals("image/png")) {
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    if (transparent) {
                        (new PngEncoder(PngEncoder.COLOR_TRUECOLOR_ALPHA)).encode(img, baos);
                    } else {
                        (new PngEncoder(PngEncoder.COLOR_TRUECOLOR)).encode(img, baos);
                    }
                    bin = baos.toByteArray();
                } else {
                    bin = JAIUtils.encondeJPEG(img);
                }

                response.setContentType(FORMAT);
                OutputStream os = response.getOutputStream();
                response.addIntHeader("Content-Length", bin.length);
                os.write(bin);

                String root = super.getServletContext().getRealPath("");
                if (bin.length > maxCacheSize) {
                    return;
                }

                for (int tries = 0; !(currCacheSize <= maxCacheSize - bin.length) && tries < 16; tries++) {
                    try {
                        File f = new File(root);
                        String[] names = f.list();
                        int i = (int) (Math.random() * names.length);
                        File bye = new File(root + "\\" + names[i]);
                        long l = bye.length();
                        bye.delete();
                        currCacheSize -= l;
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }

                if (bin.length > maxCacheSize - currCacheSize) {
                    readCache();
                    return;
                }

                FileOutputStream fos = new FileOutputStream(file);
                BufferedOutputStream bos = new BufferedOutputStream(fos);
                bos.write(bin);
                bos.close();
                fos.close();
                currCacheSize += bin.length;
                return;
            }
        }

        if (REQUEST.equals("GetFeatureInfo") == true) {

            if (!this.validateRequest(response, REQUEST, valipara)) {
                return;
            }

            int[] codes = parseLayerList(QUERY_LAYERS);
            Layer[] array = new Layer[codes.length];

            for (int i = 0; i < codes.length; i++) {
                try {
                    Layer l = buffer.getLayer(codes[i]);
                    array[i] = l;
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }

            String[] bboxs = BBOX.split(",");
            int icrs = Integer.parseInt(CRS.substring("EPSG:".length(), CRS.length()));
            BoundingBox bbox = new BoundingBox(Double.parseDouble(bboxs[0]), Double.parseDouble(bboxs[1]), Double.parseDouble(bboxs[2]), Double.parseDouble(bboxs[3]), icrs);
            int width = Integer.parseInt(WIDTH);
            int heigth = Integer.parseInt(HEIGHT);

            CRS crs = null;
            try {
                crs = buffer.getCRS(icrs);
            } catch (Exception ex) {
                ex.printStackTrace();
                response.sendError(500);
                return;
            }

            int ipix = Integer.parseInt(I);
            int jpix = Integer.parseInt(J);

            if (DISTANCE.equals("")) {
                DISTANCE = "3";
            }
            double dist = Double.parseDouble(DISTANCE);

            double[] pointXY = CoordinateUtils.translateIJtoXY(bbox, width, heigth, ipix, jpix, crs.getAxis1(), crs.getAxis2());

            double dx, dy;
            if (crs.getAxis1().equals("east") || crs.getAxis1().equals("west")) {
                dx = dist * ((bbox.getMaxx() - bbox.getMinx()) / width);
                dy = dist * ((bbox.getMaxy() - bbox.getMiny()) / heigth);
            } else {
                dx = dist * ((bbox.getMaxx() - bbox.getMinx()) / heigth);
                dy = dist * ((bbox.getMaxy() - bbox.getMiny()) / width);
            }

            WKTReader wktReader = new WKTReader();
            Geometry geo = null;

            String wkt = "POLYGON ((";
            for (double t = -180; t <= 180; t++) {
                double x = pointXY[0] + (dx * Math.cos(Math.toRadians(t)));
                double y = pointXY[1] + (dy * Math.sin(Math.toRadians(t)));
                wkt += " " + x + " " + y + ",";
            }
            double x = pointXY[0] + (dx * Math.cos(Math.toRadians(-180)));
            double y = pointXY[1] + (dy * Math.sin(Math.toRadians(-180)));
            wkt += x + " " + y + "))";

            try {
                geo = wktReader.read(wkt);
            } catch (ParseException ex) {
                ex.printStackTrace();
                response.sendError(500, "wkt invalido");
                return;
            }

            Envelope env = geo.getEnvelopeInternal();

            try {
                linkDataBase();
            } catch (Exception ex) {
                ex.printStackTrace();
                response.sendError(500, "Error Conection width Data Base");
            }

            StringBuilder format = new StringBuilder(Layer.getHeaderFormat(INFO_FORMAT, icrs, env, new URL(request.getRequestURL().toString()))); //TODO modificame
            for (int i = 0; i < array.length; i++) {

                Envelope[] listBbox = array[i].getSource().getBboxes();
                Feature[] listFeat = array[i].getSource().getFeatures();
                ArrayList features = new ArrayList();

                if (array[i].isQueryable()) {
                    for (int j = 0; j < listBbox.length; j++) {
                        if (env.intersects((Envelope) listBbox[j])) {
                            Feature feature = listFeat[j];
                            if (geo.intersects(feature.getGeom())) {
                                features.add(feature);
                            }
                        }
                    }
                }

//                Collections.sort(features, new Comparator(){
//                    public int compare(Object o1, Object o2){
//                        int v1 = ((Feature)o1).getId();
//                        int v2 = ((Feature)o2).getId();
//                        if( v1 < v2 ) return -1;
//                        if( v1 == v2 ) return 0;
//                        return 1;
//                    }
//                });

                format.append(array[i].getFormatFeatureInfo(conn, features, INFO_FORMAT, icrs));
            }
            format.append(Layer.getFooterFormat(INFO_FORMAT));

            response.setContentType(INFO_FORMAT);
            response.setCharacterEncoding("UTF-8");
            response.setContentLength(format.length());
            OutputStream os = response.getOutputStream();
            os.write(format.toString().getBytes());
            os.close();
            return;

        }

        if (REQUEST.equals("GetJSON") == true) {

            response.setContentType("text/javascript");
            response.setCharacterEncoding("UTF-8");
            OutputStream os = response.getOutputStream();
            String temp = "";

            try {

                temp = "var currectMap = atlasHandler.getMap(\"" + IDMAP + "\"); ";
                temp += "currectMap.setProjects(";
                os.write(temp.getBytes());
                byte[] json = buffer.getCapabilities("json");
                if (json != null) {
                    os.write(json);
                } else {
                    os.write("null".getBytes());
                }
                os.write("); currectMap.fireListener(currectMap.loaddataListener);".getBytes());

            } catch (Exception ex) {
                response.sendError(500, ex.getMessage());
            } finally {
                os.close();
                return;
            }
        }

        if (REQUEST.equals("GetLegend")) {
            try {
                int[] codes = parseLayerList(LAYERS);
                if (codes.length == 1) {
                    response.setContentType("image/png");
                    OutputStream os = response.getOutputStream();
                    os.write(buffer.getLayer(codes[0]).getBinalegend());
                    os.close();
                } else {
                    response.sendError(400, "Layer parameter " + LAYERS + "invalid");
                }
            } catch (Exception ex) {
                response.sendError(500, "Unexpeted Error: " + ex.getMessage());
            }


        }

        if (REQUEST.equals("GetSchema")) {
            InputStream schema = null;
            if (VERSION.equals("1.1.0")) {
                response.setContentType("text/plain;charset=UTF-8");
                schema = this.getClass().getResourceAsStream("/schemas/WMS_Capabilities_1.1.0.dtd");
            }
            if (VERSION.equals("1.1.1")) {
                response.setContentType("text/plain;charset=UTF-8");
                schema = this.getClass().getResourceAsStream("/schemas/WMS_Capabilities_1.1.1.dtd");
            }

            response.setCharacterEncoding("UTF-8");
            OutputStream os = response.getOutputStream();
            try {
                os.write(IOUtils.readFromStream(schema));
            } catch (Exception ex) {
                response.sendError(500, "Unexpeted Error: " + ex.getMessage());
            }
            os.close();
            return;
        }

        if (REQUEST.equals("GetEPSGCodeInfo")) {
            if (EPSG_CODE.equals("")) {
                response.sendError(400, "An EPSG code must be provided");
                return;
            }
            try {
                int code = Integer.parseInt(EPSG_CODE);
                response.setContentType("text/xml;charset=UTF-8");
                response.setCharacterEncoding("UTF-8");
                OutputStream os = response.getOutputStream();
                try {
                    os.write(buffer.getCRS(code).getDocument());
                } catch (Exception ex) {
                    response.sendError(500, ex.getMessage());
                }
                os.close();
                return;
            } catch (NumberFormatException ex) {
                ex.printStackTrace();
                response.sendError(400, "The EPSG code must be an integer number");
                return;
            }
        }
    }

    private boolean validateRequest(HttpServletResponse response, String request, String[] param) throws ServletException, IOException {

        String VERSION = param[0];
        String LAYERS = param[1];
        String STYLES = param[2];
        String CRS = param[3];
        String BBOX = param[4];
        String WIDTH = param[5];
        String HEIGHT = param[6];
        String QUERY_LAYERS = param[7];
        String INFO_FORMAT = param[8];
        String FEATURE_COUNT = param[9];
        String I = param[10];
        String J = param[11];
        String FORMAT = param[12];
        String DISTANCE = param[13];

        if (request.equals("GetMap")) {


            if (VERSION.equals("")) {
                response.sendError(400, "Version parameter is mandatory");
                return false;
            } else {
                if (!validateVersion(response, VERSION)) {
                    return false;
                }
            }

            if (LAYERS.equals("")) {
                response.sendError(400, "Layers parameter must not be empty");
                return false;
            }

            String[] layers = LAYERS.split(",");
            Layer[] array = new Layer[layers.length];
            if (layers.length == 0) {
                response.sendError(400, "Layers parameters must be a non empty list");
                return false;
            }

            if (STYLES.equals("")) {
                response.sendError(400, "The styles parameter is mandatory, but may be an empty string");
                return false;
            }

            if (FORMAT.equals("")) {
                response.sendError(400, "The format parameter is mandatory");
                return false;
            }

            if (CRS.equals("")) {
                if (VERSION.equals("1.1.0") || VERSION.equals("1.1.1")) {
                    response.sendError(400, "The srs parameter is mandatory");
                }

                if (VERSION.equals("1.3.0")) {
                    response.sendError(400, "The crs parameter is mandatory");
                }
                return false;
            }

            if (!CRS.startsWith("EPSG:")) {
                if (VERSION.equals("1.1.0") || VERSION.equals("1.1.1")) {
                    response.sendError(400, "The srs parameter must be in the form  EPSG:xxxx");
                }

                if (VERSION.equals("1.3.0")) {
                    response.sendError(400, "The crs parameter must be in the form  EPSG:xxxx");
                }
                return false;
            }


            //TODO VALIDAR QUE EL CODIGO EPSG SOLICIDADO CONCUERDE CON LAS CAPAS

            if (BBOX.equals("")) {
                response.sendError(400, "The bbox parameter is mandatory");
                return false;
            }

            String[] bboxs = BBOX.split(",");
            if (bboxs.length != 4) {
                response.sendError(400, "BBOX must be a list of 4 elements separated by comma.");
                return false;
            }

            try {
                double minx = Double.parseDouble(bboxs[0]);
                double miny = Double.parseDouble(bboxs[1]);
                double maxx = Double.parseDouble(bboxs[2]);
                double maxy = Double.parseDouble(bboxs[3]);
            } catch (NumberFormatException ex) {
                response.sendError(400, "BBOX must be a list of 4 elements separated by comma.");
                return false;
            }

            if (WIDTH.equals("")) {
                response.sendError(400, "The width parameter is mandatory");
                return false;
            }

            if (HEIGHT.equals("")) {
                response.sendError(400, "The height parameter is mandatory");
                return false;
            }

            try {
                int width = Integer.parseInt(WIDTH);
                int heigth = Integer.parseInt(HEIGHT);
                if (width <= 0 || heigth <= 0) {
                    response.sendError(400, "The parameters width and height must be positive numbers");
                    return false;
                }
            } catch (NumberFormatException ex) {
                response.sendError(400, "The width and heigth must be integer parameters");
                return false;
            }

        }

        if (request.equals("GetFeatureInfo")) {

            if (!validateRequest(response, "GetMap", param)) {
                return false;
            }

            if (QUERY_LAYERS.equals("")) {
                response.sendError(400, "The query_layers parameter is mandatory");
                return false;
            }

            String[] qlayers = QUERY_LAYERS.split(",");
            if (qlayers.length == 0) {
                response.sendError(400, "The query_layers parameters must be a non empty list");
                return false;
            }

            if (INFO_FORMAT.equals("")) {
                response.sendError(400, "The info_format parameter is mandatory");
                return false;
            } else if (!INFO_FORMAT.toLowerCase().equals("text/plain") && !INFO_FORMAT.toLowerCase().equals("text/html") && !INFO_FORMAT.toLowerCase().equals("application/vnd.ogc.gml")) {
                response.sendError(400, "Invalid Format, unsupported info_format " + INFO_FORMAT);
                return false;
            }

            if (FEATURE_COUNT.equals("")) {
                FEATURE_COUNT = "1";
            }

            if (I.equals("")) {
                response.sendError(400, "The parameter i is mandatory");
                return false;
            } else {
                try {
                    int ipix = Integer.parseInt(I);
                } catch (NumberFormatException ex) {
                    response.sendError(400, "The i parameter must be integer");
                    return false;
                }
            }


            if (J.equals("")) {
                response.sendError(400, "The parameters j is mandatory");
                return false;
            } else {
                try {
                    int jpix = Integer.parseInt(J);
                } catch (NumberFormatException ex) {
                    response.sendError(400, "The j parameter must be integer");
                    return false;
                }
            }

            if (!DISTANCE.equals("")) {
                try {
                    Integer.parseInt(DISTANCE);
                } catch (NumberFormatException ex) {
                    response.sendError(400, "The distance must be integer parameters");
                    return false;
                }
            }
        }

        return true;
    }

    private boolean validateVersion(HttpServletResponse response, String VERSION) throws ServletException, IOException {
        String[] pa = VERSION.split("\\.");

        if (pa.length != 3) {
            response.sendError(400, "The version parameter " + VERSION + " ,is invalid");
            return false;
        } else {
            try {
                int pv1 = Integer.parseInt(pa[0]);
                int pv2 = Integer.parseInt(pa[1]);
                int pv3 = Integer.parseInt(pa[2]);

                if (pv1 < 0 || pv2 < 0 || pv3 < 0) {
                    response.sendError(400, " The parameter version " + VERSION + " must be positive numbers");
                    return false;
                }

            } catch (NumberFormatException ex) {
                response.sendError(400, "The parameter version " + VERSION + " must be integer");
                return false;
            }
        }
        return true;
    }

    private void linkDataBase() throws Exception {
        if (!DataBaseUtils.isValidConnection(conn)) {
            try {
                conn = DataBaseUtils.createConnection(cp);
            } catch (Exception ex) {
                ex.printStackTrace();
                throw new Exception("Unable to get a connection with the data base");
            }
        }
    }

    /**
     * 
     * @param config
     * @throws javax.servlet.ServletException
     */
    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        String driver = config.getInitParameter("driver");
        String host = config.getInitParameter("host");
        String port = config.getInitParameter("port");
        String database = config.getInitParameter("database");
        String user_db = config.getInitParameter("user_db");
        String pass_db = null;
        maxCacheSize = Integer.parseInt(config.getInitParameter("cache")) * 1048576;
        try {
            pass_db = new String(Cryptography.decrypthAES(Cryptography.toBytes(config.getInitParameter("pass_db"))));
        } catch (Exception ex) {
            throw (new ServletException());
        }
        user_ser = config.getInitParameter("user_ser");
        pass_ser = config.getInitParameter("pass_ser");

        cp = new ConnectionProperties(driver, host, port, database, user_db, pass_db);
        buffer = new ServerBuffer(cp);
        try {
            linkDataBase();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        readCache();
    }

    private void deleteCache() {
        String uri = super.getServletContext().getRealPath("");
        File f = new File(uri);
        String[] names = f.list();
        for (int i = 0; i < names.length; i++) {
            File f1 = new File(uri + "\\" + names[i]);
            f1.delete();
        }
        currCacheSize = 0;
    }

    private void readCache() {
        String uri = super.getServletContext().getRealPath("");
        File f = new File(uri);
        String[] names = f.list();
        for (int i = 0; i < names.length; i++) {
            File f1 = new File(uri + "\\" + names[i]);
            currCacheSize += f1.length();
        }
        System.out.println("Cache readed: current size " + currCacheSize);
    }
}
